% AFMMGPU6 Computational experiment for the paper "Adaptive fast
% multipole method on the GPU".

% S. Engblom 2012-03-27

if exist('./examples/data/AFMMgpu6.mat','file')
  warning('Existing file AFMMgpu6.mat NOT overwritten.');
end

% arguments
ndirect = 45;
ndirect_cpu = 35;
tol = 1e-6;

% *** 1st test; uniform and two non-uniform distributions
% vs. increasing N ***

% reproducible results
rand('state',20120327);
randn('state',20120327);
fprintf('GPU, increasing N...\n');

% output
nn = round(2.^(14:0.5:24));
t = zeros(3,size(nn,2));

% accurate timings pluts smoother graph
Ntrymax = 21;
Ntrymin = 5;
Tmin = 5e20;

% build distributions
N = nn(end);
m = ones(N,1);
% (1) uniform
clear z
z{1} = complex(rand(N,1),rand(N,1));

fmm2dCUDA(z{1},m,'pot',1,'ndirect',ndirect,'tol',tol,'out','time', ...
          'cutoff',1e-15,'xopt',1e-16); %dummy

% (2) normal N(0,1/10), rejected to stay within the unit square
x = zeros(1,0);
y = zeros(1,0);
while numel(x) < N || numel(y) < N
  x = [x; abs(1/10*randn(N-numel(x),1))];
  y = [y; abs(1/10*randn(N-numel(y),1))];
  x(x > 1) = [];
  y(y > 1) = [];
end
z{2} = complex(x,y);
% (3) 'layered' distribution (combination of the above)
z{3} = complex(real(z{1}),imag(z{2}));

% dry run
tt_ = fmm2dCUDA(z{1},m,'pot',1,'ndirect',ndirect,'tol',tol,'out','time', ...
                'cutoff',1e-15,'xopt',1e-16);

% main loop
for i = 1:numel(nn)
  N = nn(i);
  for k = 1:3
    zz = z{k}(1:N); mm = m(1:N);
    j = 0;
    T = 0;
    tt = zeros(1,Ntrymax);
    while j < Ntrymin || (T < Tmin && j < Ntrymax)
      j = j+1;
      tt_ = fmm2dCUDA(zz,mm,'pot',1,'ndirect',ndirect,'tol',tol, ...
                      'out','time','cutoff',1e-15,'xopt',1e-16);
      tt(j) = tt_(10);
      cuda_timing_adaptive_n(i,k,j,:)=tt_;
      T = T+tt(j);
    end
    % form a robust mean - "trimean with discarded outliers" - to get a
    % smooth graph
    tt = sort(tt(1:j)); tt([1 end]) = [];
    % [1:j-2] = [1:a a+1:b b+1:j-2] with the first and last interval of
    % equal length
    a = floor((j-2)/3); b = j-2-a;
    t(k,i) = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));
  end
  disp(['i = ' num2str(i)]);
end

% *** 2nd test; using increasingly non-uniform distributions
% vs. uniform distribution, CPU/GPU-versions ***

% kept fix
N = 5e5;
m = rand(N,1);

% range of shape parameter
sigma2 = 1./2.^linspace(-2,5,15);

% uniform
tgpu0 = 0;
tcpu0 = 0;

% normal
tgpu1 = zeros(size(sigma2));
tcpu1 = zeros(size(sigma2));

% "layer"
tgpu2 = zeros(size(sigma2));
tcpu2 = zeros(size(sigma2));

% to get accurate timings and a smooth graph
Ntrymax = 15;
Ntrymin = 5;
Tmin = 5e20;
m = ones(N,1);

fprintf('GPU...\n');

% (0) uniform distribution
rand('state',20120328);
z = complex(rand(N,1),rand(N,1));

j = 0;
T = 0;
tt = zeros(1,Ntrymax);
while j < Ntrymin || (T < Tmin && j < Ntrymax)
  j = j+1;
  tt_ = fmm2dCUDA(z,m,'pot',1,'tol',tol,'ndirect',ndirect,'out','time', ...
                  'cutoff',1e-15,'xopt',1e-16);
  tt(j) = tt_(10);
  cuda_timing_adaptive_uniform(j,:)=tt_;
  T = T+tt(j);
end
tt = sort(tt(1:j)); tt([1 end]) = [];
a = floor((j-2)/3); b = j-2-a;
tgpu0 = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));

for i = 1:numel(sigma2)
  randn('state',20120328);

  % (1) normal distribution N(0,sigma2), rejected to the unit square
  x = zeros(1,0);
  y = zeros(1,0);
  while numel(x) < N || numel(y) < N
    x = [abs(sigma2(i)*randn(N-numel(x),1)); x];
    y = [y; abs(sigma2(i)*randn(N-numel(y),1))];
    x(x > 1) = [];
    y(y > 1) = [];
  end
  z = complex(x,y);

  j = 0;
  T = 0;
  tt = zeros(1,Ntrymax);
  while j < Ntrymin || (T < Tmin && j < Ntrymax)
    j = j+1;
    tt_ = fmm2dCUDA(z,m,'pot',1,'tol',tol,'ndirect',ndirect,'out','time', ...
                    'cutoff',1e-15,'xopt',1e-16);
    tt(j) = tt_(10);
    cuda_timing_adaptive_sigma(i,j,:)=tt_;
    T = T+tt(j);
  end
  tt = sort(tt(1:j)); tt([1 end]) = [];
  a = floor((j-2)/3); b = j-2-a;
  tgpu1(i) = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));
  
  % (2) as above, but the x-coordinate is now uniformly distributed so
  % that an artificial "boundary layer" results
  z = complex(rand(N,1),imag(z));

  % adaptive version
  j = 0;
  T = 0;
  tt = zeros(1,Ntrymax);
  while j < Ntrymin || (T < Tmin && j < Ntrymax)
    j = j+1;
    tt_ = cputime;
    tt_ = fmm2dCUDA(z,m,'pot',1,'tol',tol,'ndirect',ndirect,'out','time', ...
                    'cutoff',1e-15,'xopt',1e-16);
    tt(j) = tt_(10);
    cuda_timing_adaptive_sigma_layer(i,j,:)=tt_;
    T = T+tt(j);
  end
  tt = sort(tt(1:j)); tt([1 end]) = [];
  a = floor((j-2)/3); b = j-2-a;
  tgpu2(i) = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));

  disp(['i = ' num2str(i)]);
end

% same run as above, but for the CPU
fprintf('CPU...\n');

% (0) uniform distribution
rand('state',20120328);
z = complex(rand(N,1),rand(N,1));

fmm2dCPU(z,m,'pot',1,'tol',tol,'ndirect',ndirect_cpu,'out','time', ...
         'cutoff',1e-15,'xopt',1e-16); %dummy

j = 0;
T = 0;
tt = zeros(1,Ntrymax);
while j < Ntrymin || (T < Tmin && j < Ntrymax)
  j = j+1;
  tt_ = fmm2dCPU(z,m,'pot',1,'tol',tol,'ndirect',ndirect_cpu,'out','time', ...
                 'cutoff',1e-15,'xopt',1e-16);
  tt(j) = tt_(10);
  cpu_timing_adaptive_uniform(j,:)=tt_;
  T = T+tt(j);
end
tt = sort(tt(1:j)); tt([1 end]) = [];
a = floor((j-2)/3); b = j-2-a;
tcpu0 = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));

for i = 1:numel(sigma2)
  randn('state',20120328);

  % (1) normal distribution N(0,sigma2)
  x = zeros(1,0);
  y = zeros(1,0);
  while numel(x) < N || numel(y) < N
    x = [abs(sigma2(i)*randn(N-numel(x),1)); x];
    y = [y; abs(sigma2(i)*randn(N-numel(y),1))];
    x(x > 1) = [];
    y(y > 1) = [];
  end
  z = complex(x,y);

  j = 0;
  T = 0;
  tt = zeros(1,Ntrymax);
  while j < Ntrymin || (T < Tmin && j < Ntrymax)
    j = j+1;
    tt_ = fmm2dCPU(z,m,'pot',1,'tol',tol,'ndirect',ndirect_cpu, ...
                   'out','time','cutoff',1e-15,'xopt',1e-16);
    tt(j) = tt_(10);
    cpu_timing_adaptive_sigma(i,j,:)=tt_;
    T = T+tt(j);
  end
  tt = sort(tt(1:j)); tt([1 end]) = [];
  a = floor((j-2)/3); b = j-2-a;
  tcpu1(i) = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));
  
  % (2) "layer"
  z = complex(rand(N,1),imag(z));

  % adaptive version
  j = 0;
  T = 0;
  tt = zeros(1,Ntrymax);
  while j < Ntrymin || (T < Tmin && j < Ntrymax)
    j = j+1;
    tt_ = cputime;
    tt_ = fmm2dCPU(z,m,'pot',1,'tol',tol,'ndirect',ndirect_cpu, ...
                   'out','time','cutoff',1e-15,'xopt',1e-16);
    tt(j) = tt_(10);
    cpu_timing_adaptive_sigma_layer(i,j,:)=tt_;
    T = T+tt(j);
  end
  tt = sort(tt(1:j)); tt([1 end]) = [];
  a = floor((j-2)/3); b = j-2-a;
  tcpu2(i) = 0.5*mean(tt([1:a b+1:end]))+0.5*mean(tt(a+1:b));

  disp(['i = ' num2str(i)]);
end

if ~exist('./examples/data/AFMMgpu6.mat','file')
  save examples/data/AFMMgpu6 t nn sigma2 tgpu0 tgpu1 tgpu2 tcpu0 tcpu1 tcpu2 ... 
       cuda_timing_adaptive_n cuda_timing_adaptive_uniform cuda_timing_adaptive_sigma ...
       cuda_timing_adaptive_sigma_layer cpu_timing_adaptive_uniform ...
       cpu_timing_adaptive_sigma cpu_timing_adaptive_sigma_layer
end
