% evaltimefarfield_run Runs for the paper "A General high order two-
% dimensional panel method".
% 
% This function is used to measure the time it takes for evaluating the
% contributions of panels in the far field using FMM evaluation. 
%
% The parameters are
% 
% Nrep - Number of repetitions for each time measurement
% Neval - Number of evaluation positions
% vortexpanels - if the panels are made from vorticity (1) or source
%                strength(2)
% controlpoint - If control points in the center of the panel, or total
%                flow throught the panel will be used
% slopecorrection - If the strength of the panel should be corrected
% depending on the panel slope
% doublecontrol - Adds extra target panel and solve equations in the least
%                 square sense
function evaltimefarfield_run(Nrep,Neval,vortexpanels,controlpoint,slopecorrection,doublecontrol)
paneleval=10:500; %Number of panels to test

% Set up panels separately, this will not be included in the evaluation
% time
for m=1:length(paneleval)
  % Set up parameters for building the circle. Note that higher order
  % derivates are these two with negative sign
  anglesd=linspace(0,2*pi,paneleval(m)+1);
  panelpointsd=exp(1i*anglesd(1:end-1));
  paneldirectionsd=1i*exp(1i*anglesd(1:end-1));
  linpanels{m}=highorderpanelstrength(panelpointsd,[],[],[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  cubpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  quintpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  septpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  lincubpanels{m}=highorderpanelstrength(panelpointsd,[],[],[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  cubcubpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  quintcubpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  septcubpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  linquintpanels{m}=highorderpanelstrength(panelpointsd,[],[],[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  cubquintpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  quintquintpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
  septquintpanels{m}=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
end

% Run all tests and measure the time
for k=1:Nrep
  eval=(9*rand(1,Neval)+1).*exp(2i*pi*rand(1,Neval));
  evalstr=rand(1,Neval)-0.5;
  for m=1:length(paneleval)
    % The time when evaluating only the panels on the distination points.
    % Mesh is built with respect to the distination points
    lintime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'linshape',linpanels{m}.panels,'linstrength',linpanels{m}.strength});
    cubtime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'cubshape',cubpanels{m}.panels,cubpanels{m}.directions,'linstrength',cubpanels{m}.strength});
    quinttime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'quintshape',quintpanels{m}.panels,quintpanels{m}.directions,quintpanels{m}.directions2,'linstrength',quintpanels{m}.strength});
    septtime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'septshape',septpanels{m}.panels,septpanels{m}.directions,septpanels{m}.directions2,septpanels{m}.directions3,'linstrength',septpanels{m}.strength});
    lincubtime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'linshape',lincubpanels{m}.panels,'cubstrength',lincubpanels{m}.strength,lincubpanels{m}.strengthdir});
    cubcubtime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'cubshape',cubcubpanels{m}.panels,cubcubpanels{m}.directions,'cubstrength',cubcubpanels{m}.strength,cubcubpanels{m}.strengthdir});
    quintcubtime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'quintshape',quintcubpanels{m}.panels,quintcubpanels{m}.directions,quintcubpanels{m}.directions2,'cubstrength',quintcubpanels{m}.strength,quintcubpanels{m}.strengthdir});
    septcubtime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'septshape',septcubpanels{m}.panels,septcubpanels{m}.directions,septcubpanels{m}.directions2,septcubpanels{m}.directions3,'cubstrength',septcubpanels{m}.strength,septcubpanels{m}.strengthdir});
    linquinttime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'linshape',linquintpanels{m}.panels,'quintstrength',linquintpanels{m}.strength,linquintpanels{m}.strengthdir,linquintpanels{m}.strength2dir});
    cubquinttime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'cubshape',cubquintpanels{m}.panels,cubquintpanels{m}.directions,'quintstrength',cubquintpanels{m}.strength,cubquintpanels{m}.strengthdir, cubquintpanels{m}.strength2dir});
    quintquinttime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'quintshape',quintquintpanels{m}.panels,quintquintpanels{m}.directions,quintquintpanels{m}.directions2,'quintstrength',quintquintpanels{m}.strength,quintquintpanels{m}.strengthdir, quintquintpanels{m}.strength2dir});
    septquinttime(m,k,:)=fmm2d([],[],eval,'pot',1,'tol',1e-9,'out','time','ndirect',35,'destinationmesh',true,'sourcepanels',{'septshape',septquintpanels{m}.panels,septquintpanels{m}.directions,septquintpanels{m}.directions2,septquintpanels{m}.directions3,'quintstrength',septquintpanels{m}.strength,septquintpanels{m}.strengthdir, septquintpanels{m}.strength2dir});
    
    % The time when evaluating both contribution from panels and
    % contribution from vortices
    lintimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'linshape',linpanels{m}.panels,'linstrength',linpanels{m}.strength});
    cubtimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'cubshape',cubpanels{m}.panels,cubpanels{m}.directions,'linstrength',cubpanels{m}.strength});
    quinttimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'quintshape',quintpanels{m}.panels,quintpanels{m}.directions,quintpanels{m}.directions2,'linstrength',quintpanels{m}.strength});
    septtimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'septshape',septpanels{m}.panels,septpanels{m}.directions,septpanels{m}.directions2,septpanels{m}.directions3,'linstrength',septpanels{m}.strength});
    lincubtimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'linshape',lincubpanels{m}.panels,'cubstrength',lincubpanels{m}.strength,lincubpanels{m}.strengthdir});
    cubcubtimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'cubshape',cubcubpanels{m}.panels,cubcubpanels{m}.directions,'cubstrength',cubcubpanels{m}.strength,cubcubpanels{m}.strengthdir});
    quintcubtimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'quintshape',quintcubpanels{m}.panels,quintcubpanels{m}.directions,quintcubpanels{m}.directions2,'cubstrength',quintcubpanels{m}.strength,quintcubpanels{m}.strengthdir});
    septcubtimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'septshape',septcubpanels{m}.panels,septcubpanels{m}.directions,septcubpanels{m}.directions2,septcubpanels{m}.directions3,'cubstrength',septcubpanels{m}.strength,septcubpanels{m}.strengthdir});
    linquinttimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'linshape',linquintpanels{m}.panels,'quintstrength',linquintpanels{m}.strength,linquintpanels{m}.strengthdir,linquintpanels{m}.strength2dir});
    cubquinttimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'cubshape',cubquintpanels{m}.panels,cubquintpanels{m}.directions,'quintstrength',cubquintpanels{m}.strength,cubquintpanels{m}.strengthdir, cubquintpanels{m}.strength2dir});
    quintquinttimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'quintshape',quintquintpanels{m}.panels,quintquintpanels{m}.directions,quintquintpanels{m}.directions2,'quintstrength',quintquintpanels{m}.strength,quintquintpanels{m}.strengthdir, quintquintpanels{m}.strength2dir});
    septquinttimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35,'sourcepanels',{'septshape',septquintpanels{m}.panels,septquintpanels{m}.directions,septquintpanels{m}.directions2,septquintpanels{m}.directions3,'quintstrength',septquintpanels{m}.strength,septquintpanels{m}.strengthdir, septquintpanels{m}.strength2dir});

    % The reference time when only evaluating the vortex contribution
    reftimesrc(m,k,:)=fmm2d(eval,evalstr,'pot',1,'tol',1e-9,'out','time','ndirect',35);
  end
end

if ~exist('./examples/generalpanel/data/generalpanelfarfield.mat','file')
  save examples/generalpanel/data/generalpanelfarfield lintime cubtime quinttime septtime lincubtime cubcubtime quintcubtime septcubtime linquinttime cubquinttime quintquinttime septquinttime lintimesrc cubtimesrc quinttimesrc septtimesrc lincubtimesrc cubcubtimesrc quintcubtimesrc septcubtimesrc linquinttimesrc cubquinttimesrc quintquinttimesrc septquinttimesrc reftimesrc Neval Nrep paneleval
end
