% highorderpanelmatrix
%
% This function is used to generate the interaction matrix between the
% panels. Note that this function is only built for testing of the higher
% order panels and have not been optimized for speed. If speed is critical,
% you may consider implementing this in C++ instead.
%
% The parameters are
%
% fmmpanels        - a 2xN vector giving the start and end of the reference
%                    line for each panel
% fmmdirections    - The direction of the surface (2xN vector) (if the
%                    surface isgiven by x(s) + 1i*y(s),
%                    input dx/ds + 1i*dy/ds here.
%                    Used for cubic panels and above. Leave empty for first 
%                    order panels.
% fmm2directions   - Second order derivative (2xN vector). Used for quintic 
%                    panels and above. 
%                    Give the value of d2x/ds^2 + 1i*d2y/ds^2. Leave empty
%                    for cubic panels
% fmm3directions   - Third order derivate (2xN vector). Used for septic
%                    panels. Give the value of d3x/ds^3 + 1i*d3y/ds^3. 
%                    Leave empty for quintic panels
% strengthorder    - The desired panel strength. Valid values are 1, 3, 5
%                    and 7
% vortexpanels     - true if panels should be constructed with vorticity.
%                    false is the panels should be constructed with source
%                    strengh
% controlpoint     - true if the no penetration boundary condition should
%                    be solved at individual points on the panel surface.
%                    false if the integral over the panel surface should be
%                    solved instead
% slopecurrection  - true if the strength of the panels should be corrected
%                    by the slope of the panels to ensure that the strength
%                    is given in the direction of the panel and not in the
%                    direction of the reference line. false if the strength
%                    only should be corrected at the end points to ensure
%                    continuity
% doublecontrol    - If additional target panels should be used and the
%                    equations solved in the least square sense. 1 adds one
%                    extra target panel for each source panel. Higher
%                    values will multiply with the original number of
%                    panels (i.e. for septic where 3 targets are required,
%                    using 2 will give 6 targets)
% normalize        - Causes each panel to have equal weight in the panel
%                    matrix. Only applies if controlpoint is false, and if
%                    doublecontrol is used to solve the matrix in the least
%                    square sense, where one has to tolerate a small error
%                    for each panel
% kuttacondition   - How to solve the kutta condition. The options are
%                    0 - Do not apply any modification
%                    1 - Assume that the panel strength is zero at the
%                        trailing edge (recommended)
%                    2 - Assume that the panel strength is opposite between
%                        the upper and lower panel (basically making it
%                        zero, but gives one extra unknown)
%                    3 - Add one extra panel at the trailing edge away from
%                        the airfoil and fulfil the no-penetration
%                        condition on this panel as well
% targetonlypanels - Panels that have no source strength, but should be
%                    included when solving the boundary conditions anyway

function [M, cp, n]=highorderpanelmatrix(fmmpanels,fmmdirections,fmm2directions,fmm3directions,strengthorder,vortexpanels,controlpoint,slopecorrection,doublecontrol,kuttacondition,targetonlypanels)
kuttazeroedge=0;
kuttaoppositeedge=0;
if kuttacondition % ensure that the value is 1
  kuttazeroedge=1;
  if kuttacondition==2
    kuttaoppositeedge=1;
    kuttazeroedge=0;
  end
  kuttacondition=1;
end
targetcount=size(fmmpanels,2);
count=targetcount-targetonlypanels;

% Determine how many control elements there are for each panel add one 
% for each additional level of strenght and one extra if doublecontrol is 
% chosen
strengthordercnt=1;
if strengthorder==3
  strengthordercnt=2;
elseif strengthorder==5
  strengthordercnt=3;
elseif strengthorder==7
  strengthordercnt=4;
end
splitcount=strengthordercnt;
if length(doublecontrol)==targetcount % It is possible to give individual values for each panel for the doublecontrol parameter
  splitcount=splitcount*ones(size(doublecontrol));
  splitcount(doublecontrol==1)=splitcount(doublecontrol==1)+1;
  splitcount(doublecontrol>1)=splitcount(doublecontrol>1)*doublecontrol(doublecontrol>1);
else
  if doublecontrol
    if doublecontrol==1
      splitcount=splitcount+1;
    else
      splitcount=splitcount*doublecontrol;
    end
  end
end

%Convert to polynomial coefficients to allow all panels to run with the
%same evaluation codes later
if isempty(fmmdirections)
  fmmpanelsa=fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'linshape',fmmpanels,'linstrength',zeros(size(fmmpanels))},'out','panels');
elseif isempty(fmm2directions)
  fmmpanelsa=fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'cubshape',fmmpanels,fmmdirections,'linstrength',zeros(size(fmmpanels))},'out','panels');
elseif isempty(fmm3directions)
  fmmpanelsa=fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'quintshape',fmmpanels,fmmdirections,fmm2directions,'linstrength',zeros(size(fmmpanels))},'out','panels');
else
  fmmpanelsa=fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'septshape',fmmpanels,fmmdirections,fmm2directions,fmm3directions,'linstrength',zeros(size(fmmpanels))},'out','panels');
end

% Set up controls, either through individual control points, or by
% integrating over panel surface
if controlpoint %integration at individual control points
  % Set up suitable number of control points depending on the value of
  % splitpoint
  [controlpoints, n]=generalpanelcontrolpoint((1:2:2*splitcount-1)./(splitcount*2),fmmpanels,fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',fmmpanels,fmmpanelsa,'linstrength',zeros(size(fmmpanels))},'out','panels'));
  cp=controlpoints;
else % Evaluation through integration over panel surface
  % Set up suitable number of target panels depending on the value of
  % splitpoint
  if splitcount==1 % Same as source panels
    targetpanels=fmmpanels;
    targeta=fmmpanelsa;
  else
    [targetpanels, targeta]=splitgeneralpanel((1:splitcount-1)./(splitcount),fmmpanels,fmm2d([],[],'pot',1,'tol',0,'out','panels','sourcepanels',{'polyshape',fmmpanels,fmmpanelsa,'linstrength',zeros(size(fmmpanels))}));
  end
  cp=targetpanels;
  n=targeta;
end

if targetonlypanels~=0
  fmmpanels=fmmpanels(:,1:count);
  fmmpanelsa=fmmpanelsa(:,1:count);
end

% Temporary variable for source strength
tmpstrengthbase=[0 1;1 0];
if vortexpanels % Vortex panels means imaginary strength
  tmpstrengthbase=1i*tmpstrengthbase;
end
tmpzero=zeros(2);
tmpstrengthdir=tmpstrengthbase;
tmpstrength=tmpstrengthbase;

% Preallocate matrix
M=zeros(targetcount*splitcount+1-kuttacondition,(count+kuttacondition)*strengthordercnt-kuttacondition-kuttazeroedge);

for j=1:count
  if(j==1)
    tmppanels=[fmmpanels(:,count) fmmpanels(:,1)];
    tmppanelsa=[fmmpanelsa(:,count) fmmpanelsa(:,1)];
  else
    tmppanels=[fmmpanels(:,j-1) fmmpanels(:,j)];
    tmppanelsa=[fmmpanelsa(:,j-1) fmmpanelsa(:,j)];
  end
  
  if kuttaoppositeedge
    if j==1
      tmpstrength=[-tmpstrengthbase(:,1) tmpstrengthbase(:,2)];
    else
      tmpstrength=tmpstrengthbase;
    end
  end
  
  % Here, it is necessary to split between the different source strength
  % alternatives.
  % Set up the matrix by evaluating the contribution from the panels on
  % each control point
  if controlpoint
    if strengthorder==1
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'linstrength',tmpstrength,'outside',1}).*n);
      end
    elseif strengthorder==3
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'cubstrength',tmpstrength,tmpzero,'outside',1}).*n);
      end
      if j==1&&kuttacondition
        M(1:end-1+kuttacondition,count+j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'cubstrength',tmpzero(:,1),tmpstrengthdir(:,1),'outside',1}).*n);
        M(1:end-1+kuttacondition,count+j+1-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'cubstrength',tmpzero(:,2),tmpstrengthdir(:,2),'outside',1}).*n);
      else
        M(1:end-1+kuttacondition,count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'cubstrength',tmpzero,tmpstrengthdir,'outside',1}).*n);
      end
    elseif strengthorder==5
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpstrength,tmpzero,tmpzero,'outside',1}).*n);
      end
      if j==1&&kuttacondition
        M(1:end-1+kuttacondition,count+j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,1),tmpstrengthdir(:,1),tmpzero(:,1),'outside',1}).*n);
        M(1:end-1+kuttacondition,2*count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,1),tmpzero(:,1),tmpstrengthdir(:,1),'outside',1}).*n);
        M(1:end-1+kuttacondition,count+j+1-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,2),tmpstrengthdir(:,2),tmpzero(:,2),'outside',1}).*n);
        M(1:end-1+kuttacondition,2*count+j+1+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,2),tmpzero(:,2),tmpstrengthdir(:,2),'outside',1}).*n);
      else
        M(1:end-1+kuttacondition,count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpzero,tmpstrengthdir,tmpzero,'outside',1}).*n);
        M(1:end-1+kuttacondition,2*count+j+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpzero,tmpzero,tmpstrengthdir,'outside',1}).*n);
      end
    elseif strengthorder==7
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpstrength,tmpzero,tmpzero,tmpzero,'outside',1}).*n);
      end
      if j==1&&kuttacondition
        M(1:end-1+kuttacondition,count+j-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,1),tmpstrengthdir(:,1),tmpzero(:,1),tmpzero(:,1),'outside',1}).*n);
        M(1:end-1+kuttacondition,2*count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,1),tmpzero(:,1),tmpstrengthdir(:,1),tmpzero(:,1),'outside',1}).*n);
        M(1:end-1+kuttacondition,3*count+j+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,1),tmpzero(:,1),tmpzero(:,1),tmpstrengthdir(:,1),'outside',1}).*n);
        M(1:end-1+kuttacondition,count+j+1-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,2),tmpstrengthdir(:,2),tmpzero(:,2),tmpzero(:,2),'outside',1}).*n);
        M(1:end-1+kuttacondition,2*count+j+1+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,2),tmpzero(:,2),tmpstrengthdir(:,2),tmpzero(:,2),'outside',1}).*n);
        M(1:end-1+kuttacondition,3*count+j+1+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,2),tmpzero(:,2),tmpzero(:,2),tmpstrengthdir(:,2),'outside',1}).*n);
      else
        M(1:end-1+kuttacondition,count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpstrengthdir,tmpzero,tmpzero,'outside',1}).*n);
        M(1:end-1+kuttacondition,2*count+j+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpzero,tmpstrengthdir,tmpzero,'outside',1}).*n);
        M(1:end-1+kuttacondition,3*count+j+3*kuttacondition-kuttazeroedge)=real(fmm2d([],[],controlpoints,'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpzero,tmpzero,tmpstrengthdir,'outside',1}).*n);
      end
    end
  else % Calculate the integral of the flow through the panel
    if strengthorder==1
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'linstrength',tmpstrength,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
    elseif strengthorder==3
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'cubstrength',tmpstrength,tmpzero,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
      if j==1&&kuttacondition
        M(1:end-1+kuttacondition,count+j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'cubstrength',tmpzero(:,1),tmpstrengthdir(:,1),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,count+j+1-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'cubstrength',tmpzero(:,2),tmpstrengthdir(:,2),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      else
        M(1:end-1+kuttacondition,count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'cubstrength',tmpzero,tmpstrengthdir,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
    elseif strengthorder==5
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpstrength,tmpzero,tmpzero,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
      if j==1&&kuttacondition
        M(1:end-1+kuttacondition,count+j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,1),tmpstrengthdir(:,1),tmpzero(:,1),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,2*count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,1),tmpzero(:,1),tmpstrengthdir(:,1),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,count+j+1-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,2),tmpstrengthdir(:,2),tmpzero(:,2),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,2*count+j+1+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'quintstrength',tmpzero(:,2),tmpzero(:,2),tmpstrengthdir(:,2),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      else
        M(1:end-1+kuttacondition,count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpzero,tmpstrengthdir,tmpzero,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,2*count+j+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpzero,tmpzero,tmpstrengthdir,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
    elseif strengthorder==7
      if ~kuttazeroedge || j~=1
        M(1:end-1+kuttacondition,j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpstrength,tmpzero,tmpzero,tmpzero,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
      if j==1&&kuttacondition
        M(1:end-1+kuttacondition,count+j-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,1),tmpstrengthdir(:,1),tmpzero(:,1),tmpzero(:,1),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,2*count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,1),tmpzero(:,1),tmpstrengthdir(:,1),tmpzero(:,1),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,3*count+j+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,1),tmppanelsa(:,1),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,1),tmpzero(:,1),tmpzero(:,1),tmpstrengthdir(:,1),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,count+j+1-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,2),tmpstrengthdir(:,2),tmpzero(:,2),tmpzero(:,2),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,2*count+j+1+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,2),tmpzero(:,2),tmpstrengthdir(:,2),tmpzero(:,2),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,3*count+j+1+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels(:,2),tmppanelsa(:,2),'slopecorrection',slopecorrection,'septstrength',tmpzero(:,2),tmpzero(:,2),tmpzero(:,2),tmpstrengthdir(:,2),'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      else
        M(1:end-1+kuttacondition,count+j+kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpstrengthdir,tmpzero,tmpzero,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,2*count+j+2*kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpzero,tmpstrengthdir,tmpzero,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
        M(1:end-1+kuttacondition,3*count+j+3*kuttacondition-kuttazeroedge)=real(fmm2d([],[],'pot',1,'tol',0,'sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpzero,tmpzero,tmpstrengthdir,'outside',1},'targetpanels',{'polyshape',targetpanels,targeta}));
      end
    end
  end
  
  % To allow to set conditions for the total strength of the panels
  if ~kuttacondition
    if strengthorder==1
      str=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'linstrength',tmpstrength,'outside',1});
      if vortexpanels
        M(end,j)=sum(imag(str));
      else
        M(end,j)=sum(real(str));
      end
    elseif strengthorder==3
      str=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'cubstrength',tmpstrength,tmpzero,'outside',1});
      str2=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'cubstrength',tmpzero,tmpstrengthdir,'outside',1});
      if vortexpanels
        M(end,j)=sum(imag(str));
        M(end,count+j)=sum(imag(str2));
      else
        M(end,j)=sum(real(str));
        M(end,count+j)=sum(real(str2));
      end
    elseif strengthorder==5
      str=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpstrength,tmpzero,tmpzero,'outside',1});
      str2=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpzero,tmpstrengthdir,tmpzero,'outside',1});
      str3=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'quintstrength',tmpzero,tmpzero,tmpstrengthdir,'outside',1});
      if vortexpanels
        M(end,j)=sum(imag(str));
        M(end,count+j)=sum(imag(str2));
        M(end,2*count+j)=sum(imag(str3));
      else
        M(end,j)=sum(real(str));
        M(end,count+j)=sum(real(str2));
        M(end,2*count+j)=sum(real(str3));
      end
    elseif strengthorder==7
      str=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpstrength,tmpzero,tmpzero,tmpzero,'outside',1});
      str2=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpstrengthdir,tmpzero,tmpzero,'outside',1});
      str3=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpzero,tmpstrengthdir,tmpzero,'outside',1});
      str4=fmm2d([],[],'pot',1,'tol',0,'out','panelstrength','sourcepanels',{'polyshape',tmppanels,tmppanelsa,'slopecorrection',slopecorrection,'septstrength',tmpzero,tmpzero,tmpzero,tmpstrengthdir,'outside',1});
      if vortexpanels
        M(end,j)=sum(imag(str));
        M(end,count+j)=sum(imag(str2));
        M(end,2*count+j)=sum(imag(str3));
        M(end,3*count+j)=sum(imag(str4));
      else
        M(end,j)=sum(real(str));
        M(end,count+j)=sum(real(str2));
        M(end,2*count+j)=sum(real(str3));
        M(end,3*count+j)=sum(real(str4));
      end
    end
  end
end