% highorderpanelstrengh
%
% This function is used to calculate the panel strengh required to solve
% the no-penetration boundary condition. The function is currently limited
% to the case where only a constant flow velocity is affecting the panels.
% This function assumes that the object to be modelled by the panels is
% connected, i.e. that there will be a panel between the end point and the
% start point as well
%
% The parameters are
%
% panelpoints      - a vector with positions on the surface of the object
%                    to be modelled (input is given as complex numbers)
% paneldirections  - The direction of the surface (if the surface is
%                    given by x(s) + 1i*y(s), input dx/ds + 1i*dy/ds here.
%                    Used for cubic panels and above. Leave empty for first 
%                    order panels.
% panel2direction  - Second order derivative. Used for quintic panels and 
%                    above. Give the value of d2x/ds^2 + 1i*d2y/ds^2. Leave
%                    empty for cubic panels
% panel3directions - Third order derivate, Used for septic panels. Give the
%                    value of d3x/ds^3 + 1i*d3y/ds^3. Leave empty for
%                    quintic panels
% strengthorder    - The desired panel strength. Valid values are 1, 3, 5
%                    and 7
% vasympt          - Asymptotic flow velocity
% vortexpanels     - true if panels should be constructed with vorticity.
%                    false is the panels should be constructed with source
%                    strengh
% controlpoint     - true if the no penetration boundary condition should
%                    be solved at individual points on the panel surface.
%                    false if the integral over the panel surface should be
%                    solved instead
% slopecurrection  - true if the strength of the panels should be corrected
%                    by the slope of the panels to ensure that the strength
%                    is given in the direction of the panel and not in the
%                    direction of the reference line. false if the strength
%                    only should be corrected at the end points to ensure
%                    continuity
% doublecontrol    - If additional target panels should be used and the
%                    equations solved in the least square sense. 1 adds one
%                    extra target panel for each source panel. Higher
%                    values will multiply with the original number of
%                    panels (i.e. for septic where 3 targets are required,
%                    using 2 will give 6 targets)

function output=highorderpanelstrength(panelpoints,paneldirections,panel2directions,panel3directions,strengthorder,vasympt,vortexpanels,controlpoint,slopecorrection,doublecontrol)
if nargin<9
  slopecorrection=0;
end
if nargin<8
  controlpoint=0;
end
if nargin<7
  vortexpanels=0;
end
if strengthorder~=1 && strengthorder~=3 && strengthorder~=5 && strengthorder~=7
  error('order of strength must be either 1, 3, 5 or 7');
end

% Convert from the format of one vector with each value, to a two vector
% format where each panel is independent of the other
looppanelpoints=[panelpoints panelpoints(1)];
panels=[panelpoints;looppanelpoints(2:end)];
output.panels=panels;
if ~isempty(paneldirections)
  looppaneldirections=[paneldirections paneldirections(1)];
  directions=[paneldirections;looppaneldirections(2:end)];
  output.directions=directions;
else
  directions=[];
end
if ~isempty(panel2directions)
  looppanel2directions=[panel2directions panel2directions(1)];
  directions2=[panel2directions;looppanel2directions(2:end)];
  output.directions2=directions2;
else
  directions2=[];
end
if ~isempty(panel3directions)
  looppanel3directions=[panel3directions panel3directions(1)];
  directions3=[panel3directions;looppanel3directions(2:end)];
  output.directions3=directions3;
else
  directions3=[];
end

% Generate both the BEM matrix (M) and the right hand side for the solution
if controlpoint
  [M, controlpoints, n]=highorderpanelmatrix(panels,directions,directions2,directions3,strengthorder,vortexpanels,controlpoint,slopecorrection,doublecontrol,0,0);
  rhs=lineartargetpanelrhscontrolpoint(controlpoints,n,vasympt,false); %same for both
else
  [M, targetpanels, targeta]=highorderpanelmatrix(panels,directions,directions2,directions3,strengthorder,vortexpanels,controlpoint,slopecorrection,doublecontrol,0,0);
  rhs=generaltargetpanelrhs(targetpanels,targeta,vasympt,false); %same for both
end

% Solve the matrix equation using standard methods. 
str = (M\rhs).';
if vortexpanels
  str=1i*str;
end

% Generate output
sz=size(panels,2);
output.strength=[str(1:sz);str(2:sz) str(1)];
if strengthorder==3 || strengthorder == 5 || strengthorder == 7
  output.strengthdir=[str(sz+1:2*sz);str(sz+2:2*sz),str(sz+1)];
end
if strengthorder == 5 || strengthorder == 7
  output.strength2dir=[str(2*sz+1:3*sz);str(2*sz+2:3*sz),str(2*sz+1)];
end
if strengthorder == 7
  output.strength3dir=[str(3*sz+1:end);str(3*sz+2:end),str(3*sz+1)];
end



