% highorderpanelstrengthkutta
%
% This function is used to calculate the panel strengh required to solve
% the no-penetration boundary condition. The function is currently limited
% to the case where only a constant flow velocity is affecting the panels.
% This fuction assumes that there is a trailing edge at the start and end
% point of the panel vectors
%
% The parameters are
%
% panelpoints      - a vector with positions on the surface of the object
%                    to be modelled (input is given as complex numbers)
% paneldirections  - The direction of the surface (if the surface is
%                    given by x(s) + 1i*y(s), input dx/ds + 1i*dy/ds here.
%                    Used for cubic panels and above. Leave empty for first 
%                    order panels.
% panel2direction  - Second order derivative. Used for quintic panels and 
%                    above. Give the value of d2x/ds^2 + 1i*d2y/ds^2. Leave
%                    empty for cubic panels
% panel3directions - Third order derivate, Used for septic panels. Give the
%                    value of d3x/ds^3 + 1i*d3y/ds^3. Leave empty for
%                    quintic panels
% strengthorder    - The desired panel strength. Valid values are 1, 3, 5
%                    and 7
% vasympt          - Asymptotic flow velocity
% vortexpanels     - true if panels should be constructed with vorticity.
%                    false is the panels should be constructed with source
%                    strengh
% controlpoint     - true if the no penetration boundary condition should
%                    be solved at individual points on the panel surface.
%                    false if the integral over the panel surface should be
%                    solved instead
% slopecurrection  - true if the strength of the panels should be corrected
%                    by the slope of the panels to ensure that the strength
%                    is given in the direction of the panel and not in the
%                    direction of the reference line. false if the strength
%                    only should be corrected at the end points to ensure
%                    continuity
% doublecontrol    - If additional target panels should be used and the
%                    equations solved in the least square sense. 1 adds one
%                    extra target panel for each source panel. Higher
%                    values will multiply with the original number of
%                    panels (i.e. for septic where 3 targets are required,
%                    using 2 will give 6 targets)
% normalize        - Causes each panel to have equal weight in the panel
%                    matrix. Only applies if controlpoint is false, and if
%                    doublecontrol is used to solve the matrix in the least
%                    square sense, where one has to tolerate a small error
%                    for each panel
% kuttamethod      - How to solve the kutta condition. The options are
%                    0 - Do not apply any modification
%                    1 - Assume that the panel strength is zero at the
%                        trailing edge (recommended)
%                    2 - Assume that the panel strength is opposite between
%                        the upper and lower panel (basically making it
%                        zero, but gives one extra unknown)
%                    3 - Add one extra panel at the trailing edge away from
%                        the airfoil and fulfil the no-penetration
%                        condition on this panel as well

function output=highorderpanelstrengthkutta(panelpoints,paneldirections,panel2directions,panel3directions,strengthorder,vasympt,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod)
if nargin<11
  kuttamethod=1;
end
if nargin<10
  normalize=0;
end
if nargin<9
  doublecontrol=0;
end
if nargin<8
  slopecorrection=0;
end
if nargin<7
  controlpoint=0;
end
if strengthorder~=1 && strengthorder~=3 && strengthorder~=5 && strengthorder~=7
  error('order of strength must be either 1, 3, 5 or 7');
end

% Convert from the format of one vector with each value, to a two vector
% format where each panel is independent of the other
panels=[panelpoints(1:end-1);panelpoints(2:end)];
output.panels=panels;
if ~isempty(paneldirections)
  directions=[paneldirections(1:end-1);paneldirections(2:end)];
  output.directions=directions;
else
  directions=[];
end
if ~isempty(panel2directions)
  directions2=[panel2directions(1:end-1);panel2directions(2:end)];
  output.directions2=directions2;
else
  directions2=[];
end
if ~isempty(panel3directions)
  directions3=[panel3directions(1:end-1);panel3directions(2:end)];
  output.directions3=directions3;
else
  directions3=[];
end

targetonlypanels=0;

% If one extra panel should be added to solve the kutta condition
if kuttamethod==3
  if ~isempty(paneldirections)
    direction1=-paneldirections(1);
    direction2=paneldirections(end);
  else
    direction1=panels(1)-panels(2);
    direction2=panels(end)-panels(end-1);
  end
  direction1=direction1/abs(direction1);
  direction2=direction2/abs(direction2);
  meandirection=direction1+direction2;
  meandirection=meandirection/abs(meandirection);
  panels=[panels [mean([panelpoints(1),panelpoints(end)]);mean([panelpoints(1),panelpoints(end)])+abs(panelpoints(end)-panelpoints(end-1))*meandirection]];
  if ~isempty(paneldirections)
    directions=[directions [NaN;NaN]];
    output.directions=directions;
  end
  if ~isempty(panel2directions)
    directions2=[directions2 [NaN;NaN]];
    output.directions2=directions2;
  end
  if ~isempty(panel3directions)
    directions3=[directions3 [NaN;NaN]];
    output.directions3=directions3;
  end
  targetonlypanels=1;
end

% Generate both the BEM matrix (M) and the right hand side for the solution
dbstop if warning
if controlpoint
  [M, controlpoints, n]=highorderpanelmatrix(panels,directions,directions2,directions3,strengthorder,1,controlpoint,slopecorrection,doublecontrol,kuttamethod,targetonlypanels);
  rhs=lineartargetpanelrhscontrolpoint(controlpoints,n,vasympt,1); %same for both
else
  [M, targetpanels, targeta]=highorderpanelmatrix(panels,directions,directions2,directions3,strengthorder,1,controlpoint,slopecorrection,doublecontrol,kuttamethod,targetonlypanels);
  rhs=generaltargetpanelrhs(targetpanels,targeta,vasympt,1); %same for both
  if normalize % Both the BEM matrix and the rhs have to be normalized
    dist=transpose(abs(targetpanels(2,:)-targetpanels(1,:)));
    dist=dist.^normalize;
    M=M./(dist*ones(1,size(M,2)));
    rhs=rhs./dist;
  end
end
dbclear if warning

% Solve the matrix equation using standard methods. 
str = 1i*(M\rhs).';
sz=size(panels,2)-targetonlypanels;

% Write output data
if kuttamethod==2 % Special case, as the sign of the last element hast to be opposite
  output.strength=[str(1:sz);str(2:sz) -str(1)];
  if strengthorder==3 || strengthorder == 5 || strengthorder == 7
    output.strengthdir=[str(sz+2) str(sz+3:2*sz+1);str(sz+3:2*sz+1),str(sz+1)];
  end
  if strengthorder == 5 || strengthorder == 7
    output.strength2dir=[str(2*sz+3) str(2*sz+4:3*sz+2);str(2*sz+4:3*sz+2),str(2*sz+2)];
  end
  if strengthorder == 7
    output.strength3dir=[str(3*sz+4) str(3*sz+5:4*sz+3);str(3*sz+5:4*sz+3),str(3*sz+3)];
  end
else
  output.strength=[0 str(1:sz-1);str(1:sz-1) 0];
  if strengthorder==3 || strengthorder == 5 || strengthorder == 7
    if kuttamethod
      output.strengthdir=[str(sz+1) str(sz+2:2*sz);str(sz+2:2*sz),str(sz)];
    else
      output.strengthdir=[str(sz:2*sz-1);str(sz+1:2*sz-1),str(sz)];
    end
  end
  if strengthorder == 5 || strengthorder == 7
    if kuttamethod
      output.strength2dir=[str(2*sz+2) str(2*sz+3:3*sz+1);str(2*sz+3:3*sz+1),str(2*sz+1)];
    else
      output.strength2dir=[str(2*sz:3*sz-1);str(2*sz+1:3*sz-1),str(2*sz)];
    end
  end
  if strengthorder == 7
    if kuttamethod
      output.strength3dir=[str(3*sz+3) str(3*sz+4:4*sz+2);str(3*sz+4:4*sz+2),str(3*sz+2)];
    else
      output.strength3dir=[str(3*sz:end);str(3*sz+1:end),str(3*sz)];
    end
  end
end


