% karmantrefftzconvergence_run run script for the paper "A General high order 
% two-dimensional panel method".
%
% This test performes simulations related to the Karman Trefftz accuracy
% tests. The input parameters allows for configuring the model.
%
% The parameters are
%
% my              - Circle center for the Karman Trefftz mapping
% n               - Parameter for the Karman Trefftz mapping
% vortexpanels    - true if panels are built by vorticity strength, false
%                   for source strengt
% controlpoint    - true if control points in the center of the panels are
%                   used, false if flow through the panels are used
% slopecorrection - true if panel strength is corrected based on the slope
%                   of the panel, false if it only should be adjusted at
%                   the end points
% doublecontrol   - If target panels should be split in one (or more) extra
%                   panels and then solve the system in the least square
%                   sense
% normalize       - Normalize panels to make each panel have equal weight.
%                   Only applies for controlpoint=false as this is always
%                   true when controlpoint=true
% kuttamethod     - How to solve Kutta condition, 1 means zero vorticity at
%                   the trailing edge, 2 means opposite circulation between
%                   the upper and lower parts. 
% filename        - Name of file where results should be stored

function karmantrefftzconvergence_run(my,n,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod,filename)
if exist(filename,'file')
  disp('File already exist, aborting');
  return
end
panelcount=100; % Number of panels to study
sp=logspace(0.0000001,1,200); % Distances to loop over for the distance plot

distval=1.01;      % distances to study for the number of panel study
surfaceval=1.01;   % distance to study for the surface plot 
paneleval=[20:80 82:2:150 155:5:300]; % panels to loop over for the number of panels study

diststudy=true;    % Accuracy with respect to distance from wing
panelstudy=true;   % Accuracy with respect to number of panels 
circstudy=true;    % Accuracy in circulation
velstudy=true;     % Accuracy in velocity
surfacestudy=true; % Error over the surface

if ~velstudy
  distval=2; % make it only one element
end

if diststudy || surfacestudy
  % Preallocate arrays
  linpanels=cell(size(panelcount));
  cubpanels=cell(size(panelcount));
  quintpanels=cell(size(panelcount));
  septpanels=cell(size(panelcount));
  lincubpanels=cell(size(panelcount));
  cubcubpanels=cell(size(panelcount));
  quintcubpanels=cell(size(panelcount));
  septcubpanels=cell(size(panelcount));
  linquintpanels=cell(size(panelcount));
  cubquintpanels=cell(size(panelcount));
  quintquintpanels=cell(size(panelcount));
  septquintpanels=cell(size(panelcount));
  linseptpanels=cell(size(panelcount));
  cubseptpanels=cell(size(panelcount));
  quintseptpanels=cell(size(panelcount));
  septseptpanels=cell(size(panelcount));
  
  linerror=cell(size(panelcount));
  cuberror=cell(size(panelcount));
  quinterror=cell(size(panelcount));
  septerror=cell(size(panelcount));
  lincuberror=cell(size(panelcount));
  cubcuberror=cell(size(panelcount));
  quintcuberror=cell(size(panelcount));
  septcuberror=cell(size(panelcount));
  linquinterror=cell(size(panelcount));
  cubquinterror=cell(size(panelcount));
  quintquinterror=cell(size(panelcount));
  septquinterror=cell(size(panelcount));
  linsepterror=cell(size(panelcount));
  cubsepterror=cell(size(panelcount));
  quintsepterror=cell(size(panelcount));
  septsepterror=cell(size(panelcount));
  
  surface_pos=cell(size(panelcount));
  linerror_surf=cell(size(panelcount));
  cuberror_surf=cell(size(panelcount));
  quinterror_surf=cell(size(panelcount));
  septerror_surf=cell(size(panelcount));
  lincuberror_surf=cell(size(panelcount));
  cubcuberror_surf=cell(size(panelcount));
  quintcuberror_surf=cell(size(panelcount));
  septcuberror_surf=cell(size(panelcount));
  linquinterror_surf=cell(size(panelcount));
  cubquinterror_surf=cell(size(panelcount));
  quintquinterror_surf=cell(size(panelcount));
  septquinterror_surf=cell(size(panelcount));
  linsepterror_surf=cell(size(panelcount));
  cubsepterror_surf=cell(size(panelcount));
  quintsepterror_surf=cell(size(panelcount));
  septsepterror_surf=cell(size(panelcount));
  
  
  for k=1:length(panelcount)
    % Set up panels
    TEangle=angle(1-my);
    angles=linspace(TEangle,2*pi+TEangle,panelcount(k)+1);
    anglesdir=angles;
    anglesdir(1)=anglesdir(1)+1e-10;
    anglesdir(end)=anglesdir(end)-1e-10;

    konst=sqrt((1-real(my))^2+imag(my)^2);
    panelpoints=konst.*exp(1i*angles)+my;
    panelpointsdir=konst.*exp(1i*anglesdir)+my;
    panelpointsdir2=konst.*exp(1i*anglesdir)+my;
    panelpointsdir3=konst.*exp(1i*anglesdir)+my;
    dsdt=konst*1i*exp(1i*angles);
    d2sdt2=-konst*exp(1i*angles);
    d3sdt3=-dsdt;
    z=karmantrefftz(panelpoints,n);
    dzds=karmantrefftzd(panelpointsdir,n);
    d2zds2=karmantrefftzd2(panelpointsdir2,n);
    d3zds3=karmantrefftzd3(panelpointsdir3,n);
    paneldirections=dsdt.*dzds;
    panel2directions=d2zds2.*dsdt.^2+dzds.*d2sdt2;
    panel3directions=d3sdt3.*dzds+dsdt.*(3.*d2sdt2.*d2zds2+d3zds3.*dsdt.^2);
    panel2directions(real(z)>1.9)=NaN;
    panel3directions(real(z)>1.9)=NaN;

    linpanels{k}=highorderpanelstrengthkutta(z,[],[],[],1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    cubpanels{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    quintpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    septpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    lincubpanels{k}=highorderpanelstrengthkutta(z,[],[],[],3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    cubcubpanels{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    quintcubpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    septcubpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    linquintpanels{k}=highorderpanelstrengthkutta(z,[],[],[],5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    cubquintpanels{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    quintquintpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    septquintpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    linseptpanels{k}=highorderpanelstrengthkutta(z,[],[],[],7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    cubseptpanels{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    quintseptpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    septseptpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
    
    % How the error varies along the integration line
    if surfacestudy
      [surface_pos{k}, linerror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'linshape',linpanels{k}.panels,'linstrength',linpanels{k}.strength);
      [~, cuberror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubpanels{k}.panels,cubpanels{k}.directions,'linstrength',cubpanels{k}.strength);
      [~, quinterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintpanels{k}.panels,quintpanels{k}.directions,quintpanels{k}.directions2,'linstrength',quintpanels{k}.strength);
      [~, septerror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'septshape',septpanels{k}.panels,septpanels{k}.directions,septpanels{k}.directions2,septpanels{k}.directions3,'linstrength',septpanels{k}.strength);
      [~, lincuberror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'linshape',lincubpanels{k}.panels,'cubstrength',lincubpanels{k}.strength,lincubpanels{k}.strengthdir);
      [~, cubcuberror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubcubpanels{k}.panels,cubcubpanels{k}.directions,'cubstrength',cubcubpanels{k}.strength,cubcubpanels{k}.strengthdir);
      [~, quintcuberror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintcubpanels{k}.panels,quintcubpanels{k}.directions,quintcubpanels{k}.directions2,'cubstrength',quintcubpanels{k}.strength,quintcubpanels{k}.strengthdir);
      [~, septcuberror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'septshape',septcubpanels{k}.panels,septcubpanels{k}.directions,septcubpanels{k}.directions2,septcubpanels{k}.directions3,'cubstrength',septcubpanels{k}.strength,septcubpanels{k}.strengthdir);
      [~, linquinterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'linshape',linquintpanels{k}.panels,'quintstrength',linquintpanels{k}.strength,linquintpanels{k}.strengthdir,linquintpanels{k}.strength2dir);
      [~, cubquinterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubquintpanels{k}.panels,cubquintpanels{k}.directions,'quintstrength',cubquintpanels{k}.strength,cubquintpanels{k}.strengthdir, cubquintpanels{k}.strength2dir);
      [~, quintquinterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintquintpanels{k}.panels,quintquintpanels{k}.directions,quintquintpanels{k}.directions2,'quintstrength',quintquintpanels{k}.strength,quintquintpanels{k}.strengthdir, quintquintpanels{k}.strength2dir);
      [~, septquinterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'septshape',septquintpanels{k}.panels,septquintpanels{k}.directions,septquintpanels{k}.directions2,septquintpanels{k}.directions3,'quintstrength',septquintpanels{k}.strength,septquintpanels{k}.strengthdir, septquintpanels{k}.strength2dir);
      [~, linsepterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'linshape',linseptpanels{k}.panels,'septstrength',linseptpanels{k}.strength,linseptpanels{k}.strengthdir,linseptpanels{k}.strength2dir,linseptpanels{k}.strength3dir);
      [~, cubsepterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubseptpanels{k}.panels,cubseptpanels{k}.directions,'septstrength',cubseptpanels{k}.strength,cubseptpanels{k}.strengthdir, cubseptpanels{k}.strength2dir, cubseptpanels{k}.strength3dir);
      [~, quintsepterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintseptpanels{k}.panels,quintseptpanels{k}.directions,quintseptpanels{k}.directions2,'septstrength',quintseptpanels{k}.strength,quintseptpanels{k}.strengthdir, quintseptpanels{k}.strength2dir, quintseptpanels{k}.strength3dir);
      [~, septsepterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'septshape',septseptpanels{k}.panels,septseptpanels{k}.directions,septseptpanels{k}.directions2,septseptpanels{k}.directions3,'septstrength',septseptpanels{k}.strength,septseptpanels{k}.strengthdir, septseptpanels{k}.strength2dir, septseptpanels{k}.strength3dir);
    end


    % How the error varies as a function of distance from the airfoil
    if diststudy
      for m=1:length(sp)
        linerror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'linshape',linpanels{k}.panels,'linstrength',linpanels{k}.strength);
        cuberror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubpanels{k}.panels,cubpanels{k}.directions,'linstrength',cubpanels{k}.strength);
        quinterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintpanels{k}.panels,quintpanels{k}.directions,quintpanels{k}.directions2,'linstrength',quintpanels{k}.strength);
        septerror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'septshape',septpanels{k}.panels,septpanels{k}.directions,septpanels{k}.directions2,septpanels{k}.directions3,'linstrength',septpanels{k}.strength);
        lincuberror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'linshape',lincubpanels{k}.panels,'cubstrength',lincubpanels{k}.strength,lincubpanels{k}.strengthdir);
        cubcuberror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubcubpanels{k}.panels,cubcubpanels{k}.directions,'cubstrength',cubcubpanels{k}.strength,cubcubpanels{k}.strengthdir);
        quintcuberror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintcubpanels{k}.panels,quintcubpanels{k}.directions,quintcubpanels{k}.directions2,'cubstrength',quintcubpanels{k}.strength,quintcubpanels{k}.strengthdir);
        septcuberror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'septshape',septcubpanels{k}.panels,septcubpanels{k}.directions,septcubpanels{k}.directions2,septcubpanels{k}.directions3,'cubstrength',septcubpanels{k}.strength,septcubpanels{k}.strengthdir);
        linquinterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'linshape',linquintpanels{k}.panels,'quintstrength',linquintpanels{k}.strength,linquintpanels{k}.strengthdir,linquintpanels{k}.strength2dir);
        cubquinterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubquintpanels{k}.panels,cubquintpanels{k}.directions,'quintstrength',cubquintpanels{k}.strength,cubquintpanels{k}.strengthdir, cubquintpanels{k}.strength2dir);
        quintquinterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintquintpanels{k}.panels,quintquintpanels{k}.directions,quintquintpanels{k}.directions2,'quintstrength',quintquintpanels{k}.strength,quintquintpanels{k}.strengthdir, quintquintpanels{k}.strength2dir);
        septquinterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'septshape',septquintpanels{k}.panels,septquintpanels{k}.directions,septquintpanels{k}.directions2,septquintpanels{k}.directions3,'quintstrength',septquintpanels{k}.strength,septquintpanels{k}.strengthdir, septquintpanels{k}.strength2dir);
        linsepterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'linshape',linseptpanels{k}.panels,'septstrength',linseptpanels{k}.strength,linseptpanels{k}.strengthdir,linseptpanels{k}.strength2dir,linseptpanels{k}.strength3dir);
        cubsepterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubseptpanels{k}.panels,cubseptpanels{k}.directions,'septstrength',cubseptpanels{k}.strength,cubseptpanels{k}.strengthdir, cubseptpanels{k}.strength2dir,cubseptpanels{k}.strength3dir);
        quintsepterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintseptpanels{k}.panels,quintseptpanels{k}.directions,quintseptpanels{k}.directions2,'septstrength',quintseptpanels{k}.strength,quintseptpanels{k}.strengthdir, quintseptpanels{k}.strength2dir,quintseptpanels{k}.strength3dir);
        septsepterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'septshape',septseptpanels{k}.panels,septseptpanels{k}.directions,septseptpanels{k}.directions2,septseptpanels{k}.directions3,'septstrength',septseptpanels{k}.strength,septseptpanels{k}.strengthdir, septseptpanels{k}.strength2dir,septseptpanels{k}.strength3dir);
      end
    end
  end
end

% Accuracy as function of number of panels
if panelstudy
  % Preallocate arrays
  linerrord=cell(size(panelcount));
  cuberrord=cell(size(panelcount));
  quinterrord=cell(size(panelcount));
  septerrord=cell(size(panelcount));
  lincuberrord=cell(size(panelcount));
  cubcuberrord=cell(size(panelcount));
  quintcuberrord=cell(size(panelcount));
  septcuberrord=cell(size(panelcount));
  linquinterrord=cell(size(panelcount));
  cubquinterrord=cell(size(panelcount));
  quintquinterrord=cell(size(panelcount));
  septquinterrord=cell(size(panelcount));
  linsepterrord=cell(size(panelcount));
  cubsepterrord=cell(size(panelcount));
  quintsepterrord=cell(size(panelcount));
  septsepterrord=cell(size(panelcount));
  
  % Study all listed distances
  for k=1:length(distval)
    linerrord{k}=zeros(size(paneleval));
    cuberrord{k}=zeros(size(paneleval));
    quinterrord{k}=zeros(size(paneleval));
    septerrord{k}=zeros(size(paneleval));
    lincuberrord{k}=zeros(size(paneleval));
    cubcuberrord{k}=zeros(size(paneleval));
    quintcuberrord{k}=zeros(size(paneleval));
    septcuberrord{k}=zeros(size(paneleval));
    linquinterrord{k}=zeros(size(paneleval));
    cubquinterrord{k}=zeros(size(paneleval));
    quintquinterrord{k}=zeros(size(paneleval));
    septquinterrord{k}=zeros(size(paneleval));
    linsepterrord{k}=zeros(size(paneleval));
    cubsepterrord{k}=zeros(size(paneleval));
    quintsepterrord{k}=zeros(size(paneleval));
    septsepterrord{k}=zeros(size(paneleval));
    
    linmyerror=zeros(size(paneleval));
    cubmyerror=zeros(size(paneleval));
    quintmyerror=zeros(size(paneleval));
    septmyerror=zeros(size(paneleval));
    lincubmyerror=zeros(size(paneleval));
    cubcubmyerror=zeros(size(paneleval));
    quintcubmyerror=zeros(size(paneleval));
    septcubmyerror=zeros(size(paneleval));
    linquintmyerror=zeros(size(paneleval));
    cubquintmyerror=zeros(size(paneleval));
    quintquintmyerror=zeros(size(paneleval));
    septquintmyerror=zeros(size(paneleval));
    linseptmyerror=zeros(size(paneleval));
    cubseptmyerror=zeros(size(paneleval));
    quintseptmyerror=zeros(size(paneleval));
    septseptmyerror=zeros(size(paneleval));
    
    % Loop over the number of panels
    for m=1:length(paneleval)
      % Set up panels
      TEangle=angle(1-my);
      angles=linspace(TEangle,2*pi+TEangle,paneleval(m)+1);
      anglesdir=angles;
      anglesdir(1)=anglesdir(1)+1e-10;
      anglesdir(end)=anglesdir(end)-1e-10;
      konst=sqrt((1-real(my))^2+imag(my)^2);
      panelpoints=konst.*exp(1i*angles)+my;
      panelpointsdir=konst.*exp(1i*anglesdir)+my;
      dsdt=konst*1i*exp(1i*angles);
      d2sdt2=-konst*exp(1i*angles);
      d3sdt3=-dsdt;
      z=karmantrefftz(panelpoints,n);
      dzds=karmantrefftzd(panelpointsdir,n);
      d2zds2=karmantrefftzd2(panelpointsdir,n);
      d3zds3=karmantrefftzd3(panelpointsdir,n);
      paneldirections=dsdt.*dzds;
      panel2directions=d2zds2.*dsdt.^2+dzds.*d2sdt2;
      panel3directions=d3sdt3.*dzds+dsdt.*(3.*d2sdt2.*d2zds2+d3zds3.*dsdt.^2);
      panel2directions(real(z)>1.9)=NaN;
      panel3directions(real(z)>1.9)=NaN;
      
      linpanelsd=highorderpanelstrengthkutta(z,[],[],[],1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      cubpanelsd=highorderpanelstrengthkutta(z,paneldirections,[],[],1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      quintpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      septpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,1,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      lincubpanelsd=highorderpanelstrengthkutta(z,[],[],[],3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      cubcubpanelsd=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      quintcubpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      septcubpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,3,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      linquintpanelsd=highorderpanelstrengthkutta(z,[],[],[],5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      cubquintpanelsd=highorderpanelstrengthkutta(z,paneldirections,[],[],5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      quintquintpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      septquintpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,5,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      linseptpanelsd=highorderpanelstrengthkutta(z,[],[],[],7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      cubseptpanelsd=highorderpanelstrengthkutta(z,paneldirections,[],[],7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      quintseptpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      septseptpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,panel3directions,7,1,controlpoint,slopecorrection,doublecontrol,normalize,kuttamethod);
      
      % Accuracy in flow velocity
      if velstudy
        linerrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'linshape',linpanelsd.panels,'linstrength',linpanelsd.strength);
        cuberrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubpanelsd.panels,cubpanelsd.directions,'linstrength',cubpanelsd.strength);
        quinterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintpanelsd.panels,quintpanelsd.directions,quintpanelsd.directions2,'linstrength',quintpanelsd.strength);
        septerrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'septshape',septpanelsd.panels,septpanelsd.directions,septpanelsd.directions2,septpanelsd.directions3,'linstrength',septpanelsd.strength);
        lincuberrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'linshape',lincubpanelsd.panels,'cubstrength',lincubpanelsd.strength,lincubpanelsd.strengthdir);
        cubcuberrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubcubpanelsd.panels,cubcubpanelsd.directions,'cubstrength',cubcubpanelsd.strength,cubcubpanelsd.strengthdir);
        quintcuberrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintcubpanelsd.panels,quintcubpanelsd.directions,quintcubpanelsd.directions2,'cubstrength',quintcubpanelsd.strength,quintcubpanelsd.strengthdir);
        septcuberrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'septshape',septcubpanelsd.panels,septcubpanelsd.directions,septcubpanelsd.directions2,septcubpanelsd.directions3,'cubstrength',septcubpanelsd.strength,septcubpanelsd.strengthdir);
        linquinterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'linshape',linquintpanelsd.panels,'quintstrength',linquintpanelsd.strength,linquintpanelsd.strengthdir,linquintpanelsd.strength2dir);
        cubquinterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubquintpanelsd.panels,cubquintpanelsd.directions,'quintstrength',cubquintpanelsd.strength,cubquintpanelsd.strengthdir, cubquintpanelsd.strength2dir);
        quintquinterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintquintpanelsd.panels,quintquintpanelsd.directions,quintquintpanelsd.directions2,'quintstrength',quintquintpanelsd.strength,quintquintpanelsd.strengthdir, quintquintpanelsd.strength2dir);
        septquinterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'septshape',septquintpanelsd.panels,septquintpanelsd.directions,septquintpanelsd.directions2,septquintpanelsd.directions3,'quintstrength',septquintpanelsd.strength,septquintpanelsd.strengthdir, septquintpanelsd.strength2dir);
        linsepterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'linshape',linseptpanelsd.panels,'septstrength',linseptpanelsd.strength,linseptpanelsd.strengthdir,linseptpanelsd.strength2dir,linseptpanelsd.strength3dir);
        cubsepterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubseptpanelsd.panels,cubseptpanelsd.directions,'septstrength',cubseptpanelsd.strength,cubseptpanelsd.strengthdir, cubseptpanelsd.strength2dir, cubseptpanelsd.strength3dir);
        quintsepterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintseptpanelsd.panels,quintseptpanelsd.directions,quintseptpanelsd.directions2,'septstrength',quintseptpanelsd.strength,quintseptpanelsd.strengthdir, quintseptpanelsd.strength2dir, quintseptpanelsd.strength3dir);
        septsepterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'septshape',septseptpanelsd.panels,septseptpanelsd.directions,septseptpanelsd.directions2,septseptpanelsd.directions3,'septstrength',septseptpanelsd.strength,septseptpanelsd.strengthdir, septseptpanelsd.strength2dir, septseptpanelsd.strength3dir);
      end
      
      % Accuracy in circulation
      if circstudy
        R=sqrt((1-real(my))^2+imag(my)^2);
        Gamma=-4*pi*R*sin(asin(imag(my)/R));
        linmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'linshape',linpanelsd.panels,'linstrength',linpanelsd.strength}))))/abs(Gamma);
        cubmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubpanelsd.panels,cubpanelsd.directions,'linstrength',cubpanelsd.strength}))))/abs(Gamma);
        quintmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintpanelsd.panels,quintpanelsd.directions,quintpanelsd.directions2,'linstrength',quintpanelsd.strength}))))/abs(Gamma);
        septmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'septshape',septpanelsd.panels,septpanelsd.directions,septpanelsd.directions2,septpanelsd.directions3,'linstrength',septpanelsd.strength}))))/abs(Gamma);
        lincubmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'linshape',lincubpanelsd.panels,'cubstrength',lincubpanelsd.strength,lincubpanelsd.strengthdir}))))/abs(Gamma);
        cubcubmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubcubpanelsd.panels,cubcubpanelsd.directions,'cubstrength',cubcubpanelsd.strength,cubcubpanelsd.strengthdir}))))/abs(Gamma);
        quintcubmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintcubpanelsd.panels,quintcubpanelsd.directions,quintcubpanelsd.directions2,'cubstrength',quintcubpanelsd.strength,quintcubpanelsd.strengthdir}))))/abs(Gamma);
        septcubmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'septshape',septcubpanelsd.panels,septcubpanelsd.directions,septcubpanelsd.directions2,septcubpanelsd.directions3,'cubstrength',septcubpanelsd.strength,septcubpanelsd.strengthdir}))))/abs(Gamma);
        linquintmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'linshape',linquintpanelsd.panels,'quintstrength',linquintpanelsd.strength,linquintpanelsd.strengthdir,linquintpanelsd.strength2dir}))))/abs(Gamma);
        cubquintmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubquintpanelsd.panels,cubquintpanelsd.directions,'quintstrength',cubquintpanelsd.strength,cubquintpanelsd.strengthdir, cubquintpanelsd.strength2dir}))))/abs(Gamma);
        quintquintmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintquintpanelsd.panels,quintquintpanelsd.directions,quintquintpanelsd.directions2,'quintstrength',quintquintpanelsd.strength,quintquintpanelsd.strengthdir, quintquintpanelsd.strength2dir}))))/abs(Gamma);
        septquintmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'septshape',septquintpanelsd.panels,septquintpanelsd.directions,septquintpanelsd.directions2,septquintpanelsd.directions3,'quintstrength',septquintpanelsd.strength,septquintpanelsd.strengthdir, septquintpanelsd.strength2dir}))))/abs(Gamma);
        linseptmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'linshape',linseptpanelsd.panels,'septstrength',linseptpanelsd.strength,linseptpanelsd.strengthdir,linseptpanelsd.strength2dir,linseptpanelsd.strength3dir}))))/abs(Gamma);
        cubseptmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubseptpanelsd.panels,cubseptpanelsd.directions,'septstrength',cubseptpanelsd.strength,cubseptpanelsd.strengthdir, cubseptpanelsd.strength2dir, cubseptpanelsd.strength3dir}))))/abs(Gamma);
        quintseptmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintseptpanelsd.panels,quintseptpanelsd.directions,quintseptpanelsd.directions2,'septstrength',quintseptpanelsd.strength,quintseptpanelsd.strengthdir, quintseptpanelsd.strength2dir, quintseptpanelsd.strength3dir}))))/abs(Gamma);
        septseptmyerror(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'septshape',septseptpanelsd.panels,septseptpanelsd.directions,septseptpanelsd.directions2,septseptpanelsd.directions3,'septstrength',septseptpanelsd.strength,septseptpanelsd.strengthdir, septseptpanelsd.strength2dir, septseptpanelsd.strength3dir}))))/abs(Gamma);
      end
    end
  end
end

if ~exist(filename,'file')
  save(filename,'my','n','controlpoint','slopecorrection','doublecontrol','normalize','kuttamethod',...
    'panelcount','sp','distval','surfaceval','paneleval','diststudy','panelstudy','circstudy','velstudy','surfacestudy','surface_pos', ...
    'linerror_surf','cuberror_surf','quinterror_surf','septerror_surf','lincuberror_surf','cubcuberror_surf','quintcuberror_surf','septcuberror_surf', ...
    'linquinterror_surf','cubquinterror_surf','quintquinterror_surf','septquinterror_surf','linsepterror_surf','cubsepterror_surf','quintsepterror_surf','septsepterror_surf', ...
    'linerror','cuberror','quinterror','septerror','lincuberror','cubcuberror','quintcuberror','septcuberror', ...
    'linquinterror','cubquinterror','quintquinterror','septquinterror','linsepterror','cubsepterror','quintsepterror','septsepterror', ...
    'linerrord','cuberrord','quinterrord','septerrord','lincuberrord','cubcuberrord','quintcuberrord','septcuberrord', ...
    'linquinterrord','cubquinterrord','quintquinterrord','septquinterrord','linsepterrord','cubsepterrord','quintsepterrord','septsepterrord',...
    'linmyerror','cubmyerror','quintmyerror','septmyerror','lincubmyerror','cubcubmyerror','quintcubmyerror','septcubmyerror', ...
    'linquintmyerror','cubquintmyerror','quintquintmyerror','septquintmyerror','linseptmyerror','cubseptmyerror','quintseptmyerror','septseptmyerror')
end