% karmantrefftzconvergencecpsc_run run script for the paper "A General high order 
% two-dimensional panel method".
%
% This test performes the comparison between using control point or
% integral over the panels surface, and also the use of the correction of
% the slope
%
% my              - Circle center for the Karman Trefftz mapping
% n               - Parameter for the Karman Trefftz mapping
% doublecontrol   - If target panels should be split in one (or more) extra
%                   panels and then solve the system in the least square
%                   sense
% normalize       - Normalize panels to make each panel have equal weight.
%                   Only applies for controlpoint=false as this is always
%                   true when controlpoint=true
% kuttamethod     - How to solve Kutta condition, 1 means zero vorticity at
%                   the trailing edge, 2 means opposite circulation between
%                   the upper and lower parts. 
% filename        - Name of file where results should be stored

function karmantrefftzconvergencecpsc_run(my,n,doublecontrol,normalize,kuttamethod,filename)
if exist(filename,'file')
  disp('File already exist, aborting');
  return
end
panelcount=100; % Number of panels to study
sp=logspace(0.0000001,1,200); % Distances to loop over for the distance plot

distval=1.01;      % distances to study for the number of panel study
surfaceval=1.01;   % distance to study for the surface plot 
paneleval=[20:80 82:2:150 155:5:300]; % panels to loop over for the number of panels study

diststudy=true;    % Accuracy with respect to distance from wing
panelstudy=true;   % Accuracy with respect to number of panels 
circstudy=true;    % Accuracy in circulation
velstudy=true;     % Accuracy in velocity
surfacestudy=true; % Error over the surface

if ~velstudy
  distval=2; %make it only one element
end

if diststudy || surfacestudy
  %Preallocate arrays
  cubcubpanels=cell(size(panelcount));
  quintquintpanels=cell(size(panelcount));
  cubcubpanelscp=cell(size(panelcount));
  quintquintpanelscp=cell(size(panelcount));
  cubcubpanelssc=cell(size(panelcount));
  quintquintpanelssc=cell(size(panelcount));
  
  cubcuberror=cell(size(panelcount));
  quintquinterror=cell(size(panelcount));
  cubcuberrorcp=cell(size(panelcount));
  quintquinterrorcp=cell(size(panelcount));
  cubcuberrorsc=cell(size(panelcount));
  quintquinterrorsc=cell(size(panelcount));

  surface_pos=cell(size(panelcount));
  cubcuberror_surf=cell(size(panelcount));
  quintquinterror_surf=cell(size(panelcount));
  cubcuberrorcp_surf=cell(size(panelcount));
  quintquinterrorcp_surf=cell(size(panelcount));
  cubcuberrorsc_surf=cell(size(panelcount));
  quintquinterrorsc_surf=cell(size(panelcount));
  
  for k=1:length(panelcount)
    % Set up panels
    TEangle=angle(1-my);
    angles=linspace(TEangle,2*pi+TEangle,panelcount(k)+1);
    anglesdir=angles;
    anglesdir(1)=anglesdir(1)+1e-10;
    anglesdir(end)=anglesdir(end)-1e-10;
    konst=sqrt((1-real(my))^2+imag(my)^2);
    panelpoints=konst.*exp(1i*angles)+my;
    panelpointsdir=konst.*exp(1i*anglesdir)+my;
    panelpointsdir2=konst.*exp(1i*anglesdir)+my;
    dsdt=konst*1i*exp(1i*angles);
    d2sdt2=-konst*exp(1i*angles);
    z=karmantrefftz(panelpoints,n);
    dzds=karmantrefftzd(panelpointsdir,n);
    d2zds2=karmantrefftzd2(panelpointsdir2,n);
    paneldirections=dsdt.*dzds;
    panel2directions=d2zds2.*dsdt.^2+dzds.*d2sdt2;
    panel2directions(real(z)>1.9)=NaN;

    cubcubpanels{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,0,0,doublecontrol,normalize,kuttamethod);
    quintquintpanels{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,0,0,doublecontrol,normalize,kuttamethod);
    cubcubpanelscp{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,1,0,doublecontrol,normalize,kuttamethod);
    quintquintpanelscp{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,1,0,doublecontrol,normalize,kuttamethod);
    cubcubpanelssc{k}=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,0,1,doublecontrol,normalize,kuttamethod);
    quintquintpanelssc{k}=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,0,1,doublecontrol,normalize,kuttamethod);

    % How the error varies along the integration line
    if surfacestudy
      [surface_pos{k}, cubcuberror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubcubpanels{k}.panels,cubcubpanels{k}.directions,'cubstrength',cubcubpanels{k}.strength,cubcubpanels{k}.strengthdir);
      [~, quintquinterror_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintquintpanels{k}.panels,quintquintpanels{k}.directions,quintquintpanels{k}.directions2,'quintstrength',quintquintpanels{k}.strength,quintquintpanels{k}.strengthdir, quintquintpanels{k}.strength2dir);
      [~, cubcuberrorcp_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubcubpanelscp{k}.panels,cubcubpanelscp{k}.directions,'cubstrength',cubcubpanelscp{k}.strength,cubcubpanelscp{k}.strengthdir);
      [~, quintquinterrorcp_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintquintpanelscp{k}.panels,quintquintpanelscp{k}.directions,quintquintpanelscp{k}.directions2,'quintstrength',quintquintpanelscp{k}.strength,quintquintpanelscp{k}.strengthdir, quintquintpanelscp{k}.strength2dir);
      [~, cubcuberrorsc_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'cubshape',cubcubpanelssc{k}.panels,cubcubpanelssc{k}.directions,'cubstrength',cubcubpanelssc{k}.strength,cubcubpanelssc{k}.strengthdir);
      [~, quintquinterrorsc_surf{k}] = karmantrefftzvalidation_surface(surfaceval,n,my,'quintshape',quintquintpanelssc{k}.panels,quintquintpanelssc{k}.directions,quintquintpanelssc{k}.directions2,'quintstrength',quintquintpanelssc{k}.strength,quintquintpanelssc{k}.strengthdir, quintquintpanelssc{k}.strength2dir);
    end

    % How the error varies as a function of distance from the airfoil
    if diststudy
      for m=1:length(sp)
        cubcuberror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubcubpanels{k}.panels,cubcubpanels{k}.directions,'cubstrength',cubcubpanels{k}.strength,cubcubpanels{k}.strengthdir);
        quintquinterror{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintquintpanels{k}.panels,quintquintpanels{k}.directions,quintquintpanels{k}.directions2,'quintstrength',quintquintpanels{k}.strength,quintquintpanels{k}.strengthdir, quintquintpanels{k}.strength2dir);
        cubcuberrorcp{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubcubpanelscp{k}.panels,cubcubpanelscp{k}.directions,'cubstrength',cubcubpanelscp{k}.strength,cubcubpanelscp{k}.strengthdir);
        quintquinterrorcp{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintquintpanelscp{k}.panels,quintquintpanelscp{k}.directions,quintquintpanelscp{k}.directions2,'quintstrength',quintquintpanelscp{k}.strength,quintquintpanelscp{k}.strengthdir, quintquintpanelscp{k}.strength2dir);
        cubcuberrorsc{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'cubshape',cubcubpanelssc{k}.panels,cubcubpanelssc{k}.directions,'cubstrength',cubcubpanelssc{k}.strength,cubcubpanelssc{k}.strengthdir);
        quintquinterrorsc{k}(m)=karmantrefftzvalidationv2(sp(m),n,my,'quintshape',quintquintpanelssc{k}.panels,quintquintpanelssc{k}.directions,quintquintpanelssc{k}.directions2,'quintstrength',quintquintpanelssc{k}.strength,quintquintpanelssc{k}.strengthdir, quintquintpanelssc{k}.strength2dir);
      end
    end
  end
end

% Accuracy as function of number of panels
if panelstudy
  % Preallocate arrays
  cubcuberrord=cell(size(panelcount));
  quintquinterrord=cell(size(panelcount));
  cubcuberrordcp=cell(size(panelcount));
  quintquinterrordcp=cell(size(panelcount));
  cubcuberrordsc=cell(size(panelcount));
  quintquinterrordsc=cell(size(panelcount));
  
  cubcubmyerror=cell(size(panelcount));
  quintquintmyerror=cell(size(panelcount));
  cubcubmyerrorcp=cell(size(panelcount));
  quintquintmyerrorcp=cell(size(panelcount));
  cubcubmyerrorsc=cell(size(panelcount));
  quintquintmyerrorsc=cell(size(panelcount));
  
  % Study all listed distances
  for k=1:length(distval)
    cubcuberrord{k}=zeros(size(paneleval));
    quintquinterrord{k}=zeros(size(paneleval));
    cubcuberrordcp{k}=zeros(size(paneleval));
    quintquinterrordcp{k}=zeros(size(paneleval));
    cubcuberrordsc{k}=zeros(size(paneleval));
    quintquinterrordsc{k}=zeros(size(paneleval));
    
    cubcubmyerror{k}=zeros(size(paneleval));
    quintquintmyerror{k}=zeros(size(paneleval));
    cubcubmyerrorcp{k}=zeros(size(paneleval));
    quintquintmyerrorcp{k}=zeros(size(paneleval));
    cubcubmyerrorsc{k}=zeros(size(paneleval));
    quintquintmyerrorsc{k}=zeros(size(paneleval));
    
    for m=1:length(paneleval)
      % Set up panels
      TEangle=angle(1-my);
      angles=linspace(TEangle,2*pi+TEangle,paneleval(m)+1);
      anglesdir=angles;
      anglesdir(1)=anglesdir(1)+1e-10;
      anglesdir(end)=anglesdir(end)-1e-10;
      konst=sqrt((1-real(my))^2+imag(my)^2);
      panelpoints=konst.*exp(1i*angles)+my;
      panelpointsdir=konst.*exp(1i*anglesdir)+my;
      dsdt=konst*1i*exp(1i*angles);
      d2sdt2=-konst*exp(1i*angles);
      z=karmantrefftz(panelpoints,n);
      dzds=karmantrefftzd(panelpointsdir,n);
      d2zds2=karmantrefftzd2(panelpointsdir,n);
      paneldirections=dsdt.*dzds;
      panel2directions=d2zds2.*dsdt.^2+dzds.*d2sdt2;
      panel2directions(real(z)>1.9)=NaN;
      

      cubcubpanelsd=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,0,0,doublecontrol,normalize,kuttamethod);
      quintquintpanelsd=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,0,0,doublecontrol,normalize,kuttamethod);
      cubcubpanelsdcp=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,1,0,doublecontrol,normalize,kuttamethod);
      quintquintpanelsdcp=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,1,0,doublecontrol,normalize,kuttamethod);
      cubcubpanelsdsc=highorderpanelstrengthkutta(z,paneldirections,[],[],3,1,0,1,doublecontrol,normalize,kuttamethod);
      quintquintpanelsdsc=highorderpanelstrengthkutta(z,paneldirections,panel2directions,[],5,1,0,1,doublecontrol,normalize,kuttamethod);

      % Accuracy in flow velocity
      if velstudy
        cubcuberrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubcubpanelsd.panels,cubcubpanelsd.directions,'cubstrength',cubcubpanelsd.strength,cubcubpanelsd.strengthdir);
        quintquinterrord{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintquintpanelsd.panels,quintquintpanelsd.directions,quintquintpanelsd.directions2,'quintstrength',quintquintpanelsd.strength,quintquintpanelsd.strengthdir, quintquintpanelsd.strength2dir);
        cubcuberrordcp{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubcubpanelsdcp.panels,cubcubpanelsdcp.directions,'cubstrength',cubcubpanelsdcp.strength,cubcubpanelsdcp.strengthdir);
        quintquinterrordcp{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintquintpanelsdcp.panels,quintquintpanelsdcp.directions,quintquintpanelsdcp.directions2,'quintstrength',quintquintpanelsdcp.strength,quintquintpanelsdcp.strengthdir, quintquintpanelsdcp.strength2dir);
        cubcuberrordsc{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'cubshape',cubcubpanelsdsc.panels,cubcubpanelsdsc.directions,'cubstrength',cubcubpanelsdsc.strength,cubcubpanelsdsc.strengthdir);
        quintquinterrordsc{k}(m)=karmantrefftzvalidationv2(distval(k),n,my,'quintshape',quintquintpanelsdsc.panels,quintquintpanelsdsc.directions,quintquintpanelsdsc.directions2,'quintstrength',quintquintpanelsdsc.strength,quintquintpanelsdsc.strengthdir, quintquintpanelsdsc.strength2dir);
      end
      
      % Accuracy in circulation
      if circstudy
        R=sqrt((1-real(my))^2+imag(my)^2);
        Gamma=-4*pi*R*sin(asin(imag(my)/R));
        cubcubmyerror{k}(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubcubpanelsd.panels,cubcubpanelsd.directions,'cubstrength',cubcubpanelsd.strength,cubcubpanelsd.strengthdir}))))/abs(Gamma);
        quintquintmyerror{k}(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintquintpanelsd.panels,quintquintpanelsd.directions,quintquintpanelsd.directions2,'quintstrength',quintquintpanelsd.strength,quintquintpanelsd.strengthdir, quintquintpanelsd.strength2dir}))))/abs(Gamma);
        cubcubmyerrorcp{k}(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubcubpanelsdcp.panels,cubcubpanelsdcp.directions,'cubstrength',cubcubpanelsdcp.strength,cubcubpanelsdcp.strengthdir}))))/abs(Gamma);
        quintquintmyerrorcp{k}(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintquintpanelsdcp.panels,quintquintpanelsdcp.directions,quintquintpanelsdcp.directions2,'quintstrength',quintquintpanelsdcp.strength,quintquintpanelsdcp.strengthdir, quintquintpanelsdcp.strength2dir}))))/abs(Gamma);
        cubcubmyerrorsc{k}(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'cubshape',cubcubpanelsdsc.panels,cubcubpanelsdsc.directions,'cubstrength',cubcubpanelsdsc.strength,cubcubpanelsdsc.strengthdir}))))/abs(Gamma);
        quintquintmyerrorsc{k}(m)=abs(Gamma-imag(sum(fmm2d([],[],'tol',0,'pot',1,'out','panelstrength','sourcepanels',{'quintshape',quintquintpanelsdsc.panels,quintquintpanelsdsc.directions,quintquintpanelsdsc.directions2,'quintstrength',quintquintpanelsdsc.strength,quintquintpanelsdsc.strengthdir, quintquintpanelsdsc.strength2dir}))))/abs(Gamma);
      end
    end
  end
end

if ~exist(filename,'file')
  save(filename,'my','n','doublecontrol','normalize','kuttamethod',...
    'panelcount','sp','distval','surfaceval','paneleval','diststudy','panelstudy','circstudy','velstudy','surfacestudy','surface_pos', ...
    'cubcuberror_surf','quintquinterror_surf','cubcuberrorcp_surf','quintquinterrorcp_surf','cubcuberrorsc_surf','quintquinterrorsc_surf', ...
    'cubcuberror','quintquinterror','cubcuberrorcp','quintquinterrorcp','cubcuberrorsc','quintquinterrorsc', ...
    'cubcuberrord','quintquinterrord','cubcuberrordcp','quintquinterrordcp','cubcuberrordsc','quintquinterrordsc',...
    'cubcubmyerror','quintquintmyerror','cubcubmyerrorcp','quintquintmyerrorcp','cubcubmyerrorsc','quintquintmyerrorsc');
end