% Function to evaluate the error in flow velocity between a panel solution
% and the analytical solution of the Karman Trefftz airfoil at a distance
% away from the airfoil. This function returns the error along the
% integration line.
%
% The parameters are
% 
% distance - distance form airfoil. Note that 1 is at the airfoil surface
% n - coefficient for the Karma Trefftz mapping
% my - center for the circle in the Karman Trefftz mapping
% varargin - defines the panel
function [xvals, yvals] = karmantrefftzvalidation_surface(distance,n,my,varargin)
TEangle=angle(1-my);
endangles=endangle(TEangle,distance,n,my);
theta=TEangle+0.00005:0.0001:TEangle+2*pi-0.00005; % theta values. Avoid the trailing edge, as it will be included in theta2 and theta3
ds=getdist(theta,distance,n,my)*0.0001;            % length of each element
ds=cumsum(abs(ds));                                % The cumulative sum gives the position along the airfoil
ret=testvalue(theta,distance,n,my,varargin{:});    % Evaluate the error along this line
midangle=mean(endangles);                          % For the arc that connects the two end points, find the mid point and split the curve here
theta2=endangles(1):0.0001:midangle;               % Calculate the arc in two parts, and put one first and the second last, to make the middle of the arc the start and end point
ret2=testvalueedge(theta2,distance,n,my,varargin{:});
theta3=midangle:0.0001:endangles(2);
ret3=testvalueedge(theta3,distance,n,my,varargin{:});
xvals=[(distance-1)*(theta3-midangle), ds+(distance-1)*(midangle-endangles(1)), max(ds)+(distance-1)*(theta2-2*endangles(1)+midangle)]; % add half of the arc first, then ds, and finally the second half.
yvals=[ret3 ret ret2];


% Function to carry out the integral at a distance from the airfoil. Not
% that the curve is discontinuous at the trailing edge, and should hence
% not be integrated over
function ret=testvalue(angleval,distance,n,my,varargin)
s=sqrt((1-real(my)).^2+imag(my).^2).*exp(1i*angleval)+my;  % Position along the airfoil in the s plane
z=karmantrefftz(s,n);                                      % Map to z plane
n2=karmantrefftzd(s,n).*exp(1i*angleval);                  % Calculate normal vector
n2=n2./abs(n2);
z=z+(distance-1)*n2;                                       % Generate a point at a distance from the airfoil in the z plane
news=inversekarmantrefftz(z,n,my,distance*s);              % Use inverse mapping to calculate the position in the s plane
exact=karmantrefftzvelocity(news,my,n,1,0);                % Analytical solution
if max(abs(z-karmantrefftz(news,n)))>1e-12
  warning(['Inverse karman trefftz failed, max error is ' num2str(max(abs(z-karmantrefftz(news,n))))]);
end
approx=conj(fmm2d([],[],z,'pot',1,'tol',0,'sourcepanels',varargin)/2/pi)+1; % Numerical solution
ret=abs(exact-approx);

% Function to carry out the integral around the trailing edge, which is
% performed in a circular arc
function ret=testvalueedge(angleval,distance,n,my,varargin)
s=1;
zbase=karmantrefftz(s,n);
z=zbase+(distance-1)*exp(1i*angleval);                               % Start by generating the arc in z coordinates
news=inversekarmantrefftz(z,n,my,1+(distance-1)*exp(0.5i*angleval)); % Calculate the corresponding s coordinates for the analytical solution
if max(abs(z-karmantrefftz(news,n)))>1e-12
  warning(['Inverse karman trefftz failed, max error is ' num2str(max(abs(z-karmantrefftz(news,n))))]);
end
exact=karmantrefftzvelocity(news,my,n,1,0);                                  % Analytical solution
approx=conj(fmm2d([],[],z,'pot',1,'tol',0,'sourcepanels',varargin)/2/pi)+1;  % Panel solution
ret=abs(exact-approx);

% Calculate the length for the integration curve, given the angle and the
% distance from the airfoil
function ds=getdist(angleval,distance,n,my)
R=sqrt((1-real(my)).^2+imag(my).^2);                            % Radius of circle for mapping
s=R.*exp(1i*angleval)+my;                                       % Calculate values in s - plane
n2=karmantrefftzd(s,n).*exp(1i*angleval)*R;                     % Calculate normal vector
d3=1i.*karmantrefftzd2(s,n).*R^2.*exp(2i.*angleval)+1i.*n2;     % Use second derivative to account for curvature
n3=1i.*n2.*imag(d3.*conj(n2))./abs(n2).^3;

ds=1i*R.*karmantrefftzd(s,n).*exp(1i*angleval)+(distance-1)*n3; % the distance element ds

% Calculates the angle of the panels at the trailing edge positions some
% distance from it (the start and end points of the integral)
function ret=endangle(TEangle,distance,n,my)
angleval=[TEangle+1e-10 TEangle TEangle+2*pi-1e-10];      % Use numerical differentiation with a small distance
s=sqrt((1-real(my)).^2+imag(my).^2).*exp(1i*angleval)+my; % Calculate values in s - plane
z=karmantrefftz(s,n);                                     % Map to z - plane
n2=karmantrefftzd(s([1,3]),n).*exp(1i*angleval([1,3]));   % Calculate normal vector
n2=n2./abs(n2);
z([1,3])=z([1,3])+(distance-1)*n2;                        % Move a distance away
ret=[angle(z(3)-z(2)), angle(z(1)-z(2))];                 % Calculate the angle