% unitcircleconvergence_run run script for the paper "A General high order 
% two-dimensional panel method".
%
% This test performes all simulations related to the unit circle accuracy
% tests. The input parameters allows for configuring the model. 
%
% The parameters are
%
% vortexpanels    - true if panels are built by vorticity strength, false
%                   for source strengt
% controlpoint    - true if control points in the center of the panels are
%                   used, false if flow through the panels are used
% slopecorrection - true if panel strength is corrected based on the slope
%                   of the panel, false if it only should be adjusted at
%                   the end points
% doublecontrol   - If target panels should be split in one (or more) extra
%                   panels and then solve the system in the least square
%                   sense
% filename        - Name of file where results should be stored

function unitcircleconvergence_run(vortexpanels,controlpoint,slopecorrection,doublecontrol,filename)
if exist(filename,'file')
  disp('File already exist, aborting');
  return
end
panelcount=50; % List of numbers of panels to study for the distance study
sp=logspace(0.0000001,1,200); % Distances

distval=1.2; % List of distances to study for number of panels study
paneleval=10:100; % Number of panels

% Which studies that should be performed
diststudy=true; % Accuracy vs distance
panelstudy=true; % Accuracy vs number of panels

% Preallocate arrays
angles=cell(size(panelcount));
panelpoints=cell(size(panelcount));
paneldirections=cell(size(panelcount));
linpanels=cell(size(panelcount));
cubpanels=cell(size(panelcount));
quintpanels=cell(size(panelcount));
septpanels=cell(size(panelcount));
lincubpanels=cell(size(panelcount));
cubcubpanels=cell(size(panelcount));
quintcubpanels=cell(size(panelcount));
septcubpanels=cell(size(panelcount));
linquintpanels=cell(size(panelcount));
cubquintpanels=cell(size(panelcount));
quintquintpanels=cell(size(panelcount));
septquintpanels=cell(size(panelcount));
linseptpanels=cell(size(panelcount));
cubseptpanels=cell(size(panelcount));
quintseptpanels=cell(size(panelcount));
septseptpanels=cell(size(panelcount));

linerror=cell(size(panelcount));
cuberror=cell(size(panelcount));
quinterror=cell(size(panelcount));
septerror=cell(size(panelcount));
lincuberror=cell(size(panelcount));
cubcuberror=cell(size(panelcount));
quintcuberror=cell(size(panelcount));
septcuberror=cell(size(panelcount));
linquinterror=cell(size(panelcount));
cubquinterror=cell(size(panelcount));
quintquinterror=cell(size(panelcount));
septquinterror=cell(size(panelcount));
linsepterror=cell(size(panelcount));
cubsepterror=cell(size(panelcount));
quintsepterror=cell(size(panelcount));
septsepterror=cell(size(panelcount));

% Accuracy as a function of distance from circle
if diststudy
  for k=1:length(panelcount)
    %build circle panels
    angles{k}=linspace(0,2*pi,panelcount(k)+1);
    panelpoints{k}=exp(1i*angles{k}(1:end-1));
    paneldirections{k}=1i*exp(1i*angles{k}(1:end-1));

    linpanels{k}=highorderpanelstrength(panelpoints{k},[],[],[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    cubpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},[],[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    quintpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    septpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},-paneldirections{k},1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    lincubpanels{k}=highorderpanelstrength(panelpoints{k},[],[],[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    cubcubpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},[],[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    quintcubpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    septcubpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},-paneldirections{k},3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    linquintpanels{k}=highorderpanelstrength(panelpoints{k},[],[],[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    cubquintpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},[],[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    quintquintpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    septquintpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},-paneldirections{k},5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    linseptpanels{k}=highorderpanelstrength(panelpoints{k},[],[],[],7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    cubseptpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},[],[],7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    quintseptpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},[],7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    septseptpanels{k}=highorderpanelstrength(panelpoints{k},paneldirections{k},-panelpoints{k},-paneldirections{k},7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
    
    %evaluate for all distances
    for m=1:length(sp)
      linerror{k}(m)=unitcirclevalidation(sp(m),'linshape',linpanels{k}.panels,'linstrength',linpanels{k}.strength);
      cuberror{k}(m)=unitcirclevalidation(sp(m),'cubshape',cubpanels{k}.panels,cubpanels{k}.directions,'linstrength',cubpanels{k}.strength);
      quinterror{k}(m)=unitcirclevalidation(sp(m),'quintshape',quintpanels{k}.panels,quintpanels{k}.directions,quintpanels{k}.directions2,'linstrength',quintpanels{k}.strength);
      septerror{k}(m)=unitcirclevalidation(sp(m),'septshape',septpanels{k}.panels,septpanels{k}.directions,septpanels{k}.directions2,septpanels{k}.directions3,'linstrength',septpanels{k}.strength);
      lincuberror{k}(m)=unitcirclevalidation(sp(m),'linshape',lincubpanels{k}.panels,'cubstrength',lincubpanels{k}.strength,lincubpanels{k}.strengthdir);
      cubcuberror{k}(m)=unitcirclevalidation(sp(m),'cubshape',cubcubpanels{k}.panels,cubcubpanels{k}.directions,'cubstrength',cubcubpanels{k}.strength,cubcubpanels{k}.strengthdir);
      quintcuberror{k}(m)=unitcirclevalidation(sp(m),'quintshape',quintcubpanels{k}.panels,quintcubpanels{k}.directions,quintcubpanels{k}.directions2,'cubstrength',quintcubpanels{k}.strength,quintcubpanels{k}.strengthdir);
      septcuberror{k}(m)=unitcirclevalidation(sp(m),'septshape',septcubpanels{k}.panels,septcubpanels{k}.directions,septcubpanels{k}.directions2,septcubpanels{k}.directions3,'cubstrength',septcubpanels{k}.strength,septcubpanels{k}.strengthdir);
      linquinterror{k}(m)=unitcirclevalidation(sp(m),'linshape',linquintpanels{k}.panels,'quintstrength',linquintpanels{k}.strength,linquintpanels{k}.strengthdir,linquintpanels{k}.strength2dir);
      cubquinterror{k}(m)=unitcirclevalidation(sp(m),'cubshape',cubquintpanels{k}.panels,cubquintpanels{k}.directions,'quintstrength',cubquintpanels{k}.strength,cubquintpanels{k}.strengthdir, cubquintpanels{k}.strength2dir);
      quintquinterror{k}(m)=unitcirclevalidation(sp(m),'quintshape',quintquintpanels{k}.panels,quintquintpanels{k}.directions,quintquintpanels{k}.directions2,'quintstrength',quintquintpanels{k}.strength,quintquintpanels{k}.strengthdir, quintquintpanels{k}.strength2dir);
      septquinterror{k}(m)=unitcirclevalidation(sp(m),'septshape',septquintpanels{k}.panels,septquintpanels{k}.directions,septquintpanels{k}.directions2,septquintpanels{k}.directions3,'quintstrength',septquintpanels{k}.strength,septquintpanels{k}.strengthdir, septquintpanels{k}.strength2dir);
      linsepterror{k}(m)=unitcirclevalidation(sp(m),'linshape',linseptpanels{k}.panels,'septstrength',linseptpanels{k}.strength,linseptpanels{k}.strengthdir,linseptpanels{k}.strength2dir,linseptpanels{k}.strength3dir);
      cubsepterror{k}(m)=unitcirclevalidation(sp(m),'cubshape',cubseptpanels{k}.panels,cubseptpanels{k}.directions,'septstrength',cubseptpanels{k}.strength,cubseptpanels{k}.strengthdir, cubseptpanels{k}.strength2dir,cubseptpanels{k}.strength3dir);
      quintsepterror{k}(m)=unitcirclevalidation(sp(m),'quintshape',quintseptpanels{k}.panels,quintseptpanels{k}.directions,quintseptpanels{k}.directions2,'septstrength',quintseptpanels{k}.strength,quintseptpanels{k}.strengthdir, quintseptpanels{k}.strength2dir,quintseptpanels{k}.strength3dir);
      septsepterror{k}(m)=unitcirclevalidation(sp(m),'septshape',septseptpanels{k}.panels,septseptpanels{k}.directions,septseptpanels{k}.directions2,septseptpanels{k}.directions3,'septstrength',septseptpanels{k}.strength,septseptpanels{k}.strengthdir, septseptpanels{k}.strength2dir,septseptpanels{k}.strength3dir);
    end
  end
end

%panel count study
linerrord=cell(1,length(distval));
cuberrord=cell(1,length(distval));
quinterrord=cell(1,length(distval));
septerrord=cell(1,length(distval));
lincuberrord=cell(1,length(distval));
cubcuberrord=cell(1,length(distval));
quintcuberrord=cell(1,length(distval));
septcuberrord=cell(1,length(distval));
linquinterrord=cell(1,length(distval));
cubquinterrord=cell(1,length(distval));
quintquinterrord=cell(1,length(distval));
septquinterrord=cell(1,length(distval));
linsepterrord=cell(1,length(distval));
cubsepterrord=cell(1,length(distval));
quintsepterrord=cell(1,length(distval));
septsepterrord=cell(1,length(distval));

if panelstudy
  for k=1:length(distval) % Initial loop over distances to study
    % Preallocation
    linerrord{k}=zeros(size(paneleval));
    cuberrord{k}=zeros(size(paneleval));
    quinterrord{k}=zeros(size(paneleval));
    septerrord{k}=zeros(size(paneleval));
    lincuberrord{k}=zeros(size(paneleval));
    cubcuberrord{k}=zeros(size(paneleval));
    quintcuberrord{k}=zeros(size(paneleval));
    septcuberrord{k}=zeros(size(paneleval));
    linquinterrord{k}=zeros(size(paneleval));
    cubquinterrord{k}=zeros(size(paneleval));
    quintquinterrord{k}=zeros(size(paneleval));
    septquinterrord{k}=zeros(size(paneleval));
    linsepterrord{k}=zeros(size(paneleval));
    cubsepterrord{k}=zeros(size(paneleval));
    quintsepterrord{k}=zeros(size(paneleval));
    septsepterrord{k}=zeros(size(paneleval));
    for m=1:length(paneleval) % Loop over panel values
      anglesd=linspace(0,2*pi,paneleval(m)+1);
      panelpointsd=exp(1i*anglesd(1:end-1));
      paneldirectionsd=1i*exp(1i*anglesd(1:end-1));
      
      % Set up panels
      linpanelsd=highorderpanelstrength(panelpointsd,[],[],[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      cubpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      quintpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      septpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,1,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      lincubpanelsd=highorderpanelstrength(panelpointsd,[],[],[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      cubcubpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      quintcubpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      septcubpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,3,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      linquintpanelsd=highorderpanelstrength(panelpointsd,[],[],[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      cubquintpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      quintquintpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      septquintpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,5,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      linseptpanelsd=highorderpanelstrength(panelpointsd,[],[],[],7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      cubseptpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,[],[],7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      quintseptpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,[],7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      septseptpanelsd=highorderpanelstrength(panelpointsd,paneldirectionsd,-panelpointsd,-paneldirectionsd,7,1,vortexpanels,controlpoint,slopecorrection,doublecontrol);
      
      % Calculate the error
      linerrord{k}(m)=unitcirclevalidation(distval(k),'linshape',linpanelsd.panels,'linstrength',linpanelsd.strength);
      cuberrord{k}(m)=unitcirclevalidation(distval(k),'cubshape',cubpanelsd.panels,cubpanelsd.directions,'linstrength',cubpanelsd.strength);
      quinterrord{k}(m)=unitcirclevalidation(distval(k),'quintshape',quintpanelsd.panels,quintpanelsd.directions,quintpanelsd.directions2,'linstrength',quintpanelsd.strength);
      septerrord{k}(m)=unitcirclevalidation(distval(k),'septshape',septpanelsd.panels,septpanelsd.directions,septpanelsd.directions2,septpanelsd.directions3,'linstrength',septpanelsd.strength);
      lincuberrord{k}(m)=unitcirclevalidation(distval(k),'linshape',lincubpanelsd.panels,'cubstrength',lincubpanelsd.strength,lincubpanelsd.strengthdir);
      cubcuberrord{k}(m)=unitcirclevalidation(distval(k),'cubshape',cubcubpanelsd.panels,cubcubpanelsd.directions,'cubstrength',cubcubpanelsd.strength,cubcubpanelsd.strengthdir);
      quintcuberrord{k}(m)=unitcirclevalidation(distval(k),'quintshape',quintcubpanelsd.panels,quintcubpanelsd.directions,quintcubpanelsd.directions2,'cubstrength',quintcubpanelsd.strength,quintcubpanelsd.strengthdir);
      septcuberrord{k}(m)=unitcirclevalidation(distval(k),'septshape',septcubpanelsd.panels,septcubpanelsd.directions,septcubpanelsd.directions2,septcubpanelsd.directions3,'cubstrength',septcubpanelsd.strength,septcubpanelsd.strengthdir);
      linquinterrord{k}(m)=unitcirclevalidation(distval(k),'linshape',linquintpanelsd.panels,'quintstrength',linquintpanelsd.strength,linquintpanelsd.strengthdir,linquintpanelsd.strength2dir);
      cubquinterrord{k}(m)=unitcirclevalidation(distval(k),'cubshape',cubquintpanelsd.panels,cubquintpanelsd.directions,'quintstrength',cubquintpanelsd.strength,cubquintpanelsd.strengthdir, cubquintpanelsd.strength2dir);
      quintquinterrord{k}(m)=unitcirclevalidation(distval(k),'quintshape',quintquintpanelsd.panels,quintquintpanelsd.directions,quintquintpanelsd.directions2,'quintstrength',quintquintpanelsd.strength,quintquintpanelsd.strengthdir, quintquintpanelsd.strength2dir);
      septquinterrord{k}(m)=unitcirclevalidation(distval(k),'septshape',septquintpanelsd.panels,septquintpanelsd.directions,septquintpanelsd.directions2,septquintpanelsd.directions3,'quintstrength',septquintpanelsd.strength,septquintpanelsd.strengthdir, septquintpanelsd.strength2dir);
      linsepterrord{k}(m)=unitcirclevalidation(distval(k),'linshape',linseptpanelsd.panels,'septstrength',linseptpanelsd.strength,linseptpanelsd.strengthdir,linseptpanelsd.strength2dir,linseptpanelsd.strength3dir);
      cubsepterrord{k}(m)=unitcirclevalidation(distval(k),'cubshape',cubseptpanelsd.panels,cubseptpanelsd.directions,'septstrength',cubseptpanelsd.strength,cubseptpanelsd.strengthdir, cubseptpanelsd.strength2dir, cubseptpanelsd.strength3dir);
      quintsepterrord{k}(m)=unitcirclevalidation(distval(k),'quintshape',quintseptpanelsd.panels,quintseptpanelsd.directions,quintseptpanelsd.directions2,'septstrength',quintseptpanelsd.strength,quintseptpanelsd.strengthdir, quintseptpanelsd.strength2dir, quintseptpanelsd.strength3dir);
      septsepterrord{k}(m)=unitcirclevalidation(distval(k),'septshape',septseptpanelsd.panels,septseptpanelsd.directions,septseptpanelsd.directions2,septseptpanelsd.directions3,'septstrength',septseptpanelsd.strength,septseptpanelsd.strengthdir, septseptpanelsd.strength2dir, septseptpanelsd.strength3dir);
    end
  end
end

if ~exist(filename,'file')
  save(filename,'sp','distval','vortexpanels','controlpoint','slopecorrection','paneleval','diststudy','panelstudy','panelcount','paneleval', ...
    'linerror','cuberror','quinterror','septerror','lincuberror','cubcuberror','quintcuberror','septcuberror', ...
    'linquinterror','cubquinterror','quintquinterror','septquinterror','linsepterror','cubsepterror','quintsepterror','septsepterror', ...
    'linerrord','cuberrord','quinterrord','septerrord','lincuberrord','cubcuberrord','quintcuberrord','septcuberrord', ...
    'linquinterrord','cubquinterrord','quintquinterrord','septquinterrord','linsepterrord','cubsepterrord','quintsepterrord','septsepterrord')
end
