function [p,pe] = fmm2d(z,m,ze,varargin)
%FMM2D Fast multipole method in 2 dimensions with GPU-support.
%   P = FMM2D(Z,M) evaluates the potential P due to masses M at
%   Cartesian coordinates Z = X+iY. It is understood that the size of
%   Z matches that of M.
%
%   PE = FMM2D(Z,M,ZE) evaluates the potential at the points ZE
%   instead.
%
%   [P,PE] = FMM2D(Z,M,ZE) evaluates the potential both at Z and ZE at
%   the same time.
%
%   P = FMM2D(Z,M,...) or [...] = FMM2D(Z,M,ZE,...) allows options to 
%   be passed in the form of property/value-pairs. See the table below 
%   for available properties.
%
%   Property      Value/{Default}     Description
%   -----------------------------------------------------------------------
%   pot           {0} | 1             0: Logarithmic potential,
%                                     P = -M*LOG(ABS(Z)).
%                                     1: Harmonic (doublet) potential,
%                                     P = -M/Z.
% 
%   tol           Scalar {1e-6}       Tolerance. The choice tol = 0 means  
%                                     a direct evaluation.
% 
%   ndirect       Scalar integer {5}  The number of particles in each
%                                     box at the finest level.
% 
%   smooth        {'dirac'} |         Near-field models, see below.
%                 'rankine' |
%                 'scully' |
%                 'oseen'
% 
%   xopt          Scalar {1e-3}       Distance to the extreme point of the
%                                     smoother.
%
%   cutoff        Scalar {2e-3}       Cutoff-radius of the smoother.
%
%   cont          {true} | false      Enforce continuity to match the near- 
%                                     and far-fields.
%
%   panel         Cell-vector         Support for panels, see below.
%
%   sourcepanels  Cell-vector         Support for higher order panels, see
%                                     below
%
%   targetpanels  Cell-vector         Support for higher order targetpanels,
%                                     see below
%
%
%   printtime     true | {false}      Print timings of the algorithm.
%
%   out           {'sol'} | 'mesh' |  Output solution/mesh/time. Type
%                'time'              'type fmm2d' for more information on
%                                     the latter two outputs.
%
%   The precise definitions of the smoothers as defined via the properties
%   {'smooth','xopt','cutoff','cont'} are quite complicated. For this
%   purpose, reference implementations have been prepared. See the files
%   LOGPOT.M and INVPOT.M for details.
%
%   The following note is only valid when THETACUTOFFCHECK is
%   undefined (see MAKE): the cut-off radius of the smoother is only
%   respected in direct near-field computations. If this radius is
%   large compared to the boxes used by the algorithm in resolving the
%   computational domain, then the reference implementations in
%   LOGPOT/INVPOT are incorrect since they respect the cut-off radius
%   globally.
%
%   Support for panels is provided through the syntaxes P =
%   FMM2D(Z,M,...,'panel',{ZP,MP,...}) or [...] =
%   FMM2D(Z,M,ZE,...,'panel',{ZP,MP,...}). Here, both ZP and MP are 2-by-NP
%   complex matrices, where the first(second) row contains the first(second)
%   coordinate/strength of each of NP panels. For example, the call PE =
%   FMM2D([],[],ZE,'pot',1,'panel',{ZP,MP}) evaluates the potential at the
%   points in ZE due to panels {ZP,MP} and with no point-masses present. The
%   table below summarizes the special properties available for panels.
%
%   Property     Value/{Default}     Description
%   -----------------------------------------------------------------------
%   dir          Logical scalar or   Orientation of panels. Only meaningful
%                logical vector of   for the dirac smoother and for 
%                length NP {0}       evaluation directly on a panel, where
%                                    the potential is discontinuous.
%
%   smooth       {'dirac'} |         Near-field models, see below;
%                'rankine' |         use 0 for 'dirac', 1 for 'rankine',
%                'hat' | Scalar |    and 2 for 'hat' when using doubles.
%                vector of length NP
%
%   cutoff       Scalar or vector    Cutoff-radius of the smoother.
%                of length NP {2e-3}
%
%   The Rankine smoother is obtained by integrating the corresponding
%   point-smoother over the whole panel. Similarly, the 'hat' smoother
%   is obtained by integrating a finite element piecewise linear
%   hat-function.
%
%   Support for higher order panels is provided through the syntaxes P =
%   FMM2D(Z,M,...,'sourcepanels',{...}) or [...] =
%   FMM2D(Z,M,ZE,...,'sourcepanels',{...}). Here, the panels have to be
%   defined through a shape and a strength and other optional arguments
%
%   Property     Value/{Default}     Description
%   -----------------------------------------------------------------------
%   polyshape    1:a  2xN vector     start positions in first row
%                                    end positions in second row
%                2:nd MxN vector     polynomial coefficients (lowest order
%                                    coefficient first)
%
%   linshape     1:a  2xN vector     start positions in first row
%                                    end positions in second row
%
%   cubshape     1:a  2xN vector     start positions in first row
%                                    end positions in second row
%                2:nd 2xN vector     start directions (complex) in first row
%                                    end directions (complex) in second row
%   quintshape   1:a  2xN vector     start positions in first row
%                                    end positions in second row
%                2:nd 2xN vector     start directions in first row
%                                    end directions in second row
%                3:rd 2xN vector     start 2:nd derivative directions 
%                                    in first row
%                                    end 2:nd derivative directions 
%                                    in second row
%
%
%
%   polystrength 1:a  MxN vector     polynomial coefficients for strength
%                                    (lowest order coefficient first)
%
%   linstrength  1:a  2xN vector     start strength in first row
%                                    end strength in second row
%
%   cubstrength  1:a  2xN vector     start strength in first row
%                                    end strength in second row
%                2:nd 2xN vector     start derivative in first row
%                                    end derivative in second row
%
%
%   outside      -1,{0},1            if 1, assume evaluation is outside
%                                    panels (assuming counter clockwise
%                                    direction), -1 means inside, 0 is
%                                    turned off
%
%   slopecorrection true/{false}     corrects panel strength due to panel
%                                    slope. This is experimental and not
%                                    recommended for use in normal codes
%
%   Notes: For higher order shapes, the values for the higher order
%   derivatives should be given according to the following structure:
%   Assume that the panel is given by a function (x(s), y(s)), then first
%   derivative is (dx/ds, dy/ds), second order derivative is (d2x/ds^2,
%   d2y/ds2) etc.
%   If NaN is used as the direction in the highest order derivative terms,
%   the code will automatically reduce the panel order to the next lower
%   level.
%
%   targetpanels syntax: FMM2D(Z,M,'targetpanels',{cell vector})
%   first output in this case will be the solution of the potential
%   integrated over the panel (multiplied with the panel direction), the
%   second output argument will be the same as the first in the case
%   without panels, and correspondingly third output will be the
%   traditional second. Targetpanels are given with same syntax as source
%   panels, with the exception that no strenght is given for targets
%
%   Note: Targetpanels can only be used with point sources and high order
%   source panels, i.e. the first order panels with the 'panel' argument
%   are not supported here.
%
%   Example:
%     % N random masses
%     N = 20;
%     z = complex(rand(N,1),rand(N,1));
%     m = rand(N,1);
%
%     % mesh
%     [X,Y] = meshgrid(linspace(0,1,40));
%     e = complex(X,Y);
%
%     % direct evaluation of the Logarithmic potential with Oseen smoothing
%     p = fmm2d(z,m,e,'tol',0,'smooth','oseen');
%     figure, contour(X,Y,p);
%     hold on, plot(real(z),imag(z),'ko');
%
%     % panels
%     figure,
%     q = fmm2d([],[],e,'pot',1,'panel',{[0.25+0.5i; 0.75+0.5i],[1; 1]});
%     contour(X,Y,q,20);
%
%     % higher order panels
%     [t p q]= ...
%     fmm2d(z,m,e,'pot',1,'sourcepanels',{'polyshape',[0;1],[1;2;3;4], ...
%     'polystrength',[0+2i;1-2i;2],'outside',1},'targetpanels', ...
%     {'cubshape',[0;1+1i],[1i;1],'outside',1});
%     
%
%   See also LOGPOT, INVPOT.
  
% S. Engblom and A. Goude 2011-06-28 (timing output)
% S. Engblom and A. Goude 2011-06-03 (GPU-support+panel syntax)
% S. Engblom 2009-04-16 (Major revision, adaptivity)
% S. Engblom 2007-07-08 (Major revision)
% S. Engblom 2007-05-15 (Revision)
% S. Engblom 2006-07-10
% Based on the code 'fmm2dlp' by the same author.

% Notes:
%   -The call P = FMM2D(Z,M,Z,...) is interpreted as P = FMM2D(Z,M,...)
%   through comparison of pointers.
%
%   -The syntax Y = FMM2D(X) computes Y = EXPINT(X) very rapidly. This
%   syntax is intended for internal testing only.
%
%   -The syntax M = FMM2D(...,'out','mesh') returns the FMM mesh in a simple
%   format (see below and see also MESHPLOT for an example). This syntax is
%   mainly intended for debugging and pedagogical use.
%
%   -The syntax T = FMM2D(...,'out','time') returns detailed information on
%   the time taken by the algorithm. This syntax is intended for
%   experimental use and the result of the FMM-algorithm itself is
%   disregarded. See the description of the output format for T below.
%
% Mesh-format:
%   [M,C] = FMM2D(...,'out','mesh') additionally returns cell vectors M and
%   C containing mesh- and sparse connection matrices.
%
%   M{l} for l = 1..nlevels contains 2-by-Nl complex matrices where Nl =
%   4^(l-1) is the number of boxes at level l. M{l}(1,:) are center points
%   for each rectangle and M{l}(2,:) contains the corresponding radii
%   (i.e. distances from centers to upper right corners).
%
%   C{l}(i,j) = 1 if box i and j are strongly connected (interact directly),
%   and C{l}(i,j) = -1 whenever box i and j interact through cluster to
%   cluster interactions at level l. If all(C{l}(i,j+1:4) = -1), then a
%   cluster to cluster interaction via the parent of j is used and
%   C{l}(i,j+1:4) = -2 in order to specify this (NOTE: this feature is not
%   implemented in the current version). At the final level, C{nlevel}(i,j)
%   = 2 if box i and j interact through cluster to particle (or particle to
%   cluster) interactions. Only the strictly lower triangular part of the
%   matrix is stored. If required, C{i} = speye(size(C{i}))+C{i}''+C{i}'
%   yields the complete matrix.
%
%   See also MESHPLOT, MESHSTATS, MESHQUAL.
%
% Time-format:
%   T = FMM2D(...,'out','time') returns a column vector of length 31
%   containing precise timings of the algorithm. The exact content of T
%   depends on both the input and on the current compile.
%
%   Two timing methods are available, one uses the clock() function from
%   time.h (which is referred to as CPU timing here). The second is to use
%   the GPU built-in timing and is only available for GPU-compiled
%   versions. This is referred to as CUDA timing.
%
%   The first ten elements of T is reserved for CUDA timing of the
%   CPU-compiled version of the algorithm (although for elements #2--3, the
%   values are valid for the GPU-compiled algorithm as well). The next ten
%   elements 11--20 contain the same measurements but using CPU timing
%   instead. The elements starting from 21 are only meaningful for the
%   GPU-compiled version of the code.
%
%   Note 1: for CPU-compiled versions, both multipole evaluations and direct
%   evaluations are contained in the direct evaluation time, while for
%   GPU-compiled versions, these two are two separate items.
%
%   Note 2: The reordering times of GPU-compiled versions do not exist on
%   CPU-compiled versions, since there the arrays are never reordered.
%
%   Output vector, when 'tol' = 0 and GPU-compiled:
%     1: GPU direct sum
%     2: GPU total time (including data transfer times)
%
%   Output vector, when 'tol' = 0 and CPU-compiled:
%     1: CPU total time (CPU timing)
%     2: CPU total time (CUDA timing)
%
%   Output vector, when 'tol' > 0:
%     CUDA timing:
%     1:  Allocation
%     2:  Sort part 1, partitioning (both CPU and GPU-compiled versions)
%     3:  Sort part 2, connectivity (both CPU and GPU-compiled versions)
%     4:  Total sort
%     5:  Step 1 (initialization)
%     6:  Step 2 (m2m)
%     7:  Step 3 (m2ps)
%     8:  Step 4 (p2p)
%     9:  CPU direct evaluation
%     10: Full time of algorithm
%     CPU timing:
%     11: Allocation
%     12:
%     13:
%     14: Total sort
%     15: Step 1 (initialization)
%     16: Step 2 (m2m)
%     17: Step 3 (m2ps)
%     18: Step 4 (p2p)
%     19: CPU direct evaluation
%     20:
%     CUDA timing:
%     21: GPU sort
%     22: GPU reordering
%     23: GPU Step 1 (initialization)
%     24: GPU Step 2 (m2m)
%     25: GPU Step 3 (m2ps)
%     26: GPU Step 4 (p2p)
%     27: GPU Multipole evaluation
%     28: GPU direct evaluation
%     29: GPU reverse reordering
%     30: GPU total time
%     31: Reserved for temporary checks

% Hidden properties for experimental purposes:
%
%   Property      Value/{Default}     Description
%   -----------------------------------------------------------------------
%   channelheight Scalar {-1}         Channel potential (negative
%                                     means disabled). The channel is
%                                     understood from 0 to
%                                     1i*channelheight, and with sigma
%                                     = channelheight/2, P =
%                                     -sigma*(M*coth(sigma*(E-Z))
%                                     +conj(M)*coth(sigma*(E-conj(Z))).
%
%   sortlimits    1x6 vector          Changes the sorting algorithm to break
%                                     up large boxes, see below.
%
%   eta           Scalar/1x2 vector   Shifts the center of the split, see
%                                     below.
%
%   Smoothers for the channel potential are implemented with
%   approximately the same equations as for the harmonic potential and
%   the results will therefore differ between the direct evaluation
%   (tol = 0) and fmm evaluation.
%
%   Property sortlimits:
%
%   All values come in pairs, where the first value is the coefficient
%   for the split at the highest level (first split) and the second is
%   the value at the last split. Coefficient is linearly interpolated
%   in between. The first two values is a limit how uneven the points
%   are allowed to be. Values above 1 will split according to median,
%   0 will split in geometric midpoint. In between, if the median is
%   outside this region, the value from this coefficient will be
%   chosen instead (i.e.  box between [0,1] and median at 0.05, then
%   the value 0.7 will set the splitpoint to 0.15. The second two
%   values is an exception to the rule above. If when looking at the
%   distribution at the limit value above, if too many points are on
%   wrong side, the above rule is ignored. 0 means that the 50 % has
%   to be on each side, 1 means that there is no limit. The value 0.7
%   will mean that if more than 85 % of the points are on the wrong
%   side of the limit value from above, the first rule is ignored The
%   third two values is a coefficient that allows to choose a new
%   center point for the split if both conditions above is true. This
%   works in the same way as the second two values, and the value 0.7
%   means that if both conditions above are true, the split will be
%   with 85 % of the coefficients on one side. (This has the default
%   value as the second two coefficients if only 4 coefficients are
%   given. The default values for sortlimits are [2,2,1,1,0,0], which
%   disables sortlimits and uses median value for all splits.
%
%   Property eta:
%
%   Shifts the center of the split. If the geometric midpoint is x1
%   and median point is x2, the new split point will be given by
%   x2*eta+x1*(1-eta). If two values are given, the value for eta is
%   linearly interpolated between them, giving the first value for the
%   first split and second value for the last split. Default is 2,2,
%   which disables the shift. Useful values are [0,1].

error('.MEX-file not found on path.');
