#ifdef GENERALPANELSUPPORT
#include "../fmm.h"
#endif
#include "generalpanel.h"
#include "gaussiancoeffs.h"
#include <math.h>
#include <float.h>
#include <mex.h>
#include <string.h>
 
//a quick way of turing off all print statements
//#define SILENTMODE
#ifdef SILENTMODE
#define mexPrintf mexPrintfSilent
int mexPrintfSilent(const char * restrict format, ... ) {return 0;};
#endif

//workspace variables
#ifdef DEBUGMODE
myvector<cmplx> wksp1,wksp2,wksp3,wksp4,wksp5,wksp1big,wksp2big,wksp3big;
myvector<double> wkspd;
myvector<int> wkspi;
#else
cmplx* wksp1,*wksp2,*wksp3,*wksp4,*wksp5,*wksp1big,*wksp2big,*wksp3big;
double* wkspd;
int* wkspi;
#endif

static int generalpaneldirectinteractions;
static int generalpanelmultipoleevaluations;
static int generalpaneldirectinteractcalls;
static int directinteractexpandedroots;
static int directinteractexpandedrootsnrcoeffs;

void resetstatistics() {
  generalpaneldirectinteractions=0;
  generalpanelmultipoleevaluations=0;
  generalpaneldirectinteractcalls=0;
  directinteractexpandedroots=0;
  directinteractexpandedrootsnrcoeffs=0;
}
void printstatistics() {
  mexPrintf("total direct interactions: %d (%d calls), through multipoleexpansion: %d direct: %d (%d expanded roots (%f))\n",generalpaneldirectinteractions,generalpaneldirectinteractcalls,generalpanelmultipoleevaluations,generalpaneldirectinteractions-generalpanelmultipoleevaluations,directinteractexpandedroots,(double)directinteractexpandedrootsnrcoeffs/(double)directinteractexpandedroots);
}

//allocate variables used in the calculations
void generalpanelalloc(int aN,int bN,int Ncoeff,int slopeexp) {
  if(aN<2)
    aN=2;
  if(slopeexp)
    bN+=SQRTEXPMAX*(2*aN-1);
  int maxlength=(aN>bN?aN:bN)*2;
  int Ngauss=(bN+1+aN*Ncoeff)/2;
  int maxlength2=bN+aN*MAXROOTEXPANSIONSHIFTED;
  if(Ngauss<maxlength)
    Ngauss=maxlength;
  if(Ngauss<maxlength2)
    Ngauss=maxlength2;
  
  wksp1=(COMPLEXVECTOR)myMalloc(maxlength*sizeof(cmplx));
  wksp2=(COMPLEXVECTOR)myMalloc(maxlength*sizeof(cmplx));
  wksp3=(COMPLEXVECTOR)myMalloc(maxlength*sizeof(cmplx));
  wksp4=(COMPLEXVECTOR)myMalloc(maxlength*sizeof(cmplx));
  wksp5=(COMPLEXVECTOR)myMalloc(maxlength*sizeof(cmplx));
  wksp1big=(COMPLEXVECTOR)myMalloc(Ngauss*sizeof(cmplx));
  wksp2big=(COMPLEXVECTOR)myMalloc(Ngauss*sizeof(cmplx));
  wksp3big=(COMPLEXVECTOR)myMalloc(Ngauss*sizeof(cmplx));
  wkspd=(DOUBLEVECTOR)myMalloc(maxlength*sizeof(double));
  wkspi=(INTVECTOR)myMalloc(maxlength*sizeof(int));
}

//free all the variables allocated by generalpanelalloc()
void generalpanelfree() {
  myFree(wksp1);
  myFree(wksp2);
  myFree(wksp3);
  myFree(wksp4);
  myFree(wksp5);
  myFree(wksp1big);
  myFree(wksp2big);
  myFree(wksp3big);
  myFree(wkspd);
  myFree(wkspi);
}

//used by splitgeneralpanel.cpp
void initgeneralpanel(generalpanel *out,double xstart,double ystart,double xend, double yend,const double* ar,const double* ai,int aN,const double *br, const double *bi, int bN,double outside) {
  initgeneralvars(out,aN,bN,xstart,ystart,xend,yend,outside);
  initavectorgeneral(out,ar,ai,aN);
  initbvectorgeneral(out,br,bi,bN);
}

//initialization of the source panel
void initsourcepanel(generalpanel *out) {
  initgeneralpanelcvector(out);
  initgeneralpanelconnectingterms(out);
  initgeneralpanelbox(out);
  if(out->a_order==3) {
    if(out->areal)
      initcubicconstants_areal(&out->cub,out);
    else
      initcubicconstants_aimag(&out->cub,out);
  }
}

//initialization of the target panel
void inittargetpanel(generalpanel *out) {
  inittargetbase(out);
  initgeneralpanelcvectortarget(out);
  initgeneralpanelbox(out);
  if(out->a_order==3) {
    if(out->areal)
      initcubicconstants_areal(&out->cub,out);
    else
      initcubicconstants_aimag(&out->cub,out);
  }
}

//common initialization, should always be called
void initgeneralvars(generalpanel *out,int aN,int bN,double xstart,double ystart,double xend,double yend,double outside) {
  initgeneralvarsa(out,aN,xstart,ystart,xend,yend,outside);
  initgeneralvarsb(out,bN);
}

//common initialization, a-part only (shape polynomial)
void initgeneralvarsa(generalpanel *out,int aN,double xstart,double ystart,double xend,double yend,double outside) {
  if(aN<2)
    aN=2;
 
  out->a=(COMPLEXVECTOR)myMalloc(aN*sizeof(cmplx));
  out->c=(COMPLEXVECTOR)myMalloc(aN*sizeof(cmplx));

  //set start and end points
  COMPLEXASSIGN(out->z1, xstart, ystart);
  COMPLEXASSIGN(out->z2, xend, yend);
  COMPLEXASSIGN(out->paneldist, xend-xstart, yend-ystart);

  //set the rotation vector to transfort the coordinate system so that the panel is parallel to the real axis
  double tmp=1/COMPLEXABS(out->paneldist);
  COMPLEXASSIGN(out->rotvect, out->paneldist.re*tmp, out->paneldist.im*tmp);
  out->deltas=COMPLEXABS(out->paneldist);
  out->outside=outside;
  out->endval=out->deltas;
}
//common initialization, b-part only (strength polynomial)
void initgeneralvarsb(generalpanel *out,int bN) {
  if(bN<1)
    bN=1;

  out->b=(COMPLEXVECTOR)myMalloc(bN*sizeof(cmplx));
  out->b_original=out->b;
  out->b_unshifted=out->b;
}

//polynomial input for the shape
//NOTE: even though the code can initialize complex valued polynomials for the shape, this is not fully
//supported as singularities related to complex coefficients are not fully handled
void initavectorgeneral(generalpanel *out,const double *ar,const double* ai,int aN) {
  cmplx ctmp;
  double tmp;
  out->a_order=aN-1;
  if(ai!=NULL) { //one case for real values polynomial and one for complex valued polynomial
    for(int i=0;i<aN;i++) {
      COMPLEXASSIGN(out->a[i],ar[i],ai[i]);
    }
    for(int i=aN;i<2;i++) { //if aN=1, but a[1] to zero
      COMPLEXASSIGN(out->a[i],0,0);
    }
    out->areal=0;
    if(out->a[0].re!=0.0||out->a[0].im!=0.0) { //if lowest order term is not zero, shift the start and end position and make it zero
      COMPLEXASSIGN(ctmp,-out->a[0].re*out->rotvect.im-out->a[0].im*out->rotvect.re,out->a[0].re*out->rotvect.re-out->a[0].im*out->rotvect.im);
      COMPLEXADD(out->z1,ctmp,out->z1); //move z1 and z2 to force z1 to be the start of the panel.
      COMPLEXADD(out->z2,ctmp,out->z2);
      COMPLEXASSIGN(out->a[0],0.0,0.0);
    }
    ctmp=evalpoly(out->a,out->deltas,out->a_order+1); //calculate end point position
    tmp=ctmp.re;
    ctmp.re=-ctmp.im+out->deltas;
    ctmp.im=tmp;
    out->z2alocal=ctmp;
    COMPLEXMUL(ctmp,ctmp,out->rotvect,tmp);
    COMPLEXADD(out->z2a,ctmp,out->z1);
    
    //set up the slope variables to be used for checking if panels are connected
    COMPLEXASSIGN(ctmp,1-out->a[1].im,out->a[1].re);
    COMPLEXMUL(ctmp,ctmp,out->rotvect,tmp);
    tmp=1/COMPLEXABS(ctmp);
    COMPLEXASSIGN(out->connectvars.startslope,ctmp.re*tmp,ctmp.im*tmp);
    ctmp=evalpolyderiv(out->a,out->deltas,out->a_order+1);
    tmp=ctmp.re;
    ctmp.re=1-ctmp.im;
    ctmp.im=tmp;
    COMPLEXMUL(ctmp,ctmp,out->rotvect,tmp);
    tmp=1/COMPLEXABS(ctmp);
    COMPLEXASSIGN(out->connectvars.endslope,ctmp.re*tmp,ctmp.im*tmp);
  }
  else { //real a case, same as above, but simplified if the imaginary part is zero in all cases (this is most likely the common case)
    for(int i=0;i<aN;i++) {
      COMPLEXASSIGN(out->a[i],ar[i],0.0);
    }
    for(int i=aN;i<2;i++) { //if aN=1, but a[1] to zero
      COMPLEXASSIGN(out->a[i],0,0);
    }
    out->areal=1;
    if(out->a[0].re!=0.0) { 
      COMPLEXASSIGN(ctmp,-out->a[0].re*out->rotvect.im,out->a[0].re*out->rotvect.re);
      COMPLEXADD(out->z1,ctmp,out->z1); //move z1 and z2 to force z1 to be the start of the panel.
      COMPLEXADD(out->z2,ctmp,out->z2);
      COMPLEXASSIGN(out->a[0],0.0,0.0);
    }
    COMPLEXASSIGN(ctmp,out->deltas,evalpolyreal(out->a,out->deltas,out->a_order+1));
    out->z2alocal=ctmp;
    COMPLEXMUL(ctmp,ctmp,out->rotvect,tmp);
    COMPLEXADD(out->z2a,ctmp,out->z1);
    
    COMPLEXASSIGN(ctmp,1,out->a[1].re);
    COMPLEXMUL(ctmp,ctmp,out->rotvect,tmp);
    tmp=1/COMPLEXABS(ctmp);
    COMPLEXASSIGN(out->connectvars.startslope,ctmp.re*tmp,ctmp.im*tmp);
    COMPLEXASSIGN(ctmp,1,evalpolyderivreal(out->a,out->deltas,out->a_order+1));
    COMPLEXMUL(ctmp,ctmp,out->rotvect,tmp);
    tmp=1/COMPLEXABS(ctmp);
    COMPLEXASSIGN(out->connectvars.endslope,ctmp.re*tmp,ctmp.im*tmp);
  }
}

//linear shape, it will still generate a first order polynomial
void initavectorlinear(generalpanel *out) {
  double tmp;
  out->a_order=1;
  COMPLEXASSIGN(out->a[0],0.0,0.0);
  COMPLEXASSIGN(out->a[1],0.0,0.0); //add the linear term as zero here, since the c vector will have c[1]!=0, and we want both to have equal length for simplicity
  out->z2a=out->z2;
  COMPLEXASSIGN(out->z2alocal,out->endval,0.0);
  out->areal=1;
  cmplx paneldirection;
  COMPLEXSUB(paneldirection,out->z2,out->z1);
  tmp=1/COMPLEXABS(paneldirection);
  COMPLEXASSIGN(out->connectvars.startslope,paneldirection.re*tmp,paneldirection.im*tmp);
  out->connectvars.endslope=out->connectvars.startslope;
}

//cubic shape through directions at both ends
void initavectorcubic(generalpanel *out,const double xdirstart,const double ydirstart,const double xdirend,const double ydirend) {
  if(isnan(xdirstart)||isnan(ydirstart)||isnan(xdirend)||isnan(ydirend)) { //if nan input, reduce to cubic instead
    initavectorlinear(out);
    return;
  }
  double tmp;
  out->a_order=3;
  out->areal=1;
  cmplx paneldirection1,paneldirection2;
  COMPLEXASSIGN(paneldirection1,xdirstart,ydirstart);
  tmp=1/COMPLEXABS(paneldirection1);
  COMPLEXASSIGN(out->connectvars.startslope,paneldirection1.re*tmp,paneldirection1.im*tmp);
  COMPLEXMULC(paneldirection1,paneldirection1,out->rotvect,tmp);
  COMPLEXASSIGN(paneldirection2,xdirend,ydirend);
  tmp=1/COMPLEXABS(paneldirection2);
  COMPLEXASSIGN(out->connectvars.endslope,paneldirection2.re*tmp,paneldirection2.im*tmp);
  COMPLEXMULC(paneldirection2,paneldirection2,out->rotvect,tmp);
  if(paneldirection1.re==0||paneldirection2.re==0)
    mexWarnMsgTxt("Invalid panel slope, direction cannot be 90 degrees shifted form panel line");
  COMPLEXASSIGN(out->a[0],0.0,0.0);
  COMPLEXASSIGN(out->a[1],paneldirection1.im/paneldirection1.re,0.0);
  COMPLEXASSIGN(out->a[3],(out->a[1].re+paneldirection2.im/paneldirection2.re)/(out->deltas*out->deltas),0.0);
  COMPLEXASSIGN(out->a[2],-out->a[1].re/out->deltas-out->a[3].re*out->deltas,0.0);
  out->z2a=out->z2;
  COMPLEXASSIGN(out->z2alocal,out->endval,0.0);
}

//quintic shape through directions and directions for the first and second derivatives at both ends
void initavectorquintic(generalpanel *out,const double xdirstart,const double ydirstart,const double xdirend,const double ydirend,const double x2dirstart,const double y2dirstart,const double x2dirend,const double y2dirend) {
  if(isnan(x2dirstart)||isnan(y2dirstart)||isnan(x2dirend)||isnan(y2dirend)) { //if nan input, reduce to cubic instead
    initavectorcubic(out,xdirstart,ydirstart,xdirend,ydirend);
    return;
  }
  double tmp;
  out->a_order=5;
  out->areal=1;
  cmplx paneldirection1,paneldirection2,panel2direction1,panel2direction2;
  COMPLEXASSIGN(paneldirection1,xdirstart,ydirstart);
  COMPLEXASSIGN(paneldirection2,xdirend,ydirend);
  tmp=1/COMPLEXABS(paneldirection1);
  COMPLEXASSIGN(out->connectvars.startslope,paneldirection1.re*tmp,paneldirection1.im*tmp);
  tmp=1/COMPLEXABS(paneldirection2);
  COMPLEXASSIGN(out->connectvars.endslope,paneldirection2.re*tmp,paneldirection2.im*tmp);
  COMPLEXMULC(paneldirection1,paneldirection1,out->rotvect,tmp);
  COMPLEXASSIGN(panel2direction1,x2dirstart,y2dirstart);
  COMPLEXMULC(panel2direction1,panel2direction1,out->rotvect,tmp);
  double startderiv=paneldirection1.im/paneldirection1.re;
  double start2deriv=(panel2direction1.im*paneldirection1.re-panel2direction1.re*paneldirection1.im)/(paneldirection1.re*paneldirection1.re*paneldirection1.re);
  
  
  COMPLEXMULC(paneldirection2,paneldirection2,out->rotvect,tmp);
  COMPLEXASSIGN(panel2direction2,x2dirend,y2dirend);
  COMPLEXMULC(panel2direction2,panel2direction2,out->rotvect,tmp);
  
  double endderiv=paneldirection2.im/paneldirection2.re;
  double end2deriv=(panel2direction2.im*paneldirection2.re-panel2direction2.re*paneldirection2.im)/(paneldirection2.re*paneldirection2.re*paneldirection2.re);
  double invdeltas=1/out->deltas;
  double invdeltassqr=invdeltas*invdeltas;
  double invdeltascub=invdeltassqr*invdeltas;
  double invdeltasquant=invdeltascub*invdeltas;
  COMPLEXASSIGN(out->a[0],0.0,0.0);
  COMPLEXASSIGN(out->a[1],startderiv,0.0);
  COMPLEXASSIGN(out->a[2],0.5*start2deriv,0.0);
  double a2deltas=out->a[2].re*out->deltas;
  double a5term=(-6*out->a[1].re-2*a2deltas-6*endderiv+out->deltas*end2deriv);
  COMPLEXASSIGN(out->a[5],0.5*a5term*invdeltasquant,0.0);
  double a4term=(2*out->a[1].re+a2deltas+endderiv-a5term);
  COMPLEXASSIGN(out->a[4],a4term*invdeltascub,0.0);
  COMPLEXASSIGN(out->a[3],(-out->a[1].re-a2deltas-a4term-0.5*a5term)*invdeltassqr,0.0);
  out->z2a=out->z2;
  COMPLEXASSIGN(out->z2alocal,out->endval,0.0);
}

//septic shape through directions and directions for the first, second and third derivatives at both ends
void initavectorseptic(generalpanel *out,const double xdirstart,const double ydirstart,const double xdirend,const double ydirend,const double x2dirstart,const double y2dirstart,const double x2dirend,const double y2dirend,const double x3dirstart,const double y3dirstart,const double x3dirend,const double y3dirend) {
  if(isnan(x3dirstart)||isnan(y3dirstart)||isnan(x3dirend)||isnan(y3dirend)) { //if nan input, reduce to quintic instead
    initavectorquintic(out,xdirstart,ydirstart,xdirend,ydirend,x2dirstart,y2dirstart,x2dirend,y2dirend);
    return;
  }
  double tmp;
  out->a_order=7;
  out->areal=1;
  cmplx paneldirection1,paneldirection2,panel2direction1,panel2direction2,panel3direction1,panel3direction2;
  COMPLEXASSIGN(paneldirection1,xdirstart,ydirstart);
  COMPLEXASSIGN(paneldirection2,xdirend,ydirend);
  tmp=1/COMPLEXABS(paneldirection1);
  COMPLEXASSIGN(out->connectvars.startslope,paneldirection1.re*tmp,paneldirection1.im*tmp);
  tmp=1/COMPLEXABS(paneldirection2);
  COMPLEXASSIGN(out->connectvars.endslope,paneldirection2.re*tmp,paneldirection2.im*tmp);
  COMPLEXMULC(paneldirection1,paneldirection1,out->rotvect,tmp);
  COMPLEXASSIGN(panel2direction1,x2dirstart,y2dirstart);
  COMPLEXMULC(panel2direction1,panel2direction1,out->rotvect,tmp);
  COMPLEXASSIGN(panel3direction1,x3dirstart,y3dirstart);
  COMPLEXMULC(panel3direction1,panel3direction1,out->rotvect,tmp);
  double startderiv=paneldirection1.im/paneldirection1.re;
  double sqrdir1=paneldirection1.re*paneldirection1.re*paneldirection1.re;
  double start2deriv=(panel2direction1.im*paneldirection1.re-panel2direction1.re*paneldirection1.im)/sqrdir1;
  double start3deriv=(paneldirection1.re*(panel3direction1.im*paneldirection1.re-paneldirection1.im*panel3direction1.re-3*panel2direction1.re*panel2direction1.im)+3*paneldirection1.im*panel2direction1.re*panel2direction1.re)/(sqrdir1*paneldirection1.re*paneldirection1.re);
  
  COMPLEXMULC(paneldirection2,paneldirection2,out->rotvect,tmp);
  COMPLEXASSIGN(panel2direction2,x2dirend,y2dirend);
  COMPLEXMULC(panel2direction2,panel2direction2,out->rotvect,tmp);
  COMPLEXASSIGN(panel3direction2,x3dirend,y3dirend);
  COMPLEXMULC(panel3direction2,panel3direction2,out->rotvect,tmp);
  
  double endderiv=paneldirection2.im/paneldirection2.re;
  double sqrdir2=paneldirection2.re*paneldirection2.re*paneldirection2.re;
  double end2deriv=(panel2direction2.im*paneldirection2.re-panel2direction2.re*paneldirection2.im)/sqrdir2;
  double end3deriv=(paneldirection2.re*(panel3direction2.im*paneldirection2.re-paneldirection2.im*panel3direction2.re-3*panel2direction2.re*panel2direction2.im)+3*paneldirection2.im*panel2direction2.re*panel2direction2.re)/(sqrdir2*paneldirection2.re*paneldirection2.re);

  double invdeltas=1/out->deltas;
  double invdeltassqr=invdeltas*invdeltas;
  double invdeltascub=invdeltassqr*invdeltas;
  double invdeltasquart=invdeltascub*invdeltas;
  double invdeltasquint=invdeltasquart*invdeltas;
  double invdeltassext=invdeltasquint*invdeltas;
  COMPLEXASSIGN(out->a[0],0.0,0.0);
  COMPLEXASSIGN(out->a[1],startderiv,0.0);
  COMPLEXASSIGN(out->a[2],0.5*start2deriv,0.0);
  COMPLEXASSIGN(out->a[3],(1.0/6)*start3deriv,0.0);
  COMPLEXASSIGN(out->a[4],-(120*startderiv+90*endderiv+out->deltas*(30*start2deriv-15*end2deriv+out->deltas*(4*start3deriv+end3deriv)))*((1.0/6.0)*invdeltascub),0.0);
  COMPLEXASSIGN(out->a[5],(90*startderiv+78*endderiv+out->deltas*(20*start2deriv-14*end2deriv+out->deltas*(2*start3deriv+end3deriv)))*(0.5*invdeltasquart),0.0);
  COMPLEXASSIGN(out->a[6],-(216*startderiv+204*endderiv+out->deltas*(45*start2deriv-39*end2deriv+out->deltas*(4*start3deriv+3*end3deriv)))*((1.0/6.0)*invdeltasquint),0.0);
  COMPLEXASSIGN(out->a[7],(60*(startderiv+endderiv)+out->deltas*(12*(start2deriv-end2deriv)+out->deltas*(start3deriv+end3deriv)))*((1.0/6.0)*invdeltassext),0.0);
  
  out->z2a=out->z2;
  COMPLEXASSIGN(out->z2alocal,out->endval,0.0);
}

//polynomial coefficients for the strength distribution
void initbvectorgeneral(generalpanel *out,const double *br,const double *bi,int bN) {
  if(bi!=NULL) { //the general case with both real and imaginary part
    for(int i=0;i<bN;i++) {
      COMPLEXASSIGN(out->b[i],br[i],bi[i]);
    }
    for(out->b_order=bN-1;out->b_order>0&&out->b[out->b_order].re==0.0&&out->b[out->b_order].im==0.0;out->b_order--);
    out->b_original_order=out->b_order;
  }
  else {
    if(br!=NULL) { //safety, check that br has values
      for(int i=0;i<bN;i++) {
        COMPLEXASSIGN(out->b[i],br[i],0.0);
      }
    }
    else {
      for(int i=0;i<bN;i++) {
        COMPLEXASSIGN(out->b[i],0.0,0.0);
      }
    }
    for(out->b_order=bN-1;out->b_order>0&&out->b[out->b_order].re==0.0;out->b_order--);
    out->b_original_order=out->b_order;
  }
}

//the difference for the b vector is that the start and end points do not have to be zero, which is the case for the a vector. This gives slightly more complicated expressions for the same order
//the b vectors also has to handle scaling, which means that due to the slope of the panel, the actual strength per unit distance along the panel is different from the
//strength per unit distance along the reference line. This has to be compensated for to ensure that the strength per unit distance along the panel is independent of the
//panel direction, so that the strength per unit distance becomes equal when moving from one panel to the next.
//slopecorrection means that the entire b-polynomial is modified externally

//linear strength distribution
//NOTE: if slopecorrection is true, this means that the correction is already handled by the applyslopecorrection function, and hence the values of b should not be corrected here
void initbvectorlinear(generalpanel *out,const double gstartr, const double gstarti,const double gendr, const double gendi,int slopecorrection) {
  out->b_order=1;
  out->b_original_order=out->b_order;
  double tmp=1/out->deltas;
  double scaling1,scaling2;
  if(!slopecorrection) {
    if(out->areal) {
      scaling1=sqrt(1+out->a[1].re*out->a[1].re);
      double t2=evalpolyderivreal(out->a, out->deltas, out->a_order+1);
      scaling2=sqrt(1+t2*t2);
    }
    else {
      scaling1=sqrt((1-out->a[1].im)*(1-out->a[1].im)+out->a[1].re*out->a[1].re);
      cmplx t2=evalpolyderiv(out->a, out->deltas, out->a_order+1);
      scaling2=COMPLEXABS(t2);
    }
    COMPLEXASSIGN(out->b[0], gstartr*scaling1, gstarti*scaling1);
    COMPLEXASSIGN(out->b[1], (gendr*scaling2-out->b[0].re)*tmp, (gendi*scaling2-out->b[0].im)*tmp);
  }
  else {
    COMPLEXASSIGN(out->b[0],gstartr,gstarti);
    COMPLEXASSIGN(out->b[1],(gendr-out->b[0].re)*tmp,(gendi-out->b[0].im)*tmp);
  }
}

//cubic strength distribution.

//in this case, not only the strength has to be compensated for the panel direction, but also the derivative of the strength. This is what all the scaling variables are for
//NOTE: if slopecorrection is true, this means that the correction is already handled by the applyslopecorrection function, and hence the values of b should not be corrected here
//NOTE2: Not correctly implemented for imaginary values for out->a
void initbvectorcubic(generalpanel *out, const double gstartr, const double gstarti, const double gendr, const double gendi, double gstartdirr, double gstartdiri, double genddirr, double genddiri, int slopecorrection) {
  if(isnan(gstartdirr)||isnan(gstartdiri)||isnan(genddirr)||isnan(genddiri)) { //if nan input, reduce to linear instead
    initbvectorlinear(out,gstartr,gstarti,gendr,gendi,slopecorrection);
    return;
  }
  out->b_order=3;
  double tmp=1/out->deltas;
  double tmp2=tmp*tmp*tmp;
  double scaling1, scaling2, scalingderiv1, scalingderiv2;
  if(!slopecorrection) {
    if(out->areal) {
      double t1=0;
      double t1deriv=0;
      if(out->a_order>0) {
        t1=out->a[1].re;
        if(out->a_order>1)
          t1deriv=2*out->a[2].re;
      }
      scaling1=sqrt(1+t1*t1);
      double t2=evalpolyderivreal(out->a, out->deltas, out->a_order+1);
      double t2deriv=evalpoly2derivreal(out->a, out->deltas, out->a_order+1);
      scaling2=sqrt(1+t2*t2);
      scalingderiv1=t1*t1deriv/scaling1;
      scalingderiv2=t2*t2deriv/scaling2;
      gstartdirr*=scaling1;
      gstartdiri*=scaling1;
      genddirr*=scaling2;
      genddiri*=scaling2;
      gstartdirr+=scalingderiv1*gstartr;
      gstartdiri+=scalingderiv1*gstarti;
      genddirr+=scalingderiv2*gendr;
      genddiri+=scalingderiv2*gendi;
    }
    else { //TODO: fix this implementation if complex a is required
      scaling1=sqrt((1-out->a[1].im)*(1-out->a[1].im)+out->a[1].re*out->a[1].re);
      cmplx t2=evalpolyderiv(out->a, out->deltas, out->a_order+1);
      scaling2=COMPLEXABS(t2);
    }
    COMPLEXASSIGN(out->b[0], gstartr*scaling1, gstarti*scaling1);
    COMPLEXASSIGN(out->b[1], gstartdirr, gstartdiri);
    COMPLEXASSIGN(out->b[3], (2*(out->b[0].re-gendr*scaling2)+(out->b[1].re+genddirr)*out->deltas)*tmp2, (2*(out->b[0].im-gendi*scaling2)+(out->b[1].im+genddiri)*out->deltas)*tmp2);
    COMPLEXASSIGN(out->b[2], 0.5*tmp*(genddirr-out->b[1].re-3*out->b[3].re*out->deltas*out->deltas), 0.5*tmp*(genddiri-out->b[1].im-3*out->b[3].im*out->deltas*out->deltas));
  }
  else {
    COMPLEXASSIGN(out->b[0], gstartr, gstarti);
    COMPLEXASSIGN(out->b[1], gstartdirr, gstartdiri);
    COMPLEXASSIGN(out->b[3], (2*(out->b[0].re-gendr)+(out->b[1].re+genddirr)*out->deltas)*tmp2, (2*(out->b[0].im-gendi)+(out->b[1].im+genddiri)*out->deltas)*tmp2);
    COMPLEXASSIGN(out->b[2], 0.5*tmp*(genddirr-out->b[1].re-3*out->b[3].re*out->deltas*out->deltas), 0.5*tmp*(genddiri-out->b[1].im-3*out->b[3].im*out->deltas*out->deltas));
  }
  out->b_original_order=out->b_order;
}

//in this case, not only the strength has to be compensated for the panel direction, but also the derivative and second derivative of the strength. This is what all the scaling variables are for
//NOTE: if slopecorrection is true, this means that the correction is already handled by the applyslopecorrection function, and hence the values of b should not be corrected here
void initbvectorquintic(generalpanel *out,double gstartr, double gstarti,double gendr, double gendi,double gstartdirr, double gstartdiri, double genddirr, double genddiri,double gstartdir2r, double gstartdir2i, double genddir2r, double genddir2i, int slopecorrection) {
  if(isnan(gstartdir2r)||isnan(gstartdir2i)||isnan(genddir2r)||isnan(genddir2i)) { //if nan input, reduce to cubic instead
    initbvectorcubic(out,gstartr,gstarti,gendr,gendi,gstartdirr,gstartdiri,genddirr,genddiri,slopecorrection);
    return;
  }
  out->b_order=5;
  double invdeltas=1/out->deltas;
  double tmp=invdeltas*invdeltas;
  double invdeltascub=tmp*invdeltas;
  double invdeltasquint=invdeltascub*tmp;
  double deltassqr=out->deltas*out->deltas;
  double scaling1, scaling2, scalingderiv1, scalingderiv2, invscaling1, invscaling2;

  if(!slopecorrection) {
    if(out->areal) {
      double t1=0;
      double t1deriv=0;
      double t1deriv2=0;
      if(out->a_order>0) {
        t1=out->a[1].re;
        if(out->a_order>1) {
          t1deriv=2*out->a[2].re;
          if(out->a_order>2)
            t1deriv2=6*out->a[3].re;
        }
      }
      scaling1=sqrt(1+t1*t1);
      invscaling1=1/scaling1;
      double t2=evalpolyderivreal(out->a, out->deltas, out->a_order+1);
      double t2deriv=evalpoly2derivreal(out->a, out->deltas, out->a_order+1);
      double t2deriv2=evalpoly3derivreal(out->a, out->deltas, out->a_order+1);
      scaling2=sqrt(1+t2*t2);
      invscaling2=1/scaling2;
      scalingderiv1=t1*t1deriv*invscaling1;
      scalingderiv2=t2*t2deriv*invscaling2;
      
      //rescale parameters with respect to panel slope
      //second order derivative
      gstartdir2r*=scaling1;
      gstartdir2i*=scaling1;
      genddir2r*=scaling2;
      genddir2i*=scaling2;

      tmp=(t1deriv*t1deriv+t1*t1deriv2)*invscaling1-scalingderiv1*scalingderiv1*invscaling1;
      gstartdir2r+=2*scalingderiv1*gstartdirr+tmp*gstartr;
      gstartdir2i+=2*scalingderiv1*gstartdiri+tmp*gstarti;

      tmp=(t2deriv*t2deriv+t2*t2deriv2)*invscaling2-scalingderiv2*scalingderiv2*invscaling2;
      genddir2r+=2*scalingderiv2*genddirr+tmp*gendr;
      genddir2i+=2*scalingderiv2*genddiri+tmp*gendi;
      
      //first order derivative
      gstartdirr*=scaling1;
      gstartdiri*=scaling1;
      genddirr*=scaling2;
      genddiri*=scaling2;
      gstartdirr+=scalingderiv1*gstartr;
      gstartdiri+=scalingderiv1*gstarti;
      genddirr+=scalingderiv2*gendr;
      genddiri+=scalingderiv2*gendi;

      //panel strength
      gstartr*=scaling1;
      gstarti*=scaling1;
      gendr*=scaling2;
      gendi*=scaling2;

    }
    else {
      scaling1=sqrt((1-out->a[1].im)*(1-out->a[1].im)+out->a[1].re*out->a[1].re);
      cmplx t2=evalpolyderiv(out->a, out->deltas, out->a_order+1);
      scaling2=COMPLEXABS(t2);
      gstartr*=scaling1;
      gendr*=scaling2;
    } 
  }
  COMPLEXASSIGN(out->b[0], gstartr, gstarti);
  COMPLEXASSIGN(out->b[1], gstartdirr, gstartdiri);
  COMPLEXASSIGN(out->b[2], 0.5*gstartdir2r,0.5*gstartdir2i);
  COMPLEXASSIGN(out->b[5], (12*(gendr-gstartr)+deltassqr*(genddir2r-gstartdir2r)-6*out->deltas*(genddirr+gstartdirr))*0.5*invdeltasquint,(12*(gendi-gstarti)+out->deltas*out->deltas*(genddir2i-gstartdir2i)-6*out->deltas*(genddiri+gstartdiri))*0.5*invdeltasquint);
  COMPLEXASSIGN(out->b[4], (-2*(genddirr-gstartdirr)+out->deltas*(genddir2r+gstartdir2r)-10*out->b[5].re*deltassqr*deltassqr)*0.25*invdeltascub,(-2*(genddiri-gstartdiri)+out->deltas*(genddir2i+gstartdir2i)-10*out->b[5].im*deltassqr*deltassqr)*0.25*invdeltascub);
  COMPLEXASSIGN(out->b[3], (genddir2r-gstartdir2r-12*out->b[4].re*deltassqr-20*out->b[5].re*deltassqr*out->deltas)*(1.0/6)*invdeltas,(genddir2i-gstartdir2i-12*out->b[4].im*deltassqr-20*out->b[5].im*deltassqr*out->deltas)*(1.0/6)*invdeltas);
  out->b_original_order=out->b_order;
}

//in this case, not only the strength has to be compensated for the panel direction, but also the derivative and second derivative of the strength. This is what all the scaling variables are for
//NOTE: if slopecorrection is true, this means that the correction is already handled by the applyslopecorrection function, and hence the values of b should not be corrected here
void initbvectorseptic(generalpanel *out,double gstartr, double gstarti,double gendr, double gendi,double gstartdirr, double gstartdiri, double genddirr, double genddiri,double gstartdir2r, double gstartdir2i, double genddir2r, double genddir2i, double gstartdir3r, double gstartdir3i, double genddir3r, double genddir3i, int slopecorrection) {
  if(isnan(gstartdir3r)||isnan(gstartdir3i)||isnan(genddir3r)||isnan(genddir3i)) { //if nan input, reduce to quintic instead
    initbvectorquintic(out,gstartr,gstarti,gendr,gendi,gstartdirr,gstartdiri,genddirr,genddiri,gstartdir2r,gstartdir2i,genddir2r,genddir2i,slopecorrection);
    return;
  }
  out->b_order=7;
  double invdeltas=1/out->deltas;
  double tmp=invdeltas*invdeltas;
  double invdeltascub=tmp*invdeltas;
  double invdeltasquart=invdeltas*invdeltascub;
  double invdeltasquint=invdeltasquart*invdeltas;
  double invdeltasext=invdeltasquint*invdeltas;
  double invdeltassept=invdeltasext*invdeltas;
  double deltassqr=out->deltas*out->deltas;
  double scaling1, scaling2, scalingderiv1, scalingderiv2, scaling2deriv1, scaling2deriv2, scaling3deriv1, scaling3deriv2, invscaling1, invscaling2;
  if(!slopecorrection) {
    if(out->areal) {
      double t1=0;
      double t1deriv=0;
      double t1deriv2=0;
      double t1deriv3=0;
      if(out->a_order>0) {
        t1=out->a[1].re;
        if(out->a_order>1) {
          t1deriv=2*out->a[2].re;
          if(out->a_order>2) {
            t1deriv2=6*out->a[3].re;
            if(out->a_order>3)
              t1deriv3=24*out->a[4].re;
          }
        }
      }
      scaling1=sqrt(1+t1*t1);
      invscaling1=1/scaling1;
      double t2=evalpolyderivreal(out->a, out->deltas, out->a_order+1);
      double t2deriv=evalpoly2derivreal(out->a, out->deltas, out->a_order+1);
      double t2deriv2=evalpoly3derivreal(out->a, out->deltas, out->a_order+1);
      double t2deriv3=evalpoly4derivreal(out->a, out->deltas, out->a_order+1);
      scaling2=sqrt(1+t2*t2);
      invscaling2=1/scaling2;
      scalingderiv1=t1*t1deriv*invscaling1;
      tmp=t1deriv*t1deriv+t1*t1deriv2;
      scaling2deriv1=tmp*invscaling1-scalingderiv1*scalingderiv1*invscaling1;
      scaling3deriv1=((t1*t1deriv3+3*t1deriv*t1deriv2)+3*scalingderiv1*invscaling1*(scalingderiv1*scalingderiv1-tmp))*invscaling1;
      scalingderiv2=t2*t2deriv*invscaling2;
      tmp=t2deriv*t2deriv+t2*t2deriv2;
      scaling2deriv2=tmp*invscaling2-scalingderiv2*scalingderiv2*invscaling2;
      scaling3deriv2=((t2*t2deriv3+3*t2deriv*t2deriv2)+3*scalingderiv2*invscaling2*(scalingderiv2*scalingderiv2-tmp))*invscaling2;
      
      
      //rescale parameters with respect to panel slope
      //second order derivative
      gstartdir3r*=scaling1;
      gstartdir3i*=scaling1;
      genddir3r*=scaling2;
      genddir3i*=scaling2;

      gstartdir3r+=3*gstartdir2r*scalingderiv1+3*gstartdirr*scaling2deriv1+gstartr*scaling3deriv1;
      gstartdir3i+=3*gstartdir2i*scalingderiv1+3*gstartdiri*scaling2deriv1+gstarti*scaling3deriv1;

      genddir3r+=3*genddir2r*scalingderiv2+3*genddirr*scaling2deriv2+gendr*scaling3deriv2;
      genddir3i+=3*genddir2i*scalingderiv2+3*genddiri*scaling2deriv2+gendi*scaling3deriv2;

      //rescale parameters with respect to panel slope
      //second order derivative
      gstartdir2r*=scaling1;
      gstartdir2i*=scaling1;
      genddir2r*=scaling2;
      genddir2i*=scaling2;

      gstartdir2r+=2*scalingderiv1*gstartdirr+scaling2deriv1*gstartr;
      gstartdir2i+=2*scalingderiv1*gstartdiri+scaling2deriv1*gstarti;

      genddir2r+=2*scalingderiv2*genddirr+scaling2deriv2*gendr;
      genddir2i+=2*scalingderiv2*genddiri+scaling2deriv2*gendi;
      
      //first order derivative
      gstartdirr*=scaling1;
      gstartdiri*=scaling1;
      genddirr*=scaling2;
      genddiri*=scaling2;
      gstartdirr+=scalingderiv1*gstartr;
      gstartdiri+=scalingderiv1*gstarti;
      genddirr+=scalingderiv2*gendr;
      genddiri+=scalingderiv2*gendi;

      //panel strength
      gstartr*=scaling1;
      gstarti*=scaling1;
      gendr*=scaling2;
      gendi*=scaling2;
    }
    else {
      scaling1=sqrt((1-out->a[1].im)*(1-out->a[1].im)+out->a[1].re*out->a[1].re);
      cmplx t2=evalpolyderiv(out->a, out->deltas, out->a_order+1);
      scaling2=COMPLEXABS(t2);
      gstartr*=scaling1;
      gendr*=scaling2;
    }
  }
  COMPLEXASSIGN(out->b[0], gstartr, gstarti);
  COMPLEXASSIGN(out->b[1], gstartdirr, gstartdiri);
  COMPLEXASSIGN(out->b[2], 0.5*gstartdir2r,0.5*gstartdir2i);
  COMPLEXASSIGN(out->b[3],(1.0/6)*gstartdir3r,(1.0/6)*gstartdir3i);
  COMPLEXASSIGN(out->b[4],-(210*(gstartr-gendr)+out->deltas*(120*gstartdirr+90*genddirr+out->deltas*(30*gstartdir2r-15*genddir2r+out->deltas*(4*gstartdir3r+genddir3r))))*((1.0/6.0)*invdeltasquart),-(210*(gstarti-gendi)+out->deltas*(120*gstartdiri+90*genddiri+out->deltas*(30*gstartdir2i-15*genddir2i+out->deltas*(4*gstartdir3i+genddir3i))))*((1.0/6.0)*invdeltasquart));
  COMPLEXASSIGN(out->b[5],(168*(gstartr-gendr)+out->deltas*(90*gstartdirr+78*genddirr+out->deltas*(20*gstartdir2r-14*genddir2r+out->deltas*(2*gstartdir3r+genddir3r))))*(0.5*invdeltasquint),(168*(gstarti-gendi)+out->deltas*(90*gstartdiri+78*genddiri+out->deltas*(20*gstartdir2i-14*genddir2i+out->deltas*(2*gstartdir3i+genddir3i))))*(0.5*invdeltasquint));
  COMPLEXASSIGN(out->b[6],-(420*(gstartr-gendr)+out->deltas*(216*gstartdirr+204*genddirr+out->deltas*(45*gstartdir2r-39*genddir2r+out->deltas*(4*gstartdir3r+3*genddir3r))))*((1.0/6.0)*invdeltasext),-(420*(gstarti-gendi)+out->deltas*(216*gstartdiri+204*genddiri+out->deltas*(45*gstartdir2i-39*genddir2i+out->deltas*(4*gstartdir3i+3*genddir3i))))*((1.0/6.0)*invdeltasext));
  COMPLEXASSIGN(out->b[7],(120*(gstartr-gendr)+out->deltas*(60*(gstartdirr+genddirr)+out->deltas*(12*(gstartdir2r-genddir2r)+out->deltas*(gstartdir3r+genddir3r))))*((1.0/6.0)*invdeltassept),(120*(gstarti-gendi)+out->deltas*(60*(gstartdiri+genddiri)+out->deltas*(12*(gstartdir2i-genddir2i)+out->deltas*(gstartdir3i+genddir3i))))*((1.0/6.0)*invdeltassept));
  out->b_original_order=out->b_order;
}

//the b vector is the derivative of 1i*a+1 multiplied with 1i. The dot product is real(x*conj(n)) where n is the normal vector, hence take the conjugate here
//te b vector for the target is only used for numerical integration.
void initbvectortarget(generalpanel *out,int tindex) {
  out->b_order=out->a_order-1;
  COMPLEXASSIGN(out->b[0],-out->a[1].re,1-out->a[1].im);
  for(int i=2;i<=out->a_order;i++) {
    COMPLEXASSIGN(out->b[i-1],-i*out->a[i].re,-i*out->a[i].im);
  }
  out->target=1;
  out->targetindex=tindex;
}

//for target panels, which uses gauss law to evaluate the results, the expression will be dependent on the actual distance between the two endpoints as well
//this distance is set as targetendval. Also set the rotation vector to rotate the system so that z1 and z2a have the same imaginary part
void inittargetbase(generalpanel *out) {
  if(fabs(out->z2.re-out->z2a.re)+fabs(out->z2.im-out->z2a.im)<1e-15*out->deltas) {
    out->targetendval=out->endval;
    out->targetrotvect=out->rotvect;
  }
  else {
    cmplx ctmp;
    COMPLEXSUB(ctmp,out->z2a,out->z1);
    out->targetendval=COMPLEXABS(ctmp);
    double tmp=1/out->targetendval;
    if(out->targetendval==0) {
      COMPLEXASSIGN(out->targetrotvect, 1.0, 0.0);
    }
    else {
      COMPLEXASSIGN(out->targetrotvect, ctmp.re*tmp, ctmp.im*tmp);
    }
  }
}

//if the highest order coefficient is 0 or approximately 0, the panel order has to be reduced for the evaluation to work, as the number of roots will be lower in this case (and also to avoid divide by zero)
void reducea_order(generalpanel *out) {
  double maxval=0.0;
  double tmp;
  double tmpdeltas=out->deltas;
  int maxsignificant=1;
  tmp=(fabs(1-out->a[1].im)+fabs(out->a[1].re))*tmpdeltas;
  if(tmp>maxval)
    maxval=tmp;
  for(int i=2;i<=out->a_order;i++) {
    tmpdeltas*=out->deltas;
    tmp=(fabs(out->a[i].im)+fabs(out->a[i].re))*tmpdeltas;
    if(tmp>maxval)
      maxval=tmp;
    if(tmp>DBL_EPSILON*maxval)
      maxsignificant=i;
  }
  out->a_order=maxsignificant;
}

//set the c vector (which is the vector that is the denominator in the polynomial, i.e. the shape and it is built form the a-vector) for a source panel.
void initgeneralpanelcvector(generalpanel *out) {
  //initialize c-vector
  double tmp;
  reducea_order(out);
  if(out->areal) { //simplified the case with real coefficients
    if(out->a_order==1) { //special case for first order panels, as it is the first order term that is the highest term, and also the term that is modified differently
      cmplx ctmp;
      COMPLEXASSIGN(ctmp,out->a[1].re,-1.0);
      COMPLEXRECIPROCAL(out->invaend,ctmp,tmp);
      COMPLEXASSIGN(out->c[1],1.0,0.0);
    }
    else {
      COMPLEXASSIGN(out->invaend, 1/out->a[out->a_order].re, 0.0);
      //out->c[0] is not initialized here, as it will depend on the evaluation point
      COMPLEXASSIGN(out->c[1], out->a[1].re*out->invaend.re, -out->invaend.re); //set first order term manually
      for(int j=2;j<=out->a_order;j++) {
        COMPLEXASSIGN(out->c[j], out->a[j].re*out->invaend.re, 0.0); //set all other terms by dividing by the coefficient of the highest order term
      }
    }
  }
  else {
    if(out->a_order==1) { //special case for first order panels, as it is the first order term that is the highest term, and also the term that is modified differently
      cmplx ctmp;
      COMPLEXASSIGN(ctmp,out->a[1].re,-1.0+out->a[1].im);
      COMPLEXRECIPROCAL(out->invaend,ctmp,tmp);
      COMPLEXASSIGN(out->c[1],1.0,0.0);
    }
    else {
      COMPLEXRECIPROCAL(out->invaend, out->a[out->a_order], tmp);
      //out->c[0] is not initialized here, as it will depend on the evaluation point
      COMPLEXASSIGN(out->c[1], out->a[1].re, out->a[1].im-1); //set first order term manually
      COMPLEXMUL(out->c[1], out->c[1], out->invaend, tmp);
      for(int j=2;j<=out->a_order;j++) {
        COMPLEXMUL(out->c[j], out->a[j], out->invaend, tmp); //set all other terms by dividing by the coefficient of the highest order term
      }
    }
  }
}

//target c vector is only used for numerical integration. This is easier (especially for the smoother implementations) if invaend is not used here, which is ok since partial fraction decomposition is not used for target panels
void initgeneralpanelcvectortarget(generalpanel *out) {
  //initialize c-vector
  reducea_order(out);
  COMPLEXASSIGN(out->invaend, 1.0, 0.0);
  if(out->areal) { //simplified the case with real coefficients
    COMPLEXASSIGN(out->invaend, 1/out->a[out->a_order].re, 0.0);
    //out->c[0] is not initialized here, as it will depend on the source point
    COMPLEXASSIGN(out->c[1], out->a[1].re, -1.0);
    for(int j=2;j<=out->a_order;j++) {
      COMPLEXASSIGN(out->c[j], out->a[j].re, 0.0);
    }
  }
  else {
    //out->c[0] is not initialized here, as it will depend on the evaluation point
    COMPLEXASSIGN(out->c[1], out->a[1].re, out->a[1].im-1);
    for(int j=2;j<=out->a_order;j++) {
      out->c[j]=out->a[j];
    }
  }
}

//Connecting terms are used to determine if two panels are connected. Connected means that they have one common end point and that the slope of the panels are equal at that position. 
//Also, the strength should be equal at the connecting point. If all these conditions are met, the solution close to the panel should be smooth.
//fully connected terms can be evaluated with special expressions close to the connection point to avoid the logarithmic singularity

//this function initializes the terms that are used to determine if the panels are connected
void initgeneralpanelconnectingterms(generalpanel *restrict out) {
  double scaling1,scaling2,tmp,tmp2;
  cmplx startslope,endslope;
  if(out->areal) {
    COMPLEXASSIGN(startslope,1,out->a[1].re);
    scaling1=sqrt(1+out->a[1].re*out->a[1].re); //scaling is used to compensate for the panel direction for the b values
    double t2=evalpolyderivreal(out->a, out->endval, out->a_order+1);
    COMPLEXASSIGN(endslope,1,t2);
    scaling2=sqrt(1+t2*t2);
  }
  else {
    COMPLEXASSIGN(startslope,1-out->a[1].im,out->a[1].re);
    scaling1=COMPLEXABS(startslope);
    cmplx t2=evalpolyderiv(out->a, out->endval, out->a_order+1);
    COMPLEXASSIGN(endslope,1-t2.im,t2.re);
    scaling2=COMPLEXABS(endslope);
  }
  tmp=1/scaling1;
  cmplx tt;
  if(out->shifted) {
    cmplx tt=evalpoly(out->b,-0.5*out->endval,out->b_order+1);
    COMPLEXASSIGN(out->connectvars.startb,tt.re*tmp,tt.im*tmp);
  }
  else {
    COMPLEXASSIGN(out->connectvars.startb,out->b[0].re*tmp,out->b[0].im*tmp);
  }
  COMPLEXASSIGN(out->connectvars.startslopelocal,startslope.re*tmp,startslope.im*tmp);
  tmp=1/scaling2;
  if(out->shifted)
    tt=evalpoly(out->b,0.5*out->endval,out->b_order+1);
  else
    tt=evalpoly(out->b,out->endval,out->b_order+1);
  
  COMPLEXASSIGN(out->connectvars.endb,tt.re*tmp,tt.im*tmp);
  COMPLEXASSIGN(out->connectvars.endslopelocal,endslope.re*tmp,endslope.im*tmp);
  COMPLEXASSIGN(out->connectvars.startbterm,out->connectvars.startb.re*out->connectvars.startslopelocal.im-out->connectvars.startb.im*out->connectvars.startslopelocal.re,out->connectvars.startb.re*out->connectvars.startslopelocal.re-out->connectvars.startb.im*out->connectvars.startslopelocal.im);
  COMPLEXASSIGN(out->connectvars.endbterm,  out->connectvars.endb.re*  out->connectvars.endslopelocal.im  -out->connectvars.endb.im  *out->connectvars.endslopelocal.re,  out->connectvars.endb.re*  out->connectvars.endslopelocal.re-  out->connectvars.endb.im*  out->connectvars.endslopelocal.im);
  COMPLEXCDIV(out->connectvars.startslopeterm,out->rotvect,startslope,tmp,tmp2);
  COMPLEXLOG(out->connectvars.startlogslopeterm,out->connectvars.startslopeterm);
  COMPLEXCDIV(out->connectvars.endslopeterm,out->rotvect,endslope,tmp,tmp2);
  COMPLEXLOG(out->connectvars.endlogslopeterm,out->connectvars.endslopeterm);
}

//returns 1 if the panels are continous (which does not mean that they are fully connected, only that they belong to the same object
int determineconnectivity(generalpanel *restrict panel1,generalpanel *restrict panel2) {
  if(fabs(panel1->z2a.re-panel2->z1.re)+fabs(panel1->z2a.re-panel2->z1.re)<1e-15*fmax(panel1->deltas, panel2->deltas)) { //panels are continous
    if(fabs(panel1->connectvars.endslope.re-panel2->connectvars.startslope.re)+fabs(panel1->connectvars.endslope.im-panel2->connectvars.startslope.im)<1e-13) { //panels have continous slope
      double maxtest1=fabs(panel1->b[panel1->b_order].re)+fabs(panel1->b[panel1->b_order].im);
      for(int j=panel1->b_order;j>=0;j--)
        maxtest1=maxtest1*panel1->deltas+fabs(panel1->b[j].re)+fabs(panel1->b[j].im);
      double maxtest2=fabs(panel2->b[panel2->b_order].re)+fabs(panel2->b[panel2->b_order].im);
      for(int j=panel2->b_order;j>=0;j--)
        maxtest2=maxtest2*panel2->deltas+fabs(panel2->b[j].re)+fabs(panel2->b[j].im);
      if(fabs(panel1->connectvars.endb.re-panel2->connectvars.startb.re)+fabs(panel1->connectvars.endb.im-panel2->connectvars.startb.im)<1e-13*fmax(maxtest1, maxtest2)) { //same b-value
        panel1->next=panel2;
        panel2->prev=panel1;
        double maxtmp=fmax(panel2->endval, panel1->endval);
        panel2->connectvars.startdist=MINCONNECTDIST*maxtmp; //constants are relatively arbitrary chosen
        double mintmp=fmin(panel2->endval, panel1->endval);
        if(mintmp<10*maxtmp) //if one panel is much shorter, reduce the allowed distance
          panel2->connectvars.startdist=MAXCONNECTDIST*mintmp;
        panel1->connectvars.enddist=panel2->connectvars.startdist;
        //mexPrintf("Panels are fully connected\n");
      }
    }
    return 1;
  }
  return 0;
}

//checks if the neighboring panels are connected
void determinepanelconnections(generalpanel *panels,int count) {
  generalpanel* startpanel=&panels[0];
  for(int i=1;i<count;i++) {
    if(!determineconnectivity(&panels[i-1],&panels[i])) {
      if(startpanel!=&panels[i-1]) //check for circular connection, cannot happen with itself
        determineconnectivity(&panels[i-1],startpanel);
      startpanel=&panels[i];
    }
  }
  if(startpanel!=&panels[count-1]) //check for circular connection, cannot happen with itself
    determineconnectivity(&panels[count-1], startpanel);
}

//calculates the bounding box that encloses the panel. This is done by checking the panel endpoints and also all positions where it is parallel with any axis
void initgeneralpanelbox(generalpanel *restrict out) {
  COMPLEXVECTOR c=wksp1big;
  COMPLEXVECTOR r=wksp2big;
  COMPLEXVECTOR ap=wksp4;
  //ap, c, wksp ans wksp2 are all works space variables, should have atleast same size as length of a,except wksp3 and wkspd which should be twice as a, These are not used for input or output, but are included to avoid the need to allocate and deallocate memory in the function
  double xmin,xmax,ymin,ymax,tmp;
  int extremepoint=0; //number of valid extremepoints
  cmplx ctmp,ctmp2,cbak;

  //start by checking end points
  xmax=xmin=out->z1.re;
  ymax=ymin=out->z1.im;
  if(xmax<out->z2a.re)
    xmax=out->z2a.re;
  if(xmin>out->z2a.re)
    xmin=out->z2a.re;
  if(ymax<out->z2a.im)
    ymax=out->z2a.im;
  if(ymin>out->z2a.im)
    ymin=out->z2a.im;

  //continue with checking for extreme values between endpoints by searching for zeros in the derivative. Real and complex cases implemented separately
  if(out->areal) {
    //create derivative
    for(int j=0;j<out->a_order;j++) {
      COMPLEXASSIGN(ap[j], out->a[j+1].re*(j+1.0), 0.0);
    }
    //normalize derivative, unnecessary?
    double invvaluereal=1/ap[out->a_order-1].re;
    for(int j=0;j<out->a_order-1;j++) {
      COMPLEXASSIGN(c[j], ap[j].re*invvaluereal, 0.0);
    }
    COMPLEXASSIGN(c[out->a_order-1], 1.0, 0.0);
    cbak=c[0];
    
    //take real and imaginary parts separately by searching for zeros in the real part and imaginary parts respectively
    if(out->paneldist.im!=0.0&&fabs(out->rotvect.im)>1e-12) {  //will give inf for cot, extra check on rotvect, as very small values also can give inf from polyroots
      if(out->paneldist.re!=0.0)                               //safety, should give cot(theta)=0
        c[0].re-=out->rotvect.re/out->rotvect.im*invvaluereal; //note out->rotvect.re/out->rotvect.im=cot(out->theta)
      polyroots(r, c, out->a_order-1, wksp1,wksp2);            //find roots
      for(int j=0;j<out->a_order-1;j++) {                      //loop through roots
        if(fabs(r[j].im)<fabs(r[j].re*1e-14)&&r[j].re>0&&r[j].re<out->endval) {        //real root within the panel limits
          COMPLEXASSIGN(ctmp, r[j].re, evalpolyreal(out->a, r[j].re, out->a_order+1)); //calculate value
          COMPLEXMUL(ctmp, ctmp, out->rotvect, tmp);
          COMPLEXADD(wksp3[extremepoint], ctmp, out->z1);
          wkspd[extremepoint]=r[j].re;
          if(xmax<wksp3[extremepoint].re)     //check if it changes the max and min values
            xmax=wksp3[extremepoint].re;
          if(xmin>wksp3[extremepoint].re)
            xmin=wksp3[extremepoint].re;
          extremepoint++;
        }
      }
    }
    //same procedure for the real part to search for zeros in the y-direction
    if(out->paneldist.re!=0.0&&fabs(out->rotvect.re)>1e-12) {
      c[0]=cbak;
      if(out->paneldist.im!=0.0)
        c[0].re+=out->rotvect.im/out->rotvect.re*invvaluereal; //note out->rotvect.im/out->rotvect.re=tan(out->theta)
        polyroots(r, c, out->a_order-1, wksp1,wksp2);
        for(int j=0;j<out->a_order-1;j++) {
          if(fabs(r[j].im)<fabs(r[j].re*1e-14)&&r[j].re>0&&r[j].re<out->endval) { //real root within the panel limits
            COMPLEXASSIGN(ctmp, r[j].re, evalpolyreal(out->a, r[j].re, out->a_order+1));
            COMPLEXMUL(ctmp, ctmp, out->rotvect, tmp);
            COMPLEXADD(wksp3[extremepoint], ctmp, out->z1);
            wkspd[extremepoint]=r[j].re;
            if(ymax<wksp3[extremepoint].im)
              ymax=wksp3[extremepoint].im;
            if(ymin>wksp3[extremepoint].im)
              ymin=wksp3[extremepoint].im;
            extremepoint++;
          }
        }
    }
  }
  //same procedure as above, but for complex numbers
  else {
    for(int j=0;j<out->a_order;j++) { //derivative
      COMPLEXASSIGN(ap[j], out->a[1+j].re*(j+1.0), out->a[1+j].im*(j+1.0));
    }
    if(out->a_order>1) { //form full vector, if order=1 or less, extreme will be at endpoint
      COMPLEXASSIGN(wksp1[0], 1-ap[0].im, ap[0].re);
      COMPLEXMUL(wksp1[0], wksp1[0], out->rotvect, tmp);
      for(int j=1;j<out->a_order;j++) {
        COMPLEXASSIGN(wksp1[j], -ap[j].re*out->rotvect.im-ap[j].im*out->rotvect.re, ap[j].re*out->rotvect.re-ap[j].im*out->rotvect.im);
      }
      int tmporder=out->a_order;
      for(;tmporder>0&&fabs(wksp1[tmporder-1].im)<1e-14;tmporder--); //in case imaginary part starts with zero, reduce order
      if(tmporder>1) {
        double invvaluereal=1/wksp1[tmporder-1].im; //sort out real part
        for(int j=0;j<tmporder-1;j++) {
          COMPLEXASSIGN(c[j], wksp1[j].im*invvaluereal, 0.0);
        }
        COMPLEXASSIGN(c[tmporder-1], 1.0, 0.0);
        polyroots(r, c, tmporder-1, wksp2,wksp3); //could possibly be optimized with a method designed for real coefficients
        for(int j=0;j<tmporder-1;j++) {
          if(fabs(r[j].im)<fabs(r[j].re*1e-14)&&r[j].re>0&&r[j].re<out->deltas) { //real root within the panel limits
            ctmp2=evalpoly(out->a, r[j].re, tmporder+1);
            COMPLEXASSIGN(ctmp, r[j].re-ctmp2.im, ctmp2.re);
            COMPLEXMUL(ctmp, ctmp, out->rotvect, tmp);
            COMPLEXADD(wksp3[extremepoint], ctmp, out->z1);
            wkspd[extremepoint]=r[j].re;
            if(ymax<wksp3[extremepoint].im)
              ymax=wksp3[extremepoint].im;
            if(ymin>wksp3[extremepoint].im)
              ymin=wksp3[extremepoint].im;
            extremepoint++;
          }
        }
      }
      tmporder=out->a_order;
      for(;fabs(wksp1[tmporder-1].re)<1e-14&&tmporder>0;tmporder--); //in case imaginary part starts with zero, reduce order
      if(tmporder>1) {
        double invvaluereal=1/wksp1[tmporder-1].re;
        for(int j=0;j<tmporder-1;j++) {
          COMPLEXASSIGN(c[j], wksp1[j].re*invvaluereal, 0.0);
        }
        COMPLEXASSIGN(c[tmporder-1], 1.0, 0.0);
        polyroots(r, c, tmporder-1, wksp2,wksp3);
        for(int j=0;j<tmporder-1;j++) {
          if(fabs(r[j].im)<fabs(r[j].re*1e-14)&&r[j].re>0&&r[j].re<out->deltas) { //real root within the panel limits
            ctmp2=evalpoly(out->a, r[j].re, tmporder+1);
            COMPLEXASSIGN(ctmp, r[j].re-ctmp2.im, ctmp2.re);
            COMPLEXMUL(ctmp, ctmp, out->rotvect, tmp);
            COMPLEXADD(wksp3[extremepoint], ctmp, out->z1);
            wkspd[extremepoint]=r[j].re;
            if(xmax<ctmp.re)
              xmax=ctmp.re;
            if(xmin>ctmp.re)
              xmin=ctmp.re;
            extremepoint++;
          }
        }
      }
    }
  }
  //save extreme points for simplified search of bounding box incase of splitting the panel
  if(extremepoint>0) {
    out->extremepoints=(COMPLEXVECTOR)myMalloc(extremepoint*sizeof(cmplx));
    out->extremepointval=(DOUBLEVECTOR)myMalloc(extremepoint*sizeof(double));
    for(int i=0;i<extremepoint;i++) {
      out->extremepoints[i]=wksp3[i];
      out->extremepointval[i]=wkspd[i];
    }
  }
  out->Nextremepoints=extremepoint;
  COMPLEXASSIGN(out->z,0.5*(xmax+xmin),0.5*(ymax+ymin));
  COMPLEXASSIGN(out->d,0.5*(xmax-xmin),0.5*(ymax-ymin));
}

//this function checks the distance between a point and the panel. if maxval=1, the largest distance is returned, if maxval=0, the smallest distance is returned
//the return value is the square of the actual distance
double sqrdistancefrompanel(generalpanel *restrict currentpanel, cmplx point, int maxval) {
  COMPLEXVECTOR c=wksp4;
  COMPLEXVECTOR r=wksp1big;
  //c,r,wksp and wksp are workspace variables. size(wksp) needs to be 2 times the length of out->a
  cmplx zp, ctmp;
  double tmp,ret;
  //check endpoints
  COMPLEXSUB(zp, point, currentpanel->z1);
  ret=COMPLEXNORM(zp);
  COMPLEXSUB(ctmp, point, currentpanel->z2a);
  tmp=COMPLEXNORM(ctmp);
  if(maxval^(tmp<ret))
    ret=tmp;
  //solve problem in rotated system, distances are invariant during the rotation
  COMPLEXMULC(zp, zp, currentpanel->rotvect, tmp);
  if(currentpanel->a_order>=2) { //if order==1, the panel is linear, maximum distance has to be at an endpoint
    COMPLEXNEG(c[0], zp); //panel position-out->z
    if(currentpanel->areal) {
      COMPLEXASSIGN(c[1], 1.0, currentpanel->a[1].re); //panel position is x+1i*sum(a[i]*x^i)
      for(int j=2;j<=currentpanel->a_order;j++) {
        COMPLEXASSIGN(c[j], 0, currentpanel->a[j].re);
      }
    }
    else {
      COMPLEXASSIGN(c[1], -currentpanel->a[1].im+1.0, currentpanel->a[1].re);
      for(int j=2;j<=currentpanel->a_order;j++) {
        COMPLEXASSIGN(c[j], -currentpanel->a[j].im, currentpanel->a[j].re);
      }
    }
    polynorm(wksp2, c, currentpanel->a_order+1); //create the vector with c*conj(c) which will give the distance to the point
    //calculate derivative and check where the derivative is zero
    for(int j=1;j<currentpanel->a_order*2+1;j++)
      wksp2[j-1].re=wksp2[j].re*j;
    polyroots(r, wksp2, currentpanel->a_order*2-1, wksp1,wksp3);
    for(int j=0;j<currentpanel->a_order*2-1;j++) { //search through all roots
      if(fabs(r[j].im)<fabs(r[j].re*1e-14)&&r[j].re>0.0&&r[j].re<currentpanel->endval) { //real root within the panel limits
        ctmp=evalpoly(c, r[j].re, currentpanel->a_order+1);
        tmp=COMPLEXNORM(ctmp);
        if(maxval^(tmp<ret)) {
          ret=tmp;
        }
      }
    }
  }
  return ret;
}

//calculate the longest distance between out->z and the panel
void initgeneralpaneldist(generalpanel *restrict out) {
  out->dabssqr=sqrdistancefrompanel(out, out->z, 1);
  out->dabs=sqrt(out->dabssqr); //take square root to find the real distance.
}

//helper function to add values in splitgeneralpanel and take care of possible memory reallocations
void addtopanellist(int** list,int ind,int* alloc,int value) {
  if(list!=NULL) {
    if(ind>=*alloc) {
      *alloc=(int)(*alloc*1.2)+5;
      *list=(int*)mxRealloc(*list,*alloc*sizeof(int));
    }
    (*list)[ind]=value;
  }
}

//split the panel at the point where it crosses value. xsplit determines split direction
void splitgeneralpanel(int currentpanelindex,panellist *restrict panels,int xsplit,double value,int** leftlist,int *leftind,int* leftalloc,
                       int** rightlist,int *rightind,int* rightalloc) {
  generalpanel *currentpanel=&panels->panels[currentpanelindex];
  //start by setting up the equation system for the panel position in the x-y plane, isolate the real or imaginary part, depending on which direction that the split is performed in
  if(xsplit) { //split in x-direction
    //start by adding the start panel to the correct output list
    if(currentpanel->z1.re<value)
      addtopanellist(leftlist,(*leftind)++,leftalloc,currentpanelindex);
    else if(currentpanel->z1.re>value)
      addtopanellist(rightlist,(*rightind)++,rightalloc,currentpanelindex);
    else { //equal
      if(currentpanel->connectvars.startslope.re<0)
        addtopanellist(leftlist,(*leftind)++,leftalloc,currentpanelindex);
      else if(currentpanel->connectvars.startslope.re>0)
        addtopanellist(rightlist,(*rightind)++,rightalloc,currentpanelindex);
      else {
        if(currentpanel->z2a.re<value)
          addtopanellist(leftlist,(*leftind)++,leftalloc,currentpanelindex);
        else
          addtopanellist(rightlist,(*rightind)++,rightalloc,currentpanelindex);
      }
    }
    if(value<currentpanel->z.re-currentpanel->d.re||value>currentpanel->z.re+currentpanel->d.re) //panel will not intersect the split line as the split line is located outside the box containing the panel
      return;
    COMPLEXASSIGN(wksp1[0],currentpanel->z1.re-value,0.0);
    if(currentpanel->areal) { //simplified the case with real coefficients
      COMPLEXASSIGN(wksp1[1],currentpanel->rotvect.re-currentpanel->a[1].re*currentpanel->rotvect.im,0.0);
      for(int j=2;j<=currentpanel->a_order;j++) {
        COMPLEXASSIGN(wksp1[j],-currentpanel->a[j].re*currentpanel->rotvect.im,0.0);
      }
    }
    else {
      COMPLEXASSIGN(wksp1[1],-currentpanel->a[1].re*currentpanel->rotvect.im+(1-currentpanel->a[1].im)*currentpanel->rotvect.re,0.0);
      for(int j=2;j<=currentpanel->a_order;j++) {
        COMPLEXASSIGN(wksp1[j],-currentpanel->a[j].re*currentpanel->rotvect.im-currentpanel->a[j].im*currentpanel->rotvect.re,0.0);
      }
    }
  }
  else { //split in y direction
    //start by adding the start panel to the correct output list
    if(currentpanel->z1.im<value)
      addtopanellist(leftlist,(*leftind)++,leftalloc,currentpanelindex);
    else if(currentpanel->z1.im>value)
      addtopanellist(rightlist,(*rightind)++,rightalloc,currentpanelindex);
    else { //equal
      if(currentpanel->connectvars.startslope.im<0)
        addtopanellist(leftlist,(*leftind)++,leftalloc,currentpanelindex);
      else if(currentpanel->connectvars.startslope.im>0)
        addtopanellist(rightlist,(*rightind)++,rightalloc,currentpanelindex);
      else {
        if(currentpanel->z2a.im<value)
          addtopanellist(leftlist,(*leftind)++,leftalloc,currentpanelindex);
        else
          addtopanellist(rightlist,(*rightind)++,rightalloc,currentpanelindex);
      }
    }
    if(value<currentpanel->z.im-currentpanel->d.im||value>currentpanel->z.im+currentpanel->d.im) //panel will not intersect the split line as the split line is located outside the box containing the panel
      return;
    COMPLEXASSIGN(wksp1[0],currentpanel->z1.im-value,0.0);
    if(currentpanel->areal) { //simplified the case with real coefficients
      COMPLEXASSIGN(wksp1[1],currentpanel->a[1].re*currentpanel->rotvect.re+currentpanel->rotvect.im,0.0);
      for(int j=2;j<=currentpanel->a_order;j++) {
        COMPLEXASSIGN(wksp1[j],currentpanel->a[j].re*currentpanel->rotvect.re,0.0);
      }
    }
    else {
      COMPLEXASSIGN(wksp1[1],currentpanel->a[1].re*currentpanel->rotvect.re+(1-currentpanel->a[1].im)*currentpanel->rotvect.im,0.0);
      for(int j=2;j<=currentpanel->a_order;j++) {
        COMPLEXASSIGN(wksp1[j],currentpanel->a[j].re*currentpanel->rotvect.re-currentpanel->a[j].im*currentpanel->rotvect.im,0.0);
      }
    }
  }
  int tmporder=currentpanel->a_order;
  for(;tmporder>0&&fabs(wksp1[tmporder].re)<1e-15*currentpanel->deltas;tmporder--);
  if(tmporder==0) //panel parallel with value, no split possible
    return;
  polyroots(wksp2, wksp1, tmporder, wksp3, wksp4); //find intersection points
  int matches=0;
  for(int i=0;i<tmporder;i++) {
    if(fabs(wksp2[i].im)<fabs(wksp2[i].re*1e-14)&&wksp2[i].re>0.0&&wksp2[i].re<0.999999999999*currentpanel->endval) { //real root within the panel limits, if it is too close to the end point, skip the split to avoid unnecessary additional panels
      wkspd[matches++]=wksp2[i].re;
    }
  }
  if(matches>1) //keep elements sorted to easier handle the connectivity information
     bubblesort(wkspd,matches);
  int activepanelindex=currentpanelindex;
  double baseendval=currentpanel->endval;
  for(int i=0;i<matches;i++) { //do the split    
    //the code that performs the split, given the position along the reference line
    int oldactivepanelindex=activepanelindex;
    activepanelindex=performsplit(currentpanelindex, activepanelindex, panels, wkspd[i], baseendval, i==0?0.0:wkspd[i-1], 1);
    currentpanel=&panels->panels[currentpanelindex]; //in case of reallocation
    initgeneralpanelboxfromextremepoints(&panels->panels[activepanelindex],currentpanel,wkspd[i],i<matches-1?wkspd[i+1]:baseendval); //generate the new box information for the new panel
    initgeneralpanelboxfromextremepoints(&panels->panels[oldactivepanelindex],currentpanel,i<1?0:wkspd[i-1],wkspd[i]); //update it for the old one.

    //add panel to the proper lists
    if(xsplit) {
      if(panels->panels[activepanelindex].connectvars.startslope.re<0)
        addtopanellist(leftlist,(*leftind)++,leftalloc,activepanelindex);
      else if(currentpanel->connectvars.startslope.re>0)
        addtopanellist(rightlist,(*rightind)++,rightalloc,activepanelindex);
      else {
        if(panels->panels[activepanelindex].z2a.re<value)
          addtopanellist(leftlist,(*leftind)++,leftalloc,activepanelindex);
        else
          addtopanellist(rightlist,(*rightind)++,rightalloc,activepanelindex);
      }
    }
    else {
      if(panels->panels[activepanelindex].connectvars.startslope.im<0)
        addtopanellist(leftlist,(*leftind)++,leftalloc,activepanelindex);
      else if(currentpanel->connectvars.startslope.im>0)
        addtopanellist(rightlist,(*rightind)++,rightalloc,activepanelindex);
      else {
        if(panels->panels[activepanelindex].z2a.im<value)
          addtopanellist(leftlist,(*leftind)++,leftalloc,activepanelindex);
        else
          addtopanellist(rightlist,(*rightind)++,rightalloc,activepanelindex);
      }
    }
  }
}

void splitgeneralpanels(panellist *restrict panels,int* panelptr,int N,int xsplit,double value,int** leftlist,int *leftind,int* leftalloc,
                       int** rightlist,int *rightind,int* rightalloc) {
  *leftind=0;
  *rightind=0;
  for(int i=0;i<N;i++) {
    splitgeneralpanel(panelptr[i],panels,xsplit,value,leftlist,leftind,leftalloc,rightlist,rightind,rightalloc);
  }
}


//TODO: check if there at any point is any difference between currentpanelindex and activepanelindex? currently the same when called
//function that will split the panel based on value, which is given along the reference line.
//currentpanelindex is the index of the original panel, whose a vector correspond to value
//activepanelindex is the index of the panel to be split
//newpanelindex and return value is the index of the new panel, defaults to -1
int performsplit(int currentpanelindex,int activepanelindex, panellist *restrict panels, double value, double baseendval,double startvalue,int identical,int newpanelindex) {
  if(newpanelindex<0) //code added for splitgeneralpanel.cpp to work properly, since in that case, it can be useful to manually control the order of the panels
    newpanelindex=addpanel(panels);
  generalpanel *newpanel=&panels->panels[newpanelindex];
  generalpanel *currentpanel=&panels->panels[currentpanelindex];
  generalpanel *activepanel=&panels->panels[activepanelindex];
  allocateduplicatepanel(newpanel, activepanel, currentpanel); //allocate variables
  newpanel->Nextremepoints=0;
  activepanel->next=newpanel;
  activepanel->nextidentical=identical; //will be changed to the value of identical later, now used 
  newpanel->prev=activepanel;
  newpanel->previdentical=identical;
  newpanel->endval=baseendval-value;
  activepanel->endval=value-startvalue;
  if(newpanel->next!=NULL)
    newpanel->next->prev=newpanel;
  
  cmplx ctmp2=evalpoly(currentpanel->a, value, currentpanel->a_order+1); //determine the panel position at the split point
  cmplx ctmp;
  COMPLEXASSIGN(ctmp, value-ctmp2.im, ctmp2.re);
  double tmp;
  if(activepanel==currentpanel)
    ctmp2=ctmp;
  else { //can be done faster if the evaluation is stored form previous step, but this should be so rare (panel intersects split line more than once) that an evalpoly does not matter
    cmplx ctmp3=evalpoly(currentpanel->a, startvalue, currentpanel->a_order+1); //determine the panel position at the split point
    COMPLEXASSIGN(ctmp2, startvalue-ctmp3.im, ctmp3.re);
    COMPLEXSUB(ctmp2,ctmp,ctmp2);
  }
  activepanel->z2alocal=ctmp2;
  COMPLEXSUB(newpanel->z2alocal,newpanel->z2alocal,ctmp2); //subtract the change in start position

  COMPLEXMUL(ctmp, ctmp, currentpanel->rotvect, tmp);
  COMPLEXADD(activepanel->z2a, ctmp, currentpanel->z1); //set z2a
  COMPLEXADDC(activepanel->z2,activepanel->z1,currentpanel->rotvect,value-startvalue); //set new z2 of 
  COMPLEXSUB(ctmp,activepanel->z2a,activepanel->z2);
  COMPLEXADD(newpanel->z2,newpanel->z2,ctmp);
  newpanel->z1=activepanel->z2a;
  
  memcpy(newpanel->a, currentpanel->a, (currentpanel->a_order+1)*sizeof(cmplx));
  for (int k = 1; k <= currentpanel->a_order; k++) {
    for(int j = currentpanel->a_order-k; j < currentpanel->a_order; j++) {
      COMPLEXADDC(newpanel->a[j], newpanel->a[j], newpanel->a[j+1], value);
    }
  }
  COMPLEXASSIGN(newpanel->a[0], 0.0, 0.0);
  memcpy(newpanel->b, currentpanel->b, (currentpanel->b_order+1)*sizeof(cmplx));
  if (!currentpanel->shifted) {
    for (int k = 1; k <= currentpanel->b_order; k++) {
      for(int j = currentpanel->b_order-k; j < currentpanel->b_order; j++) {
        COMPLEXADDC(newpanel->b[j], newpanel->b[j], newpanel->b[j+1], value);
      }
    }
  }
  else { //shifted, move them in opposite directions
    double newval=0.5*value;
    for (int k = 1; k <= currentpanel->b_order; k++) {
      for(int j = currentpanel->b_order-k; j < currentpanel->b_order; j++) {
        COMPLEXADDC(newpanel->b[j], newpanel->b[j], newpanel->b[j+1], newval);
        COMPLEXSUBC(currentpanel->b[j], currentpanel->b[j], currentpanel->b[j+1], newval);
      }
    }
  }
  if(currentpanel->b!=currentpanel->b_original) {
    memcpy(newpanel->b_original, currentpanel->b_original, (currentpanel->b_order+1)*sizeof(cmplx));
    if (!currentpanel->b_original_shifted) {
      for (int k = 1; k <= currentpanel->b_original_order; k++) {
        for(int j = currentpanel->b_original_order-k; j < currentpanel->b_original_order; j++) {
          COMPLEXADDC(newpanel->b_original[j], newpanel->b_original[j], newpanel->b_original[j+1], value);
        }
      }
    }
    else { //shifted, move them in opposite directions
      double newval=0.5*value;
      for (int k = 1; k <= currentpanel->b_original_order; k++) {
        for(int j = currentpanel->b_original_order-k; j < currentpanel->b_original_order; j++) {
          COMPLEXADDC(newpanel->b_original[j], newpanel->b_original[j], newpanel->b_original[j+1], newval);
          COMPLEXSUBC(currentpanel->b_original[j], currentpanel->b_original[j], currentpanel->b_original[j+1], newval);
        }
      }
    }
  }
  if(newpanel->target)
    initgeneralpanelcvectortarget(newpanel); //create c vector for new panel.
  else
    initgeneralpanelcvector(newpanel); //create c vector for new panel.
  if(newpanel->a_order==3) {
    if(newpanel->areal)
      initcubicconstants_areal(&newpanel->cub, newpanel);
    else
      initcubicconstants_aimag(&newpanel->cub, newpanel);
  }

  //set up startslope and end slope for new panels
  COMPLEXASSIGN(ctmp,1-newpanel->a[1].im,newpanel->a[1].re);
  COMPLEXMUL(ctmp,ctmp,newpanel->rotvect,tmp);
  tmp=1/COMPLEXABS(ctmp);
  COMPLEXASSIGN(newpanel->connectvars.startslope,ctmp.re*tmp,ctmp.im*tmp);
  activepanel->connectvars.endslope=newpanel->connectvars.startslope;

  //set up connectivity information
  {
    double maxtmp=fmax(activepanel->endval, newpanel->endval);
    activepanel->connectvars.enddist=MINCONNECTDIST*maxtmp; //constants are relatively arbitrary chosen
    double mintmp=fmin(activepanel->endval, newpanel->endval);
    if(mintmp<10*maxtmp) //if one panel is much shorter, reduce the allowed distance
      activepanel->connectvars.enddist=MAXCONNECTDIST*mintmp;
    newpanel->connectvars.startdist=activepanel->connectvars.enddist;
  }

  if(activepanel->prev!=NULL) {
    double maxtmp=fmax(activepanel->endval, activepanel->prev->endval);
    activepanel->connectvars.startdist=MINCONNECTDIST*maxtmp; //constants are relatively arbitrary chosen
    double mintmp=fmin(activepanel->endval, activepanel->prev->endval);
    if(mintmp<10*maxtmp) //if one panel is much shorter, reduce the allowed distance
      activepanel->connectvars.startdist=MAXCONNECTDIST*mintmp;
    activepanel->prev->connectvars.enddist=activepanel->connectvars.startdist;
  }
  //initialize new connectivity information with updated cutoff distances, last panel case
  if(newpanel->next!=NULL) {
    double maxtmp=fmax(newpanel->endval, newpanel->next->endval);
    newpanel->connectvars.enddist=MINCONNECTDIST*maxtmp; //constants are relatively arbitrary chosen
    double mintmp=fmin(newpanel->endval, newpanel->next->endval);
    if(mintmp<10*maxtmp) //if one panel is much shorter, reduce the allowed distance
      newpanel->connectvars.enddist=MAXCONNECTDIST*mintmp;
    newpanel->next->connectvars.startdist=newpanel->connectvars.enddist;
  }
  if(!identical) {
    initgeneralpanelconnectingterms(activepanel);
    initgeneralpanelconnectingterms(newpanel);
  }
  if(activepanel->target) {
    inittargetbase(activepanel);
    inittargetbase(newpanel);
  }
  return newpanelindex;
}

//initializes the bounding box for the new panel. With extremepoints available, this is much faster as no equation has to be solved
//newpanel is the panel that will be initialized. currentpanel is the original panel, where minval and maxval are defined
void initgeneralpanelboxfromextremepoints(generalpanel *newpanel,generalpanel *currentpanel,double minval,double maxval) {  
  double xmin=newpanel->z1.re, xmax=newpanel->z1.re, ymin=newpanel->z1.im, ymax=newpanel->z1.im;
  //start by checking end points
  if(newpanel->z2a.re<xmin)
    xmin=newpanel->z2a.re;
  if(newpanel->z2a.re>xmax)
    xmax=newpanel->z2a.re;
  if(newpanel->z2a.im<ymin)
    ymin=newpanel->z2a.im;
  if(newpanel->z2a.im>ymax)
    ymax=newpanel->z2a.im;
  if(newpanel!=currentpanel) //if extremepoints are to be set, reset counter
    newpanel->Nextremepoints=0;
  for(int k=0;k<currentpanel->Nextremepoints;k++) { //check the extremepoints
    if(currentpanel->extremepointval[k]>minval&&currentpanel->extremepointval[k]<maxval) { //extremepoint belongs to this part of the panel
      if(newpanel!=currentpanel) { //update the extremepoints for the new panel
        newpanel->extremepoints[newpanel->Nextremepoints]=currentpanel->extremepoints[k];
        newpanel->extremepointval[newpanel->Nextremepoints++]=currentpanel->extremepointval[k]-minval;
      }
      if(currentpanel->extremepoints[k].re<xmin)
        xmin=currentpanel->extremepoints[k].re;
      if(currentpanel->extremepoints[k].re>xmax)
        xmax=currentpanel->extremepoints[k].re;
      if(currentpanel->extremepoints[k].im<ymin)
        ymin=currentpanel->extremepoints[k].im;
      if(currentpanel->extremepoints[k].im>ymax)
        ymax=currentpanel->extremepoints[k].im;
    }
  }
  COMPLEXASSIGN(newpanel->z, 0.5*(xmax+xmin), 0.5*(ymax+ymin));
  COMPLEXASSIGN(newpanel->d, 0.5*(xmax-xmin), 0.5*(ymax-ymin));
}

//this is a code to apply the correction due to the panel slope, which causes the panel to have a stength density correponding to the original polynomial over the panel surface and not the reference line
//this is performed by doing a series expansion of the slope correction
//NOTE: this code is mainly as a reference code. It creates additional numerical instabilities due to the large numbers of coefficients
//and results show that there are no improvements of using this method, compared to only correcting the endpoints
void applyslopeexpansion(int currentpanelindex,panellist *restrict panels,double tol,const double *coeffs,int hastarget,int** list,int *ind,int* alloc) {
  generalpanel *out=&panels->panels[currentpanelindex];
  double tmp;
  if(out->next!=NULL&&out->nextidentical) {
    out->nextidentical=0;
    out->next->previdentical=0;
  }
  if(out->prev!=NULL&&out->previdentical) {
    out->previdentical=0;
    out->prev->nextidentical=0;
  }
  for(int i=1;i<=out->a_order;i++) {//calulate derivative of a
    COMPLEXASSIGN(wksp1[i-1],i*out->a[i].re,0.0); //currently only implemented for real values of a
  }
  tmp=evalpolyreal(out->a,out->endval,out->a_order+1); //can be calculated from z2a as well, but this should not take too much time anyway

  //start by determining the maximum slope of the panel
  double maxslope=wksp1[0].re*wksp1[0].re; //slope at start point
  double minslope=maxslope;
  int positive=0,negative=0;
  if(wksp1[0].re<0)
    negative=1;
  if(wksp1[0].re>0)
    positive=1;

  //slope at end point
  tmp=evalpolyreal(wksp1,out->endval,out->a_order);
  if(tmp<0)
    negative=1;
  if(tmp>0)
    positive=1;
  tmp*=tmp;
  if(fabs(tmp)<minslope)
    minslope=fabs(tmp);
  if(fabs(tmp)>maxslope)
    maxslope=fabs(tmp);

  for(int i=1;i<out->a_order;i++) {//calulate second derivative of a
    COMPLEXASSIGN(wksp2[i-1],i*wksp1[i].re,0.0); //currently only implemented for real values of a
  }
  polyroots(wksp3, wksp2, out->a_order-2, wksp4,wksp5); //find zeros
  for(int i=0;i<out->a_order-2;i++) { //check all extreme points if any of them has higher/lower slope than the end points
    if(fabs(wksp3[i].im)<fabs(wksp3[i].re*1e-14)&&wksp3[i].re>0.0&&wksp3[i].re<out->endval) { //real root within the panel limits
      tmp=evalpolyreal(wksp1, wksp3[i].re, out->a_order);
      if(tmp<0)
        negative=1;
      if(tmp>0)
        positive=1;
      tmp*=tmp;
      if(fabs(tmp)<minslope)
        minslope=fabs(tmp);
      if(fabs(tmp)>maxslope)
        maxslope=fabs(tmp);
    }
  }
  if(positive&&negative) //correction if both negative and positive slopes are present, then 0 will exist as well
    minslope=0;

  //add correction to the square root to achive improved convergence of the series expansion
  double sqrslopeshift=0.5*(maxslope+minslope);
  maxslope-=sqrslopeshift;
  maxslope/=1+sqrslopeshift;

  //if slope is too large to achive sufficient convergence, split panel in two parts
  if(maxslope>0.5*0.5) { //TODO: fix this case (unclear if it is working at the moment, but slope expansion is very experimental)
    //mexWarnMsgTxt("slope of panel too large");
    int activepanelindex=performsplit(currentpanelindex, currentpanelindex, panels, 0.5*out->endval, out->endval,0.0,0);
    if(list!=NULL&&ind!=NULL&&alloc!=NULL)
      addtopanellist(list,(*ind)++,alloc,activepanelindex);
    out=&panels->panels[currentpanelindex]; //in case of reallocation
    initgeneralpanelboxfromextremepoints(&panels->panels[activepanelindex],&panels->panels[currentpanelindex],0,0.5*out->endval);
    initgeneralpanelboxfromextremepoints(&panels->panels[currentpanelindex],&panels->panels[currentpanelindex],0.5*out->endval,out->endval);
    applyslopeexpansion(currentpanelindex,panels,tol,coeffs,hastarget,list,ind,alloc);
    applyslopeexpansion(activepanelindex,panels,tol,coeffs,hastarget,list,ind,alloc);
    
  }
  else { //expansion can be performed
    int count=0;
    tmp=1.0;
    for(;count<SQRTEXPMAX&&tmp*fabs(coeffs[count])>tol;count++,tmp*=maxslope); //determine number of elements for convergence
    if(count>1) {
      COMPLEXVECTOR dest1=wksp1big;
      for (int k = 1; k < out->a_order; k++) { //shift polynomial to center point
        for(int j = out->a_order-1-k; j < out->a_order-1; j++) {
          COMPLEXADDC(wksp1[j], wksp1[j], wksp1[j+1], 0.5*out->endval);
        }
      }
      //normalize vector
      if(out->a_order>1) {
        COMPLEXASSIGN(wksp1[1],wksp1[1].re*out->endval,wksp1[1].im*out->endval);
      }
      double constant=out->endval;
      for(int j=2;j<out->a_order;j++) {
        constant*=out->endval;
        COMPLEXASSIGN(wksp1[j],wksp1[j].re*constant,wksp1[j].im*constant);
      }
      sqrpolyreal(wksp3, wksp1, out->a_order); //the polynomial to be expanded
      wksp3[0].re-=sqrslopeshift; //add the correction to improve convergence
      double invslope=1/(1+sqrslopeshift);
      for(int i=0;i<out->a_order*2+1;i++)
        wksp3[i].re*=invslope;
      count++;
      //create the new polynomial that is the old coefficients multiplied by the expansion
      COMPLEXVECTOR dest2=wksp2big;
      COMPLEXVECTOR vectmp;
      int currentorder=out->a_order*2-1;
      for(int i=0;i<out->a_order*2+1;i++) {
        dest1[i].re=wksp3[i].re*coeffs[count-1];
      }
      dest1[0].re+=coeffs[count-2];
      for(int i=count-2;i>0;i--) {
        multiplypolyreal(dest2, dest1, wksp3, currentorder, out->a_order*2-1);
        vectmp=dest1;
        dest1=dest2;
        dest2=vectmp;
        currentorder+=out->a_order*2-2;
        dest1[0].re+=coeffs[i-1];
      }

      double maxroot;
      if(hastarget) //targets do not have evaluation at center point implemented at the moment, therefore, the convergence need to be higher
        maxroot=2*out->a_order; //dist1 is now normalized
      else
        maxroot=0.5*1.8*out->a_order;
      double maxres=evalpolyrealabs(dest1,maxroot,currentorder)-fabs(dest1[0].re);

      //slope expansion does not only have problems with the convergence of the series, but also the convergence of the final series, due to large truncation errors between the steps 
      if(maxres<ROOTDIVISIONTOL*out->deltas) {
        out->shifted=1;
        
        memcpy(wksp2, out->b, (out->b_order+1)*sizeof(cmplx));
        double scaling=sqrt(1+sqrslopeshift);
        for(int k=0;k<=out->b_order;k++) {
          COMPLEXASSIGN(wksp2[k], wksp2[k].re*scaling, wksp2[k].im*scaling);
        }

        
        //shift b-coefficients to center around the midpoint instead (same as the expansion)
        for (int k = 1; k <= out->b_order; k++) {
          for(int j = out->b_order-k; j < out->b_order; j++) {
            COMPLEXADDC(wksp2[j], wksp2[j], wksp2[j+1], 0.5*out->endval);
          }
        }
  
        //normalize vector
        if(out->b_order>0) {
          COMPLEXASSIGN(wksp2[1],wksp2[1].re*out->endval,wksp2[1].im*out->endval);
        }
        double constant=out->endval;
        for(int j=2;j<=out->b_order;j++) {
          constant*=out->endval;
          COMPLEXASSIGN(wksp2[j],wksp2[j].re*constant,wksp2[j].im*constant);
        }
        out->normalized=1;

        //note: old b vector still stored in b_original
        out->b=(COMPLEXVECTOR)myMalloc((out->b_order+currentorder)*sizeof(cmplx));
        multiplypolyreal2(out->b, wksp2, dest1, out->b_order+1, currentorder);
        out->b_order+=currentorder-1;
        //NOTE: even though b coefficients are shifted, the a coefficients are not shifted here, instead it is handled in the direct evaluation, as there are also benefits of having the original set of values there
      }
      else { //not sufficient convergence, slit once more
        int activepanelindex=performsplit(currentpanelindex, currentpanelindex, panels, 0.5*out->endval, out->endval, 0.0, 0);
        if(list!=NULL&&ind!=NULL&&alloc!=NULL)
          addtopanellist(list,(*ind)++,alloc,activepanelindex);
        out=&panels->panels[currentpanelindex]; //in case of reallocation
        initgeneralpanelboxfromextremepoints(&panels->panels[activepanelindex], &panels->panels[currentpanelindex], 0, 0.5*out->endval);
        initgeneralpanelboxfromextremepoints(&panels->panels[currentpanelindex], &panels->panels[currentpanelindex], 0.5*out->endval, out->endval);
        applyslopeexpansion(currentpanelindex, panels, tol, coeffs, hastarget, list, ind, alloc);
        applyslopeexpansion(activepanelindex, panels, tol, coeffs, hastarget, list, ind, alloc);
      }
    }
  }
}

//normalizes the b values, should be called after all splits are performed
void normalizeb(generalpanel *currentpanel) {
  if(currentpanel->normalized)
    return;
  if(currentpanel->b==currentpanel->b_original) {
    currentpanel->b=(COMPLEXVECTOR)myMalloc((currentpanel->b_order+1)*sizeof(cmplx));
    memcpy(currentpanel->b,currentpanel->b_original,(currentpanel->b_order+1)*sizeof(cmplx));
  }
  if(currentpanel->b_order>0) {
    COMPLEXASSIGN(currentpanel->b[1],currentpanel->b[1].re*currentpanel->endval,currentpanel->b[1].im*currentpanel->endval);
  }
  double constant=currentpanel->endval;
  for(int i=2;i<=currentpanel->b_order;i++) {
    constant*=currentpanel->endval;
    COMPLEXASSIGN(currentpanel->b[i],currentpanel->b[i].re*constant,currentpanel->b[i].im*constant);
  }
  currentpanel->normalized=1;
  if(!currentpanel->shifted)
    currentpanel->b_unshifted=currentpanel->b;
}

//shifts the b values, should be called after all splits are performed
void shiftb(generalpanel *currentpanel) {
  if(currentpanel->shifted)
    return;
  if(currentpanel->b==currentpanel->b_original) {
    currentpanel->b=(COMPLEXVECTOR)myMalloc((currentpanel->b_order+1)*sizeof(cmplx));
    memcpy(currentpanel->b,currentpanel->b_original,(currentpanel->b_order+1)*sizeof(cmplx));
  }
  //shift b-coefficients to center around the midpoint instead (same as the expansion)
  double shiftval=0.5;
  if(!currentpanel->normalized)
    shiftval*=currentpanel->endval;
  for (int k = 1; k <= currentpanel->b_order; k++) {
    for(int j = currentpanel->b_order-k; j < currentpanel->b_order; j++) {
      COMPLEXADDC(currentpanel->b[j], currentpanel->b[j], currentpanel->b[j+1], shiftval);
    }
  }
  currentpanel->shifted=1;
}

//unsifts the b values to b_unshifted, should be called after all splits are performed
void unshiftb(generalpanel *currentpanel) {
  if(!currentpanel->shifted)
    return;
  if(currentpanel->b_unshifted==currentpanel->b||currentpanel->b_unshifted==currentpanel->b_original) {
    currentpanel->b_unshifted=(COMPLEXVECTOR)myMalloc((currentpanel->b_order+1)*sizeof(cmplx));
    memcpy(currentpanel->b_unshifted,currentpanel->b,(currentpanel->b_order+1)*sizeof(cmplx));
  }
  //shift b-coefficients back to the starting point
  double shiftval=-0.5;
  if(!currentpanel->normalized)
    shiftval*=currentpanel->endval;
  for (int k = 1; k <= currentpanel->b_order; k++) {
    for(int j = currentpanel->b_order-k; j < currentpanel->b_order; j++) {
      COMPLEXADDC(currentpanel->b_unshifted[j], currentpanel->b_unshifted[j], currentpanel->b_unshifted[j+1], shiftval);
    }
  }
}

//initializes multipole expansions for a source panel, using exact gaussian quadrature
void initgeneralpanelexpansion(generalpanel *restrict out, int Ncoeff) {
  
  //fvalues and pvalues are workspace variables, should be atleast the size of Ngauss
  double tmp;
  COMPLEXVECTOR c=wksp1;
  COMPLEXVECTOR fvalues=wksp1big;
  COMPLEXVECTOR pvalues=wksp2big;

  out->coeffs=(COMPLEXVECTOR)myMalloc((Ncoeff+1)*sizeof(cmplx));
  out->Ncoeffs=Ncoeff;
  int Ngauss=(out->b_order+2+(out->a_order+1)*Ncoeff)/2; //determine number of points for gaussian quadrature to be exact for highest order term
  if(Ngauss>MAXGAUSS)
    Ngauss=MAXGAUSS;

  //start by generating the vector c
  COMPLEXSUB(c[0], out->z1, out->z);
  if(out->areal) { //simplified version for real coefficients
    if(out->a_order>0) {
      COMPLEXASSIGN(c[1], 1, out->a[1].re);
      COMPLEXMUL(c[1], c[1], out->rotvect, tmp);
    }
    else { //lowest order case
      c[1]=out->rotvect;
    }
    for(int j=2;j<=out->a_order;j++) {
      COMPLEXASSIGN(c[j], -out->a[j].re*out->rotvect.im, out->a[j].re*out->rotvect.re);
    }
  }
  else {
    if(out->a_order>0) {
      COMPLEXASSIGN(c[1], 1-out->a[1].im, out->a[1].re);
      COMPLEXMUL(c[1], c[1], out->rotvect, tmp);
    }
    else {
      c[1]=out->rotvect;
    }
    for(int j=2;j<=out->a_order;j++) {
      COMPLEXASSIGN(c[j], -out->a[j].re*out->rotvect.im-out->a[j].im*out->rotvect.re, out->a[j].re*out->rotvect.re-out->a[j].im*out->rotvect.im);
    }
  }
  int bshift=1; //if polynomial is shifted to be centered around middle instead
  if(out->shifted)
    bshift=0;
  int a_order=out->a_order+1;
  if(a_order<2)
    a_order=2;

  //use exact gauss quadrature to solve the coefficients, evaluate polynomial values in each of these points
  for(int j=0;j<Ngauss;j++) {
    if(out->normalized)
      fvalues[j]=evalpoly(out->b, 0.5*(gpoints[Ngauss][j]+bshift), out->b_order+1);
    else
      fvalues[j]=evalpoly(out->b, 0.5*out->endval*(gpoints[Ngauss][j]+bshift), out->b_order+1);
    COMPLEXASSIGN(fvalues[j], fvalues[j].re*gweights[Ngauss][j], fvalues[j].im*gweights[Ngauss][j]);
    pvalues[j]=evalpoly(c, 0.5*out->endval*(gpoints[Ngauss][j]+1), a_order);
  }
  //solve for each coefficient
  COMPLEXASSIGN(out->coeffs[0],0.0,0.0); //to conform with the standard in fmm, have zero as first element;
  for(int k=0;k<Ncoeff;k++) {
    cmplx sum;
    COMPLEXASSIGN(sum, 0, 0);
    for(int j=0;j<Ngauss;j++) { //the sum in the gaussian quadrature
      COMPLEXASSIGN(sum, sum.re+fvalues[j].re, sum.im+fvalues[j].im);
    }
    COMPLEXASSIGN(out->coeffs[k+1], -/*INV2PI**/0.5*out->endval*sum.re,-/*INV2PI**/0.5*out->endval*sum.im);
    if(k!=Ncoeff-1) { //recalculate points for next coefficient. 
      for(int j=0;j<Ngauss;j++) {
        COMPLEXMUL(fvalues[j], fvalues[j], pvalues[j], tmp);
      }
    }
  }
}

//the main function for interactions between a source panel and a set of evaluation points
void generalpaneldirectinteract(double *restrict xvelocities,double *restrict yvelocities,generalpanel *restrict currentpanel,const double *restrict xpositions,const double *restrict ypositions,int N) {
  COMPLEXVECTOR d=wksp1big;
  COMPLEXVECTOR d_tmp=wksp2big;
  COMPLEXVECTOR b2=wksp3big; //b2 needs to be big as well due to the nature of polydivnowksp
  COMPLEXVECTOR r=wksp3;
  COMPLEXVECTOR rshifted=r;
  generalpaneldirectinteractcalls++; //counter to know how many evaluations that has been done
  if(currentpanel->shifted) {
    r+=currentpanel->a_order; //the temporary vector should be large enough to hold both.
  }
  //length of d and wksp should be atleast (aN-1)*MAXROOTEXPANSION+bN, length of r,b2 and wksp2 should be atleast max(aN,bN)
  cmplx zp,zp0,ctmp,ctmp2,ctmp3,ctmp4,startconnvel,endconnvel,rp;
  double tmp,tmp2,invendval=1/currentpanel->endval;
  int startconnected,endconnected;
#ifdef FORCEFULLNUMINT
  int fullnumint=1; //numerical integration instead
#else
  int fullnumint=currentpanel->numint;
#endif
  
  for(int j=0;j<N;j++) { //loop over evaluation points
    double endval=currentpanel->endval;
    startconnected=endconnected=0;
    generalpaneldirectinteractions++; //counter to know how many evaluations that has been done
    if(ypositions!=NULL) {
      COMPLEXASSIGN(zp0, xpositions[j]-currentpanel->z.re, ypositions[j]-currentpanel->z.im);
    }
    else {
      COMPLEXASSIGN(zp0, xpositions[j]-currentpanel->z.re, -currentpanel->z.im);
    }

    tmp=COMPLEXNORM(zp0);
    if(tmp>GENERALPANELCONVERGENCERADIUS*GENERALPANELCONVERGENCERADIUS*currentpanel->dabssqr) { //if possible, evaluate from multipole expansion instead. Note that all values are squared
      multipoleevaluation(xvelocities+j, yvelocities+j, zp0, currentpanel->coeffs, currentpanel->Ncoeffs, DBL_EPSILON,currentpanel->dabssqr/tmp);
      generalpanelmultipoleevaluations++;
      continue;
    }
    if(ypositions!=NULL) {
      COMPLEXASSIGN(rp, xpositions[j]-currentpanel->z1.re, ypositions[j]-currentpanel->z1.im);
    }
    else {
      COMPLEXASSIGN(rp, xpositions[j]-currentpanel->z1.re, -currentpanel->z1.im);
    }

    //check if panel is connected to another and the solution has to compensate for this
    if(currentpanel->prev!=NULL) {
      if(fabs(rp.re)+fabs(rp.im)<currentpanel->connectvars.startdist) {
        if(currentpanel->previdentical) //if they are identical, theinteraction will be handled by the other panel
          continue;
#ifdef ENDCORRECTION
        startconnected=1;
#endif
      }
    }
    //check if the panel is connected to another panel
    if(currentpanel->next!=NULL) { //compare with next->z1a.re instead of z2a, as this statement has to evaluate equally for both panels
      if(ypositions!=NULL) {
        COMPLEXASSIGN(ctmp, xpositions[j]-currentpanel->next->z1.re, ypositions[j]-currentpanel->next->z1.im);
      }
      else {
        COMPLEXASSIGN(ctmp, xpositions[j]-currentpanel->next->z1.re, -currentpanel->next->z1.im);
      }

      if(fabs(ctmp.re)+fabs(ctmp.im)<currentpanel->connectvars.enddist) { //check if point is close enough to the end point to use connected equation instead
        if(currentpanel->nextidentical) //identically connected panels, since they are identical and both have direct evaluation, continue the direct evaluation over the next panel as well
          endval+=currentpanel->next->endval;
#ifdef ENDCORRECTION
        else
          endconnected=1;
#endif
      }
    }
    COMPLEXMULC(zp, rp, currentpanel->rotvect, tmp); //transform to the panels coordinate system
    //initialize c vector
    //note: a[0]=0 in current implementation
    if(currentpanel->areal&&currentpanel->a_order!=1) {
      COMPLEXASSIGN(currentpanel->c[0], -zp.im*currentpanel->invaend.re, zp.re*currentpanel->invaend.re);
    }
    else {
      COMPLEXASSIGN(currentpanel->c[0],-zp.im,zp.re);
      COMPLEXMUL(currentpanel->c[0], currentpanel->c[0], currentpanel->invaend, tmp);
    }

    //calculate the roots of the polynomial for the partial fraction decomposition
    if(currentpanel->a_order==3&&fabs(currentpanel->invaend.re)+fabs(currentpanel->invaend.im)<1e8) { //do not use analytical solution for too bad conditioned problems
      cubicroots(r,currentpanel->c,&currentpanel->cub,d_tmp,wksp2);
    }
    else {
      polyroots(r, currentpanel->c, currentpanel->a_order, d_tmp,wksp2);
      
    }
    cmplx factor,factor0;
    factor=currentpanel->invaend;
    factor0=factor; //backup of original value for numerical integration
    
    
    int Nroots=currentpanel->a_order;
    COMPLEXVECTOR tmpwksp=currentpanel->b;
    COMPLEXVECTOR ctmpptr=currentpanel->c;
    int Ntmpwksp=currentpanel->b_order+1;
    int reduced=0;
    
    cmplx tmpvel;
    COMPLEXASSIGN(tmpvel, 0, 0);
    int numint=0;
    int b2N, dN;
    int numintorder;
    int numintnomN;
    double maxnorm=0;
    
    if(!fullnumint) { //if numerical integration should not be used
      double expansiontol;
      int maxexpansion;
      if(currentpanel->outside!=0.0) { //if the outside parameter is set, a root with imaginary part close to zero means that it is close to the panel
        for(int k=0;k<Nroots;k++) {
          double shift=currentpanel->endval*OUTSIDETHRESHOLD;
          if(fabs(r[k].im)<shift&&r[k].re>-shift&&r[k].re<currentpanel->endval+shift){ //root is on the panel
            if(r[k].im==0)
              r[k].im=fabs(r[k].im); //remove any possible negative zeros, as r[k].im<0 compares to false, while atan2 still considers them negative
            if((r[k].im<0)^(currentpanel->outside>0))
              r[k].im=-currentpanel->outside*currentpanel->endval*1e-16;
            else if(r[k].im==0&&(r[k].re==0||r[k].re==currentpanel->endval))
              r[k].im=-currentpanel->outside*currentpanel->endval*1e-16;
          }
        }
      }
      //start by determining if any root should be expanded with series expansion to improve accuracy
      //note: vector rshifted=r if polynomials are not shifted
      if(currentpanel->shifted) {  //the shifted case, allow for expansions of roots closer to zero
        expansiontol=ROOTEXPANSIONTOLSHIFTED*ROOTEXPANSIONTOLSHIFTED;
        maxexpansion=MAXROOTEXPANSIONSHIFTED;
        for(int k=0;k<Nroots;k++) { //if shifted, the roots should be moved to the center of the panel, as the a vector is not shifted.
          COMPLEXASSIGN(rshifted[k],r[k].re-0.5*currentpanel->endval,r[k].im);
        }
      }
      else {
        expansiontol=ROOTEXPANSIONTOL*ROOTEXPANSIONTOL;
        maxexpansion=MAXROOTEXPANSION;
      }

      //the loop that checks all roots and expand the necessary roots
      for(int k=0;k<Nroots;k++) {
        tmp2=COMPLEXNORM(rshifted[k]);
        if(tmp2>expansiontol*endval*endval) {
          directinteractexpandedroots++;
          COMPLEXRECIPROCAL(ctmp,rshifted[k],tmp);
          COMPLEXMULN(factor,factor,ctmp,tmp);
          //directinteractexpandedrootsnrcoeffs-=Ntmpwksp; //remove later
          if(currentpanel->normalized) {
            COMPLEXASSIGN(ctmp,ctmp.re*currentpanel->endval,ctmp.im*currentpanel->endval);
          }
          Ntmpwksp+=expandandmultiplypoly(d,tmpwksp,ctmp,DBL_EPSILON*DBL_EPSILON,endval*endval/tmp2,Ntmpwksp,maxexpansion);
          //directinteractexpandedrootsnrcoeffs+=Ntmpwksp; //remove later

          tmpwksp=d;
          d=d_tmp;
          d_tmp=tmpwksp;
          cmplx rold=rshifted[k];
          for(int m=k;m<Nroots-1;m++)
            rshifted[m]=rshifted[m+1];
          rshifted[Nroots-1]=rold;
          if(currentpanel->shifted) {//in the case shifted is false, then r and rshifted are the same vectors
            rold=r[k];
            for(int m=k;m<Nroots-1;m++)
              r[m]=r[m+1];
            r[Nroots-1]=rold;
          }
          wkspi[Nroots-1]=1;
          Nroots--;
          k--;
          reduced=1;
        }
        else if(tmp2>maxnorm)
          maxnorm=tmp2;
      }

      //recreate the polynomial where the expanded roots have been removed
      double invnormconstant;
      if(reduced||currentpanel->shifted) { //do not overwrite wksp2 later, it will be used in the endconnected case as well
        combinepoly(wksp2,rshifted,Nroots);
        ctmpptr=wksp2;

        if(currentpanel->normalized) {
          double constant=currentpanel->endval;
          if(Nroots>0) {
            COMPLEXASSIGN(ctmpptr[1],ctmpptr[1].re*currentpanel->endval,ctmpptr[1].im*currentpanel->endval);
          }
          else
            constant=1;
          for(int m=2;m<=Nroots;m++) {
            constant*=currentpanel->endval;
            COMPLEXASSIGN(ctmpptr[m],ctmpptr[m].re*constant,ctmpptr[m].im*constant);
          }
          invnormconstant=1/constant;
          for(int m=0;m<=Nroots;m++) {
            COMPLEXASSIGN(wksp2[m],wksp2[m].re*invnormconstant,wksp2[m].im*invnormconstant);
          }
        }
      }
      else if(currentpanel->normalized) { //if normalized b vector, the c vector has to be normalized during the polynomial division as well.
        wksp2[0]=ctmpptr[0];
        double constant=currentpanel->endval;
        if(Nroots>0) {
          COMPLEXASSIGN(wksp2[1],ctmpptr[1].re*currentpanel->endval,ctmpptr[1].im*currentpanel->endval);
        }
        else
          constant=1;
        for(int m=2;m<=Nroots;m++) {
          constant*=currentpanel->endval;
          COMPLEXASSIGN(wksp2[m],ctmpptr[m].re*constant,ctmpptr[m].im*constant);
        }
        invnormconstant=1/constant;
        for(int m=0;m<=Nroots;m++) {
          COMPLEXASSIGN(wksp2[m],wksp2[m].re*invnormconstant,wksp2[m].im*invnormconstant);
        }

        ctmpptr=wksp2;
      }
      
#ifdef ENDCORRECTION
      //if the panel is connected to another panel, there should be one root very close to the connected end point
      //find the connecting roots
      int startconnindex=0,endconnindex=0;
      if(startconnected) {
        double mindist=fabs(r[0].re)+fabs(r[0].im);
        for(int m=1;m<Nroots;m++) {
          double ddist=fabs(r[m].re)+fabs(r[m].im);
          if(ddist<mindist) {
            mindist=ddist;
            startconnindex=m;
          }
        }
      }
      if(endconnected) {
        double mindist=fabs(r[0].re-currentpanel->endval)+fabs(r[0].im);
        for(int m=1;m<Nroots;m++) {
          double ddist=fabs(r[m].re-currentpanel->endval)+fabs(r[m].im);
          if(ddist<mindist) {
            mindist=ddist;
            endconnindex=m;
          }
        }
      }
#endif
      //split b by dividing with c, d is the quota and b2 is the remainder
      polydivnowksp(d, b2, tmpwksp, ctmpptr, Ntmpwksp, Nroots+1); //always make sure nominator is of lower order than denominator before partial fraction decomposition

      //the different cases, depending on if the order of the denominator or the order of the nominator is highest
      if(Ntmpwksp>Nroots) {
        b2N=Nroots;
        dN=Ntmpwksp-Nroots;
      }
      else {
        b2N=Ntmpwksp;
        dN=0;
      }

      if(currentpanel->normalized) {
        for(int m=0;m<dN;m++) {
          COMPLEXASSIGN(d[m],d[m].re*invnormconstant*currentpanel->endval,d[m].im*invnormconstant*currentpanel->endval); //extra endval as it is integrated
        }
      }
      

      //search for duplicate roots, or roots very close to being duplicate, as these cause numerical truncation errors
      //note that for real coefficients for a, there should not be any duplicate roots close to the panel, i.e. with imaginary part close to zero
      int duplicatenr,duplicatecount;
      for(int m=0;m<Nroots;m++)
        wkspi[m]=0;
      for(int m=0;m<Nroots;m++) {
        if(wkspi[m])
          continue;
        duplicatecount=0;
        for(int n=m+1;n<Nroots;n++) {
          double dist=fabs(r[m].re-r[n].re)+fabs(r[m].im-r[n].im);
          if(dist*dist<DUPLICATEROOTTOL*DUPLICATEROOTTOL*maxnorm) {
#ifdef ENDCORRECTION
            if(startconnected&&n==startconnindex||m==startconnindex) { //it does not work with double roots close to the connected roots, but should not happen for normal panels
              mexWarnMsgTxt("Duplicate root detected close to panel start, results will be inaccurate");
              COMPLEXPRINT(r[m]);
              COMPLEXPRINT(r[n]);
              REALPRINT(maxnorm);
            }
            else if(endconnected&&n==endconnindex||m==endconnindex) { //it does not work with double roots close to the connected roots, but should not happen for normal panels
              mexWarnMsgTxt("Duplicate root detected close to panel end, results will be inaccurate");
            }
            else {
              duplicatecount++;
              duplicatenr=n;
            }
#else
            duplicatecount++;
            duplicatenr=n;
#endif
          }
        }
        //for double roots, use special solution where both roots are handled at once
        if(duplicatecount==1) { //double root
          cmplx dist2;
          COMPLEXSUB(dist2,r[m],r[duplicatenr]);
          //duplicates are quite rare
          cmplx rmend;
          double dist=0.5*COMPLEXABS(dist2);//fabs(dist2.re)+fabs(dist2.im);
          //ensure that the series expansions are convergent, otherwise, no root expansion
          //note that in case of no root expansions, the exact solution is used, since numerical integration seems to provide approximately as large errors for roots close to the end points
          if(dist<COMPLEXABS(r[m])&&dist<COMPLEXABS(r[duplicatenr])&&dist<sqrt((r[m].re-endval)*(r[m].re-endval)+r[m].im*r[m].im)&&dist<sqrt((r[duplicatenr].re-endval)*(r[duplicatenr].re-endval)+r[duplicatenr].im*r[duplicatenr].im)) {
            for(int n=0;n<Nroots;n++) {
              if(n==m||n==duplicatenr)
                continue;
              //if yet another root is very lose to the root, use numerical integration instead
              //we need convergence for the expansion C, which means that no other roots should be close to any of the roots. Therefore, a relatively large constant is added here to ensure to roots too close.
              if(dist>0.125*(fabs(r[n].re-r[duplicatenr].re)+fabs(r[n].im-r[duplicatenr].im))) { //add constant 4 to ensure convergence in all series
                numint=1;
              }
            }
            if(numint)
              break;
            rmend=disttoroot(xpositions[j],ypositions==NULL?0.0:ypositions[j],currentpanel,d_tmp,r[m],endval,endval*1e-3);
            COMPLEXRECIPROCAL(ctmp, rmend, tmp);
            COMPLEXRECIPROCAL(ctmp3, r[m], tmp);
            cmplx B=twotermseries(ctmp, ctmp3, dist2, DBL_EPSILON, 100);

            cmplx div,div2;
            COMPLEXASSIGN(div, 1.0, 0.0);
            COMPLEXASSIGN(div2, 1.0, 0.0);
            int mm=0;
            cmplx C;
            for(int t=0;t<Nroots;t++) {
              if(t!=m&&t!=duplicatenr) {
                COMPLEXSUB(ctmp, r[m], r[t]);
                COMPLEXMUL(div, div, ctmp, tmp);
                COMPLEXRECIPROCAL(ctmp3,ctmp,tmp);
                d_tmp[mm]=prodsum(dist2,ctmp3,DBL_EPSILON,100);
                COMPLEXSUB(ctmp, r[duplicatenr], r[t]);
                COMPLEXMUL(div2, div2, ctmp, tmp);
                mm++;
              }
            }
            ctmp=dist2;
            if(Nroots==2) {
              COMPLEXASSIGN(C,0,0);
            }
            else if(Nroots==3)
              C=d_tmp[0];
            else {
              //calculate prod (n from 0 to Nroots-2 (1+d_tmp[n])) -1
              d_tmp[(Nroots-2)*2-1]=d_tmp[Nroots-3];
              for(int i=Nroots-4;i>=0;i--) { //form cumulative sum
                COMPLEXADD(d_tmp[Nroots-2+i], d_tmp[i], d_tmp[Nroots-2+i+1]);
              }
              C=d_tmp[Nroots-2];
              COMPLEXASSIGN(ctmp3, 1.0, 0);
              for(int i=0;i<Nroots-3;i++) {
                COMPLEXMUL(ctmp, d_tmp[i], dist2, tmp);
                COMPLEXMUL(ctmp4, ctmp, d_tmp[Nroots-2+i+1], tmp);
                COMPLEXMUL(ctmp4, ctmp4, ctmp3, tmp);
                COMPLEXADD(C, C, ctmp4);
                ctmp.re=ctmp.re+1.0;
                COMPLEXMUL(ctmp3, ctmp3, ctmp, tmp);
              }
            } 
            cmplx rduplicate=rshifted[duplicatenr];
            cmplx rm=r[m];
            if(currentpanel->normalized) {
              COMPLEXASSIGN(rduplicate,rduplicate.re*invendval,rduplicate.im*invendval);
              COMPLEXASSIGN(rm,rm.re*invendval,rm.im*invendval);
            }
            cmplx beval=evalpoly(b2, rduplicate, b2N);
            COMPLEXMUL(ctmp3,C,dist2,tmp);
            ctmp3.re+=1.0;
            COMPLEXDIV(ctmp4,ctmp3,div,tmp,tmp2);
            COMPLEXRECIPROCAL(ctmp4,div2,tmp);
            COMPLEXMUL(ctmp,beval,C,tmp);
            COMPLEXDIV(ctmp3, rmend, r[m], tmp, tmp2);
            COMPLEXLOG(ctmp2, ctmp3);
            ctmp3=evalpolyprod(b2,rm,rduplicate,b2N);
            if(currentpanel->normalized) {
              COMPLEXASSIGN(ctmp3,ctmp3.re*invendval,ctmp3.im*invendval);
            }
            COMPLEXSUB(ctmp3,ctmp3,ctmp);
            COMPLEXMUL(ctmp, ctmp3, ctmp2, tmp);
            COMPLEXDIV(ctmp, ctmp, div, tmp, tmp2);
            COMPLEXADD(tmpvel, tmpvel, ctmp);

            COMPLEXMUL(B,B,beval,tmp);
            COMPLEXDIV(ctmp, B, div2, tmp, tmp2);
            COMPLEXADD(tmpvel, tmpvel, ctmp);


            //finally, remove these two roots from the list
            wkspi[m]=1;
            wkspi[duplicatenr]=1;
          }
        }
        else if(duplicatecount>1) { //root of higher multiplicity than 2, use numerical integration
          numint=1;
          break;
        }
      }
  #ifdef FORCENUMINT
      numint=1;
  #endif
      numintorder=Nroots;
      numintnomN=b2N;
      if(numint) { //for numerical integration, back up b2 and c, as these values may have to be reduced before the numerical integration 
        memcpy(wksp5,b2,sizeof(cmplx)*currentpanel->a_order);
        for(int m=0;m<Nroots;m++) //for numint, neglect all current duplicates as they will be handled by the numerical integration (Another solution would be to remove these from the polynomial, not yet implemented though)
          wkspi[m]=0;
      }
#ifdef ENDCORRECTION
      //if start is connected, use special equation to evaluate this with higher precision
      if(startconnected) {
        cmplx div;
        COMPLEXASSIGN(div, 1.0, 0.0);
        for(int t=0;t<Nroots;t++) {
          if(t!=startconnindex) {
            COMPLEXSUB(ctmp, r[startconnindex], r[t]);
            COMPLEXMUL(div, div, ctmp, tmp);
          }
        }
        cmplx G,rootconst;
        cmplx rshiftstartconn=rshifted[startconnindex];
        if(currentpanel->normalized) {
          COMPLEXASSIGN(rshiftstartconn,rshifted[startconnindex].re*invendval,rshifted[startconnindex].im*invendval);
        }
        ctmp=evalpoly(b2, rshiftstartconn, b2N);
        COMPLEXDIV(rootconst, ctmp, div, tmp, tmp2);
        COMPLEXMUL(ctmp4,rootconst,factor,tmp);

        COMPLEXSUB(G,ctmp4,currentpanel->connectvars.startbterm);

        cmplx deltax,H,logx;
        //rp initialized at beginning of function as it is identical to zp (without the rotation) if z1=z1a
        cmplx rporig=rp;
        COMPLEXMUL(rp,rp,currentpanel->connectvars.startslopeterm,tmp);
        COMPLEXASSIGN(ctmp2, r[startconnindex].re-endval, r[startconnindex].im);
        COMPLEXLOG(logx, r[startconnindex]);
        COMPLEXSUB(deltax, r[startconnindex], rp);
        if(COMPLEXNORM(deltax)<1e-28*currentpanel->endval) {
            COMPLEXASSIGN(H,0.0,0.0);
          }
        else if(COMPLEXNORM(deltax)*4<COMPLEXNORM(r[startconnindex])) { //check if the series is convergent
          COMPLEXDIV(ctmp, deltax, r[startconnindex], tmp, tmp2);
          H=onetermseries(ctmp, DBL_EPSILON, 100);
        }
        else {
          COMPLEXLOG(ctmp, rp);
          COMPLEXSUB(H, logx, ctmp);
        }

        COMPLEXLOG(ctmp3,ctmp2);

        //check branch cut
        COMPLEXDIV(ctmp,ctmp2,r[startconnindex],tmp,tmp2);

        double startcorrshift;
        if(fabs(rporig.im)+fabs(rporig.re)==0) //expression has to be modified when evaluation points are the same, as the 0 in rporig will affect several terms
          startcorrshift=PI2*myround(INV2PI*(atan2(ctmp.im,ctmp.re)-ctmp3.im));
        else
          startcorrshift=PI2*myround(INV2PI*(atan2(ctmp.im,ctmp.re)-ctmp3.im+atan2(rporig.im,rporig.re)+currentpanel->connectvars.startlogslopeterm.im+H.im));

        H.im-=startcorrshift;

        COMPLEXMUL(ctmp,ctmp3,ctmp4,tmp);
        if(fabs(r[startconnindex].re)+fabs(r[startconnindex].im)>DBL_EPSILON*currentpanel->deltas) { //prevend divergencies due to truncation errors, if the distance is close to zero, the term will be even closer to zero and hence can be put to zero
          COMPLEXMUL(ctmp2,logx,G,tmp);
        }
        else {
          COMPLEXASSIGN(ctmp2,0.0,0.0);
        }
        COMPLEXSUB(ctmp,ctmp,ctmp2);
        COMPLEXADD(ctmp2,H,currentpanel->connectvars.startlogslopeterm);
        COMPLEXMUL(ctmp2,ctmp2,currentpanel->connectvars.startbterm,tmp);
        COMPLEXSUB(startconnvel,ctmp,ctmp2);

        wkspi[startconnindex]=1;
        if(numint) {
          combinepolywithcheck(wksp4,rshifted,wkspi,Nroots);

          if(currentpanel->normalized) {
            if(numintorder>0) {
              double constant=currentpanel->endval;
              COMPLEXASSIGN(wksp4[1],wksp4[1].re*constant,wksp4[1].im*constant);
              for(int m=2;m<=numintorder;m++) {
                constant*=currentpanel->endval;
                COMPLEXASSIGN(wksp4[m],wksp4[m].re*constant,wksp4[m].im*constant);
              }
            }
          }
          reduceroot(wksp5,wksp4,rshiftstartconn, rootconst, numintorder,numintnomN); //wksp5 is the nominator, wksp4 the denominator, rootconst is the constant for the root
          if(currentpanel->normalized) {
            for(int m=0;m<numintnomN;m++) {
              COMPLEXASSIGN(wksp5[m],wksp5[m].re*invendval,wksp5[m].im*invendval);
            }
          }
          
          numintorder--;
          numintnomN=numintorder;
        }
      }

      //if end is connected, use special equation to evaluate this with higher precision
      if(endconnected) {
        ctmp2=disttoroot(xpositions[j],ypositions==NULL?0.0:ypositions[j],currentpanel,d_tmp,r[endconnindex],endval,endval*1e-3);
        if(currentpanel->outside!=0.0) { //if the outside parameter is set, a root with imaginary part close to zero means that it is close to the panel
          if(ctmp2.im==0)
            ctmp2.im=fabs(ctmp2.im); //remove any possible negative zeros, as r[k].im<0 compares to false, while atan2 still considers them negative
          if((ctmp2.im<0)^(currentpanel->outside>0))
            ctmp2.im=-currentpanel->outside*currentpanel->endval*1e-16;
          else if(ctmp2.im==0&&ctmp2.re==0)
            ctmp2.im=-currentpanel->outside*currentpanel->endval*1e-16;
        }

        cmplx div;
        COMPLEXASSIGN(div, 1.0, 0.0);
        for(int t=0;t<Nroots;t++) {
          if(t!=endconnindex) {
            COMPLEXSUB(ctmp, r[endconnindex], r[t]);
            COMPLEXMUL(div, div, ctmp, tmp);
          }
        }
        cmplx G,rootconst;
        cmplx rshiftendconn=rshifted[endconnindex];
        if(currentpanel->normalized) {
          COMPLEXASSIGN(rshiftendconn,rshifted[endconnindex].re*invendval,rshifted[endconnindex].im*invendval);
        }
        ctmp=evalpoly(b2, rshiftendconn, b2N);
        COMPLEXDIV(rootconst, ctmp, div, tmp, tmp2);

        COMPLEXMUL(ctmp4,rootconst,factor,tmp);

        COMPLEXSUB(G,ctmp4,currentpanel->connectvars.endbterm);
        

        cmplx rp,deltax,H,logxmds;
        //multiply by -i at the initialization to compensate for a i in currentpanel->connectvars.enddenom
        if(ypositions!=NULL) {
          COMPLEXASSIGN(rp, xpositions[j]-currentpanel->z2a.re, ypositions[j]-currentpanel->z2a.im);
        }
        else {
          COMPLEXASSIGN(rp, xpositions[j]-currentpanel->z2a.re, -currentpanel->z2a.im);
        }
        cmplx rporig=rp;
        COMPLEXMUL(rp,rp,currentpanel->connectvars.endslopeterm,tmp);

        COMPLEXLOG(logxmds, ctmp2);
        COMPLEXSUB(deltax, ctmp2, rp);
        if(COMPLEXNORM(deltax)<1e-28*currentpanel->endval) {
           COMPLEXASSIGN(H,0.0,0.0);
         }
         else if(COMPLEXNORM(deltax)*4<COMPLEXNORM(ctmp2)) { //check if the series is convergent
          COMPLEXDIV(ctmp, deltax, ctmp2, tmp, tmp2);
          H=onetermseries(ctmp, DBL_EPSILON, 100);
        }
        else {
          COMPLEXLOG(ctmp, rp);
          COMPLEXSUB(H, logxmds, ctmp);
        }
        COMPLEXLOG(ctmp3,r[endconnindex]);

        //correction due to branch cuts
        COMPLEXDIV(ctmp,ctmp2,r[endconnindex],tmp,tmp2);

        double endcorrshift;
        if(fabs(rporig.im)+fabs(rporig.re)==0) //expression has to be modified when evaluation points are the same, as the 0 in rporig will affect several terms
          endcorrshift=PI2*myround(INV2PI*(atan2(ctmp.im,ctmp.re)+ctmp3.im));
        else
          endcorrshift=PI2*myround(INV2PI*(atan2(ctmp.im,ctmp.re)+ctmp3.im-atan2(rporig.im,rporig.re)-currentpanel->connectvars.endlogslopeterm.im-H.im));
        H.im+=endcorrshift;

        COMPLEXMUL(ctmp,ctmp3,ctmp4,tmp);
        if(fabs(ctmp2.re)+fabs(ctmp2.im)>DBL_EPSILON*currentpanel->deltas) { //prevend divergencies due to truncation errors, if the distance is close to zero, the term will be even closer to zero and hence can be put to zero
          COMPLEXMUL(ctmp2,logxmds,G,tmp);
        }
        else {
          COMPLEXASSIGN(ctmp2,0.0,0.0);
        }
        COMPLEXSUB(ctmp,ctmp2,ctmp);
        COMPLEXADD(ctmp2,H,currentpanel->connectvars.endlogslopeterm);
        COMPLEXMUL(ctmp2,ctmp2,currentpanel->connectvars.endbterm,tmp);
        COMPLEXADD(endconnvel,ctmp,ctmp2);

        wkspi[endconnindex]=1;
        if(numint) {
          combinepolywithcheck(wksp4,rshifted,wkspi,Nroots);
          if(currentpanel->normalized) {
            if(numintorder>0) {
              double constant=currentpanel->endval;
              COMPLEXASSIGN(wksp4[1],wksp4[1].re*constant,wksp4[1].im*constant);
              for(int m=2;m<=numintorder;m++) {
                constant*=currentpanel->endval;
                COMPLEXASSIGN(wksp4[m],wksp4[m].re*constant,wksp4[m].im*constant);
              }
            }
          }
          reduceroot(wksp5,wksp4,rshiftendconn, rootconst, numintorder,numintnomN); //wksp5 is the nominator, wksp4 the denominator, rootconst is the constant for the root
          if(currentpanel->normalized) {
            for(int m=0;m<numintnomN;m++) {
              COMPLEXASSIGN(wksp5[m],wksp5[m].re*invendval,wksp5[m].im*invendval);
            }
          }
          
          numintorder--;
          numintnomN=numintorder;
        }
      }
#endif /*ENDCORRECTION*/
    }

    
    if(fullnumint) { //full numerical integration from the original polynomials
      polyquotient pol;
      pol.c=currentpanel->c;
      pol.b=currentpanel->b_original;
      pol.a=currentpanel->a;
      pol.cN=currentpanel->a_order+1;
      pol.bN=currentpanel->b_original_order+1;

      double tolbase=1e-11*(fabs(currentpanel->coeffs[1].re)+fabs(currentpanel->coeffs[1].im))/fmax(fabs(zp0.re)+fabs(zp0.im), (fabs(currentpanel->d.re)+fabs(currentpanel->d.im))*0.1);
      int Nsingularities=0;
      int endsingularities=0;

      double intstart, intend;
      if(currentpanel->b_original_shifted) {
        intend=0.5*endval;
        intstart=-intend;
      }
      else {
        intend=endval;
        intstart=0.0;
      }
      for(int i=0;i<currentpanel->a_order;i++) { //any root that has been removed through expansions can not produce any singularities anyway
        if(r[i].re>=0&&fabs(r[i].im)<endval*1e-4)
          wkspd[Nsingularities++]=rshifted[i].re;
        if((fabs(r[i].re)+fabs(r[i].im))<endval*1e-4)
          endsingularities=1;
        if((fabs(r[i].re-endval)+fabs(r[i].im))<endval*1e-4)
          endsingularities=1;
        
      }
      cmplx (*evalfunction)(double,void*);
      if(currentpanel->slopecorrection)
        evalfunction=evalpolyquotientslopecorrection;
      else
        evalfunction=evalpolyquotient;

      if(Nsingularities) {
        tmpvel=gausskonrod_singularities(evalfunction, intstart, intend, (void*)&pol, wkspd, Nsingularities, tolbase);
      }
      else if(endsingularities) {
        tmpvel=gausskonrod_endsingularities(evalfunction, intstart, intend, (void*)&pol, tolbase);
      }
      else {
        tmpvel=gausskonrod(evalfunction, intstart, intend, (void*)&pol, tolbase);
      }
      COMPLEXMUL(tmpvel, tmpvel, factor, tmp); //ignoring the case that this can be slightly simplified for areal, as factor still may be complex in some roots are expanded
      COMPLEXMULC(tmpvel, tmpvel, currentpanel->rotvect, tmp);
    }
    else if(numint) { //numerical integration of the reduced polynomials for specific roots
      COMPLEXASSIGN(tmpvel,0.0,0.0);
      for(int k=0;k<Nroots;k++) {
        if(wkspi[k]) //already taken care of
          continue;
        
        //find roots which are close to the real axis. These will be inaccurate for numerical integration and should be handled with normal equations
        if(fabs(r[k].im)<currentpanel->deltas*1e-4) {
          int duplicate=0;
          for(int n=k+1;n<Nroots;n++) {
            double dist=fabs(r[k].re-r[n].re)+fabs(r[k].im-r[n].im);
            if(dist<DUPLICATEROOTTOL*maxnorm)
              duplicate=1;
          }
          if(!duplicate) { //unless it is a duplicate root, than it has to be handled with numerical integration

            //use the traditional solution for this root
            cmplx div;
            COMPLEXASSIGN(div, 1.0, 0.0);
            for(int t=0;t<Nroots;t++) {
              if(t!=k) {
                COMPLEXSUB(ctmp, r[k], r[t]);
                COMPLEXMUL(div, div, ctmp, tmp);
              }
            }
            cmplx rshiftk=rshifted[k];
            if(currentpanel->normalized) {
              COMPLEXASSIGN(rshiftk,rshiftk.re*invendval,rshiftk.im*invendval);
            }
            ctmp=evalpoly(b2, rshiftk, b2N);

            ctmp2=disttoroot(xpositions[j],ypositions==NULL?0.0:ypositions[j],currentpanel,d_tmp,r[k],endval,endval*1e-3);
            COMPLEXDIV(ctmp3, ctmp2, r[k], tmp, tmp2);
            COMPLEXLOG(ctmp2, ctmp3);
            
            COMPLEXDIV(ctmp4, ctmp, div, tmp, tmp2);
            
            COMPLEXMUL(ctmp, ctmp4, ctmp2, tmp);
            COMPLEXADD(tmpvel, tmpvel, ctmp);
            wkspi[k]=1;
            //create new polynomial with this root excluded, is this necessary??? looks like it may be performed twice
            combinepolywithcheck(wksp4, rshifted, wkspi, Nroots);
            if(currentpanel->normalized) {
              if(numintorder>0) {
                double constant=currentpanel->endval;
                COMPLEXASSIGN(wksp4[1],wksp4[1].re*constant,wksp4[1].im*constant);
                for(int m=2;m<=numintorder;m++) {
                  constant*=currentpanel->endval;
                  COMPLEXASSIGN(wksp4[m],wksp4[m].re*constant,wksp4[m].im*constant);
                }
              }
            }
            reduceroot(wksp5, wksp4, rshiftk, ctmp4, numintorder, numintnomN); //wksp5 is the nominator, wksp4 the denominator, ctmp4 is the constant for the root
            if(currentpanel->normalized) {
              for(int m=0;m<numintnomN;m++) {
                COMPLEXASSIGN(wksp5[m],wksp5[m].re*invendval,wksp5[m].im*invendval);
              }
            }
            numintorder--;
            numintnomN=numintorder;
          }
        }
      }
      if(numintorder>0) {
        combinepolywithcheck(wksp4, rshifted, wkspi, Nroots);
        //possible alternative way of calculating wksp4, but it fails for large expanded roots
        //combinepolywithinversecheck(tmpwksp, rshifted, wkspi, currentpanel->a_order);
        //polydivnowksp(wksp4, wksp1, currentpanel->c, tmpwksp, currentpanel->a_order+1, currentpanel->a_order+1-numintorder);

        polyquotient pol;
        pol.c=wksp4;
        pol.b=wksp5;
        pol.cN=numintorder+1;
        pol.bN=numintnomN;
        pol.endval=currentpanel->endval;
        double tolbase=1e-11*(fabs(currentpanel->coeffs[1].re)+fabs(currentpanel->coeffs[1].im))/fmax(fabs(zp0.re)+fabs(zp0.im), (fabs(currentpanel->d.re)+fabs(currentpanel->d.im))*0.1);
        int Nsingularities=0;
        int endsingularities=0;
        double intstart,intend;
        if(currentpanel->shifted) {
          intend=0.5*endval;
          intstart=-intend;
        }
        else {
          intend=endval;
          intstart=0.0;
        }
        for(int i=0;i<Nroots;i++) { //any root that has been removed through expansions can not produce any singularities anyway
          if(wkspi[i])
            continue;
          //find singularities and choose apropriate method for integration
          if(r[i].re>=0&&fabs(r[i].im)<endval*1e-4)
            wkspd[Nsingularities++]=rshifted[i].re;
          if((fabs(r[i].re)+fabs(r[i].im))<endval*1e-4)
            endsingularities=1;
          if((fabs(r[i].re-endval)+fabs(r[i].im))<endval*1e-4)
            endsingularities=1;
        }
        cmplx (*evalfunction)(double,void*);
        if(currentpanel->normalized)
          evalfunction=evalpolyquotientnormalized;
        else
          evalfunction=evalpolyquotient;
        if(Nsingularities) {
          ctmp=gausskonrod_singularities(evalfunction, intstart, intend, (void*)&pol, wkspd, Nsingularities, tolbase);
        }
        else if(endsingularities) {
          ctmp=gausskonrod_endsingularities(evalfunction, intstart, intend, (void*)&pol, tolbase);
        }
        else {
          ctmp=gausskonrod(evalfunction, intstart, intend, (void*)&pol, tolbase);
        }
        COMPLEXADD(tmpvel, tmpvel, ctmp);
      }
      //handle all terms outside the quotient analytically, as these should not cause any additional problems
      if(currentpanel->normalized) {
        if(currentpanel->shifted)
          ctmp=intpolysymmetric(d,0.5,dN);
        else
          ctmp=intpoly(d, 1, dN);
      }
      else {
        if(currentpanel->shifted)
          ctmp=intpolysymmetric(d,0.5*endval,dN);
        else
          ctmp=intpoly(d, endval, dN);
      }
      COMPLEXADD(tmpvel, tmpvel, ctmp);
      COMPLEXMUL(tmpvel, tmpvel, factor, tmp); //ignoring the case that this can be slightly simplified for areal, as factor still may be complex in some roots are expanded
#ifdef ENDCORRECTION
      if(startconnected) {
        COMPLEXADD(tmpvel,tmpvel,startconnvel);
      }
      if(endconnected) {
        COMPLEXADD(tmpvel,tmpvel,endconnvel);
      }
#endif
      COMPLEXMULC(tmpvel, tmpvel, currentpanel->rotvect, tmp);
    }
    else {
      //the normal solution for all roots that does not fall into any special case
      for(int k=0;k<Nroots;k++) {
        if(wkspi[k]) //already taken care of
          continue;
        cmplx div;
        //set up the constant
        COMPLEXASSIGN(div, 1.0, 0.0);
        for(int t=0;t<Nroots;t++) {
          if(t!=k) {
            COMPLEXSUB(ctmp, r[k], r[t]);
            COMPLEXMUL(div, div, ctmp, tmp);
          }
        }
        //evaluate nominator at the root, TODO: clean up this code
        cmplx rshiftk=rshifted[k];
        if(currentpanel->normalized) {
          COMPLEXASSIGN(rshiftk,rshiftk.re*invendval,rshiftk.im*invendval);
        }
        ctmp=evalpoly(b2, rshiftk, b2N);
        ctmp2=disttoroot(xpositions[j],ypositions==NULL?0.0:ypositions[j],currentpanel,d_tmp,r[k],endval,endval*1e-3);
        COMPLEXASSIGN(ctmp2, r[k].re-endval, r[k].im);
        COMPLEXDIV(ctmp3, ctmp2, r[k], tmp, tmp2);
        COMPLEXLOG(ctmp2, ctmp3);
        
        COMPLEXMUL(ctmp, ctmp, ctmp2, tmp);
        COMPLEXDIV(ctmp, ctmp, div, tmp, tmp2);
        COMPLEXADD(tmpvel, tmpvel, ctmp);
      }

      //the term outside the quotient
      if(currentpanel->normalized) {
        if(currentpanel->shifted)
          ctmp=intpolysymmetric(d,0.5,dN); //shifted should never be identical and hence endval is always equal to currentpanel->endval
        else
          ctmp=intpoly(d, endval/currentpanel->endval, dN);
      }
      else {
        if(currentpanel->shifted)
          ctmp=intpolysymmetric(d,0.5*endval,dN);
        else
          ctmp=intpoly(d, endval, dN);
      }
      COMPLEXADD(tmpvel, tmpvel, ctmp);
      COMPLEXMUL(tmpvel, tmpvel, factor, tmp); //ignoring the case that this can be slightly simplified for areal, as factor still may be complex in some roots are expanded
      //add possible contributions form start and endconnected calculations
#ifdef ENDCORRECTION
      if(startconnected) {
        COMPLEXADD(tmpvel,tmpvel,startconnvel);
      }
      if(endconnected) {
        COMPLEXADD(tmpvel,tmpvel,endconnvel);
      }
#endif
      //rotate solution to original coordinate system
      COMPLEXMULC(tmpvel, tmpvel, currentpanel->rotvect, tmp);
    }
    
    xvelocities[j]+=tmpvel.im;
    yvelocities[j]-=tmpvel.re;
  }
}

//function to calculate the distance to a root. If root is close to end point, it will shift the polynomial and do another root calculation for higher accuracy
cmplx disttoroot(double xposition,double yposition,generalpanel *restrict currentpanel, COMPLEXVECTORRESTRICT wksp,cmplx root,double val,double cutoff,int paneltopanelmode) {
  cmplx ctmp,ctmp2;
  double tmp;
  //test if the simple formula works
  if(fabs(root.re-val)+fabs(root.im)>cutoff||val!=currentpanel->endval) { //if val!=currentpanel->endval, z2a will not be the correct value. Besides, this should not happen since the only time they are not equal are if the other end is connected
    COMPLEXASSIGN(ctmp,root.re-val,root.im);
    return ctmp;
  }
  memcpy(wksp,currentpanel->c,sizeof(cmplx)*(currentpanel->a_order+1));

  //shift the polynomial
  for (int k = 1; k <= currentpanel->a_order; k++) {
    for(int j = currentpanel->a_order-k; j < currentpanel->a_order; j++) {
      COMPLEXADDC(wksp[j], wksp[j], wksp[j+1], val);
    }
  }
  COMPLEXASSIGN(ctmp2, root.re-val, root.im);
  
  //now, recalculate the term that depends on the panel position for higher precision in the solution
  if(paneltopanelmode) { //the recalculation is slightly different depending on if it is panel to particle or panel to panel evaluation
    COMPLEXASSIGN(ctmp, currentpanel->z2alocal.im-yposition, xposition-currentpanel->z2alocal.re);
    COMPLEXMUL(wksp[0], ctmp, currentpanel->invaend, tmp);
  }
  else {
    COMPLEXASSIGN(ctmp, xposition-currentpanel->z2a.re, yposition-currentpanel->z2a.im);
    COMPLEXMULCI(wksp[0], ctmp, currentpanel->rotvect, tmp);
    COMPLEXMUL(wksp[0], wksp[0], currentpanel->invaend, tmp);
  }
  
  //iterate the root ones again for higher precision
  refineroot(&ctmp2, wksp, currentpanel->a_order);
  return ctmp2;
}

//evaluate the velocity from a multipole expansion. Much faster than the normal generalpaneldirectinteract
void multipoleevaluation(double *restrict xvelocities, double *restrict yvelocities, cmplx z, COMPLEXVECTORRESTRICT coeffs, int Ncoeff, double tol,double value) {
  double tmp;
  cmplx tmpvel, power, val;
  int i;
  COMPLEXRECIPROCAL(val, z, tmp);
  COMPLEXMUL(tmpvel, coeffs[1], val, tmp);
  
  COMPLEXMUL(power, val, val, tmp);
  COMPLEXASSIGN(tmpvel, tmpvel.re+power.re*coeffs[2].re-power.im*coeffs[2].im, tmpvel.im+power.re*coeffs[2].im+power.im*coeffs[2].re);
  double valuesqr=value*value; //This is only used to test for convergence. input value is squared already, therefore the first steps are outside the loop
  for(i=3;i<Ncoeff-3;i++) {
    COMPLEXMUL(power, power, val, tmp);
    COMPLEXASSIGN(tmpvel, tmpvel.re+power.re*coeffs[i].re-power.im*coeffs[i].im, tmpvel.im+power.re*coeffs[i].im+power.im*coeffs[i].re);
    i++;
    
    COMPLEXMUL(power, power, val, tmp);
    COMPLEXASSIGN(tmpvel, tmpvel.re+power.re*coeffs[i].re-power.im*coeffs[i].im, tmpvel.im+power.re*coeffs[i].im+power.im*coeffs[i].re);
    i++;
    
    COMPLEXMUL(power, power, val, tmp);
    COMPLEXASSIGN(tmpvel, tmpvel.re+power.re*coeffs[i].re-power.im*coeffs[i].im, tmpvel.im+power.re*coeffs[i].im+power.im*coeffs[i].re);
    i++;
    
    COMPLEXMUL(power, power, val, tmp);
    COMPLEXASSIGN(tmpvel, tmpvel.re+power.re*coeffs[i].re-power.im*coeffs[i].im, tmpvel.im+power.re*coeffs[i].im+power.im*coeffs[i].re);
    
    value*=valuesqr;
    if(value<tol) { //convergence check every fourth iteration
        xvelocities[0]+=tmpvel.re;
        yvelocities[0]+=tmpvel.im;
        return;
    }
  }
  for(;i<=Ncoeff;i++) {
    COMPLEXMUL(power, power, val, tmp);
    COMPLEXASSIGN(tmpvel, tmpvel.re+power.re*coeffs[i].re-power.im*coeffs[i].im, tmpvel.im+power.re*coeffs[i].im+power.im*coeffs[i].re);
  }

  xvelocities[0]+=tmpvel.re;
  yvelocities[0]+=tmpvel.im;

}


//Calculates the interaction of a particle on the integral through (and along) the panel. The code applies Gauss law to reduce the panel to a line before doing the calculations
//note: cutofforig is the original value for cuttoff radius. cutoff, shape and scalre are the values returned by MPexp_smooth_simplified. In principle, cuttoff=cutofforig*cutofforig, but if other smoothers are added, this may not always be the case, that is why the distiction is used
void particletogeneralpanel(double *restrict xvelocities,double *restrict yvelocities,generalpanel *restrict currentpanel,
                            const double *restrict xpositions,const double *restrict ypositions,const double *restrict xstrengths,const double *restrict ystrengths,
                            SMOOTHER smooth,double cutofforig,double cutoff,double shape,double scale,int N) {

  //length of d and wksp should be atleast (aN-1)*MAXROOTEXPANSION+bN, length of r,b2 and wksp2 should be atleast max(aN,bN)
  cmplx zp,zp0,ctmp2,ctmp3,rp;
  double tmp;
  cmplx tmpvel,strength,position;
  int fullnumint;
  double cutoffcheck=(currentpanel->dabs+cutofforig)*(currentpanel->dabs+cutofforig);

  for(int j=0;j<N;j++) {
    COMPLEXASSIGN(strength,xstrengths[j], ystrengths==NULL?0.0:ystrengths[j]);

    COMPLEXASSIGN(position,xpositions[j],ypositions==NULL?0.0:ypositions[j]);
    COMPLEXSUB(zp0, position,currentpanel->z);
    tmp=COMPLEXNORM(zp0);

    //check if numerical integration is necessary. It is only used if panel is within the smoothing radius of the particle
    fullnumint=0;
    if(smooth!=DIRAC) { //only necessary if smoother is used, as that is when analytical expression can fail
      if(tmp<cutoffcheck) { //first check, is it close enough so that it is possible that numint has to be used
        if(sqrdistancefrompanel(currentpanel, position, 0)<cutofforig*cutofforig) //the real check if the distance to the panel is smaller than cutoff radius
          fullnumint=1;
      }
    }

    if(!fullnumint) { //normal interaction
      COMPLEXSUB(rp, position,currentpanel->z1);
      COMPLEXMULC(zp, rp, currentpanel->targetrotvect, tmp);
      COMPLEXASSIGN(tmpvel,0.0,0.0);
      COMPLEXRECIPROCALN(ctmp3,zp,tmp);
      COMPLEXASSIGN(ctmp3,ctmp3.re*currentpanel->targetendval,ctmp3.im*currentpanel->targetendval);
      ctmp2=complexlogm1(ctmp3);
      int above=(int)(ctmp2.im>=0);
      COMPLEXMUL(ctmp2, ctmp2, strength, tmp);
      COMPLEXADD(tmpvel, tmpvel, ctmp2);

      //correction in case the vortex changes side of the panel when Gauss law is applied to reduce it to a line
      if(currentpanel->endval!=currentpanel->targetendval) {
         COMPLEXMULC(zp, rp, currentpanel->rotvect, tmp);
      }
      if(currentpanel->areal&&zp.re>=0&&zp.re<=currentpanel->endval) {
        double yval=evalpolyreal(currentpanel->a,zp.re,currentpanel->a_order+1);
        double refval;
        if(currentpanel->endval==currentpanel->targetendval)
          refval=0;
        else {
          refval=zp.re*currentpanel->z2alocal.im/currentpanel->z2alocal.re;
        }
        if(currentpanel->outside!=0.0) { //correction to the side if outside prameter is set
          if(fabs(zp.im-yval)<OUTSIDETHRESHOLD*currentpanel->endval) {
            if(yval==zp.im||(yval>=zp.im)^(currentpanel->outside>0))
              yval=zp.im+currentpanel->outside*currentpanel->endval; //does not matter how much higher it gets, since yval is not used for other things than comparisions
          }
        }
        if((zp.im<yval)^(!above)) { //correction if it is inside. Note that the sign depends on if the panel encloses the point above or below the reference line
          if(above) {
            COMPLEXASSIGN(tmpvel,tmpvel.re+PI2*strength.im,tmpvel.im-PI2*strength.re);
          }
          else {
            COMPLEXASSIGN(tmpvel,tmpvel.re-PI2*strength.im,tmpvel.im+PI2*strength.re);
          }
        }
      }
      
      
    }
    else { //full numerical integration from the original polynomials

      COMPLEXSUB(rp, position,currentpanel->z1);
      
      COMPLEXMULC(zp, rp, currentpanel->rotvect, tmp);
      COMPLEXASSIGN(currentpanel->c[0],-zp.im,zp.re);

      polyquotient pol;
      pol.c=currentpanel->c;
      pol.b=currentpanel->b;
      pol.cN=currentpanel->a_order+1;
      pol.bN=currentpanel->b_order+1;
      pol.cutoff=cutoff;
      pol.shape=shape;
      pol.scale=scale;

      double tolbase=1e-11/fmax(fabs(zp0.re)+fabs(zp0.im), (fabs(currentpanel->d.re)+fabs(currentpanel->d.im))*0.1);
      int Nsingularities=0;
      int endsingularities=0;

      double intstart, intend;
      intend=currentpanel->endval;
      intstart=0.0;

      //singularities should not exist here, as the whole point of smoothers is to remove them, therefore skip this part of the code that otherwise exist for the panel to particle case

      cmplx (*evalfunction)(double,void*);

      //choose proper evaluation function depending on smoother. The dirac case will not happen, but is included here as default for functionality if numint is manually enabled
      evalfunction=evalpolyquotient;
      if(smooth==RANKINE)
        evalfunction=evalpolyquotientrankine;
      else if(smooth==SCULLY)
        evalfunction=evalpolyquotientscully;
      else if(smooth==OSEEN)
        evalfunction=evalpolyquotientoseen;
      tmpvel=gausskonrod(evalfunction, intstart, intend, (void*)&pol, tolbase);
      COMPLEXMULN(tmpvel,tmpvel,strength,tmp);
    }

    xvelocities[currentpanel->targetindex]-=tmpvel.im;
    yvelocities[currentpanel->targetindex]-=tmpvel.re;
  }
}

//Calculates the interaction of a particle on the integral through (and along) the panel. The code applies Gauss law to reduce the panel to a line before doing the calculations
cmplx farexpansiontogeneralpanel(generalpanel *restrict currentpanel, generalpanel *restrict sourcepanel,double dist) {

  //length of d and wksp should be atleast (aN-1)*MAXROOTEXPANSION+bN, length of r,b2 and wksp2 should be atleast max(aN,bN)
  cmplx zp,ctmp,ctmp2,ctmp3,z1,z2,invz1,invz2,invz1base,invz2base;
  double tmp;

  COMPLEXSUB(z1, sourcepanel->z,currentpanel->z1);
  COMPLEXSUB(z2, sourcepanel->z,currentpanel->z2a);

  COMPLEXMULC(zp, z1, currentpanel->rotvect, tmp);

  if(currentpanel->endval!=currentpanel->targetendval) {
    COMPLEXMULC(zp, z1, currentpanel->targetrotvect, tmp);
  }

  //note: the first term integrates to a logarithm, the rest to 1 over polynomials
  COMPLEXRECIPROCALN(ctmp3,zp,tmp);
  COMPLEXASSIGN(ctmp3,ctmp3.re*currentpanel->targetendval,ctmp3.im*currentpanel->targetendval);
  ctmp2=complexlogm1(ctmp3);

  COMPLEXMUL(ctmp2, ctmp2, sourcepanel->coeffs[1], tmp);


  double convdistbase=sourcepanel->dabs/(dist-currentpanel->dabs);
  double convdist=convdistbase*convdistbase;

  COMPLEXRECIPROCALN(invz1,z1,tmp);
  invz1base=invz1;
  COMPLEXRECIPROCALN(invz2,z2,tmp);
  invz2base=invz2;
  COMPLEXSUB(ctmp,invz2,invz1);

  //manually take first step outside the loop, as this one is simpler
  COMPLEXMUL(ctmp,ctmp,sourcepanel->coeffs[2],tmp);
  COMPLEXSUB(ctmp2,ctmp2,ctmp);
 
  //loop through the rest of the series coefficients
  for(int i=2;i<sourcepanel->Ncoeffs&&convdist>1e-16;i++,convdist*=convdistbase) {
    COMPLEXMUL(invz1,invz1,invz1base,tmp);
    COMPLEXMUL(invz2,invz2,invz2base,tmp);
    COMPLEXSUB(ctmp,invz2,invz1);
    COMPLEXMUL(ctmp,ctmp,sourcepanel->coeffs[i+1],tmp);
    tmp=1.0/(double)i;
    COMPLEXSUBC(ctmp2,ctmp2,ctmp,tmp);
  }
  return ctmp2;
}

//The interaction of a panel to another panel
//This code does not support complex valued a-vectors, as it cannot handle the intersections between the panels in this case
//This first function mainly handles if panels are connected or not, and the special cases requried here. The actual solution
//to the panel to panel interaction is in generalpaneltogeneralpanelinner
void generalpaneltogeneralpanel(double *restrict xvelocities,double *restrict yvelocities,generalpanel *restrict currentpanel,
                            generalpanel *restrict sourcepanel,int N) {
  cmplx tmpvel,ctmp;
  double tmp;
  COMPLEXVECTOR slopeeq=wksp1;

  COMPLEXASSIGN(tmpvel,0.0,0.0);
  for(int i=0;i<N;i++) {
    double oldoutside=currentpanel->outside;
    currentpanel->outside=-sourcepanel[i].outside;
    COMPLEXSUB(ctmp, sourcepanel[i].z,currentpanel->z);
    tmp=COMPLEXNORM(ctmp);
    //start by checking if it can be evaluated through the series expansions or not
    //note that the cutoff check below requires that TARGETPANELCONVERGENCERADIUS>=2
    //convert to double in case someone enters a value as an integer, which may overflow in the multiplication
    double tmp2=(double)GENERALPANELCONVERGENCERADIUS*sourcepanel[i].dabs+currentpanel->dabs; //this distance ensures that the series are convergent
    if(tmp>tmp2*tmp2) { //for large enough distances, it is possible to make a local expansion of the vortex instead. Note that the distance for doing this is much larger here, than for source panels, as the cancellation errors are much smaller for target panels, and the evaluation of target panels is much faster
      ctmp=farexpansiontogeneralpanel(currentpanel,sourcepanel+i,sqrt(tmp));
      COMPLEXASSIGN(tmpvel,tmpvel.re-ctmp.re,tmpvel.im-ctmp.im);
      continue;
    }


    cmplx evalpanelstart,evalpanelend,evalrotvect;
    COMPLEXSUB(evalpanelstart,currentpanel->z1,sourcepanel[i].z1);
    COMPLEXMULC(evalpanelstart,evalpanelstart,sourcepanel[i].rotvect,tmp);
    COMPLEXSUB(evalpanelend,currentpanel->z2a,sourcepanel[i].z1);
    COMPLEXMULC(evalpanelend,evalpanelend,sourcepanel[i].rotvect,tmp);
    COMPLEXMULC(evalrotvect,currentpanel->rotvect,sourcepanel[i].rotvect,tmp);

    double evalpanelendval=evalpolyreal(currentpanel->a,currentpanel->deltas,currentpanel->a_order+1);
    
    //the case that the end point connects to the evaluation panel, mirror the panel to remove the divergence at the end point
    if(fabs(sourcepanel[i].z2alocal.re-evalpanelend.re)+fabs(sourcepanel[i].z2alocal.re-evalpanelend.re)<1e-8*sourcepanel[i].endval||fabs(sourcepanel[i].z2alocal.re-evalpanelstart.re)+fabs(sourcepanel[i].z2alocal.re-evalpanelstart.re)<1e-8*sourcepanel[i].endval) {
      int doubleconnected=0;
      //if connected in both ends, make 2 subpanels with haft distance in each and evaluate these two instead.
      if(fabs(evalpanelend.re)+fabs(evalpanelend.re)<1e-8*sourcepanel[i].endval||fabs(evalpanelstart.re)+fabs(evalpanelstart.re)<1e-8*sourcepanel[i].endval) {
        doubleconnected=1;
        generalpanel tmppanel2=sourcepanel[i]; //make copy of sourcepanel;
        tmppanel2.endval*=0.5;
        COMPLEXASSIGN(tmppanel2.z2alocal,tmppanel2.endval,evalpolyreal(tmppanel2.a,tmppanel2.endval,tmppanel2.a_order+1));
        ctmp=generalpaneltogeneralpanelinner(currentpanel,&tmppanel2,evalpanelstart,evalpanelend,evalrotvect,0.5); //evaluate first half directly
        
        COMPLEXADD(tmpvel,tmpvel,ctmp);
      }
      generalpanel tmppanel=sourcepanel[i]; //make copy of sourcepanel;
      tmppanel.a=wksp3big;
      tmppanel.c=wksp3big+sourcepanel[i].a_order+1;
      tmppanel.b_unshifted=wksp2big+sourcepanel[i].a_order+1;
      COMPLEXASSIGN(tmppanel.rotvect,-sourcepanel[i].rotvect.re,-sourcepanel[i].rotvect.im); //panel direction changes sign
      //create a mirrored version of the a vector, as it should move in the opposite direction
      for(int m=0;m<=sourcepanel[i].a_order;m++) {
        COMPLEXASSIGN(tmppanel.a[m],-sourcepanel[i].a[m].re,-sourcepanel[i].a[m].im); //note the extra minus sign as the direction of the axis perpendicular to the panel changes sign as well for a
        m++;
        if(m<=sourcepanel[i].a_order)
          COMPLEXASSIGN(tmppanel.a[m],sourcepanel[i].a[m].re,sourcepanel[i].a[m].im);
      }
      for (int k = 1; k <= sourcepanel[i].a_order; k++) {
        for(int j = sourcepanel[i].a_order-k; j < sourcepanel[i].a_order; j++) {
          COMPLEXSUBC(tmppanel.a[j], tmppanel.a[j], tmppanel.a[j+1], sourcepanel[i].endval);
        }
      }
      //mirror b vector as well
      for(int m=0;m<=sourcepanel[i].b_order;m++) {
        COMPLEXASSIGN(tmppanel.b_unshifted[m],sourcepanel[i].b_unshifted[m].re,sourcepanel[i].b_unshifted[m].im);
        m++;
        if(m<=sourcepanel[i].b_order)
          COMPLEXASSIGN(tmppanel.b_unshifted[m],-sourcepanel[i].b_unshifted[m].re,-sourcepanel[i].b_unshifted[m].im);
      }
      if (sourcepanel[i].normalized) {
        for (int k = 1; k <= sourcepanel[i].b_order; k++) {
          for(int j = sourcepanel[i].b_order-k; j < sourcepanel[i].b_order; j++) {
            COMPLEXSUB(tmppanel.b_unshifted[j], tmppanel.b_unshifted[j], tmppanel.b_unshifted[j+1]);
          }
        }
      }
      else {
        for (int k = 1; k <= sourcepanel[i].b_order; k++) {
          for(int j = sourcepanel[i].b_order-k; j < sourcepanel[i].b_order; j++) {
            COMPLEXSUBC(tmppanel.b_unshifted[j], tmppanel.b_unshifted[j], tmppanel.b_unshifted[j+1], sourcepanel[i].endval);
          }
        }
      }
      COMPLEXASSIGN(tmppanel.z1,sourcepanel[i].z2.re,sourcepanel[i].z2.im);
      COMPLEXASSIGN(tmppanel.z2,sourcepanel[i].z1.re,sourcepanel[i].z1.im);
      tmppanel.outside=-sourcepanel[i].outside;
      if(tmppanel.a[0].re!=0.0) {
        COMPLEXASSIGN(ctmp,-tmppanel.a[0].re*tmppanel.rotvect.im,tmppanel.a[0].re*tmppanel.rotvect.re);
        COMPLEXADD(tmppanel.z1,ctmp,tmppanel.z1); //move z1 and z2 to force z1 to be the start of the panel.
        COMPLEXADD(tmppanel.z2,ctmp,tmppanel.z2);
        COMPLEXASSIGN(tmppanel.a[0],0.0,0.0);
      }
      COMPLEXASSIGN(tmppanel.z2alocal,tmppanel.endval,evalpolyreal(tmppanel.a,tmppanel.endval,tmppanel.a_order+1));
      
      ctmp=evalpanelstart;
      COMPLEXSUB(evalpanelstart,currentpanel->z1,tmppanel.z1);
      COMPLEXMULC(evalpanelstart,evalpanelstart,tmppanel.rotvect,tmp);
      COMPLEXSUB(evalpanelend,currentpanel->z2a,tmppanel.z1);
      COMPLEXMULC(evalpanelend,evalpanelend,tmppanel.rotvect,tmp);
      COMPLEXMULC(evalrotvect,currentpanel->rotvect,tmppanel.rotvect,tmp);
      
      double endscale=1.0;
      if(doubleconnected) {
        tmppanel.endval*=0.5;
        COMPLEXASSIGN(tmppanel.z2alocal,tmppanel.endval,evalpolyreal(tmppanel.a,tmppanel.endval,tmppanel.a_order+1));
        endscale=0.5;
      }
        
      
      initgeneralpanelcvector(&tmppanel);
      ctmp=generalpaneltogeneralpanelinner(currentpanel,&tmppanel,evalpanelstart,evalpanelend,evalrotvect,endscale); //evaluate mirrored panel
    }
    else { //no connections, directly evaluate the original panel
      ctmp=generalpaneltogeneralpanelinner(currentpanel,sourcepanel+i,evalpanelstart,evalpanelend,evalrotvect,1.0);
    }
    COMPLEXADD(tmpvel,tmpvel,ctmp);
    currentpanel->outside=oldoutside;
  }
  xvelocities[currentpanel->targetindex]-=tmpvel.im;
  yvelocities[currentpanel->targetindex]-=tmpvel.re;
}

//internal function to carry out the interaction between two panels. 
cmplx generalpaneltogeneralpanelinner(generalpanel *restrict currentpanel,
                            generalpanel *restrict sourcepanel,cmplx evalpanelstart,cmplx evalpanelend,cmplx evalrotvect, double endvalscale) {
  cmplx tmpvel,ctmp,ctmp2,ctmp3;
  double tmp,tmp2;
  COMPLEXVECTOR r=wksp2;
  COMPLEXVECTOR r2=wksp3;
  COMPLEXASSIGN(tmpvel,0,0);

  //The equation has one contribution from each end point, and one combined logarithm, and also the branch cut correction

  //the first term from the starting point
  ctmp=addpaneltopanelendpointcontribution(currentpanel,sourcepanel,evalpanelstart,wksp1big,r2,wksp4,wksp5,endvalscale,1.0);
  COMPLEXADD(tmpvel,tmpvel,ctmp);

  //the second term from the end point
  ctmp=addpaneltopanelendpointcontribution(currentpanel,sourcepanel,evalpanelend,wksp2big,r2,wksp4,wksp5,endvalscale,-1.0);
  COMPLEXADD(tmpvel,tmpvel,ctmp);
  
  //Unless any root has been reduced (not used), this could be simplified through z1 and z2a (if this is to be attempted, ensure that half size panels corrects z2a, see generalpaneltogeneralpanel)
  ctmp=evalpoly(wksp1big,sourcepanel->endval,sourcepanel->a_order+1);
  ctmp2=evalpoly(wksp2big,sourcepanel->endval,sourcepanel->a_order+1);
  COMPLEXDIV(ctmp3,ctmp2,ctmp,tmp,tmp2);
  
  //now handle the main logarithm of the polynomials
  COMPLEXLOG(ctmp2,ctmp3);
  int above=(int)(ctmp2.im>=0);
  double endval=sourcepanel->endval;
  tmp2=endval;
  if(sourcepanel->normalized)
    endval=endvalscale;

  for(int k=0;k<=sourcepanel->b_order;k++) {
    COMPLEXMUL(ctmp,sourcepanel->b_unshifted[k],ctmp2,tmp);
    tmp=tmp2/(double)(k+1);
    COMPLEXADDC(tmpvel,tmpvel,ctmp,tmp);
    tmp2*=endval;
  }

  cmplx sourcepanelend;
  COMPLEXSUB(sourcepanelend,sourcepanel->z2alocal,evalpanelstart);
  
  COMPLEXMULC(sourcepanelend,sourcepanelend,evalrotvect,tmp);

  //check if the end point ends up inside the evaluation panel, i.e. if a correction has to be added to make gauss law valid
  if(sourcepanelend.re>=0&&sourcepanelend.re<=currentpanel->endval) {
    tmp=evalpolyreal(currentpanel->a,currentpanel->endval,currentpanel->a_order+1);
    double slope=tmp/currentpanel->endval;
    tmp=evalpolyreal(currentpanel->a,sourcepanelend.re,currentpanel->a_order+1);

    if(currentpanel->outside!=0.0) { //the correction if the outside parameter is set
      if(fabs(sourcepanelend.im-tmp)<OUTSIDETHRESHOLD*currentpanel->endval) {
        if(tmp==sourcepanelend.im||(tmp>=sourcepanelend.im)^(currentpanel->outside>0))
          tmp=sourcepanelend.im+currentpanel->outside*currentpanel->endval; //does not matter how much higher it gets, since yval is not used for other things than comparisions
      }
    }
    //if panel ends inside the enclosed area
    
    //first perform a proximity sign check. If the end point is very close to the straight line between the points,
    //it should have given a very small imaginary part for the logarithm. Since the sign of this is saved in "above",
    //even if the condition says that it should be inside the region, ensure that this sign correction matches with the
    //sign of the logarithm to avoid the branch cut correction due to truncation errors
    int closeproximity=0;
    int closeproximityshift=0;
    tmp2=sourcepanelend.re*slope;
    if(fabs(sourcepanelend.im-tmp2)<1e-10*sourcepanel->endval) {
      
      closeproximity=1;
      if(tmp!=tmp2&&(tmp<tmp2)^above)
        closeproximityshift=1;
    }
    
    //if the correction should be added
    if((!closeproximity&&(sourcepanelend.im<tmp)^(sourcepanelend.im<tmp2))||(closeproximity&&closeproximityshift)) { //if closeproximity, use that value, otherwise use standard test
      double tmp3=PI2*sourcepanel->endval;
      if(!above) //depending on if the evaluation panel is above or below the line between the end points
        tmp3=-tmp3;
      
      double endval=sourcepanel->endval;
      if(sourcepanel->normalized)
        endval=endvalscale;

      for(int k=0;k<=sourcepanel->b_order;k++) {
        tmp=tmp3/(double)(k+1);
        COMPLEXASSIGN(tmpvel,tmpvel.re+tmp*sourcepanel->b_unshifted[k].im,tmpvel.im-tmp*sourcepanel->b_unshifted[k].re);
        tmp3*=endval;
      }
    }
  }
  return tmpvel;
}

//this code handles the contribution from the individual roots for each term in the panel to panel interaction
cmplx addpaneltopanelendpointcontribution(generalpanel *restrict currentpanel,generalpanel *restrict sourcepanel,cmplx evalpanel,COMPLEXVECTORRESTRICT cvector,COMPLEXVECTORRESTRICT r,COMPLEXVECTORRESTRICT wksp_1,COMPLEXVECTORRESTRICT wksp_2,double endvalscale,double addsign) {
  double tmp,tmp2;
  cmplx ctmp,ctmp2,ctmp3,tmpvel,tmpval;
  COMPLEXASSIGN(tmpvel,0.0,0.0);
  memcpy(cvector+1,sourcepanel->c+1,sourcepanel->a_order*sizeof(cmplx));

  //set up the c vector, (check if it is necessary to copy it to new vector or if original vector could be used)
  if(sourcepanel->areal&&sourcepanel->a_order!=1) {
    COMPLEXASSIGN(cvector[0], -evalpanel.im*sourcepanel->invaend.re, evalpanel.re*sourcepanel->invaend.re);
  }
  else {
    COMPLEXASSIGN(cvector[0],-evalpanel.im,evalpanel.re);
    COMPLEXMUL(cvector[0], cvector[0], sourcepanel->invaend, tmp);
  }
  
  polyroots(r, cvector, sourcepanel->a_order, wksp_1,wksp_2);
  double invendvalscale;
  if(sourcepanel->normalized)
    invendvalscale=1/endvalscale;
  if(sourcepanel->outside!=0.0) { //if outside parameter is set, apply correction
    for(int k=0;k<sourcepanel->a_order;k++) {
      double shift=sourcepanel->endval*OUTSIDETHRESHOLD;
      if(fabs(r[k].im)<shift&&r[k].re>-shift&&r[k].re<sourcepanel->endval+shift){ //root is on the panel
        if(r[k].im==0)
          r[k].im=fabs(r[k].im); //remove any possible negative zeros, as r[k].im<0 compares to false, while atan2 still considers them negative
        if((r[k].im<0)^(sourcepanel->outside>0))
          r[k].im=-sourcepanel->outside*fmin(fabs(r[k].re),sourcepanel->endval)*1e-16;
      }
    }
  }
  //loop through the roots
  for(int j=0;j<sourcepanel->a_order;j++) {
    double expansionconst=10;
    if(sourcepanel->shifted) //for slope expansion, where 2 is set as the required convergence of the series
      expansionconst=2;
    if(fabs(r[j].re)+fabs(r[j].im)>expansionconst*sourcepanel->endval) { //large root, evaluate through series expansions instead
      COMPLEXRECIPROCAL(ctmp,r[j],tmp);
      COMPLEXASSIGN(ctmp,ctmp.re*sourcepanel->endval,ctmp.im*sourcepanel->endval);
      ctmp2=ctmp;
      double expvarbase=COMPLEXABS(ctmp); //used for convergence
      double expvar=1;
      
      COMPLEXASSIGN(tmpval,0.0,0.0);
      for(int k=1;expvar>1e-16;expvar*=expvarbase,k++) {
        COMPLEXASSIGN(ctmp3,0.0,0.0);
        double lambda=sourcepanel->endval;
        for(int m=0;m<=sourcepanel->b_order;m++) {
          tmp=lambda/(double)((m+1)*(m+1+k));
          COMPLEXADDC(ctmp3,ctmp3,sourcepanel->b_unshifted[m],tmp);
          if(sourcepanel->normalized)
            lambda*=endvalscale;
          else
            lambda*=sourcepanel->endval;
        }
        COMPLEXMUL(ctmp3,ctmp3,ctmp2,tmp);
        COMPLEXADD(tmpval,tmpval,ctmp3);
        COMPLEXMUL(ctmp2,ctmp2,ctmp,tmp);
      }
      //if anyone wants to overoptimize this, try xor on the sign bit of ctmp2 to change its sign with a constant that differs depending on addsign
      COMPLEXASSIGN(tmpval,tmpval.re*addsign,tmpval.im*addsign);
      COMPLEXSUB(tmpvel,tmpvel,tmpval);
    }
    else { //normal sized roots
      COMPLEXASSIGN(wksp_1[0],1.0,0.0);
      double invendval;
      if(sourcepanel->normalized) {
        invendval=endvalscale/sourcepanel->endval;
        cmplx rj;
        COMPLEXASSIGN(rj,r[j].re*invendval,r[j].im*invendval);
        wksp_1[1]=rj;
        //build table of r powers
        for(int k=2;k<=sourcepanel->b_order+1;k++) { //this is the part that causes problems for the large roots, therefore the series expansion above
          COMPLEXMUL(wksp_1[k],wksp_1[k-1],rj,tmp);
        }
      }
      else {
        
        wksp_1[1]=r[j];
        //build table of r powers
        for(int k=2;k<=sourcepanel->b_order+1;k++) { //this is the part that causes problems for the large roots, therefore the series expansion above
          COMPLEXMUL(wksp_1[k],wksp_1[k-1],r[j],tmp);
        }
      }
      cmplx tmpval;
      COMPLEXASSIGN(tmpval,0.0,0.0);
      double tmpconstant=1.0;
      if(sourcepanel->normalized)
        tmpconstant=sourcepanel->endval*invendvalscale;
      for(int k=0;k<=sourcepanel->b_order;k++) {
        tmp=tmpconstant/(k+1);
        COMPLEXASSIGN(ctmp,sourcepanel->b_unshifted[k].re*tmp,sourcepanel->b_unshifted[k].im*tmp);
        COMPLEXMADDN(tmpval,tmpval,ctmp,wksp_1[k+1],tmp);
      }

      COMPLEXVECTOR oldc=sourcepanel->c;
      sourcepanel->c=cvector; //disttoroot uses sourcepanel->c
      ctmp2=disttoroot(evalpanel.re,evalpanel.im,sourcepanel,wksp_2,r[j],sourcepanel->endval,sourcepanel->endval*1e-3,1);
      sourcepanel->c=oldc;
      if(fabs(r[j].re)+fabs(r[j].im)<1e-15*sourcepanel->endval) {
        COMPLEXASSIGN(ctmp2,0.0,0.0); //will be cancelled by multiplication by zero later in this case, which goes to zero much faster than the logarithm goes to infinity
      }
      else {
        COMPLEXDIV(ctmp3, ctmp2, r[j], tmp, tmp2);
        COMPLEXLOG(ctmp2, ctmp3);
      }
      COMPLEXMUL(ctmp2,ctmp2,tmpval,tmp);

      //if anyone wants to overoptimize this, try xor on the sign bit of ctmp2 to change its sign with a constant that differs depending on addsign
      COMPLEXASSIGN(ctmp2,ctmp2.re*addsign,ctmp2.im*addsign);

      COMPLEXSUB(tmpvel,tmpvel,ctmp2);

      tmp2=1;


      /*for k=1:length(b)
      startval=0;
      for t=k:length(b)
      startval=startval+1/(k*t)*b(t)*r(n)^(t-k);
      end
      V(m)=V(m)+startval*deltas^k;
      end*/
      if(sourcepanel->normalized) {
        double cumval=1;
        for(int k=0;k<=sourcepanel->b_order;k++) {
          COMPLEXASSIGN(tmpval,0.0,0.0);
          for(int t=k;t<=sourcepanel->b_order;t++) {
            //startval=startval+1/(k*t)*b(t)*r(j)^(k-p);
            COMPLEXMUL(ctmp,sourcepanel->b_unshifted[t],wksp_1[t-k],tmp);
            COMPLEXASSIGN(ctmp,ctmp.re*cumval,ctmp.im*cumval);
            tmp=1.0/(double)((k+1)*(t+1));
            COMPLEXADDC(tmpval,tmpval,ctmp,tmp);
          }
          cumval*=invendval;

          tmp2*=sourcepanel->endval;
          tmp=tmp2*addsign;
          COMPLEXADDC(tmpvel,tmpvel,tmpval,tmp);
        }

      }
      else {
        for(int k=0;k<=sourcepanel->b_order;k++) {
          COMPLEXASSIGN(tmpval,0.0,0.0);
          for(int t=k;t<=sourcepanel->b_order;t++) {
            //startval=startval+1/(k*t)*b(t)*r(j)^(k-p);
            COMPLEXMUL(ctmp,sourcepanel->b_unshifted[t],wksp_1[t-k],tmp);
            tmp=1.0/(double)((k+1)*(t+1));
            COMPLEXADDC(tmpval,tmpval,ctmp,tmp);
          }
          tmp2*=sourcepanel->endval;
          tmp=tmp2*addsign;
          COMPLEXADDC(tmpvel,tmpvel,tmpval,tmp);
        }
      }
    }
  }
  return tmpvel;
}

//free all internal allocated variables connected to a general panel
void freegeneralpanel(generalpanel* out) {
  if(out->a!=NULL)
    myFree(out->a);
  if(out->b!=out->b_unshifted&&out->b_unshifted!=out->b_original&&out->b_unshifted!=NULL)
    myFree(out->b_unshifted);
  if(out->b!=out->b_original&&out->b_original!=NULL)
    myFree(out->b_original);
  if(out->b!=NULL)
    myFree(out->b);
  if(out->c!=NULL)
    myFree(out->c);
  if(out->extremepoints!=NULL)
    myFree(out->extremepoints);
  if(out->extremepointval!=NULL)
    myFree(out->extremepointval);
  if(out->coeffs!=NULL)
    myFree(out->coeffs);
}

//debug function
void printgeneralpanel(generalpanel *in) {
  mexPrintf("start position:  %e+%ei, end position: %e+%ei\n",in->z1.re,in->z1.im,in->z2.re,in->z2.im);
//  mexPrintf("actual position: %e+%ei, end position: %e+%ei\n",in->z1a.re,in->z1a.im,in->z2a.re,in->z2a.im);
  mexPrintf("actual end position: %e+%ei\n",in->z2a.re,in->z2a.im);
  mexPrintf("deltas=%e endval=%e\n",in->deltas,in->endval);
  mexPrintf("z=%e+%ei d=%e+%ei dabs=%e, box=(%e - %e,%e - %e)\n",in->z.re,in->z.im,in->d.re,in->d.im,in->dabs,in->z.re-in->d.re,in->z.re+in->d.re,in->z.im-in->d.im,in->z.im+in->d.im);
  mexPrintf("a coefficients: ");
  for(int i=0;i<=in->a_order;i++)
    mexPrintf("%e+%ei ",in->a[i].re,in->a[i].im);
  mexPrintf("\ninvaend=%e+%ei\nc coefficients: ", in->invaend.re,in->invaend.im);
  for(int i=0;i<=in->a_order;i++)
    mexPrintf("%e+%ei ",in->c[i].re,in->c[i].im);
//  mexPrintf("\npaneldist=%e+%ei deltas=%e theta=%e\n",in->paneldist.re,in->paneldist.im,in->deltas,in->theta);
  mexPrintf("\npaneldist=%e+%ei deltas=%e\n",in->paneldist.re,in->paneldist.im,in->deltas);
  mexPrintf("rotvect=%e+%ei\n",in->rotvect.re,in->rotvect.im);
  mexPrintf("a_order=%d areal=%d b_order=%d\nbcoefficients: ",in->a_order,in->areal,in->b_order);
  for(int i=0;i<=in->b_order;i++)
    mexPrintf("%e+%ei ",in->b[i].re,in->b[i].im);
  mexPrintf("\nz=%e+%ei d=%e+%ei dabssqr=%e\nextremepoints (%d): ",in->z.re,in->z.im,in->d.re,in->d.im,in->dabssqr,in->Nextremepoints);
  for(int i=0;i<in->Nextremepoints;i++)
    mexPrintf("%e+%ei (%e)",in->extremepoints[i].re,in->extremepoints[i].im,in->extremepointval[i]);
  mexPrintf("\n outside=%f\ncoeffs (%d):\n",in->outside,in->Ncoeffs);
  for(int i=0;i<in->Ncoeffs;i++)
    mexPrintf("%e+%ei ",in->coeffs[i].re,in->coeffs[i].im);
  mexPrintf("\n");
}

void checkpanelintegrity(panellist *panels) {
  for(int i=0;i<panels->count;i++) {
    //mexPrintf("sourcepanel=%e+%ei to %e+%ei\n",sourcepanels->panels[i].z1.re,sourcepanels->panels[i].z1.im,sourcepanels->panels[i].z2a.re,sourcepanels->panels[i].z2a.im);
    if(panels->panels[i].prev!=NULL) {
      if(fabs(panels->panels[i].z1.re-panels->panels[i].prev->z2a.re)+fabs(panels->panels[i].z1.im-panels->panels[i].prev->z2a.im)>1e-12) {
        mexPrintf("Prev pointer failed,i=%d panel=%e+%ei to %e+%ei prev=%e+%ei to %e+%ei\n",i,panels->panels[i].z1.re,panels->panels[i].z1.im,panels->panels[i].z2a.re,panels->panels[i].z2a.im,panels->panels[i].prev->z1.re,panels->panels[i].prev->z1.im,panels->panels[i].prev->z2a.re,panels->panels[i].prev->z2a.im);
      }
    }
    if(panels->panels[i].next!=NULL) {
      if(fabs(panels->panels[i].z2a.re-panels->panels[i].next->z1.re)+fabs(panels->panels[i].z2a.im-panels->panels[i].next->z1.im)>1e-12) {
        //mexPrintf("next: %e+%ei to %e+%ei\n",sourcepanels->panels[i].next->z1.re,sourcepanels->panels[i].next->z1.im,sourcepanels->panels[i].next->z2a.re,sourcepanels->panels[i].next->z2a.im);
        mexPrintf("Next pointer failed,i=%d panel=%e+%ei to %e+%ei prev=%e+%ei to %e+%ei\n",i,panels->panels[i].z1.re,panels->panels[i].z1.im,panels->panels[i].z2a.re,panels->panels[i].z2a.im,panels->panels[i].next->z1.re,panels->panels[i].next->z1.im,panels->panels[i].next->z2a.re,panels->panels[i].next->z2a.im);
      }
    }
    cmplx r;
    COMPLEXSUB(r,panels->panels[i].z2,panels->panels[i].z1);
    if(fabs(COMPLEXABS(r)-panels->panels[i].endval)>1e-12)
      mexPrintf("endval incorrect i=%d, value is %e, should be %e (z1=%e+%ei z2=%e+%ei)\n",i,panels->panels[i].endval,COMPLEXABS(r),panels->panels[i].z1.re,panels->panels[i].z1.im,panels->panels[i].z2.re,panels->panels[i].z2.im);
  }
}

#ifdef GENERALPANELSUPPORT
//functions only for fmm implementation
void MPexp_box_generalpanel(MPexp *This,const panellist *panels, int* validinput)
//the panel version of MPexp_box_
//This is the box, panels is the panel array, and panelcount its length
{
  for(int i=0;i<panels->count;i++) {
    if(panels->panels[i].z.re+panels->panels[i].d.re>This->xmax)
      This->xmax=panels->panels[i].z.re+panels->panels[i].d.re;
    if(panels->panels[i].z.re-panels->panels[i].d.re<This->xmin)
      This->xmin=panels->panels[i].z.re-panels->panels[i].d.re;
    if(panels->panels[i].z.im+panels->panels[i].d.im>This->ymax)
      This->ymax=panels->panels[i].z.im+panels->panels[i].d.im;
    if(panels->panels[i].z.im-panels->panels[i].d.im<This->ymin)
      This->ymin=panels->panels[i].z.im-panels->panels[i].d.im;

#ifdef CHECKNANINPUT
    if(!isfinite(panels->panels[i].z.re)||!isfinite(panels->panels[i].d.re)||!isfinite(panels->panels[i].z.im)||!isfinite(panels->panels[i].d.im))
        *validinput=0;
#endif
  }
}

void createdummylistgeneral(double** dummy,int *dummyallocated,const panel* panels,const int* indices,int N,
                            panellist *sourcepanels,const int *sourceindices,int Nsource,
                            panellist *targetpanels,const int *targetindices,int Ntarget,
                            int xval,dcmplx z,dcmplx d)
/*creates list of dummy vortices to be used by MPexp_partition_dummy*/
{
  panel smallpanel;
  //should never run out of size for the traditional panels
  if(xval) {
    for(int i=0;i<N;i++) {
      panelinbox(&smallpanel, panels+indices[i], z, d);
      (*dummy)[2*i]=smallpanel.x1;
      (*dummy)[2*i+1]=smallpanel.x2;
    }
    int j=N;
    for(int i=0;i<Nsource;i++,j++) {
      if(2*j>=*dummyallocated-1) {
        *dummyallocated=(int)(*dummyallocated*1.2)+10;
        *dummy=(double*)mxRealloc(*dummy,*dummyallocated*sizeof(double));
      }
      (*dummy)[2*j]=sourcepanels->panels[sourceindices[i]].z1.re;
      (*dummy)[2*j+1]=sourcepanels->panels[sourceindices[i]].z2a.re;
    }
    for(int i=0;i<Ntarget;i++,j++) {
      if(2*j>=*dummyallocated-1) {
        *dummyallocated=(int)(*dummyallocated*1.2)+10;
        *dummy=(double*)mxRealloc(*dummy,*dummyallocated*sizeof(double));
      }
      (*dummy)[2*j]=targetpanels->panels[targetindices[i]].z1.re;
      (*dummy)[2*j+1]=targetpanels->panels[targetindices[i]].z2a.re;
    }
  }
  else {
    for(int i=0;i<N;i++) {
      panelinbox(&smallpanel, panels+indices[i], z, d);
      (*dummy)[2*i]=smallpanel.y1;
      (*dummy)[2*i+1]=smallpanel.y2;
    }
    int j=N;
    for(int i=0;i<Nsource;i++,j++) {
      if(2*j>=*dummyallocated-1) {
        *dummyallocated=(int)(*dummyallocated*1.2)+10;
        *dummy=(double*)mxRealloc(*dummy,*dummyallocated*sizeof(double));
      }
      (*dummy)[2*j]=sourcepanels->panels[sourceindices[i]].z1.im;
      (*dummy)[2*j+1]=sourcepanels->panels[sourceindices[i]].z2a.im;
    }
    for(int i=0;i<Ntarget;i++,j++) {
      if(2*j>=*dummyallocated-1) {
        *dummyallocated=(int)(*dummyallocated*1.2)+10;
        *dummy=(double*)mxRealloc(*dummy,*dummyallocated*sizeof(double));
      }
      (*dummy)[2*j]=targetpanels->panels[targetindices[i]].z1.im;
      (*dummy)[2*j+1]=targetpanels->panels[targetindices[i]].z2a.im;
    }
  }
}

void evalgeneralpanelexpansionfmm(double *xvelocity, double *yvelocity, generalpanel *restrict currentpanel,dcmplx z,dcmplx *restrict coeffs,int pcoeff) {
  cmplx dist1, dist2, distpower1, distpower2, ctmp, tmpvel;
  double tmp;
  COMPLEXASSIGN(dist1,currentpanel->z1.re-creal(z),currentpanel->z1.im-cimag(z));
  COMPLEXASSIGN(dist2,currentpanel->z2a.re-creal(z),currentpanel->z2a.im-cimag(z));
  distpower1=dist1;
  distpower2=dist2;
  COMPLEXSUB(ctmp, distpower2, distpower1);
  COMPLEXASSIGN(tmpvel,ctmp.re*creal(coeffs[0])-ctmp.im*cimag(coeffs[0]),ctmp.im*creal(coeffs[0])+ctmp.re*cimag(coeffs[0]));
  //COMPLEXMUL(tmpvel, ctmp, currentpanel->coeffs[0], tmp);
  for(int j=1;j<=pcoeff;j++) {
    COMPLEXMUL(distpower1, distpower1, dist1, tmp); //(z1-z)^(j+1)
    COMPLEXMUL(distpower2, distpower2, dist2, tmp); //(z2a-z)^(j+1)
    tmp=1.0/(j+1.0);
    COMPLEXASSIGN(ctmp, tmp*(distpower2.re-distpower1.re), tmp*(distpower2.im-distpower1.im)); //(z2a-z)^(j+1)-(z1-z)^(j+1))/(j+1)
    tmp=ctmp.re*creal(coeffs[j])-ctmp.im*cimag(coeffs[j]);
    ctmp.im=ctmp.im*creal(coeffs[j])+ctmp.re*cimag(coeffs[j]);
    ctmp.re=tmp;
    //COMPLEXMUL(ctmp, ctmp, currentpanel->coeffs[j], tmp);
    COMPLEXADD(tmpvel, tmpvel, ctmp);
  }
  xvelocity[currentpanel->targetindex]+=tmpvel.im;      //add the final multiplication by 1i
  yvelocity[currentpanel->targetindex]+=tmpvel.re;
}

//Calculates the interaction of a particle on the integral through (and along) the panel. The code applies Gauss law to reduce the panel to a line before doing the calculations
void farexpansiontogeneralpanelfmm(double *restrict xvelocity,double *restrict yvelocity,generalpanel *restrict currentpanel, dcmplx z,dcmplx *coeffs,int pcoeff) {

  //length of d and wksp should be atleast (aN-1)*MAXROOTEXPANSION+bN, length of r,b2 and wksp2 should be atleast max(aN,bN)
  cmplx zp,ctmp,ctmp2,ctmp3,z1,z2,invz1,invz2,invz1base,invz2base;
  double tmp,tmp2;

  //note that the cutoff check below requires that TARGETPANELCONVERGENCERADIUS>=2
  //convert to double in case someone enters a value as an integer, which may overflow in the multiplication

  COMPLEXASSIGN(z1, creal(z)-currentpanel->z1.re,cimag(z)-currentpanel->z1.im);
  COMPLEXASSIGN(z2, creal(z)-currentpanel->z2a.re,cimag(z)-currentpanel->z2a.im);

  COMPLEXMULC(zp, z1, currentpanel->rotvect, tmp);

  if(currentpanel->endval!=currentpanel->targetendval) {
    COMPLEXMULC(zp, z1, currentpanel->targetrotvect, tmp);
  }

  //note: the first term integrates to a logarithm, the rest to 1 over polynomials
  COMPLEXRECIPROCALN(ctmp3,zp,tmp);
  COMPLEXASSIGN(ctmp3,ctmp3.re*currentpanel->targetendval,ctmp3.im*currentpanel->targetendval);
  ctmp2=complexlogm1(ctmp3);

  tmp=ctmp2.re*creal(coeffs[1])-ctmp2.im*cimag(coeffs[1]);
  ctmp2.im=ctmp2.re*cimag(coeffs[1])+ctmp2.im*creal(coeffs[1]);
  ctmp2.re=tmp;
  //COMPLEXMUL(ctmp2, ctmp2, sourcepanel->coeffs[1], tmp);


  COMPLEXRECIPROCALN(invz1,z1,tmp);
  invz1base=invz1;
  COMPLEXRECIPROCALN(invz2,z2,tmp);
  invz2base=invz2;
  COMPLEXSUB(ctmp,invz2,invz1);

  //manually take first step outside the loop, as this one is simpler
  if(pcoeff>=2) {
    tmp=ctmp.re*creal(coeffs[2])-ctmp.im*cimag(coeffs[2]);
    ctmp.im=ctmp.re*cimag(coeffs[2])+ctmp.im*creal(coeffs[2]);
    ctmp.re=tmp;
  //COMPLEXMUL(ctmp,ctmp,sourcepanel->coeffs[2],tmp);

    COMPLEXSUB(ctmp2,ctmp2,ctmp);
  }
 
  //loop through the rest of the series coefficients
  for(int i=2;i<pcoeff;i++) {
    COMPLEXMUL(invz1,invz1,invz1base,tmp);
    COMPLEXMUL(invz2,invz2,invz2base,tmp);
    COMPLEXSUB(ctmp,invz2,invz1);
    tmp=ctmp.re*creal(coeffs[i+1])-ctmp.im*cimag(coeffs[i+1]);
    ctmp.im=ctmp.re*cimag(coeffs[i+1])+ctmp.im*creal(coeffs[i+1]);
    ctmp.re=tmp;
    //COMPLEXMUL(ctmp,ctmp,sourcepanel->coeffs[i+1],tmp);
    tmp=1.0/(double)i;
    COMPLEXSUBC(ctmp2,ctmp2,ctmp,tmp);

  }
  xvelocity[currentpanel->targetindex]+=ctmp2.im;      //add the final multiplication by 1i
  yvelocity[currentpanel->targetindex]+=ctmp2.re;
}
#endif