/* generalpanel.h */ 
/* A. Goude 2013-08-07 */
#if defined(GENERALPANELSUPPORT) && ! defined(__fmm_h)
#include "../fmm.h"
#endif

#ifndef __GENERALPANEL_H
#define __GENERALPANEL_H
#include "mex.h"
#include <math.h>
#include "stdlib.h" //for VS compatibility, restrict has to be defined here
#ifndef restrict
#ifdef __cplusplus
#define restrict __restrict
#endif
#endif

#ifndef OUTSIDETHRESHOLD
#define OUTSIDETHRESHOLD 1e-12
#endif

//#define myMalloc(size) myMalloc2(size,__FILE__,__LINE__)
//#define myCalloc(size,size2) myCalloc2(size,size2,__FILE__,__LINE__)
//#define myFree(size) myFree2(size,__FILE__,__LINE__)

//if fmm.h is not included already, define SMOOTHER
#ifndef __fmm_h
typedef enum {SCULLY = 0,RANKINE = 1,OSEEN = 2,DIRAC = 3,
              ERR_SMOOTHER = -1} SMOOTHER;
#endif

//declare some missing functions in visual studio
#if defined(_MSC_VER) && !defined(__fmm_h)
#ifndef copysign
#define copysign _copysign
#define isnan(x) _isnan(x)
inline double fmax(double x, double y) {
  return x>y?x:y;
}
inline double fmin(double x, double y) {
  return x<y?x:y;
}
#endif

#ifndef hypot
#define hypot _hypot
#endif
#endif /*defined(_MSC_VER) && !defined(__fmm_h)*/

//general structure for complex variables
typedef struct {
  double re,im;
} cmplx;

//debug mode runs all internal vector operations through the class myvector instead of through the normal c vectors, 
//which will print an error message in the case of out of bounds access
//#define DEBUGMODE
#define ENDCORRECTION

#ifdef DEBUGMODE
#include <string.h>
#include <stdio.h>
#define COMPLEXVECTOR myvector<cmplx>
#define DOUBLEVECTOR myvector<double>
#define INTVECTOR myvector<int>
#define COMPLEXVECTORRESTRICT myvector<cmplx>
#define DOUBLEVECTORRESTRICT myvector<double>
#define INTVECTORRESTRICT myvector<int>

//define the vector class that is used to check all vector operations
template <class T>
class myvector {
  T dummy;
  T *vec;
  int count;
  int deallocated;
public:
  myvector() {
    vec=NULL;
    count=0;
  }
  myvector(int N) {
    vec=(T*)myMalloc(N*sizeof(T));
    count=N;
    deallocated=0;
  }
  myvector(T* v,int N) {
    vec=v;
    count=N;
    deallocated=0;
  }
  void alloc(int N) {
    vec=(T*)(void*)myMalloc(N*sizeof(T));
    count=N;
    deallocated=0;
  }
  void free() {
    if(vec!=NULL) {
      myFree(vec);
      vec=NULL;
      deallocated=1;
    }
    else if(deallocated) {
      mexErrMsgTxt("Double free detected\n");
    }
    else {
      mexWarnMsgTxt("Free called on non-allocated vector\n");
    }
    count=0;
  }
  T& operator[](int i) {
    if(i>=0&&i<count) { //check that all operations are within the limits of the vector
      return vec[i];
    }
    char message[1000];
    sprintf(message,"Array out of bounds, attempting to access element %d of array at position %p with length %d\n",i,vec,count);
    mexErrMsgTxt(message); //note: if this is uncommented, the code below will not be reached
    mexPrintf("Array out of bounds, attempting to access element %d of array at position %p with length %d\n",i,vec,count);
    memset(&dummy,0,sizeof(T));
    return dummy;
  }
  myvector<T> operator+(int i) { //emulate the operation that a vector + a constant gives a new vector that starts at a different position
    myvector<T> ret=*this;
    ret.vec+=i;
    ret.count-=i;
    return ret;
  }
  myvector<T>& operator+=(int i) { //emulate that the position of the vector can be increased
    vec+=i;
    count-=i;
    return *this;
  }
  bool operator==(myvector<T> &in) { //comparison that two vectors are equal is performed on the vector pointer
    return in.vec==this->vec;
  }
  bool operator!=(myvector<T> &in) {
    return in.vec!=this->vec;
  }
  T* getptr(int i) {
    return &vec[i];
  }
  void setptr(T* v, int N) {
    vec=v;
    count=N;
    deallocated=0;
  }
  int getcount() {
    return count;
  }
  void operator=(void * in) { //to allow the vector to be set to NULL. 
    if(in!=NULL) {
      char message[1000];
      sprintf(message, "Cannot assign myvector a pointer value (%p), use setptr function instead\n", in);
      mexErrMsgTxt(message);
    }
    else if(vec!=NULL) {
      char message[1000];
      sprintf(message, "myvector at position %p is set to NULL without first deallocating the vector\n", vec);
      mexWarnMsgTxt(message);
    }
    else { //no error should be given if this vector is deallocated as it would be a valid operation (a warning is printed though)
      deallocated=0;
    }
  }
  bool operator==(void* in) { //allow comparison against normal vectors
    return in==(void*)vec;
  }
  bool operator!=(void* in) {
    return in!=(void*)vec;
  }
};

//overload the memcpy function to work with myvectors, and include a check that the copy is within the limits in both cases
template <class T>
inline void memcpy(myvector<T> dest,myvector<T> source, int count) {
  int newcount=count/sizeof(T);
  if(newcount>=0&&newcount<=source.getcount()&&newcount<=dest.getcount())
    memcpy(dest.getptr(0),source.getptr(0),count);
  else {
    char message[1000];
    sprintf(message,"memcpy Array out of bounds, attempting to copy %d elements between vector with %d elements at position %p and vector with %d elements at position %p\n",newcount,source.getcount(),source.getptr(0),dest.getcount(),dest.getptr(0));
    mexErrMsgTxt(message);
  }
}

//overload memset to work with myvector as well
template <class T>
inline void memset(myvector<T> dest,int value, int count) {
  int newcount=count/sizeof(T);
  if(newcount>=0&&newcount<=dest.getcount())
    memset(dest.getptr(0),value,count);
  else {
    char message[1000];
    sprintf(message,"memset Array out of bounds, attempting to set %d elements of vector with %d elements at position %p\n",newcount,dest.getcount(),dest.getptr(0));
    mexErrMsgTxt(message);
  }
}

//overload mxRealloc to work on myvector
template <class T>
inline myvector<T> mxRealloc(myvector<T> vec, mwSize size) {
  int newcount=size/sizeof(T);
  vec.setptr((T*)mxRealloc(vec.getptr(0),size),newcount);
  return vec;
}

//overload myFree, which works on both myvectors and normal vectors
template <class T>
inline void myFree(myvector<T> &vec) {
  vec.free();
}
inline void myFree(void* vec) {
  mxFree(vec);
}
//some abusing of operator overloading to enable myMalloc and myCalloc to work both the normal way and with myvector
class myMalloc {
private:
    void* ptr;
    mwSize size;
public:
    myMalloc(mwSize Size) {
      size=Size;
      ptr=mxMalloc(size);
    }
    template <class T>
    operator T*() {
      return (T*)ptr;
    }
    template <class T>
    operator myvector<T>() {
      myvector<T> myvec;
      myvec.setptr((T*)ptr,size/sizeof(T));
      return myvec;
    }
};
class myCalloc {
private:
    void* ptr;
    mwSize size;
public:
    myCalloc(mwSize n, mwSize Size) {
      size=Size*n;
      ptr=mxCalloc(n,Size);
    }
//    operator void*() {
//      return ptr;
//    }
    template <class T>
    operator myvector<T>() {
      myvector<T> myvec;
      myvec.setptr((T*)ptr,size/sizeof(T));
      return myvec;
    }
    template <class T>
    operator T*() {
      return (T*)ptr;
    }
};  
#else
#define COMPLEXVECTOR cmplx* 
#define DOUBLEVECTOR double* 
#define INTVECTOR int* 
#define COMPLEXVECTORRESTRICT cmplx *restrict
#define DOUBLEVECTORRESTRICT double *restrict  
#define INTVECTORRESTRICT int *restrict
#define myMalloc(size) mxMalloc(size)
#define myCalloc(size,size2) mxCalloc(size,size2)
#define myFree(size) mxFree(size)
#endif

//structure used as parameter for numerical intergration
typedef struct {
  COMPLEXVECTOR a;        //(optional) coefficients for slope correction
  COMPLEXVECTOR c;        //coefficients in denominator
  COMPLEXVECTOR b;        //coefficients in nominator
  int cN;                 //number of coefficients in denominator
  int bN;                 //number of coefficients in nominator
  double cutoff;
  double shape;
  double scale;
  double endval;
} polyquotient;

//constants used for analytical solution of cubic polynomials
typedef struct {
  cmplx D0,D1P1,D0c,DT1,DT2;
} cubicconstants;

//the terms used to determine if panels are connected
typedef struct {
  cmplx startslope;       //the direction of the panel at the start position
  cmplx endslope;         //the direction of the panel at the end position
  cmplx startslopelocal;  //the direction of the panel at the start position with respect to the panel direction
  cmplx endslopelocal;    //the direction of the panel at the end position with respect to the panel direction
  cmplx startb,endb;      //the value for b at panel start and end positions
  cmplx startbterm;       //the value for the b term corrected with the direction of the panel at the start
  cmplx endbterm;         //the value for the b term corrected with the direction of the panel at the end
  cmplx startlogslopeterm;//the value of log(exp(-1i*theta)/(1i*startslope))
  cmplx endlogslopeterm;  //the value of log(exp(-1i*theta)/(1i*endslope))
  cmplx startslopeterm;   //the value of exp(-1i*theta)/(1i*startslope)
  cmplx endslopeterm;     //the value of exp(-1i*theta)/(1i*endslope)
  double startdist;       //the cutoff distance for the panel expansion to be used for the start position.
  double enddist;         //the cutoff distance for the panel expansion to be used for the end position.
} connectingterms;

//in the current implementation, generalpanel is used for both source and target panels. For a pure c++ implementation, this could have been done with inheritance instead
typedef struct generalpanel {
  cmplx z1,z2;            //start and end positions of panel as used by the panel equation
  cmplx /*z1a,*/z2a;      //the actual start and end positions of the panel as evaluated from the panel equation (note: z1a will be equal to z1 due to variable transformations if startval==0)
  cmplx z2alocal;         //z2a in the local frame of reference
  COMPLEXVECTOR a;        //coefficients for the panel shape
  cmplx invaend;          //inverse of highest coefficient in a
  COMPLEXVECTOR c;        //vector used for evaluating the shape, same order as a
  cmplx paneldist;        //vector between z1 and z2
  double deltas;          //distance between z1 and z2
  double endval;          //end of integral
  //double theta;         //angle between z1 and z2
  cmplx rotvect;          //exp(1i*theta)
  int a_order;            //highest order term in the a vector
  int areal;              //does a only contain real values?
  COMPLEXVECTOR b;        //coefficients for the panel strength
  int b_order;            //highest order term in the b vector
  cmplx z;                //midpoint of the box that contains the entire panel
  cmplx d;                //distance from the center of the box to the edges
  double dabs;            //largest distance from center of box to the panel
  double dabssqr;         //square of dabs, commonly used
  COMPLEXVECTOR extremepoints; //points where the panel edge possibly can be
  DOUBLEVECTOR extremepointval;//values that generates the extremepoints
  int Nextremepoints;     //number of extremepoints
  COMPLEXVECTOR coeffs;   //multipolecoefficients of the panel around z
  int Ncoeffs;            //number of multipole coefficients
  double outside;         //which side to evaluate on, only applies for points on the panel itself. Outside is defined using the counter-clockwise direction
  cubicconstants cub;     //for cubic panels, some parts of the solution can be precomputed
  generalpanel *prev;     //in case of connected panels, the pointer to the previous panel, use NULL in the case of unconnected panels. Connected panels assumes that the direction of the panel and the strength of the panel both are continous at the connecting point
  generalpanel *next;     //in case of connected panels, the pointer to the next panel
  int previdentical;      //in the case of connected panels, if the prevoius panel is identical to the current
  int nextidentical;      //in the case of connected panels, if the next panel is identical to the current
  connectingterms connectvars;//some variables that can be reused for each connection calculation 
  int shifted;            //panel equations should be centered around the midpoint instead. Used for large numbers of b coefficients.
  int normalized;         //if b vector is normalized (and have the length 1 instead), useful for very long b vectors that otherwise overflow
  int slopecorrection;    //slope correction is applied for this panel
  COMPLEXVECTOR b_original;//in case of slope expansion, the original value for the b vector, used for numerical integration only
  COMPLEXVECTOR b_unshifted;//in case of slope expansion, the b vector for the unshifted case, only used for evaluation at target panels
  int b_original_order;   //order of b_original;
  int b_original_shifted; //if b_original is shifted
  int numint;             //always evaluate this panel through numerical integration
  int target;             //if it is a target panel instead of a source panel
  int targetindex;        //the position in the output array to write output to for a target panel
  double targetendval;    //distance between z1 and z2a
  cmplx targetrotvect;    //exp(1i*theta_target) where theta_target is the angle between z1 and z2a
} generalpanel;

//encapsulate the panel array in a struct that also keeps the count
typedef struct panellist {
  generalpanel *panels;
  int count;
  int allocated;
  int aN; //used for fmm reading of panels
  int bN; //used for fmm reading of panels
  int slopecorrection; //used for fmm reading of panels
  int normalized; //used for fmm reading of panels
  int shifted; //used for fmm reading of panels
} panellist;

//macros for complex arithmetics
#define COMPLEXASSIGN(x, y, z) {(x).re=(y);(x).im=(z);}                              //x=(y,z)
#define COMPLEXADD(x, y, z) {((x).re=(y).re+(z).re);((x).im=(y).im+(z).im);}         //x=y+z
#define COMPLEXADDXY(x, y, z, w) {((x).re=(y).re+(z));((x).im=(y).im+(w));}              //x=y+z+1i*w, z and w real
#define COMPLEXSUB(x, y, z) {((x).re=(y).re-(z).re);((x).im=(y).im-(z).im);}         //x=y-z
#define COMPLEXADDC(x, y, z, c) {((x).re=(y).re+(c)*(z).re);((x).im=(y).im+(c)*(z).im);} //x=y+c*z, c real
#define COMPLEXSUBC(x, y, z, c) {((x).re=(y).re-(c)*(z).re);((x).im=(y).im-(c)*(z).im);} //x=y-c*z, c real
//#define COMPLEXABS(x) hypot((x).re, (x).im)                                        //abs(x)
#define COMPLEXABS(x) sqrt((x).re*(x).re+ (x).im*(x).im)                                        //abs(x)
#define COMPLEXNORM(x) ((x).re*(x).re+(x).im*(x).im)                               //x*conj(x)
#define COMPLEXANGLE(x) atan2((x).im,(x).re)                                       //angle(x)
#define COMPLEXMUL(x, y, z, tmp) {(tmp)=(y).re*(z).re-(y).im*(z).im;(x).im=(y).re*(z).im+(y).im*(z).re;(x).re=tmp;}   //x=y*z
#define COMPLEXMADD(x, x2, y, z, tmp) {(tmp)=(x2).re+(y).re*(z).re-(y).im*(z).im;(x).im=(x2).im+(y).re*(z).im+(y).im*(z).re;(x).re=tmp;}    //x=x2+y*z
#define COMPLEXMADDN(x, x2, y, z, tmp) {(tmp)=(x2).re-(y).re*(z).re+(y).im*(z).im;(x).im=(x2).im-(y).re*(z).im-(y).im*(z).re;(x).re=tmp;}   //x=x2-y*z
#define COMPLEXMULN(x, y, z, tmp) {(tmp)=-(y).re*(z).re+(y).im*(z).im;(x).im=-(y).re*(z).im-(y).im*(z).re;(x).re=tmp;}//x=y*(-z)
#define COMPLEXMULI(x, y, z, tmp) {(tmp)=(y).re*(z).re-(y).im*(z).im;(x).re=-(y).re*(z).im-(y).im*(z).re;(x).im=tmp;} //x=1i*y*z
#define COMPLEXCMUL(x, y, z, tmp) {(tmp)=(y).re*(z).re+(y).im*(z).im;(x).im=(y).re*(z).im-(y).im*(z).re;(x).re=tmp;}  //x=conj(y)*z
#define COMPLEXCMULI(x, y, z, tmp) {(tmp)=(y).re*(z).re+(y).im*(z).im;(x).re=-(y).re*(z).im+(y).im*(z).re;(x).im=tmp;}//x=1i*conj(y)*z
#define COMPLEXMULC(x, y, z, tmp) {(tmp)=(y).re*(z).re+(y).im*(z).im;(x).im=-(y).re*(z).im+(y).im*(z).re;(x).re=tmp;} //x=y*conj(z)
#define COMPLEXMULCI(x, y, z, tmp) {(tmp)=(y).re*(z).re+(y).im*(z).im;(x).re=(y).re*(z).im-(y).im*(z).re;(x).im=tmp;} //x=1i*y*conj(z)
#define COMPLEXSQR(x, y, tmp) {(tmp)=(y).re*(y).re-(y).im*(y).im;(x).im=2*(y).re*(y).im;(x).re=tmp;}                  //x=y*y
#define COMPLEXLOG(x, y) {(x).re=0.5*log((y).re*(y).re+(y).im*(y).im);(x).im=atan2((y).im, (y).re);}                  //x=log(y)
#define COMPLEXLOGN(x, y) {(x).re=0.5*log((y).re*(y).re+(y).im*(y).im);(x).im=atan2(-(y).im, -(y).re);}               //x=log(-y)
#define COMPLEXRECIPROCAL(x, y, tmp) {tmp=(y).re*(y).re+(y).im*(y).im;(x).re=(y).re/tmp;(x).im=-(y).im/tmp;}          //x=1/y
#define COMPLEXRECIPROCALN(x, y, tmp) {tmp=(y).re*(y).re+(y).im*(y).im;(x).re=-(y).re/tmp;(x).im=(y).im/tmp;}          //x=-1/y
#define COMPLEXDIV(x, y, z, tmp,tmp2) {tmp=(z).re*(z).re+(z).im*(z).im;tmp2=((y).re*(z).re+(y).im*(z).im)/tmp;(x).im=((y).im*(z).re-(y).re*(z).im)/tmp;(x).re=tmp2;}          //x=y/z
#define COMPLEXCDIV(x, y, z, tmp,tmp2) {tmp=(z).re*(z).re+(z).im*(z).im;tmp2=((y).re*(z).re-(y).im*(z).im)/tmp;(x).im=-((y).im*(z).re+(y).re*(z).im)/tmp;(x).re=tmp2;}          //x=conj(y)/z
#define COMPLEXDIVI(x, y, z, tmp,tmp2) {tmp=(z).re*(z).re+(z).im*(z).im;tmp2=-((y).re*(z).re+(y).im*(z).im)/tmp;(x).re=((y).im*(z).re-(y).re*(z).im)/tmp;(x).im=tmp2;}          //x=y/iz
#define COMPLEXCDIVI(x, y, z, tmp,tmp2) {tmp=(z).re*(z).re+(z).im*(z).im;tmp2=-((y).re*(z).re-(y).im*(z).im)/tmp;(x).re=(-(y).im*(z).re-(y).re*(z).im)/tmp;(x).im=tmp2;}          //x=conj(y)/iz
#define COMPLEXDIVN(x, y, z, tmp,tmp2) {tmp=(z).re*(z).re+(z).im*(z).im;tmp2=-((y).re*(z).re+(y).im*(z).im)/tmp;(x).im=(-(y).im*(z).re+(y).re*(z).im)/tmp;(x).re=tmp2;}          //x=y/-z
#define COMPLEXDIVNI(x, y, z, tmp,tmp2) {tmp=(z).re*(z).re+(z).im*(z).im;tmp2=((y).re*(z).re+(y).im*(z).im)/tmp;(x).re=(-(y).im*(z).re+(y).re*(z).im)/tmp;(x).im=tmp2;}          //x=y/-iz
#define COMPLEXRECIPROCALSAFE(x,y,rtmp,itmp) if(fabs((y).re)>fabs((y).im)) {\
        itmp=(y).im*(1.0/(y).re);\
        rtmp=(y).re+itmp*(y).im;\
        COMPLEXASSIGN(x, 1.0/rtmp, -itmp*(1.0/rtmp));\
      }\
      else {\
        itmp=(y).re*(1.0/(y).im);\
        rtmp=(y).im+itmp*(y).re;\
        COMPLEXASSIGN(x, itmp*(1.0/rtmp), -1.0/rtmp);\
      }
#define COMPLEXDIVSAFE(x,y,z,rtmp,itmp) {COMPLEXRECIPROCALSAFE(x,z,rtmp,itmp);COMPLEXMUL(x,x,y,rtmp)};
#define COMPLEXPRINT(x) mexPrintf("%s=%.16e+%.16ei\n",#x,(x).re,(x).im);
#define REALPRINT(x) mexPrintf("%s=%.16e\n",#x,(x));
#define COMPLEXNEG(x,y) {(x).re=-(y).re;(x).im=-(y).im;}
//#define COMPLEXSQRT(x,y,tmp,tmp2) {tmp=COMPLEXABS(y);tmp2=copysign(sqrt(0.5*(tmp-y.re)),y.im);COMPLEXASSIGN(x,sqrt(0.5*(tmp+y.re)),tmp2)};

//useful numerical constants
#define INV2PI 0.159154943091895
#define PI2 6.2831853071795862
#define HALFSQRT3 0.8660254037844386
//#ifndef M_PI
//#define M_PI 3.14159265358979323846
//#endif

//constants for laguerre solver in polyroots
#define NRFRACTIONS 12
#define USEFRACTION 19 //Use a prime number in case it is locked in an even loop, unlikely that the loop is 19 steps.
#define MAXITER USEFRACTION*NRFRACTIONS*10
#define PANELCOEFFZERO 1e-25
//note that the values 64 and 1.8 are coupled, 64 coefficients are required for double precision convergence at 1.8*radius
#define GENERALPANELNCOEFF 64
#define GENERALPANELCONVERGENCERADIUS 1.8
#define TARGETPANELCONVERGENCERADIUS 1000.0 //A high value can be chosen, as the cancellation errors are small.
//#define MAXROOTEXPANSION 16   //coupled with ROOTEXPANSIONTOL, with 0.1, 16 steps should be enough for double precision tolerance
#define MAXROOTEXPANSION 64   //coupled with ROOTEXPANSIONTOL, with 0.1, 16 steps should be enough for double precision tolerance
#define MAXROOTEXPANSIONSHIFTED 64 //for shifted case (which only is used to prevend truncation errors for large numbers of coefficients, use a high value to remove all large roots
#define ROOTEXPANSIONTOLSHIFTED 0.9 //1.8/2)
#define ROOTDIVISIONTOL 1000  //the maximum allowed error 
#define ROOTEXPANSIONTOL 10   //the threshold when root expansion occurs, values closer to 1 gives higher tolerance
//#define ROOTEXPANSIONTOL 2   //the threshold when root expansion occurs, values closer to 1 gives higher tolerance
#define DUPLICATEROOTTOL 1e-4 //the threshold when duplicate root expansion occurs, values closer to 1 gives higher tolerance
#define MINCONNECTDIST 1e-4   //shortest distance for assuming that panels are connected, multiplied with the largest panel length
#define MAXCONNECTDIST 1e-3   //shortest distance for assuming that panels are connected, multiplied with the smallest panel length

//these two lists should contain the first 101 coefficients of the expansions of sqrt(1+x) and rsqrt(1+x) respectively
const double sqrtcoeffs[]={1.0,5.0000000000000000e-001, -1.2500000000000000e-001, 6.2500000000000000e-002, -3.9062500000000000e-002, 2.7343750000000000e-002, -2.0507812500000000e-002, 1.6113281250000000e-002, -1.3092041015625000e-002, 1.0910034179687500e-002, -9.2735290527343750e-003, 8.0089569091796875e-003, -7.0078372955322266e-003, 6.1992406845092773e-003, -5.5350363254547119e-003, 4.9815326929092407e-003, -4.5145140029489994e-003, 4.1161745321005583e-003, -3.7731599877588451e-003, 3.4752789360936731e-003, -3.2146330158866472e-003, 2.9850163718947442e-003, -2.7814925283564662e-003, 2.6000908417245223e-003, -2.4375851641167401e-003, 2.2913300542697357e-003, -2.1591379357541740e-003, 2.0391858282122755e-003, -1.9299437302723321e-003, 1.8301190545685910e-003, -1.7386131018401613e-003, 1.6544866614285406e-003, -1.5769325991740776e-003, 1.5052538446661652e-003, -1.4388455868132462e-003, 1.3771807759498213e-003, -1.3197982436185787e-003, 1.2662929094178256e-003, -1.2163076629934377e-003, 1.1695265990321515e-003, -1.1256693515684459e-003, 1.0844863265110638e-003, -1.0457546719928115e-003, 1.0092748578535273e-003, -9.7486776042670265e-004, 9.4237216841247919e-004, -9.1164264118163757e-004, 8.8254766327158514e-004, -8.5496804879434801e-004, 8.2879555750472523e-004, -8.0393169077958341e-004, 7.8028664105077219e-004, -7.5777837255892292e-004, 7.3633181484499108e-004, -7.1587815332151909e-004, 6.9635420368547760e-004, -6.7770185894390229e-004, 6.5986759949801012e-004, -6.4280205813168224e-004, 6.2645963292494455e-004, -6.1079814210182094e-004, 5.9577851565669425e-004, -5.8136451931016129e-004, 5.6752250694563368e-004, -5.5422119818909538e-004, 5.4143147823088543e-004, -5.2912621736200176e-004, 5.1728010801807642e-004, -5.0586951740003063e-004, 4.9487235397829084e-004, -4.8426794639304187e-004, 4.7403693344107622e-004, -4.6416116399438707e-004, 4.5462360583011890e-004, -4.4540826246869755e-004, 4.3650009721932361e-004, -4.2788496372157375e-004, 4.1954954235037426e-004, -4.1148128192055943e-004, 4.0366834618789057e-004, -3.9609956469686768e-004, 3.8876438757285163e-004, -3.8165284389773854e-004, 3.7475550334536978e-004, -3.6806344078563105e-004, 3.6156820359529632e-004, -3.5526178143956448e-004, 3.4913657831129615e-004, -3.4318538663553541e-004, 3.3740136326527363e-004, -3.3177800721085238e-004, 3.2630913896012400e-004, -3.2098888125968721e-004, 3.1581164123936970e-004, -3.1077209377278404e-004, 3.0586516597637164e-004, -3.0108602275799080e-004, 2.9643005333389815e-004, -2.9189285864001193e-004, 2.8747023956970875e-004, -2.8315818597616306e-004};
const double rsqrtcoeffs[]={1.0,-5.0000000000000000e-001, 3.7500000000000000e-001, -3.1250000000000000e-001, 2.7343750000000000e-001, -2.4609375000000000e-001, 2.2558593750000000e-001, -2.0947265625000000e-001, 1.9638061523437500e-001, -1.8547058105468750e-001, 1.7619705200195313e-001, -1.6818809509277344e-001, 1.6118025779724121e-001, -1.5498101711273193e-001, 1.4944598078727722e-001, -1.4446444809436798e-001, 1.3994993409141898e-001, -1.3583375955931842e-001, 1.3206059957155958e-001, -1.2858532063546591e-001, 1.2537068761957926e-001, -1.2238567124768450e-001, 1.1960417871932803e-001, -1.1700408787760351e-001, 1.1456650271348678e-001, -1.1227517265921705e-001, 1.1011603472346286e-001, -1.0807684889525060e-001, 1.0614690516497828e-001, -1.0431678611040969e-001, 1.0257817300856951e-001, -1.0092368634714097e-001, 9.9346753747966893e-002, -9.7841499903300738e-002, 9.6402654316487493e-002, -9.5025473540537669e-002, 9.3705675296919103e-002, -9.2439382387501268e-002, 9.1223074724507830e-002, -9.0053548125475683e-002, 8.8927878773907232e-002, -8.7843392447396168e-002, 8.6797637775403363e-002, -8.5788362917549835e-002, 8.4813495157123137e-002, -8.3871122988710647e-002, 8.2959480347529008e-002, -8.2076932684257406e-002, 8.1221964635463065e-002, -8.0393169077958349e-002, 7.9589237387178768e-002, -7.8808950746127981e-002, 7.8051172373569053e-002, -7.7314840558724063e-002, 7.6598962405402538e-002, -7.5902608201717056e-002, 7.5224906342773151e-002, -7.4565038743275150e-002, 7.3922236685143458e-002, -7.3295777052218514e-002, 7.2684978910116688e-002, -7.2089200394459998e-002, 7.1507835875149833e-002, -7.0940313368204208e-002, 7.0386092170015124e-002, -6.9844660691784230e-002, 6.9315534474422236e-002, -6.8798254366404174e-002, 6.8292384849004142e-002, -6.7797512495025858e-002, 6.7313244548632814e-002, -6.6839207615191742e-002, 6.6375046451197350e-002, -6.5920422845367241e-002, 6.5475014582898547e-002, -6.5038514485679211e-002, 6.4610629521957647e-002, -6.4191079979607268e-002, 6.3779598697686707e-002, -6.3375930351498827e-002, 6.2979830786801957e-002, -6.2591066399229123e-002, 6.2209413555331383e-002, -6.1834658051986020e-002, 6.1466594611200381e-002, -6.1105026407605088e-002, 6.0749764626165527e-002, -6.0400628047854232e-002, 6.0057442661218700e-002, -5.9720041297953427e-002, 5.9388263290742573e-002, -5.9061954151782448e-002, 5.8740965270522763e-002, -5.8425153629283397e-002, 5.8114381535510608e-002, -5.7808516369534241e-002, 5.7507430346776245e-002, -5.7211000293442345e-002, 5.6919107434802330e-002, -5.6631637195232620e-002, 5.6348479009256457e-002};
#define SQRTEXPMAX 100

//functions to operate on polynomials
cmplx evalpoly(COMPLEXVECTOR coeff,double val,int Ncoeff);
cmplx evalpoly(COMPLEXVECTOR coeff,cmplx val,int Ncoeff);
cmplx evalpoly(double *rcoeff,double *icoeff,double val,int Ncoeff);
double evalpoly(double *coeff,double val,int Ncoeff);
double evalpolyreal(COMPLEXVECTOR coeff,double val,int Ncoeff);
void polydiv(COMPLEXVECTOR ret,COMPLEXVECTOR rem,const COMPLEXVECTOR nom,const COMPLEXVECTOR denom,COMPLEXVECTOR wksp,int N,int M);
void polydivnowksp(COMPLEXVECTOR ret,COMPLEXVECTOR rem,const COMPLEXVECTOR nom,const COMPLEXVECTOR denom,int N,int M);
void polydivnowksp2(COMPLEXVECTOR ret,COMPLEXVECTOR nom,const COMPLEXVECTOR denom,int N,int M);
cmplx intpoly(COMPLEXVECTOR coeff,double val,int Ncoeff);
//inline double cot(double value) {
//  return 1.0/tan(value);
//}

//all inline functions has to be declared in the header file
//one standard implementation of the complex sqrt
inline cmplx sqrt(cmplx in) {
  double w,tmp;
  cmplx ret;
  if(fabs(in.re)>fabs(in.im)) {
    tmp=in.im/in.re;
    w=sqrt(fabs(in.re))*sqrt(0.5*(1+sqrt(1+tmp*tmp)));
  }
  else if(fabs(in.im)!=0.0) {
    tmp=in.re/in.im;
    w=sqrt(fabs(in.im))*sqrt(0.5*(fabs(tmp)+sqrt(1+tmp*tmp)));
  }
  else {
    COMPLEXASSIGN(ret,0,0);
    return ret;
  }
  if(in.re>=0) {
    COMPLEXASSIGN(ret,w,0.5*in.im/w);
    return ret;
  }
  if(in.im>=0) {
    COMPLEXASSIGN(ret,0.5*fabs(in.im)/w,w);
  }
  else {
    COMPLEXASSIGN(ret,0.5*fabs(in.im)/w,-w);
  }
  return ret;
}

//evaluate polynomial for value val
inline cmplx evalpoly(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  cmplx ret;
  COMPLEXASSIGN(ret,coeff[Ncoeff-1].re,coeff[Ncoeff-1].im);
  for(int k=Ncoeff-2;k>=0;k--) {
    COMPLEXASSIGN(ret,ret.re*val,ret.im*val);
    COMPLEXADD(ret,ret,coeff[k]);
  }
  return ret;
}

//evaluate polynomial for complex value val
inline cmplx evalpoly(COMPLEXVECTOR coeff,cmplx val,int Ncoeff) {
  double tmp;
  cmplx ret;
  COMPLEXASSIGN(ret,coeff[Ncoeff-1].re,coeff[Ncoeff-1].im);
  for(int k=Ncoeff-2;k>=0;k--) {
    COMPLEXMUL(ret,ret,val,tmp);
    COMPLEXADD(ret,ret,coeff[k]);
  }
  return ret;
}

//yet another polynomial evaluations, but when real and imaginary coefficients are split
inline cmplx evalpoly(double *rcoeff,double *icoeff,double val,int Ncoeff) {
  cmplx ret;
  COMPLEXASSIGN(ret,rcoeff[Ncoeff-1],icoeff[Ncoeff-1]);
  for(int k=Ncoeff-2;k>=0;k--) {
    COMPLEXASSIGN(ret,ret.re*val,ret.im*val);
    COMPLEXASSIGN(ret,ret.re+rcoeff[k],ret.im+icoeff[k]);
  }
  return ret;
}

//evaluate polynomial, real coefficients
inline double evalpoly(double *coeff,double val,int Ncoeff) {
  double ret=coeff[Ncoeff-1];
  for(int k=Ncoeff-2;k>=0;k--) {
    ret=ret*val+coeff[k];
  }
  return ret;
}

//evaluate polynomial, real coefficients in a COMPLEXVECTOR
inline double evalpolyreal(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  double ret=coeff[Ncoeff-1].re;
  for(int k=Ncoeff-2;k>=0;k--) {
    ret=ret*val+coeff[k].re;
  }
  return ret;
}

//evaluate polynomial, real coefficients with absolute value on each coefficient
inline double evalpolyrealabs(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  double ret=fabs(coeff[Ncoeff-1].re);
  for(int k=Ncoeff-2;k>=0;k--) {
    ret=ret*val+fabs(coeff[k].re);
  }
  return ret;
}

//evaluate the derivative of a polynomial
inline cmplx evalpolyderiv(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  double tmpval=val;
  cmplx ret;
  if(Ncoeff>1)
    ret=coeff[1];
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>2) {
    COMPLEXASSIGN(ret,ret.re+2*coeff[2].re*tmpval,ret.im+2*coeff[2].im*tmpval);
  }
  for(int k=3;k<Ncoeff;k++) {
    tmpval*=val;
    COMPLEXASSIGN(ret,ret.re+k*coeff[k].re*tmpval,ret.im+k*coeff[k].im*tmpval);
  }
  return ret;
}

//evaluate the derivative of a polynomial
inline cmplx evalpolyderiv(COMPLEXVECTOR coeff,cmplx val,int Ncoeff) {
  cmplx tmpval=val,ctmp;
  cmplx ret;
  double tmp;
  if(Ncoeff>1)
    ret=coeff[1];
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>2) {
    COMPLEXMUL(ctmp,coeff[2],tmpval,tmp);
    COMPLEXASSIGN(ret,ret.re+2*ctmp.re,ret.im+2*ctmp.im);
  }
  for(int k=3;k<Ncoeff;k++) {
    COMPLEXMUL(tmpval,tmpval,val,tmp);
    COMPLEXMUL(ctmp,coeff[k],tmpval,tmp);
    COMPLEXASSIGN(ret,ret.re+k*ctmp.re,ret.im+k*ctmp.im);
  }
  return ret;
}

//evaluate the derivative of a polynomial, use real values coeffs
inline double evalpolyderivreal(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  double tmpval=val;
  double ret;
  if(Ncoeff>1)
    ret=coeff[1].re;
  else {
    ret=0;
  }
  if(Ncoeff>2) {
    ret+=2*coeff[2].re*tmpval;
  }
  for(int k=3;k<Ncoeff;k++) {
    tmpval*=val;
    ret+=k*coeff[k].re*tmpval;
  }
  return ret;
}

//evaluate the second derivative of a polynomial
inline cmplx evalpoly2deriv(COMPLEXVECTOR coeff,cmplx val,int Ncoeff) {
  cmplx tmpval=val,ctmp;
  cmplx ret;
  double tmp;
  if(Ncoeff>2) {
    COMPLEXASSIGN(ret,coeff[2].re*2,coeff[2].im*2);
  }
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>3) {
    COMPLEXMUL(ctmp,coeff[3],tmpval,tmp);
    COMPLEXASSIGN(ret,ret.re+6*ctmp.re,ret.im+6*ctmp.im);
  }
  for(int k=4;k<Ncoeff;k++) {
    COMPLEXMUL(tmpval,tmpval,val,tmp);
    COMPLEXMUL(ctmp,coeff[k],tmpval,tmp);
    COMPLEXASSIGN(ret,ret.re+k*(k-1)*ctmp.re,ret.im+k*(k-1)*ctmp.im);
  }
  return ret;
}

//evaluate the second derivative of a polynomial
inline cmplx evalpoly2deriv(COMPLEXVECTOR coeff, double val, int Ncoeff) {
  cmplx ret;
  double tmpval=val;
  double tmp;
  if(Ncoeff>2) {
    COMPLEXASSIGN(ret,coeff[2].re*2,coeff[2].im*2);
  }
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>3) {
    COMPLEXASSIGN(ret,ret.re+6*coeff[3].re*val,ret.im+6*coeff[3].im*val);
  }
  for(int k=4;k<Ncoeff;k++) {
    tmpval*=val;
    tmp=tmpval*k*(k-1);
    COMPLEXASSIGN(ret,ret.re+tmp*coeff[k].re,ret.im+tmp*coeff[k].im);
  }
  return ret;
}

//evaluate the second derivative of a polynomial, real valued coefficients
inline double evalpoly2derivreal(COMPLEXVECTOR coeff, double val, int Ncoeff) {
  double ret;
  double tmpval=val;
  if(Ncoeff>2)
    ret=2*coeff[2].re;
  else 
    ret=0;
  if(Ncoeff>3) {
    ret+=6*coeff[3].re*val;
  }
  for(int k=4;k<Ncoeff;k++) {
    tmpval*=val;
    ret+=tmpval*k*(k-1)*coeff[k].re;
  }
  return ret;
}

//evaluate the third derivative of a polynomial
inline cmplx evalpoly3deriv(COMPLEXVECTOR coeff,cmplx val,int Ncoeff) {
  cmplx tmpval=val,ctmp;
  cmplx ret;
  double tmp;
  if(Ncoeff>3) {
    COMPLEXASSIGN(ret,coeff[3].re*6,coeff[3].im*6);
  }
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>4) {
    COMPLEXMUL(ctmp,coeff[4],tmpval,tmp);
    COMPLEXASSIGN(ret,ret.re+24*ctmp.re,ret.im+24*ctmp.im);
  }
  for(int k=5;k<Ncoeff;k++) {
    COMPLEXMUL(tmpval,tmpval,val,tmp);
    COMPLEXMUL(ctmp,coeff[k],tmpval,tmp);
    tmp=k*(k-1)*(k-2);
    COMPLEXASSIGN(ret,ret.re+tmp*ctmp.re,ret.im+tmp*ctmp.im);
  }
  return ret;
}

//evaluate the third derivative of a polynomial
inline cmplx evalpoly3deriv(COMPLEXVECTOR coeff, double val, int Ncoeff) {
  cmplx ret;
  double tmpval=val;
  double tmp;
  if(Ncoeff>3) {
    COMPLEXASSIGN(ret,coeff[3].re*6,coeff[3].im*6);
  }
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>4) {
    COMPLEXASSIGN(ret,ret.re+24*coeff[4].re*val,ret.im+24*coeff[4].im*val);
  }
  for(int k=5;k<Ncoeff;k++) {
    tmpval*=val;
    tmp=tmpval*k*(k-1)*(k-2);
    COMPLEXASSIGN(ret,ret.re+tmp*coeff[k].re,ret.im+tmp*coeff[k].im);
  }
  return ret;
}

//evaluate the third derivative of a polynomial, real valued coefficients
inline double evalpoly3derivreal(COMPLEXVECTOR coeff, double val, int Ncoeff) {
  double ret;
  double tmpval=val;
  if(Ncoeff>3)
    ret=6*coeff[3].re;
  else 
    ret=0;
  if(Ncoeff>4) {
    ret+=24*coeff[4].re*val;
  }
  for(int k=5;k<Ncoeff;k++) {
    tmpval*=val;
    ret+=tmpval*k*(k-1)*(k-2)*coeff[k].re;
  }
  return ret;
}


//evaluate the fourth derivative of a polynomial
inline cmplx evalpoly4deriv(COMPLEXVECTOR coeff,cmplx val,int Ncoeff) {
  cmplx tmpval=val,ctmp;
  cmplx ret;
  double tmp;
  if(Ncoeff>4) {
    COMPLEXASSIGN(ret,coeff[4].re*24,coeff[4].im*24);
  }
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>5) {
    COMPLEXMUL(ctmp,coeff[5],tmpval,tmp);
    COMPLEXASSIGN(ret,ret.re+120*ctmp.re,ret.im+120*ctmp.im);
  }
  for(int k=6;k<Ncoeff;k++) {
    COMPLEXMUL(tmpval,tmpval,val,tmp);
    COMPLEXMUL(ctmp,coeff[k],tmpval,tmp);
    tmp=k*(k-1)*(k-2)*(k-3);
    COMPLEXASSIGN(ret,ret.re+tmp*ctmp.re,ret.im+tmp*ctmp.im);
  }
  return ret;
}

//evaluate the fourth derivative of a polynomial
inline cmplx evalpoly4deriv(COMPLEXVECTOR coeff, double val, int Ncoeff) {
  cmplx ret;
  double tmpval=val;
  double tmp;
  if(Ncoeff>4) {
    COMPLEXASSIGN(ret,coeff[4].re*24,coeff[4].im*24);
  }
  else {
    COMPLEXASSIGN(ret,0,0);
  }
  if(Ncoeff>5) {
    COMPLEXASSIGN(ret,ret.re+120*coeff[5].re*val,ret.im+120*coeff[5].im*val);
  }
  for(int k=6;k<Ncoeff;k++) {
    tmpval*=val;
    tmp=tmpval*k*(k-1)*(k-2)*(k-3);
    COMPLEXASSIGN(ret,ret.re+tmp*coeff[k].re,ret.im+tmp*coeff[k].im);
  }
  return ret;
}

//evaluate the fourth derivative of a polynomial, real valued coefficients
inline double evalpoly4derivreal(COMPLEXVECTOR coeff, double val, int Ncoeff) {
  double ret;
  double tmpval=val;
  if(Ncoeff>4)
    ret=24*coeff[4].re;
  else 
    ret=0;
  if(Ncoeff>5) {
    ret+=120*coeff[5].re*val;
  }
  for(int k=6;k<Ncoeff;k++) {
    tmpval*=val;
    ret+=tmpval*k*(k-1)*(k-2)*(k-3)*coeff[k].re;
  }
  return ret;
}

//divide two polynomials, return new polynomial and remainder
//check if this is obsolete
inline void polydiv(COMPLEXVECTOR ret,COMPLEXVECTOR rem,/*const*/ COMPLEXVECTOR nom,/*const*/ COMPLEXVECTOR denom,COMPLEXVECTOR wksp,int N,int M) {
  double tmp;
  cmplx ctmp;
  for(int i=0;i<N;i++) {
    COMPLEXASSIGN(wksp[i],nom[i].re,nom[i].im);
  }
  for(int i=N-M;i>=0;i--) {
    COMPLEXASSIGN(ret[i],wksp[i+M-1].re,wksp[i+M-1].im);
    for(int j=0;j<M;j++) {
      COMPLEXMUL(ctmp,ret[i],denom[j],tmp);
      COMPLEXSUB(wksp[i+j],wksp[i+j],ctmp);
    }
  }
  for(int i=0;i<M;i++) {
    COMPLEXASSIGN(rem[i],wksp[i].re,wksp[i].im);
  }
}


//divide two polynomials, return new polynomial and remainder, to workspace variable to use
inline void polydivnowksp(COMPLEXVECTOR ret,COMPLEXVECTOR rem,/*const*/ COMPLEXVECTOR nom,/*const*/ COMPLEXVECTOR denom,int N,int M) {
  double tmp;
  cmplx ctmp;
  for(int i=0;i<N;i++) {
    COMPLEXASSIGN(rem[i],nom[i].re,nom[i].im);
  }
  for(int i=N-M;i>=0;i--) {
//    mexPrintf("i=%d\n",i);
    COMPLEXASSIGN(ret[i],rem[i+M-1].re,rem[i+M-1].im);
//    COMPLEXPRINT(ret[i]);
    for(int j=0;j<M;j++) {
//      COMPLEXPRINT(denom[j]);
      COMPLEXMUL(ctmp,ret[i],denom[j],tmp);
//      COMPLEXPRINT(ctmp);
//      COMPLEXPRINT(rem[i+j]);
      COMPLEXSUB(rem[i+j],rem[i+j],ctmp);
//      COMPLEXPRINT(rem[i+j]);
    }
  }
}

//divide two polynomials, return new polynomial and remainder, remainder stored in nom for output
inline void polydivnowksp2(COMPLEXVECTOR ret,COMPLEXVECTOR nom,/*const*/ COMPLEXVECTOR denom,int N,int M) {
  double tmp;
  cmplx ctmp;
  for(int i=N-M;i>=0;i--) {
    COMPLEXASSIGN(ret[i],nom[i+M-1].re,nom[i+M-1].im);
    for(int j=0;j<M;j++) {
      COMPLEXMUL(ctmp,ret[i],denom[j],tmp);
      COMPLEXSUB(nom[i+j],nom[i+j],ctmp);
    }
  }
}

//calculates the integral of a polynomial from 0 to val
inline cmplx intpoly(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  cmplx ret;
  COMPLEXASSIGN(ret,0,0);
  double xval=val;
  for(int k=0;k<Ncoeff;k++) {
    COMPLEXADDC(ret, ret, coeff[k], xval/(k+1));
    xval*=val;
  }
  return ret;
}
//calculates the integral of a polynomial from -val to val
inline cmplx intpolysymmetric(COMPLEXVECTOR coeff,double val,int Ncoeff) {
  cmplx ret;
  COMPLEXASSIGN(ret,0,0);
  double xval=val;
  val*=val;
  for(int k=0;k<Ncoeff;k+=2) { //odd order terms will give zero contribution
    COMPLEXADDC(ret, ret, coeff[k], 2*xval/(k+1));
    xval*=val;
  }
  return ret;
}

//use roots to generate a polynomial, basically the inverse to polyroots
inline void combinepoly(COMPLEXVECTORRESTRICT combined,COMPLEXVECTORRESTRICT  roots,int N) {
  double tmp;
  COMPLEXASSIGN(combined[1],1.0,0.0);
  COMPLEXNEG(combined[0],roots[0]);
//  mexPrintf("combined: N=%d\n",N);
  for(int i=1;i<N;i++) {
    COMPLEXASSIGN(combined[i+1],1.0,0.0);
    for(int j=i;j>0;j--) {
//      COMPLEXPRINT(combined[j]);
//      COMPLEXPRINT(combined[j-1]);
//      COMPLEXPRINT(roots[i]);
      COMPLEXMADDN(combined[j],combined[j-1],combined[j],roots[i],tmp); //combined[j]=combined[j+1]-combined[j]*roots[i];
//      COMPLEXPRINT(combined[j]);
    }
    COMPLEXMULN(combined[0],combined[0],roots[i],tmp);
  }
}

//combines polynomial from roots, using doubles instead of complex
inline void combinepoly(DOUBLEVECTORRESTRICT combined,DOUBLEVECTORRESTRICT roots,int N) {
  if(N==0) {
    combined[0]=1;
    return;
  }
  combined[1]=1.0;
  combined[0]=-roots[0];
  mexPrintf("combined: N=%d combined[0]=%e combined[1]=%e\n",N,combined[0],combined[1]);
  for(int i=1;i<N;i++) {
    combined[i+1]=1.0;
    for(int j=i;j>0;j--) {
      combined[j]=combined[j-1]-combined[j]*roots[i];
    }
    combined[0]*=roots[i];
  }
}

//combines polynomial from roots. Only roots with wkspi[i]==false will be included
inline int combinepolywithcheck(COMPLEXVECTORRESTRICT combined,COMPLEXVECTORRESTRICT  roots,INTVECTORRESTRICT wkspi,int N) {
  double tmp;
  COMPLEXASSIGN(combined[1],1.0,0.0);
  int count=1,start=0;
  for(;wkspi[start]&&start<N;start++);
  if(start<N) {
    COMPLEXNEG(combined[0],roots[start]);
  }
  else {
    COMPLEXASSIGN(combined[0],1.0,0.0);
    return 0;
  }
//  mexPrintf("combined: N=%d\n",N);
  
  for(int i=start+1;i<N;i++) {
    if(!wkspi[i]) {
      COMPLEXASSIGN(combined[count+1], 1.0, 0.0);
      for(int j=count;j>0;j--) {
//      COMPLEXPRINT(combined[j]);
//      COMPLEXPRINT(combined[j-1]);
//      COMPLEXPRINT(roots[i]);
        COMPLEXMADDN(combined[j], combined[j-1], combined[j], roots[i], tmp); //combined[j]=combined[j+1]-combined[j]*roots[i];
//      COMPLEXPRINT(combined[j]);
      }
      COMPLEXMULN(combined[0], combined[0], roots[i], tmp);
      count++;
    }
  }
  return count;
}

//combines polynomial from roots. Only roots with wkspi[i]==true will be included
inline int combinepolywithinversecheck(COMPLEXVECTORRESTRICT combined,COMPLEXVECTORRESTRICT  roots,INTVECTORRESTRICT wkspi,int N) {
  double tmp;
  COMPLEXASSIGN(combined[1],1.0,0.0);
  int count=1,start=0;
  for(;!wkspi[start]&&start<N;start++);
  if(start<N) {
    COMPLEXNEG(combined[0],roots[start]);
  }
  else {
    COMPLEXASSIGN(combined[0],1.0,0.0);
    return 0;
  }
//  mexPrintf("combined: N=%d\n",N);
  
  for(int i=start+1;i<N;i++) {
    if(wkspi[i]) {
      COMPLEXASSIGN(combined[count+1], 1.0, 0.0);
      for(int j=count;j>0;j--) {
//      COMPLEXPRINT(combined[j]);
//      COMPLEXPRINT(combined[j-1]);
//      COMPLEXPRINT(roots[i]);
        COMPLEXMADDN(combined[j], combined[j-1], combined[j], roots[i], tmp); //combined[j]=combined[j+1]-combined[j]*roots[i];
//      COMPLEXPRINT(combined[j]);
      }
      COMPLEXMULN(combined[0], combined[0], roots[i], tmp);
      count++;
    }
  }
  return count;
}

inline double myround(double number)
{
    return number < 0.0 ? ceil(number - 0.5) : floor(number + 0.5);
}

//functions implemented in generalpanelfunctions.cpp
int laguerre(cmplx *value, COMPLEXVECTOR coeffs,int order);
void polyroots(COMPLEXVECTOR roots,COMPLEXVECTOR coeffs,int order,COMPLEXVECTOR wksp, COMPLEXVECTOR wksp2);
void initcubicconstants_areal(cubicconstants* out,generalpanel *in);
void initcubicconstants_aimag(cubicconstants* out,generalpanel *in);
void cubicroots(COMPLEXVECTOR roots,COMPLEXVECTOR coeffs,cubicconstants *in,COMPLEXVECTOR wksp,COMPLEXVECTOR wksp2);
void quadraticroots(COMPLEXVECTOR roots,COMPLEXVECTOR coeffs);
int refineroot(cmplx *value,COMPLEXVECTOR coeffs,int order);
void polynorm(COMPLEXVECTOR ret,COMPLEXVECTOR coeff,int Ncoeff);
void polynorm(DOUBLEVECTOR ret,COMPLEXVECTOR coeff,int Ncoeff);
int expandandmultiplypoly(COMPLEXVECTOR ret,COMPLEXVECTOR in,cmplx val,double tol,double tolval,int N,int maxiter);
cmplx onetermseries(cmplx t1,double tol, int maxstep);
cmplx twotermseries(cmplx t1,cmplx t2,cmplx step,double tol, int maxstep);
cmplx prodsum(cmplx t1,cmplx t2,double tol,int maxstep);
cmplx evalpolyprod(COMPLEXVECTOR coeff,cmplx val1,cmplx val2,int Ncoeff);
void bubblesort(DOUBLEVECTOR vals,int N);
void bubblesortreal(COMPLEXVECTOR vals,int N);
cmplx gausskonrod(cmplx (*f)(double,void*),double a,double b,void* params,double tol=1e-12,int initintervals=10);
cmplx gausskonrodinner(cmplx (*f)(double,void*),double a,double b,void* params,double tol,int level);
//cmplx gausskonrod_roots(cmplx (*f)(double,void*),double a,double b,void* params,DOUBLEVECTOR roots,int Nroots,DOUBLEVECTOR wksp,double tol,int initintervals);
//cmplx gausskonrodinner_roots(cmplx (*f)(double,void*),double a,double b,void* params,double tol,double *roots,int Nroots,double *wksp,double diff2,int level);
cmplx gausskonrod_singularities(cmplx (*f)(double,void*),double a,double b,void* params,DOUBLEVECTOR singularities,int Nsingularities,double tol=1e-12,int initintervals=10);
cmplx gausskonrod_endsingularities(cmplx (*f)(double,void*),double a,double b,void* params,double tol=1e-12,int initintervals=10);
cmplx gausskonrodinner_endsingularities(cmplx (*f)(double,void*),double a,double b,double diffglobal,double offsetglobal,void* params,double tol,int level);
cmplx evalpolyquotient(double value,void* input);
cmplx evalpolyquotientnormalized(double value,void* input);
cmplx evalpolyquotientslopecorrection(double value,void* input);
cmplx evalpolyquotientslopecorrectionnormalized(double value,void* input);
cmplx evalpolyquotientrankine(double value,void* input);
cmplx evalpolyquotientscully(double value,void* input);
cmplx evalpolyquotientoseen(double value,void* input);
void MPexp_smooth_simplified(SMOOTHER smooth,double xopt,bool cont,
                   double *cutoff,double *shape,double *scale);
void allocateduplicatepanel(generalpanel *dest, generalpanel *source, generalpanel *ref);
int addpanel(panellist* panels);
void multiplypoly(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in1,COMPLEXVECTORRESTRICT in2, int N1,int N2);
void sqrpoly(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in,int N);
void multiplypolyreal(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in1,COMPLEXVECTORRESTRICT in2, int N1,int N2);
void sqrpolyreal(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in,int N);
void multiplypolyreal2(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in1,COMPLEXVECTORRESTRICT in2, int N1,int N2);
cmplx complexlogm1(cmplx x);

//functions implemented in generalpanel.cpp
void resetstatistics();
void printstatistics();
void generalpanelalloc(int aN,int bN,int Ncoeff,int slopeexp);
void generalpanelfree();
void reduceroot(COMPLEXVECTORRESTRICT  nom,COMPLEXVECTORRESTRICT  denom, cmplx root, cmplx rootconst, int Ndenom,int Nnom);
void initgeneralpanel(generalpanel *out,double xstart,double ystart,double xend, double yend,const double* ar,const double* ai,int aN,const double *br, const double *bi, int bN,double outside);
void initsourcepanel(generalpanel *out);
void inittargetpanel(generalpanel *out);
void initgeneralvars(generalpanel *out,int aN,int bN,double xstart,double ystart,double xend,double yend,double outside);
void initgeneralvarsa(generalpanel *out,int bN,double xstart,double ystart,double xend,double yend,double outside);
void initgeneralvarsb(generalpanel *out,int bN);
void initavectorgeneral(generalpanel *out,const double *ar,const double* ai,int aN);
void initavectorlinear(generalpanel *out);
void initavectorcubic(generalpanel *out,const double xdirstart,const double ydirstart,const double xdirend,const double ydirend);
void initavectorquintic(generalpanel *out,const double xdirstart,const double ydirstart,const double xdirend,const double ydirend,const double x2dirstart,const double y2dirstart,const double x2dirend,const double y2dirend);
void initavectorseptic(generalpanel *out,const double xdirstart,const double ydirstart,const double xdirend,const double ydirend,const double x2dirstart,const double y2dirstart,const double x2dirend,const double y2dirend,const double x3dirstart,const double y3dirstart,const double x3dirend,const double y3dirend);
void initbvectorgeneral(generalpanel *out,const double *br,const double *bi,int bN);
void initbvectorlinear(generalpanel *out,const double gstartr, const double gstarti,const double gendr, const double gendi, int slopecorrection);
void initbvectorcubic(generalpanel *out,const double gstartr, const double gstarti,const double gendr, const double gendi,double gstartdirr, double gstartdiri, double genddirr, double genddiri, int slopecorrection);
void initbvectorquintic(generalpanel *out,double gstartr, double gstarti,double gendr, double gendi,double gstartdirr, double gstartdiri, double genddirr, double genddiri,double gstartdir2r, double gstartdir2i, double genddir2r, double genddir2i, int slopecorrection);
void initbvectorseptic(generalpanel *out,double gstartr, double gstarti,double gendr, double gendi,double gstartdirr, double gstartdiri, double genddirr, double genddiri,double gstartdir2r, double gstartdir2i, double genddir2r, double genddir2i, double gstartdir3r, double gstartdir3i, double genddir3r, double genddir3i, int slopecorrection);
void initbvectortarget(generalpanel *out,int tindex);
void inittargetbase(generalpanel *out);
void reducea_order(generalpanel *out);
void initgeneralpanelcvector(generalpanel *out);
void initgeneralpanelcvectortarget(generalpanel *out);
void initgeneralpanelconnectingterms(generalpanel *restrict out);
void determinepanelconnections(generalpanel *panels,int count);
void initgeneralpanelbox(generalpanel *restrict out);
double sqrdistancefrompanel(generalpanel *restrict currentpanel, cmplx point, int maxval);
void initgeneralpaneldist(generalpanel *restrict out);
void splitgeneralpanel(int currentpanelindex, panellist *restrict panels, int xsplit, double value, int** leftlist, int *leftind, int* leftalloc,
                       int** rightlist, int *rightind, int* rightalloc);
void splitgeneralpanels(panellist *restrict panels,int* panelptr,int N,int xsplit,double value,int** leftlist,int *leftind,int* leftalloc,
                       int** rightlist,int *rightind,int* rightalloc);
int performsplit(int currentpanelindex,int activepanelindex, panellist *restrict panels, double value, double baseendval,double startvalue,int identical,int newpanelindex=-1);
void initgeneralpanelboxfromextremepoints(generalpanel *newpanel,generalpanel *currentpanel,double minval,double maxval);
void applyslopeexpansion(int currentpanelindex,panellist *restrict panels,double tol,const double *coeffs,int hastarget=0,int** list=NULL,int *ind=NULL,int* alloc=NULL);
void normalizeb(generalpanel *currentpanel);
void shiftb(generalpanel *currentpanel);
void unshiftb(generalpanel *currentpanel);
void initgeneralpanelexpansion(generalpanel *restrict out,int Ncoeff);
void generalpaneldirectinteract(double *restrict xvelocities,double *restrict yvelocities,generalpanel *restrict currentpanel,const double *restrict xpositions,const double *restrict ypositions,int N);
cmplx disttoroot(double xposition,double yposition,generalpanel *restrict currentpanel, COMPLEXVECTORRESTRICT wksp,cmplx root,double val,double cutoff,int paneltopanelmode=0);
void multipoleevaluation(double *restrict xvelocities, double *restrict yvelocities, cmplx z, COMPLEXVECTORRESTRICT coeffs, int Ncoeff, double tol,double value);
void particletogeneralpanel(double *restrict xvelocities,double *restrict yvelocities,generalpanel *restrict currentpanel,
                            const double *restrict xpositions,const double *restrict ypositions,const double *restrict xstrengths,const double *restrict ystrengths,
                            SMOOTHER smooth,double cutofforig,double cutoff,double shape,double scale,int N);
cmplx farexpansiontogeneralpanel(generalpanel *restrict currentpanel,generalpanel *restrict sourcepanel,double dist);
void generalpaneltogeneralpanel(double *restrict xvelocities,double *restrict yvelocities,generalpanel *restrict currentpanel,
                            generalpanel *restrict sourcepanel,int N);
cmplx generalpaneltogeneralpanelinner(generalpanel *restrict currentpanel,
                            generalpanel *restrict sourcepanel,cmplx evalpanelstart,cmplx evalpanelend,cmplx evalrotvect, double endvalscale);
cmplx addpaneltopanelendpointcontribution(generalpanel *restrict currentpanel,generalpanel *restrict sourcepanel,cmplx evalpanel,
                                          COMPLEXVECTORRESTRICT cvector,COMPLEXVECTORRESTRICT r,COMPLEXVECTORRESTRICT wksp_1,COMPLEXVECTORRESTRICT wksp_2,double endvalscale,double addsign);
void freegeneralpanel(generalpanel* out);
void printgeneralpanel(generalpanel *in);
void checkpanelintegrity(panellist *panels);
#ifdef GENERALPANELSUPPORT
void MPexp_box_generalpanel(MPexp *This,const panellist *panels, int* validinput);
void evalgeneralpanelexpansionfmm(double *xvelocity, double *yvelocity, generalpanel *restrict currentpanel,dcmplx z,dcmplx *coeffs,int pcoeff);
void farexpansiontogeneralpanelfmm(double *restrict xvelocity,double *restrict yvelocity,generalpanel *restrict currentpanel, dcmplx z,dcmplx *coeffs,int pcoeff);
#include "../panel.h"
void createdummylistgeneral(double** dummy,int *dummyallocated,const panel* panels,const int* indices,int N,
                            panellist *sourcepanels,const int *sourceindices,int Nsource,
                            panellist *targetpanels,const int *targetindices,int Ntarget,
                            int xval,dcmplx z,dcmplx d);
#endif

void* myMalloc2(size_t size,char* file,int line);
void* myCalloc2(size_t size,size_t size2,char* file,int line);
void myFree2(void* ptr,char* file,int line);
#endif /*__GENERALPANEL_H*/
