//this file contains general functions for the general panel code, such as root finding algorithms, polynomial evaluation etc.



#include "generalpanel.h"
#include <math.h>
#include <float.h>
#include <mex.h>
#include <string.h>
//Visual studio specific code, expm1 does not exist in visual studio
#if !defined(__fmm_h) && defined(_MSC_VER) 
inline double expm1(double x)
{
  if (fabs(x) < 1e-4)
    return x*(1.0+0.5*x*(0.33333333333333333333*x+1.0));
  else
    return exp(x)-1.0;
}
#endif

//debug code that can be used to track memory leaks and errors
int alloccount=0;
void* myMalloc2(size_t size,char* file,int line) {
  void* ret=mxMalloc(size);
  alloccount++;
  mexPrintf("Allocated %ld bytes at address %p at line %d in file %s alloccount=%d\n",size,ret,line,file,alloccount);
  return ret;
}
void* myCalloc2(size_t size,size_t size2,char* file,int line) {
  void* ret=mxCalloc(size,size2);
  alloccount++;
  mexPrintf("Allocated %ldx%ld bytes at address %p at line %d in file %s alloccount=%d\n",size,size2,ret,line,file,alloccount);
  return ret;
}
void myFree2(void* ptr,char* file,int line) {
  alloccount--;
  mexPrintf("Released memory at address %p at line %d in file %s alloccount=%d\n",ptr,line,file,alloccount);
  mxFree(ptr);
}

//laguerres method for solving the roots to a polynomial, this is a standard method for doing this
int laguerre(cmplx *value, COMPLEXVECTOR coeffs,int order) {
  cmplx fval,dfval,d2fval,g,h,ctmp,ctmp2,v1,v2,valuestep,newvalue,oldvalue;
  double tmp,tmp2,error,absvalue,v1sqr,v2sqr,oldfval,newfval;
  static double fractions[]={0.5,0.5,0.75,0.25,0.625,0.125,0.875,0.375,0.03125,0.02,0.01,0.005,1.0}; //just a list of values to reduce the step with in case of a limit cycle. Note that first value is not used
  if(order<=0)
    return 1;
  if(order==1) { //no point iterating if it is linear, determine solution directly instead. Removes the need to handle the order=1 case
    COMPLEXDIV(*value,coeffs[0],coeffs[1],tmp,tmp2);
    COMPLEXNEG(*value,*value);
    return 1;
  }
  if(isnan(value->re)||isnan(value->im)) // Safety check, can probably be removed in the future
    mexWarnMsgTxt("NaN sent to laguerre");
  for(int i=1;i<MAXITER;i++) { //start from 1 to not use a fraction at the first iteration
    absvalue=COMPLEXABS(*value);
    if(absvalue==0.0) { //special case if value==0, no point in evaluating the polynomial since all other terms will be zero
      fval=coeffs[0];
      dfval=coeffs[1];
      d2fval=coeffs[2];
      if(fval.re==0.0&&fval.im==0.0)
        return 1;
    }
    else { //determine the function value and its first and second derivative
      //first three terms unrolled to remove some unnecessary calculations
      COMPLEXASSIGN(fval, coeffs[order].re, coeffs[order].im);
      error=COMPLEXABS(fval);
      dfval=fval;
      COMPLEXMUL(fval, fval, *value, tmp);
      COMPLEXADD(fval, fval, coeffs[order-1]);
      error=error*absvalue+fabs(fval.re)+fabs(fval.im);//COMPLEXABS(fval);
      d2fval=dfval;
      COMPLEXMUL(dfval, dfval, *value, tmp);
      COMPLEXADD(dfval, dfval, fval);
      COMPLEXMUL(fval, fval, *value, tmp);
      COMPLEXADD(fval, fval, coeffs[order-2]);
      tmp=fabs(fval.re)+fabs(fval.im);//COMPLEXABS(fval);
      error=error*absvalue+tmp;
      for(int j=order-3;j>=0;j--) { //continue with the rest of the terms
        COMPLEXMUL(d2fval, d2fval, *value, tmp);
        COMPLEXADD(d2fval, d2fval, dfval);
        COMPLEXMUL(dfval, dfval, *value, tmp);
        COMPLEXADD(dfval, dfval, fval);
        COMPLEXMUL(fval, fval, *value, tmp);
        COMPLEXADD(fval, fval, coeffs[j]);
//        tmp=COMPLEXABS(fval);
        tmp=fabs(fval.re)+fabs(fval.im); //makes the code approximately 12 % faster for third order polynomials
        error=error*absvalue+tmp; //stopping criterion of Adams, as suggested in Numerical Recipies in C
      }
      if(tmp<=DBL_EPSILON*1.2*1.4*error) //1.2 added from experience, as values of 1.14 have been observed, 1.4 is added due to the approximation fabs(fval.re)+fabs(fval.im) instead of the real distance 
        return i;
      if(absvalue<1e-120) //second convergence criterion, added to block smaller values than minimum double value
        return i;
    }
    // added safety function to the code in case of non-convergence. After sufficient number of iterations, start blocking moves that increases the value of fval, and take half prevoius valuestep istead
    // this can happen as laguerre is not fully stable in its original implementation, but no cases have been observed where this hasn't solved the problem
    if(i>NRFRACTIONS*USEFRACTION) { 
      newfval=COMPLEXABS(fval);
      if(i>NRFRACTIONS*USEFRACTION+1&&newfval>oldfval) {
        COMPLEXASSIGN(*value,oldvalue.re-valuestep.re*0.5,oldvalue.im-valuestep.im*0.5);
        COMPLEXASSIGN(valuestep,valuestep.re*0.5,valuestep.im*0.5);
        continue;
      }
      oldvalue=*value;
      oldfval=newfval;
    }

    COMPLEXASSIGN(d2fval,d2fval.re*2,d2fval.im*2); //multiply second derivative with 2 to get the correct value
    COMPLEXDIV(g,dfval,fval,tmp,tmp2); //g=dfval/fval
    COMPLEXMUL(ctmp,g,g,tmp);
    COMPLEXDIV(ctmp2,d2fval,fval,tmp,tmp2);
    COMPLEXSUB(h,ctmp,ctmp2);          //h=((dfval/fval)^2-d2fval/fval)/2
    COMPLEXASSIGN(ctmp2,order*h.re-ctmp.re,order*h.im-ctmp.im); //(nH-G^2)
    COMPLEXASSIGN(ctmp2,ctmp2.re*(order-1),ctmp2.im*(order-1)); //(n-1)*(nH-G^2)
//    COMPLEXSQRT(ctmp2,ctmp2,tmp,tmp2);                          //sqrt((n-1)*(nH-G^2))
    ctmp2=sqrt(ctmp2);
    COMPLEXADD(v1,g,ctmp2)                                      //G+sqrt((n-1)*(nH-G^2))
    COMPLEXSUB(v2,g,ctmp2);                                     //G-sqrt((n-1)*(nH-G^2))
    v1sqr=COMPLEXNORM(v1);
    v2sqr=COMPLEXNORM(v2);
    if(v1sqr>v2sqr) { //pick the larges of the two valeus
      COMPLEXRECIPROCAL(ctmp,v1,tmp);
      COMPLEXASSIGN(valuestep,ctmp.re*order,ctmp.im*order);
    }
    else {
      if(v2sqr!=0.0) {
        COMPLEXRECIPROCAL(ctmp,v2,tmp);
        COMPLEXASSIGN(valuestep,ctmp.re*order,ctmp.im*order);
      }
      else { //pick a new guess, quite unlikely
        COMPLEXASSIGN(valuestep,(absvalue+1)*sin((double)i),(absvalue+1)*cos((double)i)); //using same expression as suggested in Numerical Recipies in C, although this should be other possible options as well
      }
    }
    
    COMPLEXSUB(newvalue,*value,valuestep);
    if(newvalue.re==value->re&&newvalue.im==value->im)
      return i;
    if(i%USEFRACTION==0&&i<=NRFRACTIONS*USEFRACTION) { //designed to break possible repeating cycles
      COMPLEXASSIGN(*value,(*value).re-valuestep.re*fractions[i/USEFRACTION],(*value).im-valuestep.im*fractions[i/USEFRACTION]);
    }
    else {
      *value=newvalue;
    }
  }
  //in case of non-convergence, print error information for debugging. If this occurs, the code should be adjusted
  if(COMPLEXABS(fval)>10*error*DBL_EPSILON) { //Only print more serious errors that may affect the results of the computation
    mexWarnMsgTxt("laguerre root finding did not converge");
    mexPrintf("error=%.16e value=%.16e missing quota=%e\n",error,COMPLEXABS(fval),COMPLEXABS(fval)/(error*DBL_EPSILON));
    COMPLEXPRINT(*value);
    COMPLEXPRINT(newvalue);
    COMPLEXPRINT(valuestep);
    for (int i=0;i<=order;i++)
      mexPrintf("coeff[%i]=%.16e+%.16ei\n",i,coeffs[i].re,coeffs[i].im);
  }
  return 0;
}

//calls laguerr for each root
void polyroots(COMPLEXVECTOR roots,COMPLEXVECTOR coeffs,int order,COMPLEXVECTOR wksp, COMPLEXVECTOR wksp2)
{
  double tmp,tmp2;
  cmplx ctmp,ctmp2;
  for(int i=0;i<=order;i++) {
    wksp[i]=coeffs[i];
  }
  if(order<=0)
      return;

  //the simple case of first order polynomial
  if(order==1) {
    COMPLEXDIV(roots[0],coeffs[0],coeffs[1],tmp,tmp2);
    COMPLEXNEG(roots[0],roots[0]);
  }
  else if(order==2) { //second order polynomial, still easy to solve analytically
    quadraticroots(roots,coeffs); //can be inaccurate due to truncation errors
    laguerre(&roots[0],coeffs,order); //refine roots for higher accuracy
    laguerre(&roots[1],coeffs,order);
  }
  //higher order polynomials
  else {
    for(int i=0;i<order-2;i++) {
      COMPLEXASSIGN(roots[i], 0, 0); //starting guess is 0
      laguerre(&roots[i], wksp, order-i); //find root
      ctmp2=wksp[order-i];
      for(int j=order-i-1;j>=0;j--) { //use polynomial division to remove root from polynomial
        COMPLEXMUL(ctmp, ctmp2, roots[i], tmp);
        COMPLEXADD(ctmp, wksp[j], ctmp);
        wksp[j]=ctmp2;
        ctmp2=ctmp;
      }
    }

    //when the order is reduced down to 2 remaining roots, use traditional solution for quadratic polynomials instead
#ifdef DEBUGMODE
    quadraticroots(myvector<cmplx>(&roots[order-2],roots.getcount()+2-order),wksp);
#else
    quadraticroots(&roots[order-2],wksp);
#endif

    //refinement step, run laguerre on the original polynomial for each root to refine the value to avoid numerical errors
    for(int i=1;i<order;i++) { //could perhaps be omitted for low order polynomials, would in that case make code approximately 20 % faster
      cmplx oldroot=roots[i];
      laguerre(&roots[i],coeffs,order);
      int duplicate=-1;
      for(int j=0;j<order;j++) { //check if two roots contain the same values
        if(j==i)
          continue;
        if(fmax(fabs(roots[i].re)+fabs(roots[i].im),fabs(roots[j].re)+fabs(roots[j].im))*1e-15>=fabs(roots[i].re-roots[j].re)+fabs(roots[i].im-roots[j].im)&&
                !(fmax(fabs(oldroot.re)+fabs(oldroot.im),fabs(roots[j].re)+fabs(roots[j].im))*1e-15>=fabs(oldroot.re-roots[j].re)+fabs(oldroot.im-roots[j].im))) {
          duplicate=j;
          break;
        }
      }

      //if duplicates are found, this can mean that one root was found twice, divide away the root and do another search to find the proper second root
      if(duplicate>=0) {
        for(int n=0;n<=order;n++) {
          wksp[n]=coeffs[n];
          wksp2[n]=roots[n];
        }
        int m=0;
        do {
          for(int j=order-m-1;j>=0;j--) { //use polynomial division to remove the duplicate root from polynomial
            COMPLEXMUL(ctmp, ctmp2, wksp2[duplicate], tmp);
            COMPLEXADD(ctmp, wksp[j], ctmp);
            wksp[j]=ctmp2;
            ctmp2=ctmp;
          }
          for(int j=duplicate;j<order-m;j++) //remove root
            wksp2[j]=wksp2[j+1];
          roots[i]=oldroot;
          laguerre(&roots[i], wksp, order); //search for new root

          //do the duplicate search again for this new root to ensure that it is no longer a duplicate
          duplicate=-1;
          for(int j=0;j<order-m;j++) {
            if(j==i)
              continue;
            if(fmax(fabs(roots[i].re)+fabs(roots[i].im),fabs(roots[j].re)+fabs(roots[j].im))*1e-15>=fabs(roots[i].re-roots[j].re)+fabs(roots[i].im-roots[j].im)&&
                    !(fmax(fabs(oldroot.re)+fabs(oldroot.im),fabs(roots[j].re)+fabs(roots[j].im))*1e-15>=fabs(oldroot.re-roots[j].re)+fabs(oldroot.im-roots[j].im))) {
              duplicate=j;
              break;
            }
          }
          m++;
        }
        while(duplicate>=0);
      }
    }
  }
}

//calculates coefficients for (coeff[0]+coeff[1]*x+coeff[2]*x^2+...)*conj((coeff[0]+coeff[1]*x+coeff[2]*x^2+...))
//note that only real part is of interest, but polyroots takes complex input
void polynorm(COMPLEXVECTOR ret,COMPLEXVECTOR coeff,int Ncoeff) { 
  for(int i=0;i<2*Ncoeff-1;i++) {
    COMPLEXASSIGN(ret[i],0.0,0.0);
  }
  for(int i=0;i<Ncoeff;i++) {
    ret[2*i].re+=coeff[i].re*coeff[i].re+coeff[i].im*coeff[i].im;
    for(int j=i+1;j<Ncoeff;j++) { //due to symmetry, only half of the coefficients are necessary
      ret[i+j].re+=2*(coeff[i].re*coeff[j].re+coeff[i].im*coeff[j].im);
    }
  }
}

//same as above, but returns results as double* instead of cmplx*
void polynorm(DOUBLEVECTOR ret,COMPLEXVECTOR coeff,int Ncoeff) {
  for(int i=0;i<2*Ncoeff-1;i++) {
    ret[i]=0.0;
  }
  for(int i=0;i<Ncoeff;i++) {
    ret[2*i]+=coeff[i].re*coeff[i].re+coeff[i].im*coeff[i].im;
    for(int j=i+1;j<Ncoeff;j++) {
      ret[i+j]+=2*(coeff[i].re*coeff[j].re+coeff[i].im*coeff[j].im);
    }
  }
}

//function to initialize constans used for analytical solution of cubic polynomials
void initcubicconstants_areal(cubicconstants* out,generalpanel *in) {
  double tmp;
  cmplx ctmp;
  COMPLEXASSIGN(out->D0,in->c[2].re*in->c[2].re-3*in->c[1].re,-3*in->c[1].im);                                                //D0=c2^2-3*c1
  COMPLEXASSIGN(out->D1P1,in->c[2].re*(2*in->c[2].re*in->c[2].re-9*in->c[1].re),-9*in->c[2].re*in->c[1].im);                  //D1P1=2*c2^3-9*c2*c1
  tmp=out->D0.re*out->D0.re-out->D0.im*out->D0.im;
  COMPLEXASSIGN(out->D0c,4*out->D0.re*(out->D0.re*out->D0.re-3*out->D0.im*out->D0.im),4*out->D0.im*(3*out->D0.re*out->D0.re-out->D0.im*out->D0.im)); //D0c=4*D0^3
  COMPLEXASSIGN(out->DT1,-27*in->c[2].re*in->c[2].re+4*27*in->c[1].re,4*27*in->c[1].im);
  COMPLEXSQR(ctmp,in->c[1],tmp);
  COMPLEXMUL(out->DT1, out->DT1, ctmp, tmp);                                                 //DT1=-27*c1^2(c2^2-4*c1)
  COMPLEXASSIGN(out->DT2,-27*18*in->c[2].re*in->c[1].re+4*27*in->c[2].re*in->c[2].re*in->c[2].re,-27*18*in->c[2].re*in->c[1].im);//DT2=-27*18*c2*c1+4*27*c2^3
}

//function to initialize constans used for analytical solution of cubic polynomials, the version for complex a vector
void initcubicconstants_aimag(cubicconstants* out,generalpanel *in) {
  double tmp;
  cmplx ctmp,ctmp2;
  COMPLEXSQR(ctmp,in->c[2],tmp);
  COMPLEXASSIGN(out->D0,ctmp.re-3*in->c[1].re,ctmp.im-3*in->c[1].im);                                   //D0=c2^2-3*c1
  COMPLEXASSIGN(ctmp2,ctmp.re*2-9*in->c[1].re,ctmp.im*2-9*in->c[1].im);
  COMPLEXMUL(out->D1P1,ctmp2,in->c[2],tmp);                                                             //D1P1=2*c2^3-9*c2*c1
  tmp=out->D0.re*out->D0.re-out->D0.im*out->D0.im;
  COMPLEXASSIGN(out->D0c,4*out->D0.re*(out->D0.re*out->D0.re-3*out->D0.im*out->D0.im),4*out->D0.im*(3*out->D0.re*out->D0.re-out->D0.im*out->D0.im)); //D0c=4*D0^3
  COMPLEXASSIGN(out->DT1,-27*(ctmp.re-4*in->c[1].re),-27*(ctmp.im-4*in->c[1].im));
  COMPLEXSQR(ctmp2,in->c[1],tmp);
  COMPLEXMUL(out->DT1, out->DT1, ctmp2, tmp);                                                 //DT1=-27*c3^2(c2^2-4*c1)
  COMPLEXASSIGN(ctmp2,-27*18*in->c[1].re+4*27*ctmp.re,-27*18*in->c[1].im+4*27*ctmp.im);
  COMPLEXMUL(out->DT2,ctmp2,in->c[2],tmp);                                               //DT2=-27*18*c2*c1+4*27*c2^3
}

//function to determine the roots of a cubic polynomial using the analytical solution for the cubic polynomial
void cubicroots(COMPLEXVECTOR roots,COMPLEXVECTOR coeffs,cubicconstants *in,COMPLEXVECTOR wksp,COMPLEXVECTOR wksp2) {
  cmplx D1,ctmp,ctmp2,ctmp3,C;
  double tmp,tmp2;
  //determine the roots of the equation from the exact solution for the roots for a complex third order equation
  COMPLEXASSIGN(D1, in->D1P1.re+27*coeffs[0].re, in->D1P1.im+27*coeffs[0].im);              //D1=D1P1+27*c4 = 2*c2^3-9*c2*c3+27*c4
  //the expression below suffers more from truncation error than the one used
//              COMPLEXMUL(ctmp,D1,D1,tmp);                                       //ctmp=D1^2
//              COMPLEXSUB(ctmp,ctmp,D0c);                                        //ctmp=D1^2-4*D0^3
  COMPLEXASSIGN(ctmp3, in->DT2.re+27*27*coeffs[0].re, in->DT2.im+27*27*coeffs[0].im);
  COMPLEXMUL(ctmp3, ctmp3, coeffs[0], tmp);
  COMPLEXADD(ctmp3, ctmp3, in->DT1);                                              //ctmp3=DT1+c4*DT2-27^2*c4^3=D1^2-4*D0^3
  ctmp2=sqrt(ctmp3);
  COMPLEXASSIGN(ctmp, 0.5*(D1.re+ctmp2.re), 0.5*(D1.im+ctmp2.im));                //ctmp=(D1+sqrt(D1^2-4*D0^3))/2
  tmp=COMPLEXANGLE(ctmp)/3;
  tmp2=pow(ctmp.re*ctmp.re+ctmp.im*ctmp.im, 1.0/6.0);
  COMPLEXASSIGN(C, tmp2*cos(tmp), tmp2*sin(tmp));                                 //C=cbrt(ctmp)
  COMPLEXDIV(ctmp, in->D0, C, tmp, tmp2);                                         //ctmp=D0/C
  COMPLEXASSIGN(roots[0], -(1.0/3.0)*(coeffs[2].re+C.re+ctmp.re), -(1.0/3.0)*(coeffs[2].im+C.im+ctmp.im)); //a=1/3*(c2+1*C+D0/(1*C))
  COMPLEXASSIGN(roots[1], -(1.0/3.0)*(coeffs[2].re-0.5*C.re-HALFSQRT3*C.im-0.5*ctmp.re+HALFSQRT3*ctmp.im), -(1.0/3.0)*(coeffs[2].im-0.5*C.im+HALFSQRT3*C.re-0.5*ctmp.im-HALFSQRT3*ctmp.re));//b=1/3*(c2+(-1+1i*sqrt(3))/2*C+D0/((-1+1i*sqrt(3))/2*C))
  COMPLEXASSIGN(roots[2], -(1.0/3.0)*(coeffs[2].re-0.5*C.re+HALFSQRT3*C.im-0.5*ctmp.re-HALFSQRT3*ctmp.im), -(1.0/3.0)*(coeffs[2].im-0.5*C.im-HALFSQRT3*C.re-0.5*ctmp.im+HALFSQRT3*ctmp.re));//c=1/3*(c2+(-1-1i*sqrt(3))/2*C+D0/((-1-1i*sqrt(3))/2*C))
  if(!refineroot(&roots[0],coeffs,3)||!refineroot(&roots[1],coeffs,3)||!refineroot(&roots[2],coeffs,3)) { //if refineroot did not work properly, use polyroots instead
    polyroots(roots,coeffs,3,wksp,wksp2);
  }
  if((fmax(fabs(roots[0].re)+fabs(roots[0].im),fabs(roots[1].re)+fabs(roots[1].im))*1e-15>=fabs(roots[0].re-roots[1].re)+fabs(roots[0].im-roots[1].im))||
     (fmax(fabs(roots[0].re)+fabs(roots[0].im),fabs(roots[2].re)+fabs(roots[2].im))*1e-15>=fabs(roots[0].re-roots[2].re)+fabs(roots[0].im-roots[2].im))||
     (fmax(fabs(roots[1].re)+fabs(roots[1].im),fabs(roots[2].re)+fabs(roots[2].im))*1e-15>=fabs(roots[1].re-roots[2].re)+fabs(roots[1].im-roots[2].im))) {
    polyroots(roots,coeffs,3,wksp,wksp2); //for duplicate roots, since this may be an incorrect results due to the refinement, use polyroots instead to find the roots, as this code handles false duplicate roots
  }
}

#define MAXREFINE 10
//function to refine a polynomial root using newton rapson iteration. One newton rapson iteration is faster than one laguerre iteration, but laguerre has more reliable convergence.
//this function can be used when the starting guess is very good.
int refineroot(cmplx *value,COMPLEXVECTOR coeffs,int order) {
  cmplx fval,dfval,ctmp;
  double tmp,tmp2,error,absvalue;
  absvalue=sqrt(value->re*value->re+value->im*value->im);  //this value should not change significantly during a refinement
  for(int i=1;i<MAXREFINE;i++) { //start from 1 to not use a fraction at the first iteration
    
    //evaluate function value and derivative
    //first two terms unrolled to remove some unnecessary calculations
    COMPLEXASSIGN(fval, coeffs[order].re, coeffs[order].im);
    error=fabs(fval.re)+fabs(fval.im);//COMPLEXABS(fval);
    dfval=fval;
    COMPLEXMUL(fval, fval, *value, tmp);
    COMPLEXADD(fval, fval, coeffs[order-1]);
    error=error*absvalue+fabs(fval.re)+fabs(fval.im);
    for(int j=order-2;j>=0;j--) { //continue with the rest of the terms
      COMPLEXMUL(dfval, dfval, *value, tmp);
      COMPLEXADD(dfval, dfval, fval);
      COMPLEXMUL(fval, fval, *value, tmp);
      COMPLEXADD(fval, fval, coeffs[j]);
      tmp=fabs(fval.re)+fabs(fval.im); //makes the code approximately 12 % faster for third order polynomials
      error=error*absvalue+tmp; //stopping criterion of Adams, as suggested in Numerical Recipies in C
    }

    //convergence check
    if(tmp<=DBL_EPSILON*1.2*1.4*error) //1.2 added from experience, as values of 1.14 have been observed, 1.4 is added due to the approximation fabs(fval.re)+fabs(fval.im) instead of the real distance
      return i;
    COMPLEXDIV(ctmp,fval,dfval,tmp,tmp2);
    COMPLEXSUB(*value,*value,ctmp);
  }
  return 0;
}

//analytical solution for quadratic polynomials
void quadraticroots(COMPLEXVECTOR roots,COMPLEXVECTOR coeffs) {
  cmplx ctmp;
  double tmp,tmp2;
  if(coeffs[2].re!=1.0||coeffs[2].im!=0.0) {
    cmplx c0,c1;
    //TODO: can be improved using that the denominator is the same in both cases.
    COMPLEXDIV(c0,coeffs[0],coeffs[2],tmp,tmp2); //normalize the highest order coefficient
    COMPLEXDIV(c1,coeffs[1],coeffs[2],tmp,tmp2);
    COMPLEXASSIGN(ctmp,c1.re*c1.re-c1.im*c1.im-4*c0.re,2*c1.re*c1.im-4*c0.im);
    ctmp=sqrt(ctmp);
    COMPLEXASSIGN(roots[0],0.5*(-c1.re+ctmp.re),0.5*(-c1.im+ctmp.im));
    COMPLEXASSIGN(roots[1],0.5*(-c1.re-ctmp.re),0.5*(-c1.im-ctmp.im));
  }
  else { //the simplified case where the highest order coefficient is 1.0
    COMPLEXASSIGN(ctmp,coeffs[1].re*coeffs[1].re-coeffs[1].im*coeffs[1].im-4*coeffs[0].re,2*coeffs[1].re*coeffs[1].im-4*coeffs[0].im);
    ctmp=sqrt(ctmp);
    COMPLEXASSIGN(roots[0],0.5*(-coeffs[1].re+ctmp.re),0.5*(-coeffs[1].im+ctmp.im));
    COMPLEXASSIGN(roots[1],0.5*(-coeffs[1].re-ctmp.re),0.5*(-coeffs[1].im-ctmp.im));
  }
}
//this code performs the series expansion of val {1/(x-val)=sum[n=0 to infinity](x/val)^n} and multiplies this polynomial to the polynomial in "in"
//note: tol should be squared, tolval should be 1/abs(val), used as an input, as it is already available where it is called
//returns the number of degrees added to the polynomial
int expandandmultiplypoly(COMPLEXVECTOR ret,COMPLEXVECTOR in,cmplx val,double tol,double tolval,int N,int maxiter) {
  cmplx newval=val;
  double tmp,newtolval=tolval;
  for(int k=0;k<N;k++) {
    ret[k]=in[k];
  }

  //the first step outside the main loop, the code adds one term of the expansion at a time
  if(maxiter>0&&newtolval>tol) {
    for(int k=0;k<N-1;k++) {
      COMPLEXMADD(ret[k+1],ret[k+1],in[k],newval,tmp);
    }
    COMPLEXMUL(ret[N],in[N-1],newval,tmp);
    newtolval*=tolval;
  }
  else
    return 0;
  int i=1;
  for(;i<maxiter&&newtolval>tol;i++,newtolval*=tolval) {
    COMPLEXMUL(newval,newval,val,tmp);
    for(int k=0;k<N-1;k++) {
      COMPLEXMADD(ret[k+i+1],ret[k+i+1],in[k],newval,tmp);
    }
    COMPLEXMUL(ret[i+N],in[N-1],newval,tmp);
  }
  return i;
}

//calculates the sum over n from 1 to maxstep (1/n)*step^(n-1)*(t1^n)
cmplx onetermseries(cmplx t1,double tol, int maxstep) {
  double tmp;
  cmplx t1tmp=t1;
  cmplx ret=t1;
  int i;
  for(i=2;i<=maxstep;i++) {
    COMPLEXMUL(t1tmp,t1tmp,t1,tmp);
    COMPLEXADDC(ret,ret,t1tmp,1.0/((double)i));
    if(fabs(t1tmp.re)+fabs(t1tmp.im)<tol*(fabs(ret.re)+fabs(ret.im)))
      break;
  }
  return ret;
}
//calculates the sum over n from 1 to maxstep (1/n)*step^(n-1)*(t1^n-t2^n)
cmplx twotermseries(cmplx t1,cmplx t2,cmplx step,double tol, int maxstep) {
  double tmp;
  cmplx t1tmp=t1;
  cmplx t2tmp=t2;
  cmplx steptmp=step;
  cmplx ret;
  cmplx ctmp;
  COMPLEXSUB(ret,t1,t2);
  int i;
  for(i=2;i<=maxstep;i++) {
    COMPLEXMUL(t1tmp,t1tmp,t1,tmp);
    COMPLEXMUL(t2tmp,t2tmp,t2,tmp);
    COMPLEXSUB(ctmp,t1tmp,t2tmp);
    COMPLEXMUL(ctmp,ctmp,steptmp,tmp);
    COMPLEXADDC(ret,ret,ctmp,1.0/((double)i));
    if(fabs(ctmp.re)+fabs(ctmp.im)<tol*(fabs(ret.re)+fabs(ret.im)))
      break;
    COMPLEXMUL(steptmp,steptmp,step,tmp);
  }
  return ret;
}
//calculates the sum over n from 1 to maxstep t1^(n-1)*t2^n
cmplx prodsum(cmplx t1,cmplx t2,double tol,int maxstep) {
  double tmp;
  cmplx steptmp;
  cmplx ret=t2;
  cmplx step;
  COMPLEXMUL(step,t1,t2,tmp);
  COMPLEXMUL(steptmp,step,t2,tmp);
  COMPLEXADD(ret,ret,steptmp);
  for(int i=2;i<=maxstep;i++) {
    COMPLEXMUL(steptmp,steptmp,step,tmp);
    COMPLEXADD(ret,ret,steptmp);
    if((fabs(ret.re)+fabs(ret.im))*tol>(fabs(steptmp.re)+fabs(steptmp.im)))
      break;
  }
  //COMPLEXPRINT(steptmp);
  //mexPrintf("Prodsum did not converge, tol=%e\n",(fabs(steptmp.re)+fabs(steptmp.im))/(fabs(ret.re)+fabs(ret.im)));
  //COMPLEXPRINT(ret);
  return ret;
}
//evaluates sum(n=1 to Ncoeff-1)coeff[n]*{sum(m=0 to n-1)val1^m*val2^(n-1-m)}
cmplx evalpolyprod(COMPLEXVECTOR coeff,cmplx val1,cmplx val2,int Ncoeff) {
  cmplx ret;
  cmplx oldsum,val2tmp;
  double tmp;
  if(Ncoeff<2) {
    COMPLEXASSIGN(ret,0,0);
    return ret;
  }
  ret=coeff[1];
  if(Ncoeff>2) {
    COMPLEXADD(oldsum,val1,val2);
    COMPLEXMADD(ret,ret,oldsum,coeff[2],tmp);
    val2tmp=val2;
  }
  for(int k=3;k<Ncoeff;k++) {
    COMPLEXMUL(oldsum,oldsum,val1,tmp);
    COMPLEXMUL(val2tmp,val2tmp,val2,tmp);
    COMPLEXADD(oldsum,oldsum,val2tmp);
    COMPLEXMADD(ret,ret,oldsum,coeff[k],tmp);
  }
  return ret;
}

//constans used for gauss konrod quadrature
#define MAXGAUSSKONRODLEVEL 30
const double konrod15points[]={-0.9914553711208126392068547,
-0.9491079123427585245261897,
-0.8648644233597690727897128,
-0.7415311855993944398638648,
-0.5860872354676911302941448,
-0.4058451513773971669066064,
-0.2077849550078984676006894,
0.0,
0.2077849550078984676006894,
0.4058451513773971669066064,
0.5860872354676911302941448,
0.7415311855993944398638648,
0.8648644233597690727897128,
0.9491079123427585245261897,
0.9914553711208126392068547};
const double konrod15weights[]={0.0229353220105292249637320,
0.0630920926299785532907007,
0.1047900103222501838398763,
0.1406532597155259187451896,
0.1690047266392679028265834,
0.1903505780647854099132564,
0.2044329400752988924141620,
0.2094821410847278280129992,
0.2044329400752988924141620,
0.1903505780647854099132564,
0.1690047266392679028265834,
0.1406532597155259187451896,
0.1047900103222501838398763,
0.0630920926299785532907007,
0.0229353220105292249637320};
const double gauss7weights[]={0.0,
0.1294849661688696932706114,
0.0,
0.2797053914892766679014678,
0.0,
0.3818300505051189449503698,
0.0,
0.4179591836734693877551020,
0.0,
0.3818300505051189449503698,
0.0,
0.2797053914892766679014678,
0.0,
0.1294849661688696932706114,
0.0};

//the basic function to call for numerical integration using gauss konrod quadrature. This function assumes that there are no sigularities in the interval. For singularities, se functions below
//the initial function splits the interval in inintervals steps to get a suitable number of starting points
//f should be the function to integrate, and params can be used to send additional data to that function
cmplx gausskonrod(cmplx (*f)(double,void*),double a,double b,void* params,double tol,int initintervals) {
  cmplx ret,ans;
  COMPLEXASSIGN(ret,0,0);
  double diff=(b-a)/initintervals;
  for(int i=0;i<initintervals;i++) {
    ans=gausskonrodinner(f,a+i*diff,a+(i+1)*diff,params,tol/initintervals,1);
    COMPLEXADD(ret,ret,ans);
  }
  return ret;
}

//the internal recursive part of gauss konrod that adaptively refines the integral if the precision is not sufficient
cmplx gausskonrodinner(cmplx (*f)(double,void*),double a,double b,void* params,double tol,int level) {
  cmplx ret,ctmp,ans,gaussum;
  COMPLEXASSIGN(ret,0,0);
  COMPLEXASSIGN(gaussum,0,0);
  double diff=b-a;
  double offset1=0.5*diff;
  double offset2=0.5*(b+a);
  for(int i=0;i<15;i++) {
    ans=f(konrod15points[i]*offset1+offset2,params); //evaluate the function in the 15 points
    COMPLEXADDC(ret,ret,ans,konrod15weights[i]); //calculate both values
    COMPLEXADDC(gaussum,gaussum,ans,gauss7weights[i]);//calculate both values
  }
  COMPLEXSUB(ctmp,ret,gaussum); //the difference between the two estimates
  double error=200*offset1*COMPLEXABS(ctmp); //the error estimate
  error*=sqrt(error);
  if(error<=tol||level>MAXGAUSSKONRODLEVEL) { //check for convergence
    COMPLEXASSIGN(ret,ret.re*offset1,ret.im*offset1);
    return ret;
  }
  //if not converged, split interval in 2 and continue
  ret=gausskonrodinner(f,a,a+offset1,params,tol/2,level+1);
  ctmp=gausskonrodinner(f,a+offset1,b,params,tol/2,level+1);
  COMPLEXADD(ret,ret,ctmp);
  return ret;
}

//simple implementation of bubble sort for sorting short vectors
void bubblesort(DOUBLEVECTOR vals,int N) {
  for(int i=0;i<N-1;i++) {
    int swapped=0;
    for(int j=0;j<N-1-i;j++) {
      if(vals[j]>vals[j+1]) {
        double tmp=vals[j+1];
        vals[j+1]=vals[j];
        vals[j]=tmp;
        swapped=1;
      }
    }
    if(!swapped)
      break;
  }
}

//sorts a vector of complex elements by theri real part
void bubblesortreal(COMPLEXVECTOR vals,int N) {
  for(int i=0;i<N-1;i++) {
    int swapped=0;
    for(int j=0;j<N-1-i;j++) {
      if(vals[j].re>vals[j+1].re) {
        cmplx tmp=vals[j+1];
        vals[j+1]=vals[j];
        vals[j]=tmp;
        swapped=1;
      }
    }
    if(!swapped)
      break;
  }
}

//a version of gauss konrod quadrature that handles singularities better than the original one through splitting the interval to cause the sigularities to be located at the end points, otherwise works as original gauss konrod
cmplx gausskonrod_singularities(cmplx (*f)(double,void*),double a,double b,void* params,DOUBLEVECTOR singularities,int Nsingularities,double tol,int initintervals) {
  cmplx ret,ctmp;
  bubblesort(singularities,Nsingularities); //number of singularities should be low, bubble sort is simple and does not require a workspace
  int i=0;
  for(;i<Nsingularities&&singularities[i]<a;i++); //ignore singularities outside interval
  if(i>=Nsingularities)
    ret=gausskonrod_endsingularities(f,a,b,params,tol,initintervals);
  else {
    ret=gausskonrod_endsingularities(f,a,singularities[i],params,tol,initintervals);
    i++;
    for(;i<Nsingularities&&singularities[i]<b;i++) {
      REALPRINT(singularities[i]);
      ctmp=gausskonrod_endsingularities(f,singularities[i-1],singularities[i],params,tol,initintervals);
      COMPLEXADD(ret,ret,ctmp);
    }
    ctmp=gausskonrod_endsingularities(f,singularities[i-1],b,params,tol,initintervals);
    COMPLEXADD(ret,ret,ctmp);
  }
  return ret;
}

//performs gauss kondrod quadrature using variable transform to remove endpoint singularies. The variable transform is the same as the one used in quadgk in matlab
cmplx gausskonrod_endsingularities(cmplx (*f)(double,void*),double a,double b,void* params,double tol,int initintervals) {
  cmplx ret,ans;
  COMPLEXASSIGN(ret,0,0);
  double diff=2.0/double(initintervals);
  for(int i=0;i<initintervals;i++) {
    ans=gausskonrodinner_endsingularities(f,-1+i*diff,-1+(i+1)*diff,(b-a)*0.25,0.5*(b+a),params,tol/initintervals,1);
    COMPLEXADD(ret,ret,ans);
  }
  return ret;
}

//the recursive version part of the gauss konrod quadrature with end point singularities included
cmplx gausskonrodinner_endsingularities(cmplx (*f)(double,void*),double a,double b,double diffglobal,double offsetglobal,void* params,double tol,int level) {
  cmplx ret,ctmp,ans,gaussum;
  COMPLEXASSIGN(ret,0,0);
  COMPLEXASSIGN(gaussum,0,0);
  double diff=b-a;
  double offset1=0.5*diff;
  double offset2=0.5*(b+a);
  for(int i=0;i<15;i++) {
    double localpoint=konrod15points[i]*offset1+offset2;
    double tmp=localpoint*localpoint;
    localpoint=diffglobal*localpoint*(3-tmp)+offsetglobal; //(B-A)/4*x*(1-x^2)+(B+A)/2
    ans=f(localpoint,params);
    tmp=3*diffglobal*(1-tmp); //  3/4*(B-A)*(1-x^2)
    COMPLEXASSIGN(ans,ans.re*tmp,ans.im*tmp);
    
    COMPLEXADDC(ret,ret,ans,konrod15weights[i]);
    COMPLEXADDC(gaussum,gaussum,ans,gauss7weights[i]);
  }
  COMPLEXSUB(ctmp,ret,gaussum);
  //please note that this error estimate appears to have issues with functions that are not continous or have continous derivative 
  double error=200*offset1*COMPLEXABS(ctmp);
  error*=sqrt(error);
  if(error<=tol||level>MAXGAUSSKONRODLEVEL) {
    COMPLEXASSIGN(ret,ret.re*offset1,ret.im*offset1);
    return ret;
  }
  ret=gausskonrodinner_endsingularities(f,a,a+offset1,diffglobal,offsetglobal,params,tol/*/2*/,level+1); //note:no need to reduce tol with 2, as the answer is divided by the step length afterwards
  ctmp=gausskonrodinner_endsingularities(f,a+offset1,b,diffglobal,offsetglobal,params,tol/*/2*/,level+1);
  COMPLEXADD(ret,ret,ctmp);
  return ret;
}

//function to evaluate the quotient of two polynomials, which is the equation to solve for the general panel interaction
cmplx evalpolyquotient(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  COMPLEXDIV(nom,nom,denom,tmp,tmp2);
  return nom;
}

//function to evaluate the quotient of two polynomials, which is the equation to solve for the general panel interaction
cmplx evalpolyquotientnormalized(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value/pol->endval,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  COMPLEXDIV(nom,nom,denom,tmp,tmp2);
  return nom;
}

//function to evaluate the quotient of two polynomials with the correction due to the slope of the line. Used for numerical integration of the case when slope correction is used
cmplx evalpolyquotientslopecorrection(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  double corr=evalpolyderivreal(pol->a,value,pol->cN);
  corr=sqrt(1+corr*corr);
  COMPLEXDIV(nom,nom,denom,tmp,tmp2);
  COMPLEXASSIGN(nom,nom.re*corr,nom.im*corr);
  return nom;
}

//function to evaluate the quotient of two polynomials with the correction due to the slope of the line. Used for numerical integration of the case when slope correction is used
cmplx evalpolyquotientslopecorrectionnormalized(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value/pol->endval,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  double corr=evalpolyderivreal(pol->a,value,pol->cN);
  corr=sqrt(1+corr*corr);
  COMPLEXDIV(nom,nom,denom,tmp,tmp2);
  COMPLEXASSIGN(nom,nom.re*corr,nom.im*corr);
  return nom;
}

//code that can be used to numerically integrate the contribution from one particle to a panel when the rankine smoother is used
//there may be some numerical errors with this one, maybe due to the numerical integration. error is about 1e-9 compared to reference implementation which should not be an issue in any normal case
cmplx evalpolyquotientrankine(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  tmp=COMPLEXNORM(denom);
  if(tmp>=pol->cutoff) { //expanded COMPLEXDIV as tmp is already calculated
    tmp2=(nom.re*denom.re+nom.im*denom.im)/tmp;
    nom.im=(nom.im*denom.re-nom.re*denom.im)/tmp;
    nom.re=tmp2;
  }
  else {
    nom.re*=pol->shape;
    nom.im*=pol->shape;
    COMPLEXMULC(nom,nom,denom,tmp);
  }
  return nom;
}

//code that can be used to numerically integrate the contribution from one particle to a panel when the scully smoother is used
cmplx evalpolyquotientscully(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  tmp=COMPLEXNORM(denom);
  if(tmp<pol->cutoff) { //add shape to tmp and scale to nominator, then continue as usual
    tmp+=pol->shape;
    nom.re*=pol->scale;
    nom.im*=pol->scale;
  }
  //expanded COMPLEXDIV as tmp is already calculated
  tmp2=(nom.re*denom.re+nom.im*denom.im)/tmp;
  nom.im=(nom.im*denom.re-nom.re*denom.im)/tmp;
  nom.re=tmp2;
  return nom;
}

//code that can be used to numerically integrate the contribution from one particle to a panel when the oseen smoother is used
cmplx evalpolyquotientoseen(double value,void* input) {
  polyquotient* pol=(polyquotient*)input;
  double tmp,tmp2;
  cmplx nom=evalpoly(pol->b,value,pol->bN);
  cmplx denom=evalpoly(pol->c,value,pol->cN);
  tmp=COMPLEXNORM(denom);
  if(tmp!=0) {
    if(tmp<pol->cutoff) { //add shape to tmp and scale to nominator, then continue as usual
      double P=-expm1(-pol->shape*tmp)*pol->scale;
      nom.re*=P;
      nom.im*=P;
    }
  }
  else
  {
    COMPLEXASSIGN(nom,0.0,0.0);
    return nom;
  }
  //expanded COMPLEXDIV as tmp is already calculated
  tmp2=(nom.re*denom.re+nom.im*denom.im)/tmp;
  nom.im=(nom.im*denom.re-nom.re*denom.im)/tmp;
  nom.re=tmp2;
  return nom;
}

//this function is copied from FMM2D and is originally written by Stefan Engblom http://user.it.uu.se/~stefane/freeware.html, kept here for compatibility with the implementation of that code
//funciton is simplified for only the harmonic potential. It is only useful here if not compiled together with FMM2D
void MPexp_smooth_simplified(SMOOTHER smooth,double xopt,bool cont,
                   double *cutoff,double *shape,double *scale)
/* Precomputes the three parameters (cutoff,shape,scale) given
   (pot,smooth,xopt,cutoff,cont). This yields a slicker code for
   mpexp_directInteract(). */
{
  *scale = 1.0;
  switch (smooth) {
  case DIRAC:
    break;

  case RANKINE:
    *cutoff = xopt*xopt;
    *shape = 1.0/(*cutoff);
    break;

  case SCULLY:
    *cutoff = *cutoff**cutoff;
    *shape = xopt*xopt;
    if (cont)
      *scale = 1.0+*shape/(*cutoff);
    break;

  case OSEEN:
    *cutoff = *cutoff**cutoff;
    *shape = 1.2564312086261696770/(xopt*xopt);
    if (cont)
      *scale = -1.0/expm1(-*shape**cutoff);
    break;
  }
}

//uses partial fraction decomposition to remove the contribution of one root from the quotient of the two polynomials
//note: Nnom can be atmost equal to Ndenom and atleast 1
void reduceroot(COMPLEXVECTORRESTRICT  nom,COMPLEXVECTORRESTRICT  denom, cmplx root, cmplx rootconst, int Ndenom,int Nnom) {
  //the reduction equation is ans[m-1]=root*ans[m]+nom[m]-rootconst*denom[m] which is looped. Care is taken if any of the terms should not be included. ans[m] is zero for the first step
  cmplx ctmp,ctmp2,cbak;
  double tmp;
  int i=Ndenom-2;
  if(Ndenom==1) { //Need atleast Ndenom==2 to reduce a root
    COMPLEXASSIGN(nom[0],0.0,0.0);
    return;
  }
  if(Nnom>=Ndenom) { //the case where Nnom==Ndenom, the typical case if bN>=aN-1
    cbak=nom[Ndenom-2];
    COMPLEXMUL(ctmp,rootconst,denom[Ndenom-1],tmp); //first step, exclude the contribution from the last step, as it does not exist
    COMPLEXSUB(nom[Ndenom-2],nom[Ndenom-1],ctmp);
  }
  else { //if bN is smaller, exclude the contributions from nom until there are values to use
    if(Nnom==Ndenom-1) { //if only one less
      cbak=nom[Ndenom-2];
      COMPLEXMULN(nom[Ndenom-2],rootconst,denom[Ndenom-1],tmp); //first step, exclude the contribution from the last step, as it does not exist
    }
    else {
      COMPLEXMULN(nom[Ndenom-2],rootconst,denom[Ndenom-1],tmp); //first step, exclude the contribution from the last step, as it does not exist
      for(;i>Nnom;i--) { //loop until there are values available for nominator
        COMPLEXMUL(ctmp,rootconst,denom[i],tmp);
        COMPLEXMUL(ctmp2,root,nom[i],tmp);
        COMPLEXSUB(nom[i-1],ctmp2,ctmp);
      }
      //last step handled manually, as cbak needs to be set
      cbak=nom[i-1];
      COMPLEXMUL(ctmp, rootconst, denom[i], tmp);
      COMPLEXMUL(ctmp2, root, nom[i], tmp);
      COMPLEXSUB(nom[i-1], ctmp2, ctmp);
      i--;
    }
  }
  for(;i>0;i--) { //the main loop where there are values for all variables
    COMPLEXMUL(ctmp,rootconst,denom[i],tmp);
    COMPLEXSUB(ctmp,ctmp,cbak);
    cbak=nom[i-1];
    COMPLEXMUL(ctmp2, root, nom[i], tmp);
    COMPLEXSUB(nom[i-1],ctmp2,ctmp);
  }
}

//code that allocates a panel, which is a copy of another panel. Note that code does not copy the vectors as well, buth the other elements are copied
//ref is used as the source for the extreme points
void allocateduplicatepanel(generalpanel *dest, generalpanel *source, generalpanel *ref) {
  memcpy(dest,source,sizeof(generalpanel));

  if(dest->a!=NULL)
    dest->a=(COMPLEXVECTOR)myMalloc((dest->a_order+1)*sizeof(cmplx));
  if(dest->b!=dest->b_original&&dest->b_original!=NULL) {
    dest->b_original=(COMPLEXVECTOR)myMalloc((dest->b_original_order+1)*sizeof(cmplx));
    if(dest->b!=NULL)
      dest->b=(COMPLEXVECTOR)myMalloc((dest->b_order+1)*sizeof(cmplx));
  }
  else if(dest->b!=NULL) {
    dest->b=(COMPLEXVECTOR)myMalloc((dest->b_order+1)*sizeof(cmplx));
    dest->b_original=dest->b;
  }
  if(source->b!=source->b_unshifted&&source->b_original!=source->b_unshifted&&source->b_unshifted!=NULL)
    dest->b_unshifted=(COMPLEXVECTOR)myMalloc((dest->b_order+1)*sizeof(cmplx));
  else
    dest->b_unshifted=dest->b;

  if(dest->c!=NULL)
    dest->c=(COMPLEXVECTOR)myMalloc((dest->a_order+1)*sizeof(cmplx));
  if(dest->extremepoints!=NULL)
    dest->extremepoints=(COMPLEXVECTOR)myMalloc(ref->Nextremepoints*sizeof(cmplx));
  if(dest->extremepointval!=NULL)
    dest->extremepointval=(DOUBLEVECTOR)myMalloc(ref->Nextremepoints*sizeof(double));
  if(dest->coeffs!=NULL)
    dest->coeffs=(COMPLEXVECTOR)myMalloc(dest->Ncoeffs*sizeof(cmplx));
}

//adds a panel to a panellist. Automatically reallocates the array if necessary
int addpanel(panellist* panels) {
  if(panels->count==panels->allocated) {
    //checkpanelintegrity(panels);
    panels->allocated=(int)(panels->allocated*1.2)+10;
    generalpanel *newpanels=(generalpanel*)mxMalloc(panels->allocated*sizeof(generalpanel));
    memcpy(newpanels,panels->panels,panels->count*sizeof(generalpanel));
    memset(newpanels+panels->count,0,(panels->allocated-panels->count)*sizeof(generalpanel));
    for(int i=0;i<panels->count;i++) {
      if(panels->panels[i].prev!=NULL)
        newpanels[i].prev=newpanels+(panels->panels[i].prev-panels->panels);
      if(panels->panels[i].next!=NULL)
        newpanels[i].next=newpanels+(panels->panels[i].next-panels->panels);
    }
    mxFree(panels->panels);
    panels->panels=newpanels;
    //checkpanelintegrity(panels);
  }
  panels->count++;
  
  return panels->count-1;
}

//multiplication of two polynomials, creates a new polynomial which is the product of the two input ones
void multiplypoly(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in1,COMPLEXVECTORRESTRICT in2, int N1,int N2) {
  double tmp;
  memset(dest,0,sizeof(cmplx)*(N1+N2-1));
  for(int i=0;i<N1;i++) {
    for(int j=0;j<N2;j++) {
      COMPLEXMADD(dest[i+j],dest[i+j],in1[i],in2[j],tmp);
    }
  }
}

//creates a polynomial which is the square of the input polynomial
void sqrpoly(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in,int N) {
  memset(dest,0,sizeof(cmplx)*(2*N-1));
  for(int i=0;i<N;i++) {
    COMPLEXASSIGN(dest[2*i],dest[2*i].re+in[i].re*in[i].re-in[i].im*in[i].im,dest[2*i].im+2*in[i].re*in[i].im);
    for(int j=i+1;j<N;j++) {
      COMPLEXASSIGN(dest[i+j],dest[i+j].re+2*(in[i].re*in[j].re-in[i].im*in[j].im),dest[i+j].im+2*(in[i].re*in[j].im-in[i].im*in[j].re));
    }
  }
}

//multiplies two polynomials, where both polynomials only have real elements
void multiplypolyreal(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in1,COMPLEXVECTORRESTRICT in2, int N1,int N2) {
  memset(dest,0,sizeof(cmplx)*(N1+N2-1));
  for(int i=0;i<N1;i++) {
    for(int j=0;j<N2;j++) {
      dest[i+j].re+=in1[i].re*in2[j].re;
    }
  }
}

//creates a square polynomial where the input polynomial only have real coefficients
void sqrpolyreal(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in,int N) {
  memset(dest,0,sizeof(cmplx)*(2*N-1));
  for(int i=0;i<N;i++) {
    dest[2*i].re+=in[i].re*in[i].re;
    for(int j=i+1;j<N;j++) {
      dest[i+j].re+=2*in[i].re*in[j].re;
    }
  }
}
//assumes that in2 is real, but not in1
void multiplypolyreal2(COMPLEXVECTORRESTRICT dest,COMPLEXVECTORRESTRICT in1,COMPLEXVECTORRESTRICT in2, int N1,int N2) {
  double tmp;
  memset(dest,0,sizeof(cmplx)*(N1+N2-1));
  for(int i=0;i<N1;i++) {
    for(int j=0;j<N2;j++) {
      COMPLEXASSIGN(dest[i+j],dest[i+j].re+in1[i].re*in2[j].re,dest[i+j].im+in1[i].im*in2[j].re);
    }
  }
}

//calculates log(1+x) for small x using series expansions. Could possibly be made faster with tabulated values for 1/i
cmplx complexlogm1(cmplx x) {
  double dist=COMPLEXNORM(x);
  double conv=dist;
  double tmp;
  if(dist>0.025) {
    cmplx ret;
    x.re+=1.0;
    COMPLEXLOG(ret,x);
    return ret;
  }
  cmplx ret=x;
  cmplx xcum;
  COMPLEXSQR(xcum,x,tmp);
  COMPLEXSUBC(ret,ret,xcum,0.5);
  for(double i=3;dist>1e-17*conv;i+=1.0,dist*=conv) {
    COMPLEXMUL(xcum,xcum,x,tmp);
    tmp=1.0/i;
    COMPLEXADDC(ret,ret,xcum,tmp);

    i+=1.0;
    COMPLEXMUL(xcum,xcum,x,tmp);
    tmp=1.0/i;
    COMPLEXSUBC(ret,ret,xcum,tmp);
  }
  return ret;
}